#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <iomanip>
#include <assert.h>
#include <string.h>

#include "basestat.h"
#include "util.h"

using namespace std;

basestat::basestat(){
  init("");
}

basestat::~basestat(){
  assert(cout << "basestat::~basestat()" << endl);
}

void basestat::clear(){
  init("");
}

void basestat::init(string s){
  A = T = C = G = N = 0;
  a = t = c = g = n = 0;
    
  pos = 0;
  repeat = unknown = false;
    
  name=s;
    
  rs.clear();
  re.clear();
  us.clear();
  ue.clear();

  for(int i=0; i<4; i++) cdf[i].init();
  cdf[0].setcodon("ATG"); // met
  cdf[1].setcodon("TAA"); // stop
  cdf[2].setcodon("TAG"); // stop
  cdf[3].setcodon("TGA"); // stop

  for(int i=0; i<3; i++){
    startF[i].clear();
    startR[i].clear();
    stopF[i].clear();
    stopR[i].clear();
  }
}

void basestat::finishup(){
  // cleanups
  if (unknown) ue.push_back(pos-1);
  if (repeat) re.push_back(pos-1);
}

void basestat::addseq(char* seq){
  seqlen_t l = (seqlen_t)strlen(seq);
  for (seqlen_t i=0; i<l; i++){
    char ch = seq[i];
    
    switch(ch){
    case 'm': case 'r': case 'w': case 's': case 'y': 
    case 'k': case 'v': case 'h': case 'd': case 'b':  
       ch = 'n';
       break;
    case 'M': case 'R': case 'W': case 'S': case 'Y': 
    case 'K': case 'V': case 'H': case 'D': case 'B':  
       ch = 'N';
       break;
    }

    // statistics
    switch(ch){
    case 'A': A++; break;
    case 'T': T++; break;
    case 'C': C++; break;
    case 'G': G++; break;
    case 'a': a++; break;
    case 't': t++; break;
    case 'c': c++; break;
    case 'g': g++; break;
    case 'N': N++; break;
    case 'n': n++; break;
    }


    // unknown region
    switch(ch){
    case 'N': case 'n': 
      if (!unknown){
	unknown = true;
	us.push_back(pos);  }
      break;
    case 'A': case 'T': case 'C': case 'G': 
    case 'a': case 't': case 'c': case 'g': 
      if (unknown){
	unknown = false;
	ue.push_back(pos-1); }
      break;
    }

    // repeat region
    switch(ch){
    case 'A': case 'T': case 'C': case 'G': case 'N':
      if (repeat){
	repeat = false;
	re.push_back(pos-1);  }
      pos++;
      break;
    case 'a': case 't': case 'c': case 'g': case 'n':
      if (!repeat){
	repeat = true;
	rs.push_back(pos);  }
      pos++;
      break;
    }

    // ORF detection
    for(int f=0; f<4; f++){
      seqlen_t b = cdf[f].addbase(ch);
      if (b==0) continue;

      bool fwd = ( b>0 );
      
      if(fwd) b=b-3;
      else b=-(b-1);

      if(f==0){
	if (fwd) startF[b%3].push_back(b);
	else     startR[b%3].push_back(b);
      } else {
	if (fwd) stopF[b%3].push_back(b);
	else     stopR[b%3].push_back(b);
      }
    }
  }
}

int basestat::disp_summary(bool verbose){
  cout << summary(verbose);
  return 1;
}

string basestat::summary(bool verbose){
  ostringstream os;

  // print summary
  seqlen_t total = totalbases();
  seqlen_t nrep =  nonrepeats();
  seqlen_t rep =   repeats();
    

  os.setf(ios::fixed);
  os.precision(2);

  os << "                 Non-repeat"
     << "                   Repeat"
     << "                    Total" << endl;

  os << "---------------------------------------"
     << "---------------------------------------" << endl;

  os << "A: " << setw(14) << itos_comma(A) 
     << " ("  << setw( 6) << (float)A/total*100 << "%)"
     << " "   << setw(14) << itos_comma(a) 
     << " ("  << setw( 6) << (float)a/total*100 << "%)"
     << " "   << setw(14) << itos_comma(a+A) 
     << " ("  << setw( 6) << (float)(a+A)/total*100 << "%)" << endl;
  os << "T: " << setw(14) << itos_comma(T) 
     << " ("  << setw( 6) << (float)T/total*100 << "%)" 
     << " "   << setw(14) << itos_comma(t) 
     << " ("  << setw( 6) << (float)t/total*100 << "%)"
     << " "   << setw(14) << itos_comma(t+T) 
     << " ("  << setw( 6) << (float)(t+T)/total*100 << "%)" << endl;
  os << "C: " << setw(14) << itos_comma(C) 
     << " ("  << setw( 6) << (float)C/total*100 << "%)"
     << " "   << setw(14) << itos_comma(c) 
     << " ("  << setw( 6) << (float)c/total*100 << "%)"
     << " "   << setw(14) << itos_comma(c+C) 
     << " ("  << setw( 6) << (float)(c+C)/total*100 << "%)" << endl;
  os << "G: " << setw(14) << itos_comma(G) 
     << " ("  << setw( 6) << (float)G/total*100 << "%)"
     << " "   << setw(14) << itos_comma(g) 
     << " ("  << setw( 6) << (float)g/total*100 << "%)"
     << " "   << setw(14) << itos_comma(g+G) 
     << " ("  << setw( 6) << (float)(g+G)/total*100 << "%)" << endl;

  os << "N: " << setw(14) << itos_comma(N) 
     << " ("  << setw( 6) << (float)N/total*100 << "%)"
     << " "   << setw(14) << itos_comma(n) 
     << " ("  << setw( 6) << (float)n/total*100 << "%)"
     << " "   << setw(14) << itos_comma(n+N) 
     << " ("  << setw( 6) << (float)(n+N)/total*100 << "%)" << endl;

  os << "---------------------------------------"
     << "---------------------------------------" << endl;

  os << "*: " << setw(14) << itos_comma(nrep) 
     << " ("  << setw( 6) << (float)nrep/total*100 << "%)"
     << " "   << setw(14) << itos_comma(rep) 
     << " ("  << setw( 6) << (float)rep/total*100 << "%)"
     << " "   << setw(14) << itos_comma(nrep+rep) 
     << " ("  << setw( 6) << (float)(nrep+rep)/total*100 << "%)" << endl;


  if(verbose){
    os << "Repeat sequences:" << rs.size() << endl;
    for (int i=0; i<(int)rs.size(); i++)
      os << rs[i] << "-" << re[i] << endl;
    os << endl;
  }

  if(verbose){
    os << "Unknown regions:" << us.size() << endl;
    for (int i=0; i<(int)us.size(); i++)
      os << us[i] << "-" << ue[i] << endl;
    os << endl;
  }

  return os.str();
}

int basestat::disp_summary(){
  disp_summary(false);
  return 1;
}


// ----------------------------------------------------------------------

void codonfind::init(){
  pos=0;
  statF = statR = 0;
}

void codonfind::setcodon(const char* c){
  for(int i=0; i<3; i++){
    cdF[i] = toupper(c[i]);
    cdR[2-i] = ( cdF[i] == 'A' ? 'T' :
		 cdF[i] == 'T' ? 'A' :
		 cdF[i] == 'C' ? 'G' : 'C' );
  }
}

seqlen_t codonfind::addbase(char b){
  // return value is positive:  pos-3 points head of the codon (+ strand)
  // return value is negative: -pos-1 points head of the codon (- strand)
  // return value is 0: no match

  char base = toupper(b);
  if(base!='A' && base!='T' && base!='C' && base!='G' && base!='N') return 0;

  if(cdF[statF] == base) statF++; else statF=0;
  if(cdR[statR] == base) statR++; else statR=0;
  pos++;

  if(statF==3){ statF=0; return  pos; };
  if(statR==3){ statR=0; return -pos; };
  return 0;
}
