#include "colormap.h"
#include <math.h>

//HSV color conversion (taken from google: http://www.cs.rit.edu/~ncs/color/t_convert.html)
// r,g,b values are from 0 to 1
// h = [0,360], s = [0,1], v = [0,1]
//		if s == 0, then h = -1 (undefined)

double MIN(double a,double b,double c){
  if(a<b)
    if(a<c)
      return a;
    else return c;
  else if(b<c)
    return b;
  else return c;
}

double MAX(double a,double b,double c){
  if(a>b)
    if(a>c)
      return a;
    else return c;
  else if(b>c)
    return b;
  else return c;
}

void RGBtoHSV( double r, double g, double b, double *h, double *s, double *v )
{
	double min, max, delta;

	min = MIN( r, g, b );
	max = MAX( r, g, b );
	*v = max;				// v

	delta = max - min;

	if( max != 0 )
		*s = delta / max;		// s
	else {
		// r = g = b = 0		// s = 0, v is undefined
		*s = 0;
		*h = -1;
		return;
	}

	if( r == max )
		*h = ( g - b ) / delta;		// between yellow & magenta
	else if( g == max )
		*h = 2 + ( b - r ) / delta;	// between cyan & yellow
	else
		*h = 4 + ( r - g ) / delta;	// between magenta & cyan

	*h *= 60;				// degrees
	if( *h < 0 )
		*h += 360;

}

void HSVtoRGB( double *r, double *g, double *b, double h, double s, double v )
{
	int i;
	double f, p, q, t;

	if( s == 0 ) {
		// achromatic (grey)
		*r = *g = *b = v;
		return;
	}

	h /= 60;			// sector 0 to 5
	i = (int)floor( h );
	f = h - i;			// factorial part of h
	p = v * ( 1 - s );
	q = v * ( 1 - s * f );
	t = v * ( 1 - s * ( 1 - f ) );

	switch( i ) {
		case 0:
			*r = v;
			*g = t;
			*b = p;
			break;
		case 1:
			*r = q;
			*g = v;
			*b = p;
			break;
	case 2:
			*r = p;
			*g = v;
			*b = t;
			break;
		case 3:
			*r = p;
			*g = q;
			*b = v;
			break;
		case 4:
			*r = t;
			*g = p;
			*b = v;
			break;
		default:		// case 5:
			*r = v;
			*g = p;
			*b = q;
			break;
	}

}

/*
void RGBtoHSV(Vec3f rgb,Vec3f &hsv){
  RGBtoHSV(rgb[0],rgb[1],rgb[2],&hsv[0],&hsv[1],&hsv[2]);
}

void HSVtoRGB(Vec3f hsv,Vec3f &rgb){
  HSVtoRGB(&rgb[0],&rgb[1],&rgb[2],hsv[0],hsv[1],hsv[2]);
}
*/
