#include "confwin.h"
#include "gmv.h"
#include "mainwindow.h"

#include <sstream>

using namespace std;

// ----------------------------------------------------------------------
// handlers

void preferenceDialog::activate(GtkWidget *w){
  if(w==m_f_load){ status.status_copy->load_prefs(); redisp_all(); }
  if(w==m_f_save){ status.status_copy->save_prefs(); redisp_all(); }
  if(w==m_f_reset) response(GTK_RESPONSE_REJECT);
  if(w==m_f_apply) response(GTK_RESPONSE_APPLY);
  if(w==m_f_close) response(GTK_RESPONSE_CANCEL);
}

void preferenceDialog::response(gint response){
  /*
     GTK_RESPONSE_APPLY : apply
     GTK_RESPONSE_OK:     ok
     GTK_RESPONSE_REJECT: defaults
     GTK_RESPONSE_CANCEL: cancel
   */

  switch(response){
  case GTK_RESPONSE_OK:  
  case GTK_RESPONSE_APPLY: 
    if(!confirm_to_flush()) return;

    for(unsigned int i=0; i<status.confwin_tables.size(); i++){
      for(unsigned int j=0; j<status.confwin_tables.at(i).entries.size(); j++){
	configt_entry* oresama = &status.confwin_tables.at(i).entries.at(j);
	GtkWidget* wi = oresama->form;
	switch(oresama->type){
	case CONFIGT_BOOL:
	  *(bool*)oresama->value = gtk_toggle_button_get_active((GtkToggleButton*)wi);
	  break;
	case CONFIGT_DOUBLE: 
	  *(double*)oresama->value=gtk_spin_button_get_value(GTK_SPIN_BUTTON(wi));
	  break;
	case CONFIGT_INT: 
	  *(int*)oresama->value=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(wi));
	  break;
	case CONFIGT_STRING:
	  *(string*)oresama->value = gtk_entry_get_text((GtkEntry*)wi);
	  break;
	}
      }
    }

    mv_status *original, *newone;
    original = &status;
    newone = status.status_copy;
    
    //    cout << "original at " << original <<endl;
    //    cout << "newone at   " << newone << endl;
    original->copy(*newone);

    status.status_copy = newone;
    if(response == GTK_RESPONSE_OK){
      status.confwin_tables.clear();
      handlers.remove_dialog(HANDLER_DIALOG);
      gtk_widget_destroy((GtkWidget*)dialog);
      isOpen = false;
      status.lsb.setByStatus();
      free(newone);
    }
    flush_and_redraw();
    break;
  case GTK_RESPONSE_REJECT: 
    // set default
    for(unsigned int i=0; i<status.confwin_tables.size(); i++){
      status.confwin_tables.at(i).set_defaults();
      status.confwin_tables.at(i).redisp_values();
    }
    break;
  default:              
    status.confwin_tables.clear();
    handlers.remove_dialog(HANDLER_DIALOG);
    gtk_widget_destroy(GTK_WIDGET(dialog));
    isOpen = false;
    free(status.status_copy);
  }
}

void preferenceDialog::redisp_all(){
  for(unsigned int i=0; i<status.confwin_tables.size(); i++){
    status.confwin_tables.at(i).redisp_values();
  }
}


// ----------------------------------------------------------------------
// table management

void config_table::clear(){
  entries.clear();
}

void config_table::set_defaults(){
  assert(cout << "config_table::set_defaults() called." << endl);

  for(unsigned int i=0; i<entries.size(); i++){
    switch(entries[i].type){
    case CONFIGT_BOOL:
      *((bool*)(entries[i].value)) = entries[i].d.b; break;
    case CONFIGT_INT:
      *((int*)(entries[i].value)) = entries[i].d.i; break;
    case CONFIGT_DOUBLE:
      *((double*)(entries[i].value)) = entries[i].d.d; break;
    case CONFIGT_STRING:
      ((string*)(entries[i].value))->assign((entries[i].d.s)->c_str()); 
      break;
    }
  }
  redisp_values();
}

void config_table::redisp_values(){
  //  assert(cout << "config_table::redisp_values() called." << endl);

  for(unsigned int i=0; i<entries.size(); i++){
    switch(entries.at(i).type){
    case CONFIGT_INT:
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(entries.at(i).form),
				(double)(*(int*)entries.at(i).value));
      break;
    case CONFIGT_DOUBLE:
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(entries.at(i).form),
				*(double*)entries.at(i).value);
      break;
    case CONFIGT_STRING: 
      gtk_entry_set_text((GtkEntry*)entries.at(i).form, 
			 (const gchar*)(*(string*)entries.at(i).value).c_str());
      break;
    case CONFIGT_BOOL:
      gtk_toggle_button_set_active((GtkToggleButton*)entries.at(i).form, 
				   *(bool*)entries.at(i).value);
      break;
    }
  }

  //  assert(cout << "config_table::redisp_values() return." << endl);
}

void config_table::add_check(const char *s, bool* b, bool db){
  configt_entry te;

  te.label = gtk_label_new(s);
  te.form  = gtk_check_button_new_with_label("ON");
  gtk_toggle_button_set_active((GtkToggleButton*)te.form, (gboolean)(*b));

  te.d.b = db;
  te.type = CONFIGT_BOOL;
  te.value = (void*)b;
  entries.push_back(te);
}

void config_table::add_entry(const char *s, void* i, int t, 
			     int default_i, double default_d, string default_s,
			     double min, double max, double step){
  configt_entry te;

  //  assert(cout << "config_table::add_entry() called. type=" << t << endl);
  fflush(stdout);

  te.label = gtk_label_new(s);

  switch(t){
  case CONFIGT_STRING:
    te.form  = gtk_entry_new(); 
    break;
  default:
    te.form  = gtk_spin_button_new_with_range(min, max, step); break;
  }

  gtk_entry_set_alignment((GtkEntry*)te.form, 1); // justify to right

  switch(t){
  case CONFIGT_INT:
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(te.form), (double)(*(int*)i));
    break;
  case CONFIGT_DOUBLE: 
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(te.form), *(double*)i);
    break;
  case CONFIGT_STRING: 
    gtk_entry_set_text((GtkEntry*)te.form, ((string*)i)->c_str()); break;
  }
    
  te.type = t;
  te.value = (void*)i;
  
  if(t == CONFIGT_INT) te.d.i = default_i;
  if(t == CONFIGT_DOUBLE ) te.d.d = default_d;
  if(t == CONFIGT_STRING ){ te.d.s = new string; te.d.s->assign(default_s.c_str()); }

  entries.push_back(te);
  //  assert(cout << "config_table::add_entry() return." << endl);
}

void config_table::add_entry(const char *s, string *st, string ds)
{
  //  assert(cout << "config_table::add_entry() for string: default=" << ds << endl);
  add_entry(s, (void*)st, CONFIGT_STRING,   0, 0, ds, 0, 0, 0);
}

void config_table::add_entry(const char *s, int *i, int di, 
			     int min, int max){
  //  assert(cout << "config_table::add_entry() for int: default=" << di << endl);
  add_entry(s, (void*)i, CONFIGT_INT,   di, 0, "", (double)min, (double)max, 1);
}

void config_table::add_entry(const char *s, double *d, double dd,
			     double min, double max){
  //  assert(cout << "config_table::add_entry() for double: default=" << dd << endl);
  add_entry(s, (void*)d, CONFIGT_DOUBLE, 0, dd, "", min, max, 0.1);
}

GtkWidget* config_table::create_table(){
  table = gtk_table_new(entries.size(), 2, true);
    
  for(unsigned int i=0; i<entries.size(); i++){
    gtk_misc_set_alignment(GTK_MISC(entries[i].label), 0, 0.5);
    gtk_table_attach((GtkTable*)table, entries[i].label, 0, 1, i, i+1, 
		     (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 
		     (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 10, 0);
    gtk_table_attach_defaults((GtkTable*)table, entries[i].form,  1, 2, i, i+1);

    /*
    if(entries[i].type != CONFIGT_BOOL)
      g_signal_connect(entries[i].form, "focus_out_event",
		       G_CALLBACK(confwin_checkinput), &(entries[i]));
    */
  }
  return table;
}

// ----------------------------------------------------------------------
// initialize everything

void preferenceDialog::init(GtkWidget *mainWindow){
  isOpen = true;
  assert(cout << "handle_Confbutton(): configure button pressed" << endl);

  mv_status* status_new = (mv_status*)malloc(sizeof(mv_status));
  status.status_copy = status_new;

  status_new->copy(status);

  // basic containers
  GtkWidget *vbox, *frameHbox, *agVbox, *ldVbox;
  GtkWidget *a_frame, *g_frame, *l_frame, *d_frame; // sequence, advanced, general, link, dot

  dialog = gtk_dialog_new_with_buttons ("GMV Preferences",
					GTK_WINDOW(mainWindow),
					GTK_DIALOG_DESTROY_WITH_PARENT,
					GTK_STOCK_APPLY,
					GTK_RESPONSE_APPLY,
					GTK_STOCK_OK,
					GTK_RESPONSE_OK,
					"_Defaults ",
					GTK_RESPONSE_REJECT,
					GTK_STOCK_CANCEL,
					GTK_RESPONSE_CANCEL,
					NULL);

  gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(status.mainwindow));
  gtk_window_set_modal(GTK_WINDOW(dialog), true);
  
  // menubar
  
  GtkWidget *mb;
  GtkWidget *file_menu, *file_item;

  file_menu = gtk_menu_new();
  file_item = gtk_menu_item_new_with_mnemonic("_File");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_item), file_menu);

  m_f_load = gtk_menu_item_new_with_mnemonic("_Load preferences");
  m_f_save = gtk_menu_item_new_with_mnemonic("_Save preferences");
  m_f_reset = gtk_menu_item_new_with_mnemonic("_Reset to default values");
  m_f_apply = gtk_menu_item_new_with_mnemonic("_Apply preferences");
  m_f_close = gtk_menu_item_new_with_mnemonic("_Close");

  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_f_load);
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_f_save);
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_f_reset);
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_f_apply);
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_f_close);

  mb = gtk_menu_bar_new();
  gtk_menu_shell_append(GTK_MENU_SHELL(mb), file_item);

  gtk_widget_show_all(mb);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), mb);


  g_signal_connect(m_f_load, "activate", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(m_f_save, "activate", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(m_f_reset, "activate", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(m_f_apply, "activate", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(m_f_close, "activate", 
		   G_CALLBACK(handle_activate), (gpointer)this);

  handlers.connect(m_f_load, HANDLER_DIALOG);
  handlers.connect(m_f_save, HANDLER_DIALOG);
  handlers.connect(m_f_reset, HANDLER_DIALOG);
  handlers.connect(m_f_apply, HANDLER_DIALOG);
  handlers.connect(m_f_close, HANDLER_DIALOG);

  // setup main contents
  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);

  frameHbox  = gtk_hbox_new(TRUE,0);
  gtk_box_pack_start(GTK_BOX(vbox), frameHbox,  TRUE, TRUE, 0);

  a_frame  = gtk_frame_new("Advanced Options");
  g_frame  = gtk_frame_new("General Options");
  l_frame  = gtk_frame_new("Chrom Link Options");
  d_frame  = gtk_frame_new("Dot Plot Options");

  ldVbox   = gtk_vbox_new(FALSE,0);
  gtk_box_pack_start(GTK_BOX(ldVbox), l_frame,  FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(ldVbox), d_frame,  FALSE, FALSE, 0);

  agVbox   = gtk_vbox_new(FALSE,0);
  gtk_box_pack_start(GTK_BOX(agVbox), g_frame,  FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(agVbox), a_frame,  FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(frameHbox), agVbox,   FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(frameHbox), ldVbox,   FALSE, FALSE, 0);

  g_signal_connect(dialog, "response", 
		   G_CALLBACK(handle_dialogResponse), (gpointer)this);

  // making & filling table layout for each variables
  config_table a_table, g_table, l_table, d_table;

  g_table.add_entry("Default width",          &status_new->default_width,
		    DEFAULT_WIDTH, DEFAULT_WIDTH_MIN, DEFAULT_WIDTH_MAX);  
  g_table.add_entry("Default height",         &status_new->default_height,
		    DEFAULT_HEIGHT, DEFAULT_HEIGHT_MIN, DEFAULT_HEIGHT_MAX);
  g_table.add_entry("Sequence thickness",     &status_new->sequence_thickness,
		    DEFAULT_SEQUENCE_THICKNESS,
		    SEQUENCE_THICKNESS_MIN, SEQUENCE_THICKNESS_MAX);
  g_table.add_entry("ORF zone thickness",     &status_new->orf_thickness,
		    DEFAULT_ORF_THICKNESS,
		    SEQUENCE_THICKNESS_MIN, SEQUENCE_THICKNESS_MAX);
  g_table.add_entry("Plot color saturation",  &status_new->saturation,
		    DEFAULT_SATURATION, SATURATION_MIN, SATURATION_MAX);
  g_table.add_entry("Forward color value",    &status_new->fwdvalue,
		    DEFAULT_VALUE, FWDVALUE_MIN, FWDVALUE_MAX);  
  g_table.add_entry("Backward color value",   &status_new->bwdvalue,
		    DEFAULT_VALUE, BWDVALUE_MIN, BWDVALUE_MAX);  
  g_table.add_entry("Background color value", &status_new->bgvalue,
		    DEFAULT_VALUE, BGVALUE_MIN, BGVALUE_MAX);  
  g_table.add_entry("Alpha channel value (high)",    &status_new->alpha,
		    DEFAULT_ALPHA, ALPHA_MIN, ALPHA_MAX);
  g_table.add_entry("Alpha channel value (low)",    &status_new->alpha_low,
		    DEFAULT_ALPHA_LOW, ALPHA_MIN, ALPHA_MAX);
  g_table.add_check("Display sequence #",     &status_new->disp_seqno     ,true);
  g_table.add_entry("Font size (general)", &status_new->font_size_general,
		    DEFAULT_FONT_SIZE_GENERAL, FONT_SIZE_MIN, FONT_SIZE_MAX);
  g_table.add_entry("Minimal ORF length", &status_new->min_orf,
		    DEFAULT_MINORF, MINORF_MIN, MINORF_MAX);

  gtk_container_add(GTK_CONTAINER(g_frame), g_table.create_table());

  a_table.add_entry("Minimum size",           &status_new->min_size,
		    DEFAULT_MIN_SIZE, MIN_SIZE_MIN, MIN_SIZE_MAX);
  a_table.add_entry("Maximum size",           &status_new->max_size,
		    DEFAULT_MAX_SIZE, MAX_SIZE_MIN, MAX_SIZE_MAX);
  a_table.add_entry("Zoom step factor",       &status_new->zoom_factor,
		    DEFAULT_ZOOM_FACTOR, ZOOM_FACTOR_MIN, ZOOM_FACTOR_MAX);
  a_table.add_check("Antialiasing",           &status_new->antialias,
		    DEFAULT_ANTIALIAS);
  a_table.add_entry("Image cache size (MB)",  &status_new->pbcache_cap,
		    DEFAULT_PBCACHE_CAP, PBCACHE_CAP_MIN, PBCACHE_CAP_MAX);

  gtk_container_add(GTK_CONTAINER(a_frame), a_table.create_table());

  l_table.add_entry("Minimum thickness",      &status_new->min_thickness,
		    DEFAULT_MIN_THICKNESS, MIN_THICKNESS_MIN,MIN_THICKNESS_MAX);
  l_table.add_entry("Maximum thickness",      &status_new->max_thickness,
		    DEFAULT_MAX_THICKNESS, MAX_THICKNESS_MIN,MAX_THICKNESS_MAX);
  l_table.add_entry("Zoomcursor width (bp)",  &status_new->zoomcursor_width_bp,
		    DEFAULT_ZOOMCURSOR_WIDTH_BP,
		    ZOOMCURSOR_WIDTH_BP_MIN, ZOOMCURSOR_WIDTH_BP_MAX);
  l_table.add_entry("Zoomcursor width (px)",  &status_new->zoomcursor_width_px,
		    DEFAULT_ZOOMCURSOR_WIDTH_PX,
		    ZOOMCURSOR_WIDTH_PX_MIN, ZOOMCURSOR_WIDTH_PX_MAX);
  l_table.add_entry("Stroke/Fill value ratio",  &status_new->fill_stroke_ratio,
		    DEFAULT_FILL_STROKE_RATIO, 
		    FILL_STROKE_RATIO_MIN, FILL_STROKE_RATIO_MAX);
  l_table.add_entry("Endcap length (relative)",  &status_new->chrlink_endcap,
		    DEFAULT_CHRLINK_ENDCAP, 
		    CHRLINK_ENDCAP_MIN, CHRLINK_ENDCAP_MAX);
  l_table.add_check("Display range",          &status_new->disp_range ,true);

  l_table.add_check("Show gene markers",  &status_new->show_gene_marker ,true);
  //  l_table.add_check("Show gene names",    &status_new->show_gene_name_forced, false);
  //  FIXME: here should be a combo box for gene_name_onoff_method
  //  l_table.add_check("Auto toggle gene names",  &status_new->show_gene_name_auto ,true);
  l_table.add_entry("Toggle threshold", &status_new->show_gene_name_threshold,
		    DEFAULT_GENE_NAME_THRESHOLD,
		    GENE_NAME_THRESHOLD_MIN, GENE_NAME_THRESHOLD_MAX);

  l_table.add_entry("Font size (Annotation)", &status_new->font_size_gene,
		    DEFAULT_FONT_SIZE_GENE, FONT_SIZE_MIN, FONT_SIZE_MAX);
  l_table.add_entry("Gene marker width", &status_new->genewidth,
		    DEFAULT_GENE_WIDTH, GENE_WIDTH_MIN, GENE_WIDTH_MAX);
  l_table.add_entry("Gene marker spacing", &status_new->genepadding,
		    DEFAULT_GENE_PADDING, GENE_PADDING_MIN, GENE_PADDING_MAX);
  l_table.add_entry("Gene marker margin", &status_new->genemargin,
		    DEFAULT_GENE_MARGIN, GENE_MARGIN_MIN, GENE_MARGIN_MAX);

  gtk_container_add(GTK_CONTAINER(l_frame), l_table.create_table());

  d_table.add_entry("Start-circle radius",    &status_new->dot_radius,
		    DEFAULT_DOT_RADIUS, DOT_RADIUS_MIN, DOT_RADIUS_MAX);
  d_table.add_entry("End-circle radius",      &status_new->dot_radius_end ,
		    DEFAULT_DOT_RADIUS_END, 
		    DOT_RADIUS_END_MIN, DOT_RADIUS_END_MAX);
  d_table.add_entry("Fwd/Bwd hue difference", &status_new->huediff_fb,
		    DEFAULT_HUEDIFF, HUEDIFF_FB_MIN, HUEDIFF_FB_MAX);
  gtk_container_add(GTK_CONTAINER(d_frame), d_table.create_table());

  //  status.confwin_tables.push_back(s_table);
  status.confwin_tables.push_back(g_table);
  status.confwin_tables.push_back(a_table);
  status.confwin_tables.push_back(l_table);
  status.confwin_tables.push_back(d_table);

  status_new->copy(status);

  // register buttons
  GList *buttons, *b;
  buttons = gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area));
  for(b=buttons; b; b=b->next) handlers.connect(b->data, HANDLER_DIALOG);
  g_list_free(buttons);

  // That's all. now show everything.
  gtk_widget_show_all(dialog);
}
