using namespace std;

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <assert.h>

#ifndef __MINGW32__
# include <netinet/in.h>
# include <netdb.h>
# include <sys/socket.h>
#else
# include <winsock2.h>
#endif

#include <glib.h>

#include <iostream>
#include <string>
#include <map>
#include <vector>

#include "dasclient.h"
#include "util.h"

string url_encode(string s){
  string ss="";
  char temp[4];
  
  for(size_t i=0; i<s.size(); i++){
    sprintf(temp, "%%%02x", s[i]);
    ss += temp;
  }
  return ss;
}

// ----------------------------------------------------------------------
// put resulting content in result, and return true if succeeded.
bool get_by_http_session(string*, const char*, int, const char*, int);

bool get_by_http(string* result, const char* host, int port, const char* path){
  return get_by_http_session(result, host, port, path, 0);
}

bool get_by_http_session(string* result, const char* host, 
			 int port, const char* path, int nest_level){
  if(nest_level > 10) return false; // too many forwards

  const int buffer_size = 1024;

  int sock_fd;
  struct sockaddr_in server_sockadr;
  struct hostent *server_hostent;
  gchar *chunk_buf = (gchar*)malloc(buffer_size+1);

  *result="";

  // get server IP Address
  assert(cerr << "get_by_http(): gethostbyname() \"" << host << "\" " << gethostbyname(host) << "\n");
  if( (server_hostent = gethostbyname(host)) == NULL) return false;

  // specify server IP Addres & port
  memset((char*)&server_sockadr, 0, sizeof(server_sockadr));
  server_sockadr.sin_family=PF_INET;
  server_sockadr.sin_port=htons(port);
  memcpy((char*)&server_sockadr.sin_addr, server_hostent->h_addr, 
	server_hostent->h_length);

  // Open TCP Socket, then connect it
  assert(cerr << "get_by_http(): socket()\n");
  if( (sock_fd = socket(PF_INET, SOCK_STREAM, 0)) < 0) return false;
  assert(cerr << "get_by_http(): connect()\n");
  if( (connect(sock_fd, (struct sockaddr*)&server_sockadr,
	       sizeof(server_sockadr))) < 0) return false;

  GIOChannel* sock;
  GError* err;
  gsize size;
  sock = g_io_channel_unix_new(sock_fd);

  err = NULL;
  snprintf(chunk_buf, buffer_size, "GET %s HTTP/1.1\r\nHost: %s\r\n\r\n", path, host);
  //  snprintf(chunk_buf, buffer_size, "GET %s HTTP/1.0\n\n", path); // force non-chunk

  assert(cerr << "get_by_http(): GET method\n");
  g_io_channel_write_chars(sock, chunk_buf, -1, &size, &err);
  g_io_channel_flush(sock, &err);
  if(err != NULL) return false;

  // receive request
  bool receiving_body = false;
  bool receiving = true;
  bool chunked = false;
  gchar *recv_buf;
  int chunk_buf_size = buffer_size;
  int content_length = 0;

  int status_code=0;
  string location; // if forwarded by responce 301 or 302

  assert(cerr << "get_by_http(): receiving...\n");
  do{
    if(!receiving_body){
      assert(cerr << "get_by_http(): header\n");

      g_io_channel_read_line(sock, &recv_buf, &size, NULL, &err);
      if(err != NULL) return false;

      if( recv_buf[0]=='\n' || recv_buf[0]=='\r'){
	if (status_code == 200)
	  receiving_body = true;
	else {
	  break;
	}
      }

      if(strncmp(recv_buf, "Transfer-Encoding: chunked", 26)==0) chunked = true;
      if(strncmp(recv_buf, "Content-Length:", 15)==0){
	char* c = recv_buf;
	while(*c!=' ') c++;
	sscanf(c, " %d", &content_length);
      }

      if(strncmp(recv_buf, "Location:", 9)==0){
	location = recv_buf;
	size_t eol = 0;
	for(int i=1; i<=2; i++){
	  if (location[location.length()-i] == '\r' ||
	      location[location.length()-i] == '\n' ) eol ++;
	}
	location = location.substr(10, location.length()-eol-10);
      }

      if(strncmp(recv_buf, "HTTP", 4)==0){ // succeed?
	char* c = recv_buf;
	while(*c!=' ') c++;
	sscanf(c, " %d", &status_code);

	if( ! (status_code == 200 ||
	       status_code == 301 ||
	       status_code == 302 ) ) return false;
      }

      g_free(recv_buf);
    } else {
      assert(cerr << "get_by_http(): body\n");
      if(chunked){
	// chunked transfer: get chunk size first:
	int chunk_size;
	g_io_channel_read_line(sock, &recv_buf, NULL, NULL, &err);

	//	cerr << "chunked body: " << recv_buf;

	if(err != NULL) return false;
	sscanf(recv_buf, "%x", &chunk_size);

	if(chunk_size==0){ receiving=false; break; }
	
	if(chunk_buf_size < chunk_size+1){
	  chunk_buf=(gchar*)realloc((void*)chunk_buf, chunk_size+1);
	  chunk_buf_size = chunk_size+1;
	}
	g_io_channel_read_chars(sock, chunk_buf, chunk_size, &size, &err);
	if(err != NULL) return false;
	chunk_buf[size]=0;
	*result += (char*)chunk_buf;

	// cr+lf after the chunk must be skipped
	g_io_channel_read_chars(sock, chunk_buf, 2, &size, &err);
	if(err != NULL) return false;
      } else {
	// non-chunked, bulk transfer: just read
	g_io_channel_read_line(sock, &recv_buf, &size, NULL, &err);

	content_length -= size;

	if(err != NULL) return false;
	if(size==0 || content_length==0) break;
	*result += (char*)recv_buf;
	g_free(recv_buf);
      }
    }
  } while(receiving);
  g_io_channel_shutdown(sock, true, NULL);
  close(sock_fd);
   
  if (status_code == 301 || status_code == 302){
    string ho, pa;
    int po;

    if(!parseURL((const string)location, &ho, &po, &pa)) return false;
    return get_by_http_session(result, ho.c_str(), po, pa.c_str(), nest_level+1);
  }

  return true;
}

// ----------------------------------------------------------------------
// parse URL to put host, port and path. return false if failed.
bool parseURL(const string URL, string* host, int *port, string* path){
  *port = 80;

  // URL must begin by "http://"
  if(URL.substr(0, 7) != "http://") return false;

  size_t p;
  if( (p = URL.substr(7).find_first_of(":")) == string::npos){ // no port spec
    if( (p = URL.substr(7).find_first_of("/")) != string::npos){
      *host = URL.substr(7, p);
      *path = URL.substr(7 + p);
    } else return false;
  } else {
    size_t pp;
    if( (pp = URL.substr(7+p+1).find_first_of("/")) != string::npos){
      *host = URL.substr(7, p);
      *port = atoi( URL.substr(7+p+1, pp).c_str() );
      *path = URL.substr(7+p+1 + pp);
    } else return false;
  }

  return true;
}

// ----------------------------------------------------------------------
// abstracted XML parser

xmlParser::xmlParser(){
  // setup parser context
  //   p = { xml_parser_start_element, NULL, xml_parser_text, NULL, NULL};
  p.start_element = xml_parser_start_element;
  p.end_element = xml_parser_end_element;
  p.text = xml_parser_text;
  p.passthrough = xml_parser_text; // is this OK?
  p.error = NULL;
  pc = g_markup_parse_context_new(&p, (GMarkupParseFlags)0,
				  (gpointer)this, NULL);
}

xmlParser::~xmlParser(){
 g_markup_parse_context_free(pc); 
}

void xml_parser_start_element(GMarkupParseContext* context,
			      const gchar* elem, 
			      const gchar** attr_names, 
			      const gchar** attr_vals,
			      gpointer parser, GError** gerr){
  ((xmlParser*)parser)->start_element(context, elem, attr_names, attr_vals, gerr);
}


void xml_parser_end_element(GMarkupParseContext* context, const gchar* elem, 
			    gpointer parser, GError** gerr){
  ((xmlParser*)parser)->end_element(context, elem, gerr);
}

void xml_parser_text(GMarkupParseContext* context, 
		     const gchar* text, 
		     gsize text_len, 
		     gpointer parser, GError** gerr){
  ((xmlParser*)parser)->text(context, text, text_len, gerr);
}



// ----------------------------------------------------------------------
// DSN

void das_dsnParser::start_element(GMarkupParseContext* context, const gchar* elem, 
			      const gchar** attr_names, const gchar** attr_vals,
			      GError** gerr){
  string e = elem;
  string id;

  gchar** an = (gchar**)attr_names;
  gchar** av = (gchar**)attr_vals;
  while(*an!=NULL){
    if(strcasecmp(*an, "id") == 0) id=*av; 
    an++; av++;
  }

  if( strcasecmp(elem,"SOURCE")==0 ) current_id = id;
}

void das_dsnParser::end_element(GMarkupParseContext* context, const gchar* elem, 
			    GError** gerr){

  if(strcasecmp(elem, "DSN")==0) 
    dsnmap->insert(pair<string,das_dsn>(current_id,current_dsn));
}

void das_dsnParser::text(GMarkupParseContext* context, const gchar* text, 
		     gsize text_len, GError** gerr){
  string t = text;
  if(t.find_first_not_of(" \n\r\t") == string::npos) return;
  //cout << "text: \"" << t << "\"" << endl;

  gchar* elem = (gchar*)g_markup_parse_context_get_element(context);
  if(elem != NULL){
    if(strcasecmp(elem, "MAPMASTER") == 0) current_dsn.mapmaster = text;
    if(strcasecmp(elem, "DESCRIPTION") == 0) current_dsn.desc = text;
    if(strcasecmp(elem, "SOURCE") == 0) current_dsn.source = text;
  }
}

bool das_dsnParser::parse(const string& dsn_string, map<string,das_dsn>& ds){
  GError *gerr = NULL;
  dsnmap = &ds;
  g_markup_parse_context_parse(pc, dsn_string.c_str(),
			       dsn_string.size(), &gerr);
  if(gerr != NULL) return false;
  return true;
}

// ----------------------------------------------------------------------
// entry_points parser

void das_entrypointParser::start_element(GMarkupParseContext* context, 
				     const gchar* elem, 
				     const gchar** attr_names, 
				     const gchar** attr_vals,
				     GError** gerr){
  gchar** an = (gchar**)attr_names;
  gchar** av = (gchar**)attr_vals;
  if(strcasecmp(elem, "SEGMENT")==0){
    while(*an!=NULL){
      if(strcasecmp(*an, "id")==0) current_id = *av;
      if(strcasecmp(*an, "start")==0)
	current_ep.start = (unsigned)atol(*av);
      if(strcasecmp(*an, "stop")==0) 
	current_ep.stop = (unsigned)atol(*av);
      if(strcasecmp(*an, "orientation")==0) 
	current_ep.reverse = (*av[0]=='-');
      if(strcasecmp(*an, "subparts")==0) 
	current_ep.subparts = (strcasecmp(*av, "no") != 0);
      an++; av++;
    }
  }
}

void das_entrypointParser::end_element(GMarkupParseContext* context,
				   const gchar* elem, 
				   GError** gerr){
  if(strcasecmp(elem, "SEGMENT")==0) 
    epmap->insert(pair<string,das_entrypoint>(current_id,current_ep));
}

void das_entrypointParser::text(GMarkupParseContext* context, const gchar* text, 
			    gsize text_len, GError** gerr){
  string t = text;
  if(t.find_first_not_of(" \n\r\t") == string::npos) return;
  current_ep.segment = t;
}

bool das_entrypointParser::parse(const string& dsn_string, 
			     map<string,das_entrypoint> &m){
  epmap = &m;
  GError *gerr = NULL;
  g_markup_parse_context_parse(pc, dsn_string.c_str(),
			       dsn_string.size(), &gerr);
  if(gerr != NULL) return false;
  return true;
}

// ----------------------------------------------------------------------
// types parser

void das_typeParser::start_element(GMarkupParseContext* context, const gchar* elem, 
			       const gchar** attr_names, 
			       const gchar** attr_vals, GError** gerr){
  gchar** an = (gchar**)attr_names;
  gchar** av = (gchar**)attr_vals;
  if(strcasecmp(elem, "TYPE")==0){
    while(*an!=NULL){
      if(strcasecmp(*an, "id")==0) current_id = *av;
      if(strcasecmp(*an, "category")==0) current_dt.category = *av;
      an++; av++;
    }
  }
}

void das_typeParser::end_element(GMarkupParseContext* context, const gchar* elem, 
			     GError** gerr){
  if(strcasecmp(elem, "TYPE")==0) 
    dtmap->insert(pair<string,das_type>(current_id,current_dt));
};

void das_typeParser::text(GMarkupParseContext* context, const gchar* text, 
		      gsize text_len, GError** gerr){
  string t = text;
  if(t.find_first_not_of(" \n\r\t") == string::npos) return;
  current_dt.type = t;
}

bool das_typeParser::parse(const string& dsn_string, map<string,das_type> &m){
  dtmap = &m;
  GError *gerr = NULL;
  g_markup_parse_context_parse(pc, dsn_string.c_str(),
			       dsn_string.size(), &gerr);
  if(gerr != NULL) return false;
  return true;
}

// ----------------------------------------------------------------------
// features parser


void das_featureParser::start_element(GMarkupParseContext* context, 
				   const gchar* elem, 
				   const gchar** attr_names,
				   const gchar** attr_vals,
				   GError** gerr){
  gchar** an = (gchar**)attr_names;
  gchar** av = (gchar**)attr_vals;

  if(strcasecmp(elem, "FEATURE")==0){
    current_f.clear();
    while(*an!=NULL){
      if(strcasecmp(*an, "id")==0) current_f.id = *av;
      if(strcasecmp(*an, "label")==0) current_f.label = *av;
      an++; av++; }
  }

  if(strcasecmp(elem, "TYPE")==0){
    while(*an!=NULL){
      if(strcasecmp(*an, "id")==0) current_f.type = *av;
      an++; av++; }
  }

  if(strcasecmp(elem, "LINK")==0){
    while(*an!=NULL){
      if(strcasecmp(*an, "href")==0) current_f.url.push_back("DAS_URL:" + (string)*av);
      an++; av++; }
  }

  if(strcasecmp(elem, "GROUP")==0){
    while(*an!=NULL){
      if(strcasecmp(*an, "id")==0) current_f.group = *av;
      an++; av++; }
  }
}

void das_featureParser::end_element(GMarkupParseContext* context, const gchar* elem, 
				 GError** gerr){
  if(strcasecmp(elem, "FEATURE")==0) 
    fvec->push_back(current_f);
}

void das_featureParser::text(GMarkupParseContext* context, const gchar* text, 
			  gsize text_len, GError** gerr){
  string t = text;
  if(t.find_first_not_of(" \n\r\t") == string::npos) return;


  gchar* elem = (gchar*)g_markup_parse_context_get_element(context);
  if(elem != NULL){
    if(strcasecmp(elem, "ORIENTATION") == 0) 
      current_f.reverse = ( text[0]=='-' );
    if(strcasecmp(elem, "START") == 0) current_f.start=(atoll(text));
    if(strcasecmp(elem, "STOP")  == 0) current_f.stop =(atoll(text));
    if(strcasecmp(elem, "END")   == 0) current_f.stop =(atoll(text));
    if(strcasecmp(elem, "SCORE") == 0) current_f.score=(atof(text));
  }
}

bool das_featureParser::parse(const string& dsn_string, vector<das_feature> &m){
  fvec = &m;
  GError *gerr = NULL;
  g_markup_parse_context_parse(pc, dsn_string.c_str(),
			       dsn_string.size(), &gerr);
  if(gerr != NULL) return false;
  return true;
}



// ----------------------------------------------------------------------

/*

int main(int argc, char *argv[]){
  string host, path;
  int port;

  string das_data;
  
  string source;
  vector<string> type_requests;
  string segment;
  int segment_from, segment_to;

  // dummy source configuration
  source="hg18";

  type_requests.push_back("knownGene");
  type_requests.push_back("refGene");

  segment = "1";
  segment_from = 1;
  segment_to = 10000;

  // ----------------------------------------------------------------------
  // getting started with DSN entry URL


  //  string dsn_url = "http://yasu/who/biography.html.ja";
  // string dsn_url = "http://das.hgc.jp/cgi-bin/das/kegg/dsn";
  string dsn_url = "http://genome.cse.ucsc.edu/cgi-bin/das/dsn";

  if( !parseURL(dsn_url.c_str(), &host, &port, &path)){
    cout << "URL parse error" << endl;
    return -1;
  }

  // ----------------------------------------------------------------------
  // DSN entry 

  // get dsn by HTTP
  if(!get_by_http(&das_data, host.c_str(), port, path.c_str())) {
    cout << "http error" << endl;
    return -1;
  }

  // parse dsn
  das_dsnParser dp;
  map<string,das_dsn> dsnmap;
  if(!dp.parse(das_data, dsnmap)){
    cout << "DSN parse error" << endl;
    return -1;
  }

  // dump dsn
//   for(map<string,das_dsn>::iterator i=dsnmap.begin(); i!=dsnmap.end(); i++){
//     cout << i->first << "\t" << i->second.source << "\t" << i->second.mapmaster << "\t" << i->second.desc << endl;
//   }

  // ----------------------------------------------------------------------
  // get entry_points

  map<string,das_entrypoint> epmap;

  // parse URL
  if( !parseURL(dsnmap[source].mapmaster+"/entry_points", &host, &port, &path)){
    cout << "URL parse error" << endl;
    return -1;
  }

  // get by HTTP
  if(!get_by_http(&das_data, host.c_str(), port, path.c_str())){
    cout << "http error" << endl;
    return -1;
  }

  // parse it
  das_entrypointParser ep;
  if(!ep.parse(das_data, epmap)){
    cout << "Das_Entrypoints parse error" << endl;
    return -1;
  }

//   for(map<string,das_entrypoint>::iterator i=epmap.begin(); i!=epmap.end(); i++){
//     cout << i->first << "\t" << i->second.start << "-" << i->second.stop << "\t" << i->second.segment << endl;
//   }

  // ----------------------------------------------------------------------
  // get types

  map<string,das_type> dtmap;

  // parse URL
  if( !parseURL(dsnmap[source].mapmaster+"/types", &host, &port, &path)){
    cout << "URL parse error" << endl;
    return -1;
  }

  // get by HTTP
  if(!get_by_http(&das_data, host.c_str(), port, path.c_str())){
    cout << "http error" << endl;
    return -1;
  }

  // parse it
  das_typeParser tp;
  if(!tp.parse(das_data, dtmap)){
    cout << "Entrypoints parse error" << endl;
    return -1;
  }

//   for(map<string,das_type>::iterator i=dtmap.begin(); i!=dtmap.end(); i++){
//     cout << i->first << "\t" << i->second.category << "\t" << i->second.type << endl;
//   }

  // ----------------------------------------------------------------------
  // get features

  vector<das_feature> fvec;

  // parse URL
  char feature_range[100];
  snprintf(feature_range, 99, ":%d,%d", segment_from, segment_to);

  string feature_URL = ( dsnmap[source].mapmaster + "/features?segment=" + 
 			 segment + feature_range );

  for(vector<string>::iterator i=type_requests.begin(); i!=type_requests.end();
      i++) feature_URL += ";type=" + *i;
  
   //  string feature_URL = "http://localhost/~yasu/features.txt"; // for debug
  
  if( !parseURL(feature_URL, &host, &port, &path)){
    cout << "URL parse error" << endl;
    return -1;
  }

  // get by HTTP
  if(!get_by_http(&das_data, host.c_str(), port, path.c_str())){
    cout << "http error" << endl;
    return -1;
  }


  // parse it
  das_featureParser fp;
  if(!fp.parse(das_data, fvec)){
    cout << "Entrypoints parse error" << endl;
    return -1;
  }
  das_data = ""; // cleanup required to save memory


  for(vector<das_feature>::iterator i=fvec.begin(); i!=fvec.end(); i++){
    cout 
      << i->id << "\t"
      << i->type << "\t"
      << i->start << "\t"
      << i->stop << "\t"
      << ( i->reverse ? '-' : '+' ) << endl;
  }
}

*/
