using namespace std;


#include <gtk/gtk.h>
#include <string>
#include <set>
#include <ctype.h>
#include <assert.h>

#include <iostream>

#include "dialogs.h"
#include "messages.h"
#include "gmv_defs.h"
#include "gmv.h"
#include "opts.h"
#include "dblinks.h"
#include "util.h"
#include "annotation.h"

// ----------------------------------------------------------------------
// report formatter

void report::add_internal(string &sss, bool &h, unsigned &ff, string ss){ 
  if(!h) sss+="\t"; 
  else h=false; 
  sss+=ss;
    
  if(fw.size()<ff+1) fw.push_back(ss.length());
  else if(fw[ff]<ss.length()) fw[ff]=ss.length();
  ff++;
}

void report::add_header(string ss){
  add_internal(header, head_header, fh, ss); 
}

void report::add(string ss){
  add_internal(s, head, f, ss);
}

PangoTabArray* report::tabs(GtkWidget* w){ 
  PangoContext *pc;
  PangoLayout *textlayout;
  PangoFontDescription *desc;
    
  pc = gtk_widget_create_pango_context(w);
  textlayout = pango_layout_new(pc);
  
  pango_layout_set_text(textlayout, "WWWWWWWW", -1);
  desc = pango_font_description_from_string("Courier,Fixed 10");
  pango_layout_set_font_description(textlayout, desc);

  int width, height;
  pango_layout_get_size (textlayout, &width, &height);
  double cw = (double)width/7.5/PANGO_SCALE;

  pango_font_description_free(desc);
  g_object_unref(textlayout);
  g_object_unref(pc);

  PangoTabArray* ta=pango_tab_array_new(fw.size(), true);
  unsigned int p=0;
  for(unsigned i=0; i<fw.size(); i++){
    p=p+fw[i]+2;
    pango_tab_array_set_tab(ta, i, PANGO_TAB_LEFT, (gint)(p*cw));
  }
  return ta;
};


// ----------------------------------------------------------------------
// report dialog

gpointer repd_subthread(gpointer dialog){
  string cogpath = ((reportDialog*)dialog)->cog_load_path();
  if( cogpath!="" ){
    mr.loadcog(cogpath);
  }

  vector<filterTrack> filterTracksToCalc;
  vector<annotationTrack*>* filterTracks = 
    &(((reportDialog*)dialog)->filterTracks);

  filterTracks->clear();

  pbar_reset("Calculating anchor scores from expression data...");

  for (unsigned int i=0; i<mr.nseqs(status.toggle_seq); i++){
    for(unsigned int j=0; j<mr.annotations[status.seq_lookup(i)].size(); j++){
      annotationTrack* thistrack =  mr.annotations[status.seq_lookup(i)].at(j);

      assert(cout << "repd_subthread(): s" << i << " t" << j
	     << "(" << thistrack->get_name() << ")\t has: " 
	     << ( thistrack->isTraceable()  ? "traceability, " : "")
	     << ( thistrack->hasScore()     ? "score, " : "")
	     << ( thistrack->filterActive() ? "score filter " : "") 
	     << endl);

      if(thistrack->hasScore()){
	filterTracks->push_back(thistrack);
	if(!thistrack->hasAncScores()) filterTracksToCalc.push_back(filterTrack(thistrack,i));
      }
    }
  }

  for(unsigned i=0; i<filterTracksToCalc.size(); i++){
    pbar_set_task((double)i/filterTracksToCalc.size(),
		  (double)(i+1)/filterTracksToCalc.size());
    filterTracksToCalc[i].first->
      calcAncScores(filterTracksToCalc[i].second, pbar_set_fraction);
  }

  // ------------------------------
  pbar_reset("Calculating gene scores from expression data...");

  filterTracksToCalc.clear();
  for (unsigned int i=0; i<mr.nseqs(status.toggle_seq); i++){
    for(unsigned int j=0; j<mr.annotations[status.seq_lookup(i)].size(); j++){
      annotationTrack* thistrack =  mr.annotations[status.seq_lookup(i)].at(j);

      if(thistrack->hasScore()){
	if(!thistrack->checkAnnotExpScores(i)) 
	  filterTracksToCalc.push_back(filterTrack(thistrack,i));
      }
    }
  }

  //  cout << filterTracksToCalc.size() << " tracks to calc" << endl;

  for(unsigned i=0; i<filterTracksToCalc.size(); i++){
    pbar_set_task((double)i/filterTracksToCalc.size(),
		  (double)(i+1)/filterTracksToCalc.size());
    filterTracksToCalc[i].first->
      calcAnnotExpScores(filterTracksToCalc[i].second, pbar_set_fraction);
  }

  // ------------------------------

  pbar_reset("Applying anchors and their scores on annotation data...");

  filterTracksToCalc.clear();
  for (unsigned int i=0; i<mr.nseqs(status.toggle_seq); i++){
    for(unsigned int j=0; j<mr.annotations[status.seq_lookup(i)].size(); j++){
      annotationTrack* thistrack =  mr.annotations[status.seq_lookup(i)].at(j);
      if( thistrack->isFilterable() )
	filterTracksToCalc.push_back(filterTrack(thistrack,i));
    }
  }

  for(unsigned i=0; i<filterTracksToCalc.size(); i++){
    pbar_set_task((double)i/filterTracksToCalc.size(),
		 (double)(i+1)/filterTracksToCalc.size());
    filterTracksToCalc[i].first->
      calcAnnotScores(filterTracksToCalc[i].second, pbar_set_fraction);
  }

  // ------------------------------

  ((reportDialog*)dialog)->generateReports();
  ((reportDialog*)dialog)->subthread_terminated();
  return NULL;
}

bool repd_idle(gpointer dialog){
  if(((reportDialog*)dialog)->is_thread_running()){

    set_progbar();
  } else {
    ((reportDialog*)dialog)->showit();
    gtk_progress_bar_set_fraction((GtkProgressBar*)status.progbar, 0);
    gtk_progress_bar_set_text((GtkProgressBar*)status.progbar, "Idle");
    gtk_widget_hide(status.progbar);
    return false;
  }
  return true;
}

void reportDialog::showit(){
  if(!widgets_ready){
    gtk_widget_destroy(pleaseWait);

    // ------------------------------
    // prepare notebook 
    nb = gtk_notebook_new();
    gtk_notebook_set_scrollable(GTK_NOTEBOOK(nb), true);
    gtk_box_pack_start_defaults(GTK_BOX(GTK_DIALOG(dialog)->vbox), nb);

    // source reports
    vector<string> sources;
    for(unsigned s=0; s<mr.acols()/3; s++){
      if(mr.sources_size(s) > 0) 
	sources.push_back("Seq #" + itos(s+1)+ ": segments");
    }
  
    // new tracks & prepare reports
    int tracks=0;
    vector<string> tracknames;
    reportable.clear();

    for(unsigned s=0; s<mr.acols()/3; s++){
      for(rvector<annotationTrack*>::iterator t=mr.annotations[s].begin_all();
	  t!=mr.annotations[s].end_all(); t++){
	if( (*t)->size() != 0){
	  tracks++;
	  tracknames.push_back("Seq #" + itos(s+1) + ": " + (*t)->get_name());
	  reportable.push_back( (*t)->isFilterable() );
	}
      }
    }

    reports=(GtkTextBuffer**)malloc(sizeof(GtkTextBuffer*) * (tracks+specialReports));
    views=(GtkTextView**)malloc(sizeof(GtkTextView*) * (tracks+specialReports));
    selectors=(GtkWidget**)malloc(sizeof(GtkWidget*) * (tracks+specialReports));

    PangoFontDescription* pfd = pango_font_description_from_string("Courier,Fixed 10");
  
    int pageno=0;
    for(int i=0; i<tracks+specialReports; i++){
      if(i>=specialReports && !reportable[i-specialReports]) continue;
      GtkWidget* l = 
	gtk_label_new((i==0) ? "Anchors" :
		      (i==gcrep_page) ? "Gene Connection" :
		      (i>=srep_firstpage && i<specialReports) ? 
		      sources[i-srep_firstpage].c_str():
		      tracknames[i-specialReports].c_str());

      GtkWidget* tv = gtk_text_view_new_with_buffer(NULL);
      views[pageno]   = GTK_TEXT_VIEW(tv);
      reports[pageno] = gtk_text_view_get_buffer(GTK_TEXT_VIEW(tv));
      gtk_text_view_set_editable(GTK_TEXT_VIEW(tv), false);
      gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(tv), false);

      gtk_widget_modify_font(tv, pfd);

      GtkWidget *sw = gtk_scrolled_window_new(NULL,NULL);

      gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), tv);
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), 
				     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
      
      gtk_notebook_append_page(GTK_NOTEBOOK(nb), sw, l);
      pageno++;
    }
    pango_font_description_free(pfd);

    // ------------------------------
    // prepare general options
    GtkWidget *optionFrame = gtk_frame_new("Report Options");
    GtkWidget *optionFrameBox = gtk_vbox_new(false, 0);

    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), optionFrame, 
		       false, false, 0);
    gtk_container_add(GTK_CONTAINER(optionFrame), optionFrameBox);

    GtkWidget *optionBox = gtk_table_new(1, 3, true);
    hideInvisible = gtk_check_button_new_with_label("Hide invisibles");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(hideInvisible), true);
    showScoreDetail = gtk_check_button_new_with_label("Show score details");
    showFeatureTypes = gtk_check_button_new_with_label("Show feature types");
    applyOptions = gtk_button_new_with_label("Apply Options");
    saveButton = gtk_button_new_with_label("Save Report(s)");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(hideInvisible), 
				 hide_invisible);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(showScoreDetail), 
				 show_score_detail);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(showFeatureTypes), 
				 show_feature_types);
  
    gtk_table_attach_defaults(GTK_TABLE(optionBox), hideInvisible, 
			      0, 1, 0, 1);
    gtk_table_attach_defaults(GTK_TABLE(optionBox), showScoreDetail, 
			      1, 2, 0, 1);
    gtk_table_attach_defaults(GTK_TABLE(optionBox), showFeatureTypes, 
			      2, 3, 0, 1);

    gtk_box_pack_start(GTK_BOX(optionFrameBox), optionBox, 
		       false, false, 0);

    handlers.connect(hideInvisible, HANDLER_REPD);
    handlers.connect(showScoreDetail, HANDLER_REPD);
    handlers.connect(showFeatureTypes, HANDLER_REPD);

    // ------------------------------
    // COG reference options

    GtkWidget *cog_option_expander = 
      gtk_expander_new_with_mnemonic("COG reference options (requires downloaded COG database file)");
    GtkWidget *cog_option_expander_box = gtk_hbox_new(false, 0);
    cog_option_box = gtk_table_new(1, 3, true);

    GtkWidget *dummyLabel = gtk_label_new("");
    gtk_widget_set_size_request(dummyLabel, 40, 0);

    gtk_box_pack_start(GTK_BOX(cog_option_expander_box), dummyLabel, 
		       false, false, 0);
    gtk_box_pack_end(GTK_BOX(cog_option_expander_box), cog_option_box, 
		     true, true, 0);
    gtk_container_add(GTK_CONTAINER(cog_option_expander), 
		      cog_option_expander_box);
    gtk_box_pack_start(GTK_BOX(optionFrameBox), cog_option_expander, 
		       false, false, 0);

    // ------------------------------
    // Feature filtering
    unsigned fs = features_vec.size();
    GtkWidget *featureFilterBox = 
      gtk_expander_new_with_mnemonic("Filter annotations by feature");
    gtk_box_pack_start(GTK_BOX(optionFrameBox), featureFilterBox, 
		       false, false, 0);

    dummyLabel = gtk_label_new("");
    gtk_widget_set_size_request(dummyLabel, 40, 0);

    GtkWidget *featureBox = gtk_hbox_new(false, 0);
    GtkWidget *featureTable = gtk_table_new(1, 4, true);
    gtk_container_add(GTK_CONTAINER(featureFilterBox), featureBox);
    gtk_box_pack_start(GTK_BOX(featureBox), dummyLabel, false, false, 0);
    gtk_box_pack_end(GTK_BOX(featureBox), featureTable, true, true, 0);

    feature_checks=(GtkCheckButton**)malloc(sizeof(GtkCheckButton*) * fs);
    for(unsigned i=0; i<fs; i++){
      GtkWidget *tb = gtk_check_button_new_with_label(features_vec[i].c_str());
      feature_checks[i]=GTK_CHECK_BUTTON(tb);
      gtk_table_attach_defaults(GTK_TABLE(featureTable), tb,
				i%4, i%4+1, i/4, i/4+1);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tb), features_en[features_vec[i]]);
    }

    feature_filter_all_on   = gtk_button_new_with_label("On all features");
    feature_filter_all_off = gtk_button_new_with_label("Off all features");

    int button_ypos = ( fs%4 < 3 ) ? fs/4 : fs/4+1;

    gtk_table_attach_defaults(GTK_TABLE(featureTable), 
			      feature_filter_all_on, 2, 3, 
			      button_ypos, button_ypos+1);
    gtk_table_attach_defaults(GTK_TABLE(featureTable), 
			      feature_filter_all_off, 3, 4, 
			      button_ypos, button_ypos+1);

    // signals
    g_signal_connect(feature_filter_all_on, "clicked", 
		     G_CALLBACK(handle_activate), (gpointer)this);
    g_signal_connect(feature_filter_all_off, "clicked", 
		     G_CALLBACK(handle_activate), (gpointer)this);

    handlers.connect(feature_filter_all_on,  HANDLER_REPD);
    handlers.connect(feature_filter_all_off, HANDLER_REPD);

    for(unsigned i=0; i<fs; i++) 
      handlers.connect(feature_checks[i], HANDLER_REPD);

    // ------------------------------
    // Gene Connection Report Filters
    if(gcrep_page != -1){
      GtkWidget *gcrepFilterTable;
      GtkWidget *gcrepFilterBox = 
	gtk_expander_new_with_mnemonic("Gene connection report filter");
      gtk_box_pack_start(GTK_BOX(optionFrameBox), gcrepFilterBox, 
			 false, false, 0);

      dummyLabel = gtk_label_new("");
      gtk_widget_set_size_request(dummyLabel, 40, 0);

      featureBox = gtk_hbox_new(false, 0);
      gcrepFilterTable = gtk_table_new(3, 3, true);
      gtk_container_add(GTK_CONTAINER(gcrepFilterBox), featureBox);
      gtk_box_pack_start(GTK_BOX(featureBox), dummyLabel, false, false, 0);
      gtk_box_pack_end(GTK_BOX(featureBox), gcrepFilterTable, true, true, 0);

      radio_num_anc = gtk_radio_button_new_with_label(NULL, "# of anchors");
      scale_num_anc = (num_anc_min != num_anc_max) ?
	gtk_hscale_new_with_range((gdouble)num_anc_min, 
				  (gdouble)num_anc_max, 1) : NULL;
      spbtn_num_anc = gtk_spin_button_new_with_range((gdouble)num_anc_min, (gdouble)num_anc_max, 1);
      gtk_table_attach_defaults(GTK_TABLE(gcrepFilterTable), radio_num_anc,
				0, 1, 0, 1);
      gtk_table_attach_defaults(GTK_TABLE(gcrepFilterTable), spbtn_num_anc,
				1, 2, 0, 1);
      if(scale_num_anc != NULL){
	gtk_table_attach_defaults(GTK_TABLE(gcrepFilterTable), scale_num_anc,
				  2, 3, 0, 1);

	gtk_scale_set_draw_value(GTK_SCALE(scale_num_anc), false);
	g_signal_connect(scale_num_anc, "value-changed",
			 G_CALLBACK(handle_activate), (gpointer)this);

	handlers.connect(scale_num_anc, HANDLER_REPD);
      }

      g_signal_connect(spbtn_num_anc, "value-changed",
 		       G_CALLBACK(handle_activate), (gpointer)this);

      handlers.connect(radio_num_anc, HANDLER_REPD);
      handlers.connect(spbtn_num_anc, HANDLER_REPD);
      
      // if tf-idf score is available:
      if( !mr.tfidf.empty() ){
	radio_tfidf_avg = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio_num_anc), "tfidf average");
	radio_tfidf_sum = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(radio_num_anc), "tfidf sum");
	
	double log_tfidf_avg_min = log10(tfidf_avg_min)-0.1;
	double log_tfidf_avg_max = log10(tfidf_avg_max)+0.1;
	double log_tfidf_sum_min = log10(tfidf_sum_min)-0.1;
	double log_tfidf_sum_max = log10(tfidf_sum_max)+0.1;

	scale_tfidf_avg = gtk_hscale_new_with_range(log_tfidf_avg_min, log_tfidf_avg_max, (log_tfidf_avg_max-log_tfidf_avg_min)/1000);
	scale_tfidf_sum = gtk_hscale_new_with_range(log_tfidf_sum_min, log_tfidf_sum_max, (log_tfidf_sum_max-log_tfidf_sum_min)/1000);
	spbtn_tfidf_avg = gtk_spin_button_new_with_range(tfidf_avg_min, tfidf_avg_max, 0.01);
	spbtn_tfidf_sum = gtk_spin_button_new_with_range(tfidf_sum_min, tfidf_sum_max, 0.01);

	gtk_scale_set_draw_value(GTK_SCALE(scale_tfidf_avg), false);
	gtk_scale_set_draw_value(GTK_SCALE(scale_tfidf_sum), false);

	gtk_table_attach_defaults(GTK_TABLE(gcrepFilterTable), radio_tfidf_avg,
				  0, 1, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(gcrepFilterTable), spbtn_tfidf_avg,
				  1, 2, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(gcrepFilterTable), scale_tfidf_avg,
				  2, 3, 1, 2);
	gtk_table_attach_defaults(GTK_TABLE(gcrepFilterTable), radio_tfidf_sum,
				  0, 1, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(gcrepFilterTable), spbtn_tfidf_sum,
				  1, 2, 2, 3);
	gtk_table_attach_defaults(GTK_TABLE(gcrepFilterTable), scale_tfidf_sum,
				  2, 3, 2, 3);

	g_signal_connect(scale_tfidf_avg, "value-changed",
			 G_CALLBACK(handle_activate), (gpointer)this);
	g_signal_connect(scale_tfidf_sum, "value-changed",
			 G_CALLBACK(handle_activate), (gpointer)this);

	g_signal_connect(spbtn_tfidf_avg, "value-changed",
			 G_CALLBACK(handle_activate), (gpointer)this);
	g_signal_connect(spbtn_tfidf_sum, "value-changed",
			 G_CALLBACK(handle_activate), (gpointer)this);
	
	handlers.connect(radio_tfidf_avg, HANDLER_REPD);
	handlers.connect(spbtn_tfidf_avg, HANDLER_REPD);
	handlers.connect(scale_tfidf_avg, HANDLER_REPD);
	handlers.connect(radio_tfidf_sum, HANDLER_REPD);
	handlers.connect(spbtn_tfidf_sum, HANDLER_REPD);
	handlers.connect(scale_tfidf_sum, HANDLER_REPD);
      }

      // COG ortholog detection

      if( !mr.has_cog ){
	loadcog_button = gtk_button_new_with_label("Load COG ortholog list \"whog\"");
	gtk_table_attach_defaults(GTK_TABLE(cog_option_box), loadcog_button,
				  0, 1, 0, 1);
	gtk_widget_show_all( cog_option_box );

	g_signal_connect(loadcog_button, "clicked",
			 G_CALLBACK(handle_activate), (gpointer)this);
	handlers.connect(loadcog_button, HANDLER_REPD);
	cog_option_box_ready = false;
      }    
    }

    // buttons
    optionBox = gtk_table_new(1, 3, true);
    gtk_table_attach_defaults(GTK_TABLE(optionBox), applyOptions, 
			      1, 2, 0, 1);
    gtk_table_attach_defaults(GTK_TABLE(optionBox), saveButton, 
			      2, 3, 0, 1);

    gtk_box_pack_start(GTK_BOX(optionFrameBox), optionBox, 
		       false, false, 0);

    g_signal_connect(applyOptions, "clicked", G_CALLBACK(handle_activate),
		     (gpointer)this);
    g_signal_connect(saveButton, "clicked", G_CALLBACK(handle_activate),
		     (gpointer)this);

    handlers.connect(applyOptions, HANDLER_REPD);
    handlers.connect(saveButton, HANDLER_REPD);

    gtk_widget_set_size_request(dialog, -1, 400);
    gtk_widget_show_all(dialog);

    // response
    g_signal_connect(dialog, "response", G_CALLBACK(handle_dialogResponse), 
		     (gpointer)this);
  }

  if( (widgets_ready && loadcog) || (!widgets_ready && mr.has_cog) ){
    // for only when COG is loaded
    check_ortholog = gtk_check_button_new_with_label("Check ortholog by COG");
    show_cog_group = gtk_check_button_new_with_label("Show COG group");
    check_cog_gene = gtk_check_button_new_with_label("Check COG gene");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_ortholog), true);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_cog_group), false);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_cog_gene), false);
    gtk_table_attach_defaults(GTK_TABLE(cog_option_box), check_ortholog,
			      0, 1, 0, 1);
    gtk_table_attach_defaults(GTK_TABLE(cog_option_box), show_cog_group,
			      1, 2, 0, 1);
    gtk_table_attach_defaults(GTK_TABLE(cog_option_box), check_cog_gene,
			      2, 3, 0, 1);

    gtk_widget_show_all(cog_option_box);
    loadcog = false;
    cog_option_box_ready = true;
  } 

  if( !(widgets_ready && loadcog) ){  
    // usually, report display
    displayReports(); 
  }

  handlers.unblock();
  widgets_ready = true;
}

bool reportDialog::is_thread_running(){
  bool r;
  g_mutex_lock(repo_mutex);
  r = thread_running;
  g_mutex_unlock(repo_mutex);
  return r;
}

void reportDialog::subthread_terminated(){
  g_mutex_lock(repo_mutex);
  thread_running = false;
  g_mutex_unlock(repo_mutex);
}

void reportDialog::init(GtkWidget* mainwindow){
  if(isOpen) return;

  repo_mutex = g_mutex_new();
  thread_running = false;
  widgets_ready = false;
  has_gcrep_stat = false;
  loadcog = false;
  cog_option_box_ready = false;

  arep.clear();
  grep.clear();
  srep.clear();

  features.clear();
  features_vec.clear();
  features_en.clear();

  hide_invisible = true;
  show_score_detail = false;
  show_feature_types = false;

  dialog = gtk_dialog_new_with_buttons ("Anchor / Gene filter report",
					GTK_WINDOW(mainwindow),
					GTK_DIALOG_DESTROY_WITH_PARENT,
					GTK_STOCK_CLOSE,
					GTK_RESPONSE_NONE,
					NULL);

  gtk_window_set_modal(GTK_WINDOW(dialog), true);
  
  pleaseWait = gtk_label_new("Generating statistics... Please wait.");
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
		     pleaseWait, false, false, 0);

  gtk_widget_show_all(dialog);

  GList *buttons, *b;
  buttons = gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area));
  for(b=buttons; b; b=b->next) handlers.connect(b->data, HANDLER_REPD);
  g_list_free(buttons);

  handlers.block();

  // launch subthread & idle func
  thread_running = true;
  g_thread_create((GThreadFunc)repd_subthread, this, false, NULL);    
  g_timeout_add(PROGBAR_UPDATE_INTERVAL, (GSourceFunc)repd_idle, (gpointer)this);

  // re-initialize
  specialReports = 0;
  gcrep_page = -1;
  srep_firstpage = -1;

  isOpen = true;
}

void reportDialog::response(gint r){
  if(!isOpen) return;

  gtk_widget_destroy(dialog);
  free(reports);
  free(views);
  free(selectors);
  free(feature_checks);
  g_mutex_free(repo_mutex);
  handlers.remove_dialog(HANDLER_REPD);
  isOpen = false;
}

void reportDialog::displayReports(){
 
  PangoTabArray* ta;
  gtk_text_buffer_set_text(reports[0], arep.str().c_str(), arep.str().size());
  ta = arep.tabs(GTK_WIDGET(views[0]));
  gtk_text_view_set_tabs(views[0], ta);
  pango_tab_array_free(ta);

  if(!gcrep.isEmpty()){
    gtk_text_buffer_set_text(reports[1], gcrep.str().c_str(), gcrep.str().size());

    ta = gcrep.tabs(GTK_WIDGET(views[1]));
    gtk_text_view_set_tabs(views[1], ta);
    pango_tab_array_free(ta);
  }

  //  cout << "srep size " << srep.size() << endl;
  for(unsigned s=0; s<srep.size(); s++){
    unsigned page = s+srep_firstpage;
    gtk_text_buffer_set_text(reports[page],
			     srep[s].str().c_str(), srep[s].str().size());
    ta = srep[s].tabs(GTK_WIDGET(views[page]));
    gtk_text_view_set_tabs(views[page], ta);
    pango_tab_array_free(ta);
  }

  unsigned tc=specialReports;
  for(unsigned s=0; s<mr.acols()/3; s++){
    for(rvector<annotationTrack*>::iterator t=mr.annotations[s].begin_all();
	t!=mr.annotations[s].end_all(); t++){

      if((*t)->isFilterable() && (*t)->size() != 0){
	if(!grep[tc-specialReports].isEmpty()){
	  gtk_text_buffer_set_text(reports[tc], grep[tc-specialReports].str().c_str(), grep[tc-specialReports].str().size());
	  ta = grep[tc-specialReports].tabs(GTK_WIDGET(views[tc]));
	  gtk_text_view_set_tabs(views[tc], ta);
	  pango_tab_array_free(ta);
	} else {
	  gtk_text_buffer_set_text(reports[tc], "", 0);
	}
	tc++;
      }
    }
  }
}

void reportDialog::generateReports(){
  arep.clear();
  grep.clear();
  gcrep.clear();
  
  // ------------------------------
  // anchor filter report
  pbar_reset("Generating anchor filter report...");
  arep = ancReport();

  // ------------------------------
  // gene filter report

  // count filterable tracks
  int tc=0;
  for(unsigned s=0; s<mr.acols()/3; s++){
    for(rvector<annotationTrack*>::iterator t=mr.annotations[s].begin_all();
	t!=mr.annotations[s].end_all(); t++) tc++;
  }

  // generate reports
  int tc2=0;
  for(unsigned s=0; s<mr.acols()/3; s++){
    for(rvector<annotationTrack*>::iterator t=mr.annotations[s].begin_all();
	t!=mr.annotations[s].end_all(); t++){
      tc2++;

      pbar_reset("Generating gene filter report...");
      //      cout << "gene filter report for: " << ((annotationTrack*)*t)->get_name() << endl;
      report gr = geneReport(s, *t, tc2-1, tc);
      //      if(!gr.isEmpty()) grep.push_back(gr);
      grep.push_back(gr);
    }
  }

  if(features_vec.empty()){
    for(set<string>::iterator i=features.begin(); i!=features.end(); i++){
      features_vec.push_back(*i);
      features_en[*i]=true;
    }
  }

  // ------------------------------
  // segment reports

  pbar_reset("Generating segment report...");
  for(unsigned s=0; s<mr.acols()/3; s++){
    srep.push_back(segmentReport(s));
    pbar_set_fraction((double)s/mr.acols());
  }

  // ------------------------------
  // gene connection report

  pbar_reset("Generating gene connection report...");
  gcrep = geneConnectionReport();

  if(specialReports == 0){
    specialReports = 1;
    if (!gcrep.isEmpty()) gcrep_page = specialReports++;
    bool has_multi_sources = false;
    for(int s=0; s<(int)mr.acols()/3; s++){
      if(mr.sources_size(s)>0){
	if(!has_multi_sources) srep_firstpage = specialReports;
	has_multi_sources = true;
	specialReports++;
      }
    }
  }
}

void reportDialog::activate(GtkWidget *w){
  if(w==applyOptions){ 
    for(unsigned i=0; i<features_vec.size(); i++){
      features_en[features_vec[i]] = 
	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(feature_checks[i]));
    }

    hide_invisible = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(hideInvisible));
    show_score_detail = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(showScoreDetail));
    show_feature_types = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(showFeatureTypes ));

    handlers.block();
    thread_running = true;
    g_thread_create((GThreadFunc)repd_subthread, this, false, NULL);    
    g_timeout_add(PROGBAR_UPDATE_INTERVAL, (GSourceFunc)repd_idle, (gpointer)this);
  }

  if(w==feature_filter_all_on){
    for(unsigned i=0; i<features_vec.size(); i++){
      features_en[features_vec[i]]=true;
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(feature_checks[i]), true);
    }
  }

  if(w==feature_filter_all_off){
    for(unsigned i=0; i<features_vec.size(); i++){
      features_en[features_vec[i]]=false;
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(feature_checks[i]), false);
    }
  }

  // ------------------------------
  // gene connection report filter

  if(w==scale_num_anc) 
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spbtn_num_anc),
			      gtk_range_get_value(GTK_RANGE(w)));

  if(w==spbtn_num_anc && scale_num_anc != NULL) 
    gtk_range_set_value(GTK_RANGE(scale_num_anc), 
			gtk_spin_button_get_value(GTK_SPIN_BUTTON(w)));

  if(w==scale_tfidf_avg) 
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spbtn_tfidf_avg),
			      pow(10, gtk_range_get_value(GTK_RANGE(w))));

  if(w==scale_tfidf_sum) 
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spbtn_tfidf_sum),
			      pow(10, gtk_range_get_value(GTK_RANGE(w))));

  if(w==spbtn_tfidf_avg) 
    gtk_range_set_value(GTK_RANGE(scale_tfidf_avg), 
			log10(gtk_spin_button_get_value(GTK_SPIN_BUTTON(w))));
  if(w==spbtn_tfidf_sum) 
    gtk_range_set_value(GTK_RANGE(scale_tfidf_sum), 
			log10(gtk_spin_button_get_value(GTK_SPIN_BUTTON(w))));

  // Load COG
  if(w==loadcog_button){
    GtkWidget *dialog;
    bool file_selected = false;
  
    dialog = gtk_file_chooser_dialog_new ("Load COG ortholog list \"whog\"",
					  GTK_WINDOW(status.mainwindow),
					  GTK_FILE_CHOOSER_ACTION_OPEN,
					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					  GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					  (gpointer)NULL);
  
    GtkFileFilter *filter = gtk_file_filter_new();
    gtk_file_filter_add_pattern(filter, "whog");
    gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filter);

    if (gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
      file_selected = true;

    if(file_selected){
      char *filename;
    
      filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));

      // windows compatibility hack
      char *fn = filename;
      while(*fn != 0){
	if(*fn=='\\') *fn='/';
	fn++;
      }
      // end windows compatibility hack

      cogpath = filename;
      loadcog = true;

      g_free(filename);

      handlers.remove(loadcog_button);
      gtk_widget_destroy(loadcog_button);
    }
    gtk_widget_destroy (dialog);

    // launch subthread & idle func
    if(loadcog){
      thread_running = true;
      g_thread_create((GThreadFunc)repd_subthread, this, false, NULL);    
      g_timeout_add(PROGBAR_UPDATE_INTERVAL, (GSourceFunc)repd_idle, (gpointer)this);
    }
  }

  // ------------------------------
  // save reports

  if(w==saveButton){
    vector<GtkWidget*> pages;
    for(int p=0; p<gtk_notebook_get_n_pages(GTK_NOTEBOOK(nb)); p++){
      pages.push_back(gtk_notebook_get_nth_page(GTK_NOTEBOOK(nb), p));
    }

    GtkWidget *sel;
    sel = gtk_file_chooser_dialog_new("Save Report(s)",
				      GTK_WINDOW(dialog),
				      GTK_FILE_CHOOSER_ACTION_SAVE,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
				      (gpointer)NULL);
    gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(sel), TRUE);
      
    gtk_window_set_modal(GTK_WINDOW(sel), true);

    unsigned int tracks=0;
    vector<string> tracknames;
    for(unsigned s=0; s<mr.acols()/3; s++){
      for(rvector<annotationTrack*>::iterator t=mr.annotations[s].begin_all();
	  t!=mr.annotations[s].end_all(); t++){
	tracknames.push_back("Seq#" + itos(s) + " : " + (*t)->get_name());
	tracks++;
      }
    }

    // options
    GtkWidget *optbox;
    optbox = gtk_table_new(tracks/3+1, 3, false);
    for(unsigned i=0; i<pages.size(); i++){
      GtkWidget* l = gtk_check_button_new_with_label(gtk_label_get_text(GTK_LABEL(gtk_notebook_get_tab_label(GTK_NOTEBOOK(nb), pages[i]))));
      gtk_table_attach_defaults(GTK_TABLE(optbox), l, i%3, i%3+1, i/3, i/3+1);
      if(i==(unsigned)gtk_notebook_get_current_page(GTK_NOTEBOOK(nb)))
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(l), true);
      selectors[i] = l;
    }

    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sel)->vbox), optbox, false, false, 0);
    gtk_widget_show_all(optbox);

    if(gtk_dialog_run(GTK_DIALOG(sel)) == GTK_RESPONSE_ACCEPT){
      char *filename;
      filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(sel));

      ofstream os;
      os.open(filename);
      if(os != NULL){
	for(unsigned i=0; i<pages.size(); i++){
	  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(selectors[i]))){
	    GtkTextIter b, e;
	    gtk_text_buffer_get_start_iter(reports[i], &b);
	    gtk_text_buffer_get_end_iter(reports[i], &e);
	    gchar* rs = gtk_text_buffer_get_text(reports[i], &b, &e, true);
	    os << rs << endl;
	    g_free(rs);
	  }
	}
	os.close();
      }
      g_free(filename);
    }
    gtk_widget_destroy(sel);
  }
}

report reportDialog::ancReport(){
  // ------------------------------
  // anchor report

  report r;
  bool firstline = true;

  string caption;
  for(unsigned s=0; s<mr.acols()/3; s++){
    if(s!=0) caption+=" + ";
    caption += mr.getbs_direct(s)->name;
  }
  r.set_caption(caption+"\n");

  for(unsigned anc=0; anc<mr.arows(); anc++){
    if(!mr.anchorFilter(anc) && hide_invisible) continue;

    // anchors
    for(unsigned c=0; c<mr.acols(); c+=3){
      string ancS = mr.raa_physical(anc,c+2) == 0 ? "-" : "+";
      seqlen_t ancF = mr.raa_physical(anc,c);
      seqlen_t ancT = mr.raa_physical(anc,c+1);
      ancF++; ancT++;

      r.add( itos(ancF) );
      r.add( itos(ancT) );
      r.add( ancS );
    }

    if(firstline){
      for(unsigned c=0; c<mr.acols(); c+=3){
	r.add_header("from");
	r.add_header("to");
	r.add_header("strand");
      }
    }

    // visibility
    r.add( mr.anchorFilter(anc) ? "1" : "0" );
    if(firstline) r.add_header("visible");

    // tf-idf score
    r.add( (!mr.tfidf.empty()) ? itos(mr.tfidf.at(anc)): "0" );
    if(firstline) r.add_header("tfidf");

    // expression scores
    for(vector<annotationTrack*>::iterator ite=filterTracks.begin();
	ite!=filterTracks.end(); ite++){
      // average
      double avg = (*ite)->ancScore(anc);
      r.add( itos( avg ) );

      // variance
      double variance = 0;
      if((*ite)->ancScoreIndivisual_size(anc) != 0){
	for(vector<double>::const_iterator score=(*ite)->ancScoreIndivisual_begin(anc);
	    score!=(*ite)->ancScoreIndivisual_end(anc); score++){
	  variance += (*score - avg) * (*score-avg);
	}
	variance /= (*ite)->ancScoreIndivisual_size(anc);
	r.add( itos( variance ) );
      } else r.add("0");

      if(show_score_detail){
	string scores;
	for(vector<double>::const_iterator score=(*ite)->ancScoreIndivisual_begin(anc);
	    score!=(*ite)->ancScoreIndivisual_end(anc); score++){
	  if(score!=(*ite)->ancScoreIndivisual_begin(anc)) scores+=",";
	  scores += (itos(*score));
	}
	r.add(scores);
      }

      if(firstline) r.add_header((*ite)->get_name()+" avg");
      if(firstline) r.add_header((*ite)->get_name()+" variance");
      if(firstline && show_score_detail) r.add_header( (*ite)->get_name()+" scores" );
    }

    r.endl();
    if(firstline){
      r.end_header();
      firstline = false;
    }
    
    pbar_set_fraction( (double)anc/mr.arows() );
  }
  return r;
}

report reportDialog::geneReport(int seq, annotationTrack* t, int position ,int total){
  bool firstline = true;
  report r;

  if(t->isFilterable()){
    r.set_caption( "seq #" + itos(seq+1) +
		   " (" + t->get_name() + ")\n" ); 

    int ac=0;
    for(unsigned i=0; i<t->size(); i++){
      annotation anno = mr.rat(t->at(i));
      annotation* native_anno = mr.rat_native(t->at(i));
      if( ( ! (!mr.geneFilter(t, ac) && hide_invisible) ) &&
	  ( features_en[anno.atype] || features_vec.empty() ) ){
	
	// gene name
	string anno_name = anno.getname(status.name_by);
	if(anno_name == "") 
	  anno_name=(!show_feature_types?("("+anno.atype+")") : "---");
	r.add(anno_name);
	if(show_feature_types) r.add(anno.atype);

	features.insert(anno.atype);

	// from, to, strand
	r.add( itos(anno.begin+1) );
	r.add( itos(anno.end) );
	r.add( anno.reverse ? "-" : "+" );

	if(firstline){
	  r.add_header("name");
	  if(show_feature_types) r.add_header("type");
	  r.add_header("from");
	  r.add_header("to");
	  r.add_header("strand");
	} 

	// visibility
	r.add( ( mr.geneFilter(t, ac) ) ? "1" : "0" );
	if(firstline) r.add_header("visible");
	
	// tfidf-avg, min, max
	if( t->hasAnnotScores()  && (!mr.tfidf.empty()) ){
	  // anchor lookup
	  set<int> matches;
	  mr.lookup_anchor_segments_direct(seq, native_anno->begin, 
					   native_anno->end, matches);

	  r.add( itos((int)matches.size()) );
	  r.add( itos(anno.tfidf_avg) );
	  r.add( itos(anno.tfidf_min) );
	  r.add( itos(anno.tfidf_max) );

	  // variance 
	  if( matches.size() != 0){
	    double variance=0;
	    for(set<int>::iterator score=matches.begin();
		score!=matches.end(); score++)
	      variance += ( (mr.tfidf.at(*score) - anno.tfidf_avg) *
			    (mr.tfidf.at(*score) - anno.tfidf_avg) );
	    variance /= matches.size();
	    r.add( itos(variance) );
	  } else r.add("0");

	  if(show_score_detail){
	    string scores;
	    for(set<int>::iterator score=matches.begin();
		score!=matches.end(); score++){
	      if(score!=matches.begin()) scores+=",";
	      scores += (itos(mr.tfidf.at(*score)));
	    }
	    r.add(scores);
	  }

	  if(firstline){
	    r.add_header("#ancs");
	    r.add_header("tf-idf avg");
	    r.add_header("tf-idf min");
	    r.add_header("tf-idf max");
	    r.add_header("tf-idf variance");
	  }

	  if(show_score_detail && firstline) r.add_header("tf-idf scores");
	}

	// expscores
	for(vector<annotationTrack*>::iterator ite=filterTracks.begin();
	    ite!=filterTracks.end(); ite++){
	  if( !(*ite)->hasGenScoreForTrack((annotationTrack*)t) ) continue;
	  if( *ite == t ) continue; // no self score

	  double avg = (*ite)->genScore((annotationTrack*)t, ac);
	  r.add( itos( avg ) );

	  if( (*ite)->expScores_size((annotationTrack*)t, ac) != 0){
	    double variance = 0;
	    for(vector<double>::const_iterator score=
		  (*ite)->expScores_begin((annotationTrack*)t, ac);
		score != (*ite)->expScores_end((annotationTrack*)t, ac); 
		score++){	  
	      variance += ( *score - avg ) * ( *score - avg );
	    }
	    variance /= (*ite)->expScores_size((annotationTrack*)t, ac);
	    r.add(itos(variance));
	  } else r.add("0");
 
	    
	  if(firstline) r.add_header( (*ite)->get_name()+" avg" );
	  if(firstline) r.add_header( (*ite)->get_name()+" variance" );

	  if(show_score_detail && *ite!=(annotationTrack*)t ){
	    string scores;
	    for(vector<double>::const_iterator score=
		  (*ite)->expScores_begin((annotationTrack*)t, ac);
		score != (*ite)->expScores_end((annotationTrack*)t, ac); score++){
	      if(score!= (*ite)->expScores_begin((annotationTrack*)t, ac)) scores+=",";
	      scores += (itos(*score));
	    }
	    r.add(scores);
	    if(firstline) r.add_header( (*ite)->get_name()+" scores" );
	  }
	}

	r.endl();

	if(firstline){
	  r.end_header();

	  firstline = false;
	}
      }
      ac++;

      pbar_set_fraction( (double)ac/(t->size()*total) + (double)position/total );
    }
  } else {
    // is not filterable: this track has score
  }
  return r;
}

report reportDialog::segmentReport(unsigned s){
  report r;
  r.add_header("Name");
  r.add_header("Original From");
  r.add_header("Original To");
  r.add_header("Offset");
  r.add_header("Adjusted From");
  r.add_header("Adjusted To");
  r.add_header("#ancs");
  r.add_header("#visible ancs");
  r.add_header("Last reorder reference");
  r.end_header();

  for(unsigned i=0; i<mr.sources_size(s); i++){
    const seq_source* src = mr.get_source_native(s, i);
    r.add(src->getstr());
    r.add(itos(src->begin));
    r.add(itos(src->end));
    
    seqlen_t offset = mr.get_segment_offset(s, src->begin, src->end);
    r.add(itos(offset));
    r.add(itos(src->begin + offset));
    r.add(itos(src->end + offset));

    set<int> ancs;
    int visible_ancs = 0;

    mr.lookup_anchor_segments_direct(s, src->begin, src->end, ancs);
    for(set<int>::iterator a=ancs.begin(); a!=ancs.end(); a++)
      if(mr.anchorFilter(*a)) visible_ancs++;
    r.add(itos((int)ancs.size()));
    r.add(itos(visible_ancs));
    r.add(itos(src->last_reference));
    r.endl();
  }

  return r;
}

// generate gene connection report:
// annotationTrack::calcAnnotScores() must be done prior to call this
report reportDialog::geneConnectionReport(){
  report r;
  gene_sets_t gene_sets;
  int total, ortho, newortho, skipped;
  total = ortho = newortho = skipped = 0;
  set<string> species, cogs;

  bool check_cog = false;
  bool show_cog  = false;
  bool check_cog_g = false;
  if(mr.has_cog){
    if(cog_option_box_ready){
      check_cog = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_ortholog));
      show_cog  = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(show_cog_group));
      check_cog_g = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_cog_gene));
    } else {
      check_cog = true;
      show_cog = false;
      check_cog_g = false;
    }
  }

  vector<annotationTrack*> tracks;

  // take first filterable track: this should be replaced GUI based selector
  int valid_tracks=0;
  for(unsigned s=0; s<mr.acols()/3; s++){
    bool found=false;
    for(rvector<annotationTrack*>::iterator t=mr.annotations[s].begin_all();
	t!=mr.annotations[s].end_all(); t++){
      if((*t)->isFilterable()){
	tracks.push_back((annotationTrack*)(*t));
	found=true;
	valid_tracks++;
	break;
      }
    }
    if(!found){
      tracks.push_back(NULL);
    }
  }
  if(valid_tracks<=1) return r; // nothing to report!

  // scan tracks
  for(unsigned trk=0; trk<tracks.size(); trk++){ 
    if(tracks[trk]==NULL) continue;
    for(unsigned gene=0; gene<tracks[trk]->size(); gene++){
      annotation* anno = mr.rat_native(tracks[trk]->at(gene));
      if(!features_en[anno->atype] &&
	 !features_vec.empty()) continue;

      // scan anchors goin' through the gene
      set<int> matches;
      mr.lookup_anchor_segments_direct(tracks[trk]->bound_to(),
				       anno->begin, anno->end, matches);

      for(set<int>::iterator anc=matches.begin(); anc!=matches.end(); anc++){

	// matchvec has list of genes which are connected to the anchor *anc
	matchvec_t matchvec(tracks.size());
	
	// push myself into matchvec
	matchvec[trk].push_back(gene);

	// check connected genes in other tracks
	for(unsigned t=0; t<tracks.size(); t++){
	  if(t==trk) continue; // skip myself
	  if(tracks[t]==NULL){ // no valid track
	    matchvec[t].push_back(-1); continue; }

	  set<int> genes;
	  tracks[t]->lookup_segment(mr.raa_native(*anc,
						  tracks[t]->bound_to()*3),
				    mr.raa_native(*anc,
						  tracks[t]->bound_to()*3+1),
				    genes);

	  if(genes.size() == 0){ // not found
	    matchvec[t].push_back(-1); continue; }
	  
	  for(set<int>::iterator g = genes.begin(); g!=genes.end(); g++){
	    if(!features_en[mr.rat_native(tracks[t]->at(*g))->atype] && 
	       !features_vec.empty()) continue;

	    matchvec[t].push_back(*g);
	  }
	}

	// unfold matchvec
	gene_set_t gene_set(tracks.size());
	gcrep_unfold(gene_sets, matchvec, gene_set, 0, (int)tracks.size(), (int)*anc);

	pbar_set_fraction( (double)trk/tracks.size() + 
			   (double)gene/(tracks[trk]->size() * tracks.size()) );
      }
    }
  }

  // gene_sets to report
  for(unsigned t=0; t<tracks.size(); t++){
    if(tracks[t]!=NULL)
      r.add_header(tracks[t]->get_name());
    else 
      r.add_header("No track");

    if(show_feature_types) r.add_header("type");
    if(check_cog_g) r.add_header("COG gene");
  }

  if(check_cog) r.add_header("COG group");
  if(show_cog) r.add_header("COG ID");
  r.add_header("#ancs");
  if(!mr.tfidf.empty()){
    r.add_header("tfidf sum");
    r.add_header("tfidf avg");
    if(show_score_detail) r.add_header("tfidf scores");
  }
  r.end_header();

  // determine how to filter
  bool has_tfidf = !mr.tfidf.empty();
  bool filter_by_num_anc, filter_by_tfidf_avg, filter_by_tfidf_sum;
  unsigned min_num_anc;
  double min_tfidf_avg, min_tfidf_sum;
  filter_by_num_anc = filter_by_tfidf_avg = filter_by_tfidf_sum = false;
  min_num_anc = 0;
  min_tfidf_avg = min_tfidf_sum = 0;
  
  if(widgets_ready){
    min_num_anc = (unsigned)gtk_spin_button_get_value(GTK_SPIN_BUTTON(spbtn_num_anc));
    filter_by_num_anc = 
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_num_anc));
    if ( !has_tfidf ){
      filter_by_tfidf_avg = filter_by_tfidf_sum = false;
    } else {
      filter_by_tfidf_avg = 
	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_tfidf_avg));
      filter_by_tfidf_sum = 
	gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(radio_tfidf_sum));
      min_tfidf_avg = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spbtn_tfidf_avg));
      min_tfidf_sum = gtk_spin_button_get_value(GTK_SPIN_BUTTON(spbtn_tfidf_sum));
    }
  }

  // and let's go 
  for(gene_sets_t::iterator i=gene_sets.begin(); i!=gene_sets.end(); i++){
    total++;

    // do basic statistics (required for filter)
    unsigned num_anc = i->second.size();
    double tfidf_sum=0;
    if(has_tfidf)
      for(set<int>::iterator j=i->second.begin(); j!=i->second.end(); j++) 
	tfidf_sum += mr.tfidf.at(*j);
    double tfidf_avg = tfidf_sum/num_anc;

    // apply filter
    if( ( filter_by_num_anc && num_anc < min_num_anc ) ||
	( filter_by_tfidf_avg && tfidf_avg < min_tfidf_avg ) ||
	( filter_by_tfidf_sum && tfidf_sum < min_tfidf_sum ) ){
      skipped++;
      continue;
    }

    // gene names and feature types
    vector<string> locustags;
    bool was_unknown = false;
    for(unsigned j=0; j<tracks.size(); j++) {
      if(tracks[j]==NULL || i->first.at(j)==-1){
	r.add("N/A");
	if(show_feature_types) r.add("N/A");
	if(check_cog_g){ if(show_cog) r.add("N/A"); else r.add("0"); };
	was_unknown = true;
      } else {
	annotation* gene = mr.rat_native(tracks[j]->at(i->first.at(j)));
	string gene_name = gene->getname(status.name_by);
	if(gene_name == "") 
	  gene_name=(!show_feature_types?("("+gene->atype+")") : "---");
	r.add(gene_name);

	if(show_feature_types) r.add(mr.rat_native(tracks[j]->at(i->first.at(j)))->atype);
	locustags.push_back(gene->locus);

	if(check_cog_g){
	  string cog = mr.gene_cog[gene->locus];
	  if(cog==""){ if(show_cog) cog=("---"); else cog="0"; }
	  else { if(!show_cog) cog="1"; }
	  r.add(cog);
	}
      }
    }
    if(was_unknown) newortho++;

    // COG ortholog
    if(check_cog || show_cog){
      bool isortho = true;
      string cog = mr.gene_cog[locustags[0]];
      for(unsigned l=0; l<locustags.size(); l++){
	string spe = mr.gene_specie[locustags[l]];
	if(spe!="") species.insert(spe);
	if(cog != mr.gene_cog[locustags[l]]) isortho=false;
	if(cog =="" || locustags[l]=="") isortho = false;
	if(!isortho) break;
      }
      if(locustags.size() != tracks.size()) isortho = false;
      if(isortho) cogs.insert(mr.gene_cog[locustags[0]]);
      if(isortho) ortho++;

      if(check_cog) r.add( (isortho ? "1" : "0") );
      if(show_cog)  r.add( (isortho ? mr.gene_cog[locustags[0]] : "---") );
    }

    // # ancs
    r.add( itos((int)(num_anc)) );

    // tfidf
    if(has_tfidf){
      r.add( itos(tfidf_sum) );
      r.add( itos(tfidf_avg) );
      
      if(show_score_detail){
	string sd;
	for(set<int>::iterator j=i->second.begin(); j!=i->second.end(); j++) 
	  sd += ( string(*j!=0 ? "," : "") + 
		  itos(mr.tfidf.at(*j) ) );
	r.add(sd);
      }
    }
    r.endl();

    // do statistics
    if( !has_gcrep_stat ){
      if(i==gene_sets.begin()){
	num_anc_min = num_anc_max = num_anc;
	if(!mr.tfidf.empty()){
	  tfidf_sum_min = tfidf_sum_max = tfidf_sum;
	  tfidf_avg_min = tfidf_avg_max = tfidf_avg; 
	}
      } else {
	num_anc_min = min(num_anc_min, (unsigned)(num_anc));
	num_anc_max = max(num_anc_max, (unsigned)(num_anc));
	tfidf_sum_min = min(tfidf_sum_min, tfidf_sum);
	tfidf_sum_max = max(tfidf_sum_max, tfidf_sum);
	tfidf_avg_min = min(tfidf_avg_min, tfidf_avg);
	tfidf_avg_max = max(tfidf_avg_max, tfidf_avg);
      }
    }
  }

//   cout << "gcrep: #anc " << num_anc_min << "-" << num_anc_max << endl
//        << "       tfidf-sum " << tfidf_sum_min << "-" << tfidf_sum_max << endl
//        << "       tfidf-avg " << tfidf_avg_min << "-" << tfidf_avg_max << endl;
    

  string caption = (itos(total) + " gene sets detected (" +
		    itos(total-skipped) + " shown, " +
		    itos(skipped) + " filtered out)" +
		    ( ", " + itos(newortho) + 
		      "/" + itos(total-skipped) +
		      " has newly predicted gene") +
		    "\n");


  if(species.size() == tracks.size()){
    unsigned total_ortho_set=0;
    unsigned cog_groups=0;
    for(map<string,string>::iterator c=mr.cogs.begin(); c!=mr.cogs.end(); c++){
      unsigned num_ortho_set=1;
      // c->first = cog id
      for(set<string>::iterator s=species.begin(); s!=species.end(); s++){
	num_ortho_set *= mr.cog_specie.count(pair<string, string>(c->first, *s));
      }
      total_ortho_set+=num_ortho_set;
      if(num_ortho_set!=0) cog_groups++;
    }


    caption += (itos(ortho) + 
		"/" + itos(total-skipped) +
		" ortholog sets found in COG while COG has " + 
		itos(total_ortho_set) + 
		"(" + itos((double)(ortho*100)/total_ortho_set) + "%), " +
		itos((int)cogs.size()) + "/" + itos(cog_groups) + " COG groups detected" +
		"\n");
    for(set<string>::iterator i=species.begin(); i!=species.end(); i++){
      caption += *i + ": " + itos(int(mr.specie_gene.count(*i))) + " ";
    }
    caption += "\n";
  }

  r.set_caption(caption);

  has_gcrep_stat = true;
  return r;
}

void reportDialog::gcrep_unfold(gene_sets_t& gene_sets, matchvec_t& matchvec, 
				gene_set_t& gene_set, int track, int tracks, 
				int anchor){
  for(unsigned i=0; i<matchvec[track].size(); i++){
    gene_set[track]=matchvec[track].at(i);
    if(track != tracks-1){
      // we have more track(s)
      gcrep_unfold(gene_sets, matchvec, gene_set, track+1, tracks, anchor);
    } else {
      // the last track
      gene_sets[gene_set].insert(anchor);
    }
  }
}
