using namespace std;

#include <gtk/gtk.h>
#include <string>
#include <set>
#include <ctype.h>

#include <iostream>

#include "dialogs.h"
#include "messages.h"
#include "gmv_defs.h"
#include "gmv.h"
#include "opts.h"
#include "dblinks.h"
#include "util.h"

#include "murasaki-logo.xpm"
#include "../image/beerware.h"


// ----------------------------------------------------------------------
// widget/dialog/assistant abstraction handlers

gboolean handle_release(GtkWidget* w, GdkEventButton *e, gpointer p){
  ((generalWidgets*)p)->release(w);
  return false;
}

gboolean handle_activate(GtkWidget* w, gpointer p){
  ((generalWidgets*)p)->activate(w);
  return false;
}

void handle_dialogResponse(GtkDialog *d, gint r, gpointer dialog){
  ((generalDialog*)dialog)->response(r);
}

generalAssistant::~generalAssistant(){
};

void assistant_apply(GtkAssistant *a, gpointer p){
  ((generalAssistant*)p)->apply();
}

void assistant_close(GtkAssistant *a, gpointer p){
  ((generalAssistant*)p)->close();
}

void assistant_cancel(GtkAssistant *a, gpointer p){
  ((generalAssistant*)p)->cancel();
}

void assistant_prepare(GtkAssistant *a, GtkWidget *w, gpointer p){
  ((generalAssistant*)p)->prepare(w);
}

// ----------------------------------------------------------------------
// manual sequence source sorter

bool is_any_sortable_seq(){
  // test whether there's any sortable sequence or not
  bool some_multi_source_seq = false;
  for(int s=0; s<(int)mr.acols()/3; s++){
    if(mr.sources_size(s) > 1){
      some_multi_source_seq = true; break;
    }
  }

  if(!some_multi_source_seq){ // not sortable
    GtkWidget* dialog = gtk_message_dialog_new(GTK_WINDOW(status.mainwindow),
					       (GtkDialogFlags)
					       (GTK_DIALOG_MODAL | 
						GTK_DIALOG_DESTROY_WITH_PARENT),
					       GTK_MESSAGE_ERROR,
					       GTK_BUTTONS_OK,
					       "No sortable sequence found!\n\
Target sequence must have multiple chromosomes.");

    gtk_dialog_run(GTK_DIALOG(dialog));
    gtk_widget_destroy(dialog);
    return false;
  }
  return true;
}

void manual_source_sort_dialog::init(GtkWidget *parent){
  if(!is_any_sortable_seq()) return;

  isOpen = true;
  dialog = gtk_dialog_new_with_buttons("Sort Sequence Sources by Manual",
			      GTK_WINDOW(status.mainwindow),
			      (GtkDialogFlags)
			      (GTK_DIALOG_MODAL | 
			       GTK_DIALOG_DESTROY_WITH_PARENT),
			      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, 
			      GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
			      NULL);

  segmentList.clear();
  seqnoList.clear();

  // prepare cols
  std::vector<generalList<segItem>::coltype> cols;
  generalList<segItem>::coltype col;

  col.is_editable = false; col.name = "#";
  col.data_type = G_TYPE_INT; 
  col.view_type = generalList<segItem>::renderer_text; cols.push_back(col);

  col.data_type = G_TYPE_STRING; col.name = "Segment name";
  col.view_type = generalList<segItem>::renderer_text; cols.push_back(col);

  notebook = gtk_notebook_new();
  gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), true);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
		     notebook, true, true, 0);

  // prepare lists
  for(int s=0; s<(int)mr.acols()/3; s++){
    if(mr.sources_size(s) > 1){
      seqnoList.push_back(s);
      segmentList.push_back(new generalList<segItem>);

      // setup list
      map<seqlen_t, int> segments;  // to reorder segments by current position
      segmentList[s]->set_types(cols);
      for(unsigned src=0; src<mr.sources_size(s); src++)
	segments.insert(pair<seqlen_t, int>(mr.get_source(s, src).begin, src));

      for(map<seqlen_t,int>::iterator i=segments.begin(); 
	  i!=segments.end(); i++)
	segmentList[s]->add_initial_item(segItem(i->second, mr.get_source_native(s, i->second)->getstr()));
      segmentList[s]->create_model();

      // instantiate the list widget
      GtkWidget *sw = gtk_scrolled_window_new(FALSE, FALSE);
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), 
				     GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
      GtkWidget *w = GTK_WIDGET(segmentList[s]->create_view(GTK_SELECTION_MULTIPLE));
      gtk_container_add(GTK_CONTAINER(sw), w);

      gtk_notebook_append_page(GTK_NOTEBOOK(notebook), sw, 
			       gtk_label_new(string("Seq #" + itos(s+1)).c_str()));
    }
  }

  // buttons
  reset_all = gtk_button_new_with_label("Reset all sequence");
  reset_seq = gtk_button_new_with_label("Reset this sequence");
  
  GtkWidget *hb = gtk_hbox_new(true, 0);
  gtk_box_pack_start(GTK_BOX(hb), reset_all, true, true, 3);
  gtk_box_pack_start(GTK_BOX(hb), reset_seq, true, true, 3);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hb, false, false, 0);

  // signals + stg more
  g_signal_connect(dialog, "response", G_CALLBACK(handle_dialogResponse), this);
  g_signal_connect(reset_all, "clicked", G_CALLBACK(handle_activate), this);
  g_signal_connect(reset_seq, "clicked", G_CALLBACK(handle_activate), this);

  gtk_widget_set_size_request(dialog, -1, 300);

  // focus to OK button
  GList *l;
  l = gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area));
  gtk_widget_grab_focus(GTK_WIDGET(l->next->data));
  g_list_free(l);

  // launch it!
  gtk_widget_show_all(dialog);
  gtk_main();


  // but don't remove this :)
  for(vector<generalList<segItem>*>::iterator i=segmentList.begin();
      i!=segmentList.end(); i++) delete *i;
}

void manual_source_sort_dialog::reset_page(int p){
  map<int, int> item_map;
  vector<segItem> newitems;

  int seq=seqnoList[p];

  for(int i=0; 
      i<(int)segmentList[seq]->items.size(); i++)
    item_map[segmentList[seq]->items[i].number()] = i;

  for(int i=0; i<(int)item_map.size(); i++)
    newitems.push_back(segmentList[seq]->items[item_map[i]]);

  segmentList[seq]->replace(newitems);
}

void manual_source_sort_dialog::activate(GtkWidget* w){
  if(w==reset_all)
    for(int p=0; p<(int)seqnoList.size(); p++)
      reset_page(p);

  if(w==reset_seq)
    reset_page(gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)));
}

void manual_source_sort_dialog::response(gint r){
  map<int, multimap<seqlen_t, unsigned> > srcmap;

  if(r == GTK_RESPONSE_ACCEPT){
    for(unsigned s=0; s<seqnoList.size(); s++){
      int seq = seqnoList[s];
      for(unsigned seg=0; seg<segmentList[s]->items.size(); seg++){
	srcmap[seq].insert(pair<seqlen_t, unsigned>(seg+1, segmentList[s]->items[seg].number()));
      }
    }
  }

  gtk_widget_destroy(dialog);
  isOpen = false;

  if(!srcmap.empty()){
    mr.sort_sequence_sources(srcmap);
    redraw();
  }

  gtk_main_quit();
}

// ----------------------------------------------------------------------
// auto sequence source sorter

void source_sort_dialog::activate(GtkWidget* w){
  if(w == sel_all || w == desel_all ){
    for(map<GtkWidget*, int>::iterator i=target_seqs.begin();
	i!=target_seqs.end(); i++)
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(i->first), w==sel_all);
  }
}

void source_sort_dialog::init(GtkWidget *parent){
  if(!is_any_sortable_seq()) return;

  // OK, let's start.
  isOpen = true;
  dialog = gtk_dialog_new_with_buttons("Sort Sequence Sources by Anchors",
			      GTK_WINDOW(status.mainwindow),
			      (GtkDialogFlags)
			      (GTK_DIALOG_MODAL | 
			       GTK_DIALOG_DESTROY_WITH_PARENT),
			      GTK_STOCK_OK, GTK_RESPONSE_ACCEPT, 
			      GTK_STOCK_CANCEL, GTK_RESPONSE_REJECT,
			      NULL);

  GtkWidget* tb = gtk_table_new(2, 2, true);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), tb);

  GtkWidget* lf = gtk_frame_new("Sort chromosomes on:");
  GtkWidget* rf = gtk_frame_new("Along:");
  GtkWidget* mf = gtk_frame_new("Method:");
  gtk_table_attach_defaults(GTK_TABLE(tb), lf, 0, 1, 0, 2);
  gtk_table_attach_defaults(GTK_TABLE(tb), rf, 1, 2, 0, 1);
  gtk_table_attach_defaults(GTK_TABLE(tb), mf, 1, 2, 1, 2);

  GtkWidget* targetbox = gtk_vbox_new(true, 0);
  gtk_container_add(GTK_CONTAINER(lf), targetbox);

  GtkWidget* referencebox = gtk_vbox_new(false, 0);
  gtk_container_add(GTK_CONTAINER(rf), referencebox);
  reference = gtk_combo_box_new_text();
  gtk_box_pack_start(GTK_BOX(referencebox), reference, false, false, 0);
  keep_target_order = gtk_check_button_new_with_label("Keep segment order of reference sequence");
  apply_anchor_filter = gtk_check_button_new_with_label("Apply anchor filter");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(apply_anchor_filter), true);
  gtk_box_pack_start(GTK_BOX(referencebox), keep_target_order, false, false, 0);
  gtk_box_pack_start(GTK_BOX(referencebox), apply_anchor_filter, false, false, 0);

  GtkWidget* methodbox = gtk_vbox_new(false, 0);
  gtk_container_add(GTK_CONTAINER(mf), methodbox);
  method = gtk_combo_box_new_text();
  gtk_box_pack_start(GTK_BOX(methodbox), method, false, false, 0);

  gtk_combo_box_append_text(GTK_COMBO_BOX(method), 
			    "Average (weight by anchor length)");
  gtk_combo_box_append_text(GTK_COMBO_BOX(method), 
			    "Average (unweighted)");
  gtk_combo_box_append_text(GTK_COMBO_BOX(method),
			    "Average (median of anchors' length)");
  gtk_combo_box_append_text(GTK_COMBO_BOX(method),
			    "Longest anchor in the segment");
  gtk_combo_box_set_active(GTK_COMBO_BOX(method), 0);

  target_seqs.clear(); // map<GtkWidget*, int> target_seqs;
  for(int s=0; s<(int)mr.acols()/3; s++){
    string seqID = string("Sequence #" + itos(s+1));
    if(mr.sources_size(s) > 1){
      GtkWidget *cb = gtk_check_button_new_with_label(seqID.c_str());
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(cb), true);
      gtk_box_pack_start(GTK_BOX(targetbox), cb, true, true, 0);
      target_seqs.insert(pair<GtkWidget*, int>(cb, s));
    }
    gtk_combo_box_append_text(GTK_COMBO_BOX(reference), seqID.c_str());
  }
  gtk_combo_box_set_active(GTK_COMBO_BOX(reference), 0);

  // {,de}select all
  GtkWidget *selbox = gtk_hbox_new(true, 0);
  sel_all = gtk_button_new_with_label("Select all");
  desel_all = gtk_button_new_with_label("Deselect all");

  gtk_box_pack_start(GTK_BOX(selbox), sel_all, true, true, 0);
  gtk_box_pack_start(GTK_BOX(selbox), desel_all, true, true, 0);
  gtk_box_pack_start(GTK_BOX(targetbox), selbox, true, true, 0);

  // signals
  g_signal_connect(dialog, "response", G_CALLBACK(handle_dialogResponse), this);
    
  g_signal_connect(sel_all, "clicked", G_CALLBACK(handle_activate), this);
  g_signal_connect(desel_all, "clicked", G_CALLBACK(handle_activate), this);

  // focus to OK button
  GList *l;
  l = gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area));
  gtk_widget_grab_focus(GTK_WIDGET(l->next->data));
  g_list_free(l);

  // launch it!
  gtk_widget_show_all(dialog);
  gtk_main();
}

void source_sort_dialog::response(int r){
  vector<int> target_seq_list ;
  int reference_seq = 0;
  int method_no = 0;
  bool keep = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(keep_target_order));
  bool filter = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(apply_anchor_filter));

  if(r == GTK_RESPONSE_ACCEPT){
    reference_seq = gtk_combo_box_get_active(GTK_COMBO_BOX(reference));
    method_no = gtk_combo_box_get_active(GTK_COMBO_BOX(method));

    for(map<GtkWidget*, int>::iterator ite=target_seqs.begin();
	ite!=target_seqs.end(); ite++){
      if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ite->first)) &&
	 ite->second != reference_seq)
	target_seq_list.push_back(ite->second);
    }
  }

  gtk_widget_destroy(dialog);
  isOpen = false;

  if(!target_seq_list.empty()){
    mr.sort_sequence_sources(reference_seq, target_seq_list, 
			     method_no, keep, filter, NULL);
    redraw();
  }
  // init() will return, and the instance of this class is freed.
  gtk_main_quit(); 
}



// ----------------------------------------------------------------------
// confirmation dialog

bool confirm_to_flush(){
  GtkWidget *d;
  d = gtk_message_dialog_new (GTK_WINDOW(status.mainwindow),
			      GTK_DIALOG_DESTROY_WITH_PARENT,
			      GTK_MESSAGE_WARNING,
			      GTK_BUTTONS_OK_CANCEL,
			      "Caches will be flushed!");
  
  gtk_window_set_modal(GTK_WINDOW(d), true);
  gtk_window_set_transient_for(GTK_WINDOW(d), GTK_WINDOW(status.mainwindow));

  gtk_message_dialog_format_secondary_text(GTK_MESSAGE_DIALOG(d),
					   "Undo/Redo history will be flushed!\n\(Bookmarks are still available)");

  bool ret = (gtk_dialog_run(GTK_DIALOG(d)) == GTK_RESPONSE_OK) ? true : false;
  gtk_widget_destroy(d);

  return ret;
}


// ----------------------------------------------------------------------
// dialog abstraction

generalWidgets::~generalWidgets(){};
generalDialog::~generalDialog(){};

// ----------------------------------------------------------------------
// "Statistics" dialog

gpointer statd_subthread(gpointer dialog){ // generate detailed stat
  if(!mr.has_detailed_statistics) opt_anchor_statistics();
  if(!mr.has_sequence_statistics) mr.load_seq_files(true, false);
  
  ((statisticsDialog*)dialog)->subthread_terminated();
  return NULL;
}

bool statd_idle(gpointer dialog){
  if(((statisticsDialog*)dialog)->is_thread_running()){

    set_progbar();
  } else {
    ((statisticsDialog*)dialog)->show_new_stats();
    flush_and_redraw();
    return false;
  }
  return true;
}

bool statisticsDialog::is_thread_running(){
  bool running;
  g_mutex_lock(stat_mutex);
  running = thread_running;
  g_mutex_unlock(stat_mutex);
  return running;
}

void statisticsDialog::subthread_terminated(){
  g_mutex_lock(stat_mutex);
  thread_running = false;
  g_mutex_unlock(stat_mutex);
}

void statisticsDialog::show_new_stats(){
  gtk_widget_destroy(generatedetail);
  show_stats();
  gtk_widget_show_all(vbox);
  
  gtk_progress_bar_set_fraction((GtkProgressBar*)status.progbar, 0);
  gtk_progress_bar_set_text((GtkProgressBar*)status.progbar, "Idle");
  
  handlers.unblock();
}

void statisticsDialog::attachLabel(GtkWidget *table, string s, guint x1, 
				   guint x2, guint y1, guint y2){
  GtkWidget* l = gtk_label_new(s.c_str());
  gtk_table_attach((GtkTable*)table, l, x1, x2, y1, y2, 
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), 10, 0);
  gtk_misc_set_alignment(GTK_MISC(l), 1, 0);
}

void statisticsDialog::response(gint r){
  if(thread_running) return;
  isOpen = false;

  g_mutex_free(stat_mutex);
  gtk_widget_destroy(dialog);
  handlers.remove_dialog(HANDLER_STATD);
}


void statisticsDialog::activate(GtkWidget* w){
  // ------------------------------
  if(w==generatedetail){
    if(!confirm_to_flush()) return;

    handlers.remove_dialog(HANDLER_STATD);

    // clear dialog->vbox
    GList *widgetlist = gtk_container_get_children(GTK_CONTAINER(vbox));
    for(GList* widget=widgetlist; widget; widget=widget->next){
      gtk_widget_destroy((GtkWidget*)(widget->data));
    }
    g_list_free(widgetlist);

    // place new widgets in dialog->vbox
    generatedetail = gtk_label_new("Generating detailed statistics...");
    gtk_box_pack_start(GTK_BOX(vbox), generatedetail, false, false, 0);
    gtk_widget_show_all(dialog);

    // reregistrate action area to handler table
    GList *buttons, *b;
    buttons = gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area));
    for(b=buttons; b; b=b->next) handlers.connect(b->data, HANDLER_STATD);
    g_list_free(buttons);


    // prepare before launch
    g_mutex_lock(stat_mutex);
    thread_running = true;
    g_mutex_unlock(stat_mutex);

    handlers.block();

    // launch it
    g_thread_create((GThreadFunc)statd_subthread, this, false, NULL);    
    g_timeout_add(PROGBAR_UPDATE_INTERVAL, (GSourceFunc)statd_idle, (gpointer)this);
  }
  
  // ------------------------------
  if(w==savehtml || w==savetext){

    GtkWidget* chooser;
    string title = (string)"Save Statistics in "+((w==savehtml)?"HTML":"Text");
    string ext = (w==savehtml) ? (string)".html" : (string)".txt";

    chooser = gtk_file_chooser_dialog_new(title.c_str(),
					  GTK_WINDOW(status.mainwindow),
					  GTK_FILE_CHOOSER_ACTION_SAVE,
					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					  GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
					  (gpointer)NULL);
    
    GtkFileFilter *filter = gtk_file_filter_new();

    gtk_file_filter_add_pattern(filter, string("*"+ext).c_str());
    gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(chooser),
				  string(status.basename+
					 "-stat"+ext).c_str());

    gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(chooser), filter);
    gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(chooser), 
						   TRUE);

    GtkWidget* open_browser;
    open_browser = gtk_check_button_new_with_mnemonic("Open in _Browser");
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(chooser)->vbox), open_browser, false, false, 0);
    if( w==savehtml ){
      gtk_widget_show_all(GTK_DIALOG(chooser)->vbox);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(open_browser), true);
    }

    bool filesave, browse;
    if (gtk_dialog_run (GTK_DIALOG (chooser))==GTK_RESPONSE_ACCEPT){
      char *filename;
      filesave = true;
      filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
      browse = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(open_browser));

      ofstream report;
      report.open(filename);
      if( w==savehtml) report << mr.summarize_in_html(); 
      else report << mr.summarize_in_text();
      report.close();

      char *path=strdup(string((string)"file://"+(string)filename).c_str());
      char *p=path;
      if(path[8]==':'){ // windows hack
	while(*p != 0){
	  if(*p=='\\') *p='/';
	  p++;
	}
      }

      if(browse) launch_browser(path);
      free(path);

      g_free(filename);
    }
    gtk_widget_destroy (chooser);    
  }

}

void statisticsDialog::show_stats(){
//   ofstream htmlsummary;
//   string htmlname = string(status.basename+"-stat.html"); 
//   htmlsummary.open(htmlname.c_str());
//   htmlsummary << mr.summarize_in_html() << endl;
//   htmlsummary.close();

  GtkWidget *num_anchors;
  string na = "# of anchors: " + itos_comma(mr.arows());
  num_anchors=gtk_label_new(na.c_str());
  gtk_box_pack_start(GTK_BOX(vbox), num_anchors, FALSE, FALSE, 0);

  // ------------------------------
  int seqno = 0;

  if( !mr.has_detailed_statistics && !mr.has_sequence_statistics ){
    GtkWidget *l;
    l = gtk_label_new("No sequence/anchor statistics is available yet.");
    gtk_box_pack_start(GTK_BOX(vbox), l, FALSE, FALSE, 0);

  } else {
    while(seqno < MAXSEQ && mr.bs[seqno] != NULL){
      basestat *b = mr.bs[seqno];

      GtkWidget* frame = gtk_frame_new(string("Sequence #" + 
					      itos(seqno+1) + ": " +
					      b->name).c_str());
      GtkWidget* table = gtk_table_new(10, 9, false); // A, T, C, G, N, Total x Rep, non-rep, total
      GtkWidget *fvbox, *fb, *l, *hs, *vs;

      gtk_box_pack_start(GTK_BOX(vbox), frame , FALSE, FALSE, 0);
      fb = gtk_hbox_new(FALSE, 0);
      fvbox = gtk_vbox_new(FALSE, 0);
      gtk_container_add(GTK_CONTAINER(frame), fvbox);

      // anchor coverage statistics
      if(mr.has_detailed_statistics){
	seqlen_t anchorlen = mr.anchor_ranges.at(seqno).total_length();
	seqlen_t tf = mr.anchor_ranges_f.at(seqno).total_length();
	seqlen_t tb = mr.anchor_ranges_b.at(seqno).total_length();

	l = gtk_label_new(string(itos_comma(anchorlen)+ "bp ("+
				 itos((double)anchorlen*100/b->totalbases(),2)+
				 "%) covered by anchor(s): "+

				 "Forward "+
				 itos_comma(tf)+"bp ("+ 
				 itos((double)tf*100/b->totalbases(),2) + "%)"+
				 " / Backward "+
				 itos_comma(tb)+"bp ("+ 
				 itos((double)tb*100/b->totalbases(),2) + "%) "
				 ).c_str());

	gtk_misc_set_alignment(GTK_MISC(l),    0, 0.5);
	gtk_box_pack_start(GTK_BOX(fvbox), l, FALSE, FALSE, 0);

	// # of anchors 
	l = gtk_label_new(string("Forward anchors: "
				 +itos_comma(mr.anchors_num_f[seqno])+
				 " / Backward anchors: "
				 +itos_comma(mr.anchors_num_b[seqno])).c_str());
	gtk_misc_set_alignment(GTK_MISC(l),    0, 0.5);
	gtk_box_pack_start(GTK_BOX(fvbox), l, FALSE, FALSE, 0);
      }

      if(mr.has_sequence_statistics){
	l = gtk_hseparator_new();
	gtk_box_pack_start(GTK_BOX(fvbox), l, FALSE, FALSE, 0);

	// base ratio table
	gtk_box_pack_start(GTK_BOX(fvbox), fb, FALSE, FALSE, 0);
	gtk_box_pack_start(GTK_BOX(fb), table, FALSE, FALSE, 0);

	l = gtk_label_new("A");
	gtk_table_attach_defaults((GtkTable*)table, l, 0, 1, 2, 3);

	l = gtk_label_new("T");
	gtk_table_attach_defaults((GtkTable*)table, l, 0, 1, 3, 4);

	l = gtk_label_new("C");
	gtk_table_attach_defaults((GtkTable*)table, l, 0, 1, 4, 5);

	l = gtk_label_new("G");
	gtk_table_attach_defaults((GtkTable*)table, l, 0, 1, 5, 6);

	l = gtk_label_new("N");
	gtk_table_attach_defaults((GtkTable*)table, l, 0, 1, 6, 7);
    
	l = gtk_label_new("Total");
	gtk_widget_set_size_request((GtkWidget*)l, 70, -1);
	gtk_table_attach_defaults((GtkTable*)table, l, 0, 1, 8, 9);

	l = gtk_label_new("Non-repeat");
	gtk_widget_set_size_request((GtkWidget*)l, 200, -1);
	gtk_table_attach_defaults((GtkTable*)table, l, 2, 4, 0, 1);

	l = gtk_label_new("Repeat");
	gtk_widget_set_size_request((GtkWidget*)l, 200, -1);
	gtk_table_attach_defaults((GtkTable*)table, l, 5, 7, 0, 1);

	l = gtk_label_new("Total");
	gtk_widget_set_size_request((GtkWidget*)l, 200, -1);
	gtk_table_attach_defaults((GtkTable*)table, l, 8, 10, 0, 1);

	// H separator
	hs = gtk_hseparator_new();
	gtk_table_attach_defaults((GtkTable*)table, hs, 0, 10, 1, 2);

	hs = gtk_hseparator_new();
	gtk_table_attach_defaults((GtkTable*)table, hs, 0, 10, 7, 8);

	// V separator
	vs = gtk_vseparator_new();
	gtk_table_attach_defaults((GtkTable*)table, vs, 1, 2, 0, 9);

	// non-repeat (bp)
	attachLabel(table, itos_comma(b->A), 2, 3, 2, 3);
	attachLabel(table, itos_comma(b->T), 2, 3, 3, 4);
	attachLabel(table, itos_comma(b->C), 2, 3, 4, 5);
	attachLabel(table, itos_comma(b->G), 2, 3, 5, 6);
	attachLabel(table, itos_comma(b->N), 2, 3, 6, 7);
	attachLabel(table, itos_comma(b->nonrepeats()), 2, 3, 8, 9);

	// non-repeat (%)
	attachLabel(table, string("("+itos((double)b->A*100/b->totalbases(), 2)+"%)"), 3, 4, 2, 3);
	attachLabel(table, string("("+itos((double)b->T*100/b->totalbases(), 2)+"%)"), 3, 4, 3, 4);
	attachLabel(table, string("("+itos((double)b->C*100/b->totalbases(), 2)+"%)"), 3, 4, 4, 5);
	attachLabel(table, string("("+itos((double)b->G*100/b->totalbases(), 2)+"%)"), 3, 4, 5, 6);
	attachLabel(table, string("("+itos((double)b->N*100/b->totalbases(), 2)+"%)"), 3, 4, 6, 7);
	attachLabel(table, string("("+itos((double)b->nonrepeats()*100/b->totalbases(), 2)+"%)"), 3, 4, 8, 9);

	// V separator
	vs = gtk_vseparator_new();
	gtk_table_attach_defaults((GtkTable*)table, vs, 4, 5, 0, 9);

	// repeat (bp)
	attachLabel(table, itos_comma(b->a), 5, 6, 2, 3);
	attachLabel(table, itos_comma(b->t), 5, 6, 3, 4);
	attachLabel(table, itos_comma(b->c), 5, 6, 4, 5);
	attachLabel(table, itos_comma(b->g), 5, 6, 5, 6);
	attachLabel(table, itos_comma(b->n), 5, 6, 6, 7);
	attachLabel(table, itos_comma(b->repeats()), 5, 6, 8, 9);

	// repeat (%)
	attachLabel(table, string("("+itos((double)b->a*100/b->totalbases(), 2)+"%)"), 6, 7, 2, 3);
	attachLabel(table, string("("+itos((double)b->t*100/b->totalbases(), 2)+"%)"), 6, 7, 3, 4);
	attachLabel(table, string("("+itos((double)b->c*100/b->totalbases(), 2)+"%)"), 6, 7, 4, 5);
	attachLabel(table, string("("+itos((double)b->g*100/b->totalbases(), 2)+"%)"), 6, 7, 5, 6);
	attachLabel(table, string("("+itos((double)b->n*100/b->totalbases(), 2)+"%)"), 6, 7, 6, 7);
	attachLabel(table, string("("+itos((double)b->repeats()*100/b->totalbases(), 2)+"%)"), 6, 7, 8, 9);

	// V separator
	vs = gtk_vseparator_new();
	gtk_table_attach_defaults((GtkTable*)table, vs, 7, 8, 0, 9);

	// total (bp)
	attachLabel(table, itos_comma(b->a+b->A), 8, 9, 2, 3);
	attachLabel(table, itos_comma(b->t+b->T), 8, 9, 3, 4);
	attachLabel(table, itos_comma(b->c+b->C), 8, 9, 4, 5);
	attachLabel(table, itos_comma(b->g+b->G), 8, 9, 5, 6);
	attachLabel(table, itos_comma(b->n+b->N), 8, 9, 6, 7);
	attachLabel(table, itos_comma(b->totalbases()), 8, 9, 8, 9);

	// total (%)
	attachLabel(table, string("("+itos((double)(b->a+b->A)*100/b->totalbases(), 2)+"%)"), 9, 10, 2, 3);
	attachLabel(table, string("("+itos((double)(b->t+b->T)*100/b->totalbases(), 2)+"%)"), 9, 10, 3, 4);
	attachLabel(table, string("("+itos((double)(b->c+b->C)*100/b->totalbases(), 2)+"%)"), 9, 10, 4, 5);
	attachLabel(table, string("("+itos((double)(b->g+b->G)*100/b->totalbases(), 2)+"%)"), 9, 10, 5, 6);
	attachLabel(table, string("("+itos((double)(b->n+b->N)*100/b->totalbases(), 2)+"%)"), 9, 10, 6, 7);
	attachLabel(table, string("(100.00%)"), 9, 10, 8, 9);
      }

      seqno++;
    }
  }

  if(!mr.has_detailed_statistics || !mr.has_sequence_statistics){
    generatedetail = gtk_button_new_with_mnemonic("_Generate Details");
    gtk_box_pack_start(GTK_BOX(vbox), generatedetail , FALSE, FALSE, 0);    
    g_signal_connect(generatedetail, "clicked", 
		     G_CALLBACK(handle_activate), (gpointer)this);
  }

  if(mr.has_detailed_statistics || mr.has_sequence_statistics){
    GtkWidget *hbox = gtk_hbox_new(true, 0);
    savehtml = gtk_button_new_with_mnemonic("Save in _HTML");
    savetext = gtk_button_new_with_mnemonic("Save in _Text");
    gtk_box_pack_start(GTK_BOX(hbox), savehtml, true, true, 0);
    gtk_box_pack_start(GTK_BOX(hbox), savetext, true, true, 0);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, false, false, 0);

    g_signal_connect(savehtml, "clicked", 
		     G_CALLBACK(handle_activate), (gpointer)this);
    g_signal_connect(savetext, "clicked", 
		     G_CALLBACK(handle_activate), (gpointer)this);
  }
}

void statisticsDialog::init(GtkWidget *mainwindow){
  isOpen = true;
  dialog = gtk_dialog_new_with_buttons ("Sequence Statistics",
					GTK_WINDOW(mainwindow),
					GTK_DIALOG_DESTROY_WITH_PARENT,
					GTK_STOCK_CLOSE,
					GTK_RESPONSE_NONE,
					NULL);

  gtk_window_set_modal(GTK_WINDOW(dialog), true);

  vbox = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), vbox);

  show_stats();

  stat_mutex = g_mutex_new();
  thread_running = false;

  // signals
  g_signal_connect(dialog, "response", G_CALLBACK(handle_dialogResponse), 
		   (gpointer)this);


  GList *buttons, *b;
  buttons = gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area));
  for(b=buttons; b; b=b->next) handlers.connect(b->data, HANDLER_STATD);
  g_list_free(buttons);

  gtk_widget_show_all (dialog);
}

// ----------------------------------------------------------------------
// "About" Dialog

void aboutDialog(GtkWidget *mainwindow){
  GtkWidget *dialog, *nb, *mlogo, *vlabel, *hlabel, *viewport;

  dialog = gtk_dialog_new_with_buttons ("About GMV",
					GTK_WINDOW(mainwindow),
					GTK_DIALOG_DESTROY_WITH_PARENT,
					GTK_STOCK_OK,
					GTK_RESPONSE_NONE,
					NULL);

  // version label
  vlabel = gtk_label_new(version_string().c_str());
   
  // murasaki logo
  mlogo = gtk_image_new_from_pixbuf(gdk_pixbuf_new_from_xpm_data(murasaki_logo));

  GtkWidget *mlogo_hb, *mlogo_vb;
  mlogo_hb = gtk_hbox_new(FALSE, 0);
  mlogo_vb = gtk_vbox_new(FALSE, 0);

  gtk_box_pack_start((GtkBox*)mlogo_hb, (GtkWidget*)mlogo,
		       TRUE, FALSE, 0);
  gtk_box_pack_start((GtkBox*)mlogo_vb, mlogo_hb, TRUE, FALSE, 0);

  // notebook
  nb = gtk_notebook_new();

  // page 0: license
  hlabel = gtk_label_new("");
  gtk_label_set_markup(GTK_LABEL(hlabel), copyright_string().c_str());
  gtk_label_set_line_wrap(GTK_LABEL(hlabel), true);

  viewport = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_set_size_request(viewport, -1, 200);
  gtk_scrolled_window_set_policy((GtkScrolledWindow*)viewport, 
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  gtk_scrolled_window_add_with_viewport((GtkScrolledWindow*)viewport, hlabel);

  gtk_notebook_append_page(GTK_NOTEBOOK(nb), 
			   viewport, gtk_label_new("License"));

  // page 1: history
  hlabel = gtk_label_new("");
  gtk_label_set_markup(GTK_LABEL(hlabel), history_string().c_str());
  gtk_label_set_line_wrap(GTK_LABEL(hlabel), true);

  viewport = gtk_scrolled_window_new(NULL, NULL);
  gtk_widget_set_size_request(viewport, -1, 200);
  gtk_scrolled_window_set_policy((GtkScrolledWindow*)viewport, 
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  gtk_scrolled_window_add_with_viewport((GtkScrolledWindow*)viewport, hlabel);

  gtk_notebook_append_page(GTK_NOTEBOOK(nb), 
			   viewport, gtk_label_new("History"));


  // page 2: ingredients
  GtkWidget *ingre_hb, *ingre_vb;
  
  ingre_vb = gtk_vbox_new(false, 0);
  ingre_hb = gtk_hbox_new(false, 0);
  gtk_box_pack_start(GTK_BOX(ingre_vb), ingre_hb, false, false, 0);
  gtk_box_pack_start(GTK_BOX(ingre_hb), 
		     gtk_image_new_from_pixbuf(gdk_pixbuf_new_from_inline(-1, beerware, FALSE, NULL)), false, false, 0);

  hlabel = gtk_label_new("");
  gtk_label_set_line_wrap(GTK_LABEL(hlabel), true);
  gtk_label_set_markup(GTK_LABEL(hlabel),
		       "\n\
<b>Ingredients:</b>\n\
- Caffeine (Tea): 40%\n\
- Sugar: 20%\n\
- Beer: 10%\n\
- Art of C++: 10%\n\
- Inspiration: 5%\n\
- Luck: 5%\n\
- Others: 10%\n\
");
  
  gtk_box_pack_start(GTK_BOX(ingre_hb), hlabel, false, false, 10);

  gtk_notebook_append_page(GTK_NOTEBOOK(nb), 
			   ingre_vb, gtk_label_new("Ingredients"));


  // pack 'em
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), mlogo_vb,  FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), vlabel, FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), nb, FALSE, FALSE, 0);

  gtk_widget_show_all(dialog);
  gtk_dialog_run(GTK_DIALOG(dialog));
  gtk_widget_destroy(dialog);
}

// ----------------------------------------------------------------------
// geneInfoDialog

// ------------------------------
// page abstraction
void gidPage::flush_hilights(){
  // flush all highlights
  for(int i=0; i<MAXSEQ; i++){ 
    if(gmhtable[i].size()!=0)
      for(unsigned j=gmhtable[i].size(); j!=0; j--) gmhtable[i].at(j-1).flush(pixbuf);
  }
}

gidPage::~gidPage(){};

// ------------------------------
// gid_browse

void gid_browse::cleanup(){
  for(int i=0; i<MAXSEQ; i++){ genetable[i].clear(); gmhtable[i].clear(); }
  //  delete maingene;  // this causes crash, but i don't know why...
}

void gid_browse::clear(){
  maingene->clear();

  for(int i=0; i<MAXSEQ; i++){ 
    for(unsigned j=0; j<genetable[i].size(); j++) genetable[i].at(j).clear();
    if(gmhtable[i].size()!=0)
      for(unsigned j=gmhtable[i].size(); j!=0; j--) gmhtable[i].at(j-1).clear();
  }

  for(unsigned int s=0; s<mr.acols()/3; s++){ 
    for(unsigned int i=0; i<genetable[s].size(); i++){
      gtk_widget_hide_all(genetable[s].at(i).widget());
    }
  }
}

GtkWidget* gid_browse::prepare(){
  GtkWidget *vbox;

  // initialize stuff
  vbox   = gtk_vbox_new(false, 0);
  follow_mouse_check = gtk_check_button_new_with_mnemonic("Follow to _Mouse");
  sticky_disp_check = gtk_check_button_new_with_mnemonic("_Sticky display");
  show_connected_check = gtk_check_button_new_with_mnemonic("Show _connected genes");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(follow_mouse_check), false);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(sticky_disp_check), true);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(show_connected_check), true);

  // attach 'em
  viewport = gtk_scrolled_window_new(NULL,NULL);
  maingene = new gid_infoWidgets();

  table = gtk_table_new(mr.acols()/3, 1, false);
  gtk_table_attach_defaults(GTK_TABLE(table), maingene->init(false), 0, 1, 0, 1);

  gtk_scrolled_window_add_with_viewport((GtkScrolledWindow*)viewport, table);
  gtk_scrolled_window_set_policy((GtkScrolledWindow*)viewport, 
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_widget_set_size_request(viewport, 300, -1);

  gtk_box_pack_start(GTK_BOX(vbox), viewport, true, true, 0);
  gtk_box_pack_start(GTK_BOX(vbox), follow_mouse_check, false, false, 0);
  gtk_box_pack_start(GTK_BOX(vbox), show_connected_check, false, false, 0);
  gtk_box_pack_end(GTK_BOX(vbox), sticky_disp_check, false, false, 0);

  // connect and show
  g_signal_connect(follow_mouse_check, "toggled", 
  		   G_CALLBACK(handle_activate), (gpointer)this);

  g_signal_connect(show_connected_check, "toggled", 
  		   G_CALLBACK(handle_activate), (gpointer)this);

  // initialize
  handlers.set_sensitive(sticky_disp_check, followMouse());
  return vbox;
}

void gid_browse::activate(GtkWidget *w){
  if(w==show_connected_check){
    clear();
    set_annotation(last_annotation, last_sequence);
  }

  // called only when toggled
  handlers.set_sensitive(sticky_disp_check, followMouse());

  if(showConnected()){
    // show detail
    gtk_scrolled_window_set_policy((GtkScrolledWindow*)viewport,
   				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  } else {
    // hide detail
    gtk_scrolled_window_set_policy((GtkScrolledWindow*)viewport,
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  }
}

void gid_browse::set_annotation(atIte ai, int seq){ // seq = geom #
  annotation a = mr.rat(ai);

  last_annotation = ai;
  last_sequence = seq;

  seqlen_t begin = a.begin;
  seqlen_t end   = a.end;
  if(begin>end) swap(begin,end);

  if(showConnected()){
    vector <set <annotationIndex> > related_genes(MAXSEQ);

    set<int> ancs; // anchors attached on gene "a" on "seq"
    mr.lookup_anchor_segments(seq, begin, end, ancs);
    for(set<int>::iterator anc=ancs.begin(); anc!=ancs.end(); anc++){
      if(!mr.anchorFilter(*anc)) continue;

      // connected genes on other seqs/tracks
      for(unsigned int s=0; s<mr.acols()/3; s++){
	seqlen_t af, at;
	af = abs(mr.raa_physical(*anc, s*3));
	at = abs(mr.raa_physical(*anc, s*3+1));
	if(af>at) swap(af, at);

	// scan tracks
	for(unsigned int t=0; 
	    t<mr.annoTracks(s).size(); t++){
	  if(!mr.annoTracks(s).at(t)->isTraceable()) continue;
	  
	  set<int> genes;
	  mr.annoTracks(s).at(t)->lookup_segment(af, at, genes);
	  for(set<int>::iterator gene=genes.begin(); gene!=genes.end(); gene++)
	    related_genes.at(s).insert(annotationIndex(s, t, *gene));
	}
      }
    }

    // count genes
    unsigned int genes[MAXSEQ];
    for(unsigned int s=0; s<mr.acols()/3; s++) genes[s]=related_genes[s].size();

    // prepare & show widgets
    for(unsigned int s=0; s<mr.acols()/3; s++){ 
      if(s!=(unsigned int)seq && s < mr.nseqs(status.toggle_seq)){

	// add if insufficient
	if(genetable[s].size() < genes[s]){
	  while(genetable[s].size() != genes[s]){
	    unsigned int pos = genetable[s].size();
	    genetable[s].push_back(gid_infoWidgets());
	    gtk_table_attach_defaults(GTK_TABLE(table),
				      genetable[s].at(pos).init(false),
				      pos, pos+1, s+1, s+2);
	    gmhtable[s].push_back(gmhilight());
	  }
	}

	// table: show & hide extras
	for(unsigned int i=0; i<genetable[s].size(); i++){
	  if(i<genes[s])
	    gtk_widget_show_all(genetable[s].at(i).widget());
	  else
	    gtk_widget_hide_all(genetable[s].at(i).widget());
	}

	// table: set annotations
	int c=0;
	for(set<annotationIndex>::iterator i=related_genes[s].begin(); 
	    i!=related_genes[s].end(); i++){
	  genetable[s].at(c).set_annotation(mr.annotations[status.seq_lookup(i->seq)].at(i->track)->at(i->no), s);
	  c++;
	}
	      
	// hilight
	unsigned hilights = 0;
	for(gene_markers_type::iterator j=status.gene_markers.begin();
	    j!=status.gene_markers.end(); j++){
	  annotationIndex *ref = (annotationIndex*)&(j->ai);
	  annotationIndex ref2 = annotationIndex(status.seq_rlookup(ref->seq),
						 ref->track, ref->no);

	  if(related_genes[s].find(ref2) !=  related_genes[s].end()){
	    gmhtable[s].at(hilights).focus(pixbuf, j->rect);
	    hilights++;
	  }
	}

	for(unsigned i=hilights; i<gmhtable[s].size(); i++){
	  gmhtable[s].at(i).clear();
	}
      } else { // just hide all
	for(unsigned int i=0; i<genetable[s].size(); i++)
	  gtk_widget_hide_all(genetable[s].at(i).widget());
	for(unsigned int i=0; i<gmhtable[s].size(); i++)
	  gmhtable[s].at(i).clear();
      }
    }
  }

  //  assert(cout << "gid_browse::set_annotation() : seq#" << seq << " "<< a->getname_auto() << " " << status.seq_lookup(seq) << endl);
  maingene->set_annotation(ai, seq);
}


bool gid_browse::isSticky() const{
  return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(sticky_disp_check));
}

bool gid_browse::showConnected() const{
  return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(show_connected_check));
}

bool gid_browse::followMouse() const{
  return gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(follow_mouse_check));
}

// ------------------------------
// gid_search

void gid_search::import_prefs(pref* p){
  p->getprefa(opts.name,    "GID_search_name");
  p->getprefa(opts.product, "GID_search_product");
  p->getprefa(opts.locus,   "GID_search_locus");
  p->getprefa(opts.note,    "GID_search_note");
  p->getprefa(opts.func,    "GID_search_func");
  p->getprefa(opts.casei,   "GID_search_casei");
}

void gid_search::export_prefs(pref* p){
  p->setpref("gmv_auto", opts.name,    "GID_search_name");
  p->setpref("gmv_auto", opts.product, "GID_search_product");
  p->setpref("gmv_auto", opts.locus,   "GID_search_locus");
  p->setpref("gmv_auto", opts.note,    "GID_search_note");
  p->setpref("gmv_auto", opts.func,    "GID_search_func");
  p->setpref("gmv_auto", opts.casei,   "GID_search_casei");
}

GtkWidget* gid_search::prepare(){
  resultsReady = false;

  GtkWidget* vbox;
  vbox = gtk_vbox_new(false, 0);
  
  // Option frame
  GtkWidget *optframe, *optframebox;
  optframe = gtk_frame_new("Query Options");
  optframebox = gtk_vbox_new(false, 0);
  gtk_container_add(GTK_CONTAINER(optframe), optframebox);

  // fields
  GtkWidget *fieldexpander, *fieldbox;
  fieldexpander = gtk_expander_new("Search in...");
  fieldbox = gtk_vbox_new(false, 0);
  gtk_container_add(GTK_CONTAINER(fieldexpander), fieldbox);
  gtk_box_pack_start(GTK_BOX(optframebox), fieldexpander, false, false, 0);
  
  name     = chkbox_setup(&opts.name,    "Name");
  product  = chkbox_setup(&opts.product, "Product");
  locus    = chkbox_setup(&opts.locus,   "Locus");
  note     = chkbox_setup(&opts.note,    "Note");
  function = chkbox_setup(&opts.func,    "Function");

  gtk_box_pack_start(GTK_BOX(fieldbox), name,     false, false, 0);
  gtk_box_pack_start(GTK_BOX(fieldbox), product,  false, false, 0);
  gtk_box_pack_start(GTK_BOX(fieldbox), locus,    false, false, 0);
  gtk_box_pack_start(GTK_BOX(fieldbox), note,     false, false, 0);
  gtk_box_pack_start(GTK_BOX(fieldbox), function, false, false, 0);

  // sequences
  GtkWidget *seqexpander, *seqbox;
  seqexpander = gtk_expander_new("Target sequences");
  seqbox = gtk_vbox_new(false, 0);
  gtk_container_add(GTK_CONTAINER(seqexpander), seqbox);
  gtk_box_pack_start(GTK_BOX(optframebox), seqexpander, false, false, 0);
  for(unsigned i=0; i<mr.acols()/3; i++){
    seqCheck[i] = gtk_check_button_new_with_mnemonic(string("Seq #_" + 
							    itos(i+1)).c_str());
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(seqCheck[i]), true);
    gtk_box_pack_start(GTK_BOX(seqbox), seqCheck[i], false, false, 0);
  }

  // orientation
  orientation = gtk_combo_box_new_text();
  gtk_combo_box_append_text(GTK_COMBO_BOX(orientation), "Search both strands");
  gtk_combo_box_append_text(GTK_COMBO_BOX(orientation), "Forward strand");
  gtk_combo_box_append_text(GTK_COMBO_BOX(orientation), "Backward strand");
  gtk_combo_box_set_active(GTK_COMBO_BOX(orientation), 0);
  gtk_box_pack_start(GTK_BOX(optframebox), orientation, false, false, 0);

  // search frame
  GtkWidget *termframe, *termframebox;
  termframe = gtk_frame_new("Find");
  termframebox = gtk_vbox_new(false, 0);
  gtk_container_add(GTK_CONTAINER(termframe), termframebox);
  gtk_box_pack_start(GTK_BOX(vbox), termframe, false, false, 0);

  queryTerm = gtk_entry_new();
  gtk_box_pack_start(GTK_BOX(termframebox), queryTerm, false, false, 0);
  gtk_box_pack_start(GTK_BOX(termframebox), optframe, false, false, 0);

  GtkWidget *termframe_lowerbox;
  termframe_lowerbox = gtk_hbox_new(false, 0);
  
  ignoreCase = chkbox_setup(&opts.casei, "Case _insensitive");
  queryButton = gtk_button_new_with_mnemonic("_Query");
  gtk_box_pack_start(GTK_BOX(termframe_lowerbox), ignoreCase, false, false, 0);
  gtk_box_pack_end(GTK_BOX(termframe_lowerbox), queryButton, false, false, 0);

  gtk_box_pack_start(GTK_BOX(termframebox), termframe_lowerbox, 
		     false, false, 0);

  g_signal_connect(queryButton, "clicked",
		   G_CALLBACK(handle_activate), (void*)this);
  handlers.connect(queryButton, HANDLER_GID);

  // ------------------------------
  // result frame
  GtkWidget *resultframe;
  GtkWidget *resultframebox, *resultViewport;
  GtkWidget *resultbuttonbox;

  // ---------
  // prepare

  // resultframebox
  resultStatusLabel = gtk_label_new(GID_SEARCH_DEFAULT_STATUS);
  resultframe = gtk_frame_new("Query Results");
  resultframebox = gtk_vbox_new(false, 0);

  // resultviewport & resultBox
  resultViewport = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(resultViewport), 
			 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  resultBox = gtk_vbox_new(false, 0);

  // nav buttons
  resultbuttonbox = gtk_hbox_new(false, 0);
  resultHead = gtk_button_new_from_stock(GTK_STOCK_GOTO_FIRST);
  resultPrev = gtk_button_new_from_stock(GTK_STOCK_GO_BACK);
  resultNext = gtk_button_new_from_stock(GTK_STOCK_GO_FORWARD);
  resultTail = gtk_button_new_from_stock(GTK_STOCK_GOTO_LAST);

  g_signal_connect(resultHead, "clicked",
		   G_CALLBACK(handle_activate), (void*)this);
  g_signal_connect(resultPrev, "clicked",
		   G_CALLBACK(handle_activate), (void*)this);
  g_signal_connect(resultNext, "clicked",
		   G_CALLBACK(handle_activate), (void*)this);
  g_signal_connect(resultTail, "clicked",
		   G_CALLBACK(handle_activate), (void*)this);

  handlers.connect(resultHead, HANDLER_GID);
  handlers.connect(resultPrev, HANDLER_GID);
  handlers.connect(resultNext, HANDLER_GID);
  handlers.connect(resultTail, HANDLER_GID);

  // ---------
  // packing

  // resultframebox
  gtk_container_add(GTK_CONTAINER(resultframe), resultframebox);

  gtk_box_pack_start(GTK_BOX(resultframebox), resultViewport, true, true, 0);
  gtk_box_pack_start(GTK_BOX(resultframebox), resultStatusLabel, false, false, 0);
  gtk_box_pack_end(GTK_BOX(resultframebox), resultbuttonbox, false, false, 0);

  // resultviewport & resultBox
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(resultViewport), resultBox);
  gtk_box_pack_start(GTK_BOX(vbox), resultframe, true, true, 0);

  // nav buttons
  gtk_box_pack_start(GTK_BOX(resultbuttonbox), resultHead, true, true, 0);
  gtk_box_pack_start(GTK_BOX(resultbuttonbox), resultPrev, true, true, 0);
  gtk_box_pack_start(GTK_BOX(resultbuttonbox), resultNext, true, true, 0);
  gtk_box_pack_end(GTK_BOX(resultbuttonbox), resultTail,   true, true, 0);

  return vbox;
}

void gid_search::activate(GtkWidget *w){
  if( w == resultHead ){
    matchlist_from = 0;
    showResults();
    return;
  }

  if( w == resultTail ){
    matchlist_from = (matches.size()/GID_SEARCH_RESULTS_TO_SHOW) * GID_SEARCH_RESULTS_TO_SHOW;
    showResults();
  }

  if( w == resultPrev ){
    if(matchlist_from > GID_SEARCH_RESULTS_TO_SHOW) 
      matchlist_from -= GID_SEARCH_RESULTS_TO_SHOW;
    showResults();
    return;
  }

  if( w == resultNext ){
    if(matchlist_from+GID_SEARCH_RESULTS_TO_SHOW < matches.size()) 
      matchlist_from += GID_SEARCH_RESULTS_TO_SHOW;
    showResults();
    return;
  }

  if( w == queryButton ){
    char* queryString = (char*)gtk_entry_get_text(GTK_ENTRY(queryTerm));
    char* q = queryString;

    // check whether query is empty or not
    while( *q != 0 ){ if(*q!=' ' && *q!='\t') break; else q++; }
    if(*q==0) return; 

    // prepare query
    bool ic = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ignoreCase));
    char* qq = strdup(queryString);

    if(ic){
      char *qqq = qq;
      while(*qqq!=0){
	*qqq = (char)toupper((int)*qqq); qqq++;
      }
    }

#ifndef USE_GLIB_REGEX
    regex_t reg_instance;
    regex_t* reg;
    int regcompstat;
    reg = &reg_instance;

    regcompstat = regcomp(reg, qq, REG_EXTENDED);
    if(regcompstat!=0) return;
#else
    GPatternSpec* reg = g_pattern_spec_new(string("*"+(string)qq+"*").c_str());
#endif

    // ready to search, and go
    matches.clear();
    matchlist_from = 0;
    
    for(unsigned i=0; i<mr.nseqs(status.toggle_seq); i++){
      unsigned skip=0;
      for(int j=0; j<status.seq_lookup(i); j++) if(!status.toggle_seq[j]) skip++;
      
      if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(seqCheck[status.seq_lookup(i)]))) continue;
      unsigned track = 0;

      // search
      unsigned ns = status.seq_lookup(i);

      for(rvector<annotationTrack*>::iterator j = mr.annotations[ns].begin();
	  j != mr.annotations[ns].end(); j++){

	atIte gene;
	int c = 0;
	for(unsigned a=0; a<((annotationTrack*)*j)->size(); a++){
	  gene = ((annotationTrack*)*j)->at(a);
	  if(mr.geneFilter((annotationTrack*)*j, a) && 
	     mr.rat(gene).match(reg, ic,
				gtk_combo_box_get_active(GTK_COMBO_BOX(orientation)),
				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(name)),
				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(product)),
				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(locus)),
				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(note)),
				gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(function))
				)){
	    annotationIndex ref = annotationIndex(ns, track, c);
	    matches.push_back(pair<atIte, annotationIndex>(gene, ref));
	  }
	  c++;
	}

	track++;
      }
    }

    // show 'em
    showResults();
  
    // match hilights
    hilight();
 
    // cleanup
    free(qq);
#ifndef USE_GLIB_REGEX
    regfree(reg);
#else
    g_pattern_spec_free(reg);
#endif
    return;
  }
}

void gid_search::showResults(){
  if(!resultsReady){
    for(unsigned i=0; i<GID_SEARCH_RESULTS_TO_SHOW; i++){
      queryResult[i].init(true);
      gtk_box_pack_start(GTK_BOX(resultBox),
			 queryResult[i].widget(), false, false, 0);
    }
    resultsReady = true;
  }

  unsigned matchcount;
  unsigned matchlist_to = matchlist_from + GID_SEARCH_RESULTS_TO_SHOW;
  if(matchlist_to >= matches.size()) matchlist_to=matches.size();

  unsigned entryNo = 0;
  for(matchcount=matchlist_from; matchcount<matchlist_to; matchcount++){
    queryResult[entryNo].set_annotation_nsn(matches[matchcount].first, 
					    matches[matchcount].second.seq);
    gtk_widget_show_all(queryResult[entryNo].widget());
    entryNo++;
  }

  gtk_label_set_text(GTK_LABEL(resultStatusLabel), 
		     string(itos(matchlist_from)+"-"+itos(matchlist_to)
			    +" of "+itos((unsigned)matches.size())).c_str());

  // hide extra entries
  for(unsigned i=matchcount-matchlist_from; i<GID_SEARCH_RESULTS_TO_SHOW; i++)
    gtk_widget_hide_all(queryResult[i].widget());

}

void gid_search::hilight(){
  unsigned hilightcount = 0;
  clear_hilights();
  assert(cout << "gid_search::hilight()" << endl);
  
  assert(cout << "hilight() : " << matches.size() << " matches" << endl);
  
  for(matchvec::iterator i=matches.begin(); i!=matches.end(); i++){
    int seq = status.seq_rlookup(i->second.seq); // i->second.seq = native seq#
    int offset = mr.get_offset_t(seq);
    annotation a = mr.rat(i->first);

    // highlight if it's in range
    if( ( (offset+a.begin) >= status.superzoom_from && 
	  (offset+a.begin) <= status.superzoom_to ) ||
	( (offset+a.end) >= status.superzoom_from && 
	  (offset+a.end) <= status.superzoom_to ) ){
      // find the marker

      // *i is in range: find the marker.
      for( gene_markers_type::iterator j=status.gene_markers.begin();
	   j!=status.gene_markers.end(); j++ ){
	if( j->ai == i->second ){ // hit! yeah!
	  if(hilightcount >= gmhtable[0].size()) 
	    gmhtable[0].push_back(gmhilight());
	  gmhtable[0].at(hilightcount).focus(pixbuf, j->rect);
	  hilightcount++;
	}
      }

      //      unsigned p=0;
      //
//       while( p<status.gene_refs[seq].size() ){
// 	if( status.gene_refs[seq].at(p) == i->second ){ // hit!
// 	  if(hilightcount >= gmhtable[0].size()) 
// 	    gmhtable[0].push_back(gmhilight());
// 	  gmhtable[0].at(hilightcount).focus(pixbuf,
// 					     status.gene_markers[seq].at(p));
// 	  hilightcount++;
// 	}
// 	p++;
//       }
    }
  }

  assert(cout << "hilight() : done" << endl);
}

void gid_search::cleanup(){
  assert(cout << "gid_search::cleanup()" << endl);
  clear_hilights();
  gmhtable[0].clear();
}

void gid_search::clear_hilights(){
  if(gmhtable[0].size()!=0)
    for(unsigned j=gmhtable[0].size(); j!=0; j--) gmhtable[0].at(j-1).clear();
}

void gid_search::clear(){
  assert(cout << "gid_search::clear()" << endl);
  clear_hilights();
  if(resultsReady){
    for(unsigned i=0; i<GID_SEARCH_RESULTS_TO_SHOW; i++)
      gtk_widget_hide_all(queryResult[i].widget());
  }
  matches.clear();
  //  gtk_entry_set_text(GTK_ENTRY(queryTerm), "");
  gtk_label_set_text(GTK_LABEL(resultStatusLabel), GID_SEARCH_DEFAULT_STATUS);
}

// ------------------------------
// gid

void geneInfoDialog::setByStatus(){
  if(!isOpen) return;
  string ap = mr.annotationProfile();
  string sp = status.sequenceProfile();

  if (ap==annotationProfile && sp==sequenceProfile) return;

  assert(cout << "geneInfoDialog::setByStatus() : cleaning up pages..." << endl);

  annotationProfile = ap;
  sequenceProfile = sp;

  for(unsigned i=0; i<pages.size(); i++) pages[i]->clear();
}

void geneInfoDialog::init(GtkWidget *mainwindow){
  if(isOpen) return;
  isOpen = true;
  pages.clear();

  annotationProfile = mr.annotationProfile();
  sequenceProfile = status.sequenceProfile();

  // prepare dialog
  dialog = gtk_dialog_new_with_buttons ("Gene Information",
					GTK_WINDOW(mainwindow),
					GTK_DIALOG_DESTROY_WITH_PARENT,
					GTK_STOCK_CLOSE,
					GTK_RESPONSE_OK,
					NULL);
  // prepare notebook
  notebook = gtk_notebook_new();
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), notebook);

  // setup pages
  pages.push_back((gidPage*)&browse);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), browse.prepare(), 
			   gtk_label_new("Browse"));

  pages.push_back((gidPage*)&search);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), search.prepare(), 
			   gtk_label_new("Search"));

  // signals
  g_signal_connect(dialog, "response",
		   G_CALLBACK(handle_dialogResponse), (gpointer)this);

  // connect signals in action area
  GList *buttons, *b;
  buttons = gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area));
  for(b=buttons; b; b=b->next){
    handlers.connect(b->data, HANDLER_GID);
  }
  g_list_free(buttons);

  // here we go
  int main_x, main_y, main_w, main_h;
  gtk_window_get_position(GTK_WINDOW(mainwindow), &main_x, &main_y);
  gtk_window_get_size(GTK_WINDOW(mainwindow), &main_w, &main_h);

  gtk_window_set_default_size(GTK_WINDOW(dialog), 150, main_h);
  gtk_window_move(GTK_WINDOW(dialog), main_x+main_w , main_y);
  gtk_widget_show_all(dialog);
}

void geneInfoDialog::activate(GtkWidget *w){
  if(!isOpen) return;
  for(unsigned i=0; i<pages.size(); i++) pages[i]->activate(w);
}

void geneInfoDialog::response(gint r){
  if(!isOpen) return;
  handlers.remove_dialog(HANDLER_GID);

  // clear widgets
  for(unsigned i=0; i<pages.size(); i++) pages[i]->clear();

  isOpen = false;
  gtk_widget_destroy(dialog);
  for(unsigned i=0; i<pages.size(); i++) pages[i]->cleanup();
  //  for(unsigned i=0; i<pages.size(); i++) delete pages[0];
  //  delete (gid_search*)pages[1];
  //  delete (gid_browse*)pages[0];
  pages.clear();
}

void geneInfoDialog::clear(){
  if(!isOpen) return;
  for(unsigned i=0; i<pages.size(); i++) pages[i]->clear();
}

void geneInfoDialog::set_annotation(atIte a, int seq){
  if(!isOpen) return;

  if(gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)) == GID_PAGE_BROWSE) 
    browse.set_annotation(a, seq);
}

void geneInfoDialog::flush_hilights(){
  if(!isOpen) return;
  assert(cout << "geneInfoDialog::flush_hilights()" << endl);
  for(unsigned i=0; i<pages.size(); i++) pages[i]->flush_hilights();
}

void geneInfoDialog::redraw_hilights(){
  if(isOpen) search.hilight();
}


bool geneInfoDialog::isBrowserSticky() const{ 
  if(!isOpen) return false;
  return browse.isSticky(); };

bool geneInfoDialog::browserShowConnected() const{
  if(!isOpen) return false;
  return browse.showConnected(); 
}

bool geneInfoDialog::browserFollowMouse() const{
  if(!isOpen) return false;
  return browse.followMouse(); 
}

void geneInfoDialog::clearBrowser(){
  if(!isOpen) return;
  return browse.clear(); 
}


// ------------------------------
// GID Info widget

void gid_infoWidgets::activate(GtkWidget *w){
  if(w==dbmenu){
    int n = gtk_combo_box_get_active(GTK_COMBO_BOX(dbmenu));

    if(n==0) return;

    if((unsigned)n <= dbrefTable.size()){
      // indivisual
      launch_browser(dbrefTable[n-1].c_str());
    } else {
      // all
      for(unsigned i=0; i<dbrefTable.size(); i++)
	launch_browser(dbrefTable[i].c_str());
    }
    return;
  }

  if(w==go){
//     cout << anno->getname_auto() << endl;
//     cout << "seq #" << sequence << " " << anno->begin << "-" << anno->end << endl;

    int seq = status.seq_rlookup(sequence);

    annotation a = mr.rat(anno);
    seqlen_t gf = a.begin;
    seqlen_t gt = a.end;

    int overwraps=0;
    seqlen_t offsets[MAXSEQ];
    for(unsigned s=0; s<mr.acols()/3; s++) offsets[s]=0;

    bool score_loaded = !mr.tfidf.empty();
    for(unsigned i=0; i<mr.arows(); i++){
      // tf-idf filter to justify
      if(!mr.anchorFilter(i)) continue;

      if(score_loaded){ 
	if(mr.tfidf.at(i) < mr.tfidf.getmin()) continue;
	if(mr.tfidf.at(i) > mr.tfidf.getmax() &&
	   (!mr.tfidf.empty())) continue;
      }
      seqlen_t af, at;
      af = abs(mr.raa_physical(i, seq*3));
      at = abs(mr.raa_physical(i, seq*3+1));
      if(af>at){ seqlen_t tmp=af; af=at; at=tmp;  }
      
      // FIXME: with segment tree
      if((af<=gf && gf<=at) || (af<=gt && gt<=at) ||
	 (gf<=af && at<=gt)){
	overwraps++;
	
	for(unsigned s=0; s<mr.acols()/3; s++){
	  seqlen_t of = abs(mr.raa_physical(i, s*3));
	  seqlen_t ot = abs(mr.raa_physical(i, s*3+1));
	  
	  offsets[s] += ((af+at)-(of+ot))/2;
	}
      }
    }

    if(overwraps != 0){
      // adjust offset
      mr.clear_offset();
      for(unsigned s=0; s<mr.acols()/3; s++){
	offsets[s]/=overwraps;
	mr.set_offset(status.seq_lookup(s), offsets[s]);
      }
    }

    // then zoom & center it (even if there is no anchors)
    status.plot_type = PLOT_SUPERZOOM;
    seqlen_t center = mr.get_offset_t(seq) + (gf+gt)/2;
    seqlen_t glen = abs(gt-gf);
    status.superzoom_from = center - 2*glen;
    status.superzoom_to = center + 2*glen;
    if(status.superzoom_from<0) status.superzoom_from=0;
    if(status.superzoom_to>mr.maxbases(status.toggle_seq))
      status.superzoom_to=mr.maxbases(status.toggle_seq);
    redraw();
  }
}

void gid_infoWidgets::clear(){
  gtk_frame_set_label(GTK_FRAME(frame),      "Gene Information");
  gtk_label_set_text(GTK_LABEL(range),       "");
  gtk_label_set_text(GTK_LABEL(type),        "");
  gtk_label_set_text(GTK_LABEL(name),        "");
  gtk_label_set_text(GTK_LABEL(product),     "");
  gtk_label_set_text(GTK_LABEL(locus),       "");
  gtk_label_set_text(GTK_LABEL(orientation), "");
  gtk_label_set_text(GTK_LABEL(note),        "");
  gtk_label_set_text(GTK_LABEL(function),    "");
  gtk_label_set_text(GTK_LABEL(translation), "");

  dbrefTable.clear();
  if (dbmenuActive) gtk_widget_destroy(dbmenu);
  dbmenuActive = false;
}

void gid_infoWidgets::set_annotation(atIte a, int seq){
  set_annotation_nsn(a, status.seq_lookup(seq));
}

void gid_infoWidgets::set_annotation_nsn(atIte ai, int seq){
  anno = ai;
  sequence = seq;

  annotation a = mr.rat(ai);

  seqlen_t begin = a.begin;
  seqlen_t end   = a.end;
  
  // lookup source
  string source;
  int source_count=0;
  for(unsigned i=0; i<mr.sources_size(seq); i++){
    seq_source src = mr.get_source(seq, i);
    seqlen_t sbegin = src.begin;
    seqlen_t send   = src.end;

    if(sbegin<=begin && begin<=send && sbegin<=end && end<=send){
      if (src.getstr_domestic() != "")
	source = ( "(" + src.getstr_domestic() + ":" +
		   itos(begin-sbegin) + "-" + itos(end-sbegin) + ")" );
    }
    source_count++;
  }
  if(source_count < 2) source="";

  // set labels
  string s = ("Sequence #" +itos(seq+1));
  string r = (itos(begin) + "-" + itos(end)) + source;
  string o = (a.reverse) ? "Backward" : "Forward"; 

  
  gtk_frame_set_label(GTK_FRAME(frame),      s.c_str());
  gtk_label_set_text(GTK_LABEL(range),       r.c_str());
  gtk_label_set_text(GTK_LABEL(type),        a.atype.c_str());
  gtk_label_set_text(GTK_LABEL(name),        a.name.c_str());
  gtk_label_set_text(GTK_LABEL(product),     a.product.c_str());
  gtk_label_set_text(GTK_LABEL(locus),       a.locus.c_str());
  gtk_label_set_text(GTK_LABEL(note),        a.note.c_str());
  gtk_label_set_text(GTK_LABEL(function),    a.function.c_str());
  gtk_label_set_text(GTK_LABEL(translation), a.translation.c_str());
  gtk_label_set_text(GTK_LABEL(orientation), o.c_str());


  // DBlinks should not overwritten!
  if(dbmenuActive) gtk_widget_destroy(dbmenu);
  dbmenuActive = false;
  dbrefTable.clear();

  set<string> *dbref = &a.dbref;
  vector<string> dbmenuText;
  for(set<string>::iterator i=dbref->begin(); i!=dbref->end(); i++){
    //      cout << "dbref=" << *i << endl;
    dblink* d=dblinks;
    string dbname, accession;
    char url[1025];
    while(d->dbname != NULL){
      if(i->compare(0, strlen(d->dbtag), (string)d->dbtag) == 0){
	int pos = i->find(":");
	dbname = (string)d->dbname;
	accession = i->substr(pos+1, i->length()-pos-1);
	snprintf(url, 1024, d->url, accession.c_str());
      }
      d++;
    }
    if( dbname!="" && accession!="" ){
      dbmenuText.push_back( dbname + "(" + accession + ")" );
      dbrefTable.push_back(url);
    }
  }
  
  if(dbrefTable.size() > 1) dbmenuText.push_back("Open All");

  if(dbrefTable.size() > 0){
    char select[1025];
    snprintf(select, 1024, "Select... (%d entries)", (int)dbrefTable.size());

    dbmenu      = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX(dbmenu), select);
    gtk_combo_box_set_active(GTK_COMBO_BOX(dbmenu), 0);
    gtk_table_attach(GTK_TABLE(table), dbmenu,      1, 2, db_pos, db_pos+1,
		     GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
    gtk_widget_show(dbmenu);

    for(unsigned i=0; i<dbmenuText.size(); i++)
      gtk_combo_box_append_text(GTK_COMBO_BOX(dbmenu), dbmenuText[i].c_str());

    g_signal_connect(dbmenu, "changed", 
		     G_CALLBACK(handle_activate), (gpointer)this);

    dbmenuActive = true;
  }
}

gboolean toggle_label_mode(GtkWidget *useless, GdkEventButton *eb, gpointer w){
  GtkWidget  *label = (GtkWidget*)w;

  if(gtk_label_get_line_wrap(GTK_LABEL(label))){
    gtk_label_set_line_wrap(GTK_LABEL(label), false);
    gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_END);
  } else {
    gtk_label_set_line_wrap(GTK_LABEL(label), true);
#ifndef OLD_GTK_COMPAT
    gtk_label_set_line_wrap_mode(GTK_LABEL(label), PANGO_WRAP_WORD_CHAR);
#endif
    gtk_label_set_ellipsize(GTK_LABEL(label), PANGO_ELLIPSIZE_NONE);
  }
  return true;
}

GtkWidget* gid_infoWidgets::setup_label(const char* s){
  GtkWidget *l = gtk_label_new(s);
  gtk_misc_set_alignment(GTK_MISC(l), 0, 0);
  return l;
}

GtkWidget* gid_infoWidgets::setup_label_toggle(GtkWidget *label, GtkWidget *toggledlabel){
  GtkWidget* eb = gtk_event_box_new();

  string l = string(gtk_label_get_text(GTK_LABEL(label)));
  gtk_label_set_markup(GTK_LABEL(label), string("<u>" + l + "</u>").c_str());

  gtk_label_set_ellipsize(GTK_LABEL(toggledlabel), PANGO_ELLIPSIZE_END);
  gtk_label_set_selectable(GTK_LABEL(toggledlabel), true);
  gtk_container_add(GTK_CONTAINER(eb), label);
  gtk_widget_add_events(eb, GDK_BUTTON_PRESS_MASK);
  gtk_tooltips_set_tip(tips, eb, "Click to toggle to show ellipsized/full", "");
  g_signal_connect(eb, "button-press-event", 
		   G_CALLBACK(toggle_label_mode), toggledlabel);
  return eb;
}

GtkWidget* gid_infoWidgets::init(bool show_goButton){
  frame=gtk_frame_new("Gene Information");
  table=gtk_table_new(7, 2, false);

  GtkWidget* hbox = gtk_hbox_new(false, 0);

  tips = gtk_tooltips_new();

  rl = setup_label("Range:");
  tl = setup_label("Type:");
  nl = setup_label("Name: ");      
  pl = setup_label("Product:");  
  ll = setup_label("Locus:");    
  ol = setup_label("Orientation:");
  dl = setup_label("DB Links:");
  nol= setup_label("Note:");
  ful= setup_label("Function:");
  trl= setup_label("Translation:");

  range       = setup_label("");
  type        = setup_label("");
  name        = setup_label("");
  product     = setup_label("");
  locus       = setup_label("");  
  orientation = setup_label("");
  note        = setup_label("");
  function    = setup_label("");
  translation = setup_label("");

  gtk_label_set_selectable(GTK_LABEL(name), true);
  gtk_label_set_selectable(GTK_LABEL(product), true);
  gtk_label_set_selectable(GTK_LABEL(locus), true);

  GtkWidget *nlb, *plb, *nolb, *fulb, *trlb;

  nlb  = setup_label_toggle(nl,  name);
  plb  = setup_label_toggle(pl,  product);
  nolb = setup_label_toggle(nol, note);
  fulb = setup_label_toggle(ful, function);
  trlb = setup_label_toggle(trl, translation);

  gtk_table_attach(GTK_TABLE(table), rl,  0, 1, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), tl,  0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), nlb, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), plb, 0, 1, 3, 4, GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), ll,  0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), ol,  0, 1, 5, 6, GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), nolb,0, 1, 6, 7, GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), fulb,0, 1, 7, 8, GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), trlb,0, 1, 8, 9, GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), dl,  0, 1, db_pos, db_pos+1,
		   GTK_FILL, GTK_FILL, 0, 0);


  gtk_box_pack_start(GTK_BOX(hbox), range, true, true, 0);
  if(show_goButton){
    go = gtk_button_new_with_label("Go");
    handlers.connect(go, HANDLER_GID);
    gtk_box_pack_end(GTK_BOX(hbox), go, false, false, 0);
    g_signal_connect(go, "clicked", 
		     G_CALLBACK(handle_activate), (gpointer)this);
    handlers.connect(go, HANDLER_GID);
  } else {
    go = NULL;
  }

  gtk_table_attach(GTK_TABLE(table), hbox,        1, 2, 0, 1,
		   GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), type,        1, 2, 1, 2,
		   GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), name,        1, 2, 2, 3,
		   GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), product,     1, 2, 3, 4,
		   GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), locus,       1, 2, 4, 5,
		   GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), orientation, 1, 2, 5, 6,
		   GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), note,        1, 2, 6, 7,
		   GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), function,    1, 2, 7, 8,
		   GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), translation, 1, 2, 8, 9,
		   GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);

  gtk_container_add(GTK_CONTAINER(frame), table);

  dbmenuActive = false;
  dbrefTable.clear();

  return widget();
}
