using namespace std;

#ifndef _DIALOGS_H_
#define _DIALOGS_H_

#include <iostream>
#include <vector>
#include "gmv_defs.h"
#include "mainwindow.h"
#include "annotation.h"

#include "dialog_base.h"
#include "util.h"

#include "general_list.h"

#define GID_PAGE_BROWSE  0
#define GID_PAGE_SEARCH  1

#define MZD_RANGESCALE (w == rangeScale)
#define MZD_FROMSCALE  (w == fromScale)
#define MZD_TOSCALE    (w == toScale)
#define MZD_FROMBOX    (w == fromBox)
#define MZD_TOBOX      (w == toBox)
#define MZD_RANGEBOX   (w == rangeBox)
#define MZD_ZIN        (w == range_zin)
#define MZD_ZOUT       (w == range_zout)
#define MZD_GONEXT     (w == goNext)
#define MZD_GOPREV     (w == goPrev)
#define MZD_GOHEAD     (w == goHead)
#define MZD_GOTAIL     (w == goTail)

#define GID_SEARCH_RESULTS_TO_SHOW 50
#define GID_SEARCH_DEFAULT_STATUS "Click \"Query\" to execute query."

// ----------------------------------------------------------------------
// confirmation dialog
bool confirm_to_flush();

// ----------------------------------------------------------------------
// sequence souce sorter

class manual_source_sort_dialog : public generalDialog {
public:
  void activate(GtkWidget*);
  void release(GtkWidget*){};

  void init(GtkWidget*);
  void response(int);

private:
  class segItem : public generalListItem<int, string, int, int> {
  public:
    // v is reserved for future extension
    int number() const { return w; };
    int number(int i){ t=i+1; w=i; return w; };
    string name() const { return u; };
    string name(string& s){ u=s; return u; };
    segItem(){};
    segItem(int i, string s){ number(i); name(s); };
  };

  void reset_page(int);
  GtkWidget *notebook, *reset_all, *reset_seq;
  vector <generalList<segItem>*> segmentList;
  vector <int> seqnoList;
};

class source_sort_dialog : public generalDialog {
public:
  void activate(GtkWidget*);
  void release(GtkWidget*){};

  void init(GtkWidget*);
  void response(int);

private:
  GtkWidget *keep_target_order, *apply_anchor_filter;
  GtkWidget *sel_all, *desel_all;

  GtkWidget *reference, *method;
  map<GtkWidget*, int> target_seqs;
};

// ----------------------------------------------------------------------
// report

gpointer repd_subthread(gpointer);
bool repd_idle(gpointer);

class report {
public:
  report(){ s=""; header=""; fw.clear(); f=fh=0; l=0; head=true; 
    head_header=true; caption=""; };
  void clear(){ report(); };

  void set_caption(string ss){ caption=ss; };

  void add_header(string ss);
  void add(string ss);

  void end_header(){ header+="\n"; };
  void endl(){ f=0; s+="\n"; l++; head=true; f=0; };

  string str() const { return caption+header+s; };
  bool isEmpty() const { return(s.size()==0); }

  PangoTabArray* tabs(GtkWidget *w);

private:
  void add_internal(string &sss, bool &h, unsigned &ff, string ss);

  bool head_header;
  bool head;
  string header;

  string s;
  string caption;
  vector<unsigned> fw;
  int l;
  unsigned int f, fh;
};

class reportDialog : public generalDialog{
public:
  void init(GtkWidget *);
  void release(GtkWidget*){};
  void activate(GtkWidget*);
  void response(gint);

  void showit();

  bool is_thread_running();
  void subthread_terminated();
  void generateReports();

  vector<annotationTrack*> filterTracks;

  string cog_load_path(){ return string(loadcog ? cogpath : "");};

protected:
  GtkTextBuffer **reports;
  GtkTextView   **views;
  GtkWidget     **selectors;

  GtkWidget *pleaseWait;

  GMutex* repo_mutex;

  bool thread_running;
  bool widgets_ready;

  // report options 
  GtkWidget *cog_option_box;
  GtkWidget *hideInvisible, *showScoreDetail, *showFeatureTypes;
  GtkWidget *applyOptions;
  GtkWidget *saveButton;
  GtkWidget *nb;

  bool cog_option_box_ready;

  bool hide_invisible, show_score_detail, show_feature_types;
  GtkWidget *loadcog_button;
  GtkWidget *check_ortholog, *show_cog_group, *check_cog_gene;
  bool loadcog;
  string cogpath;

  // report storage
  int specialReports;
  int gcrep_page, srep_firstpage;
  report arep, gcrep; 
  vector<report> srep; // segment report
  vector<report> grep; // gene filter report
  vector<bool> reportable;

  // feature filter option
  GtkCheckButton **feature_checks;

  set<string> features;
  vector<string> features_vec;
  map<string, bool> features_en;

  GtkWidget *feature_filter_all_off;
  GtkWidget *feature_filter_all_on;

  // gene connection report statistics
  bool has_gcrep_stat;
  double tfidf_sum_max, tfidf_sum_min;
  double tfidf_avg_max, tfidf_avg_min;
  unsigned int num_anc_min, num_anc_max;

  GtkWidget *radio_num_anc, *radio_tfidf_avg, *radio_tfidf_sum;
  GtkWidget *scale_num_anc, *scale_tfidf_avg, *scale_tfidf_sum;
  GtkWidget *spbtn_num_anc, *spbtn_tfidf_avg, *spbtn_tfidf_sum;

  //
  
  void displayReports();

  report ancReport();
  report geneReport(int, annotationTrack*, int, int);
  report geneConnectionReport();
  report segmentReport(unsigned);

  typedef vector<int> gene_set_t;           // geneConnectionReport::gene_set
  typedef map< gene_set_t, set<int> > gene_sets_t;    // geneConnectionReport::gene_sets
  typedef vector< vector<int> > matchvec_t; // geneConnectionReport::matchvec

  void gcrep_unfold(gene_sets_t&, matchvec_t&, gene_set_t&, int, int, int);

};

// ----------------------------------------------------------------------
// statistics

gpointer statd_subthread(gpointer);
bool statd_idle(gpointer);

class statisticsDialog : public generalDialog{
public:
  void init(GtkWidget *);
  void release(GtkWidget*){};
  void activate(GtkWidget*);
  void response(gint);

  void show_stats();
  void subthread_terminated();
  bool is_thread_running();
  void show_new_stats();

protected:
  void attachLabel(GtkWidget*, string, guint, guint, guint, guint);
  GtkWidget *generatedetail;
  GtkWidget *vbox;

  GtkWidget *savehtml, *savetext;

  GMutex *stat_mutex;
  bool thread_running;
};


// ----------------------------------------------------------------------
// configure

class preferenceDialog : public generalDialog{
public:
  void init(GtkWidget *);
  void release(GtkWidget*){};
  void activate(GtkWidget*);
  void response(gint);

protected:
  GtkWidget *m_f_load, *m_f_save, *m_f_reset, *m_f_apply, *m_f_close;

  void redisp_all();
};

// ----------------------------------------------------------------------
// about

void aboutDialog(GtkWidget*);

// ----------------------------------------------------------------------
// gene info dialog

// ------------------------------
// gene info summary

class gid_infoWidgets : public generalWidgets {
public:
  GtkWidget* init(bool);
  GtkWidget* widget(){ return frame; };
  void set_annotation(atIte, int);      // geometrical seq #
  void set_annotation_nsn(atIte, int);  // native seq #
  void clear();
  void activate(GtkWidget *w);
  void release(GtkWidget *w){};

  void cleanup(); 

private:
#define db_pos 9

  GtkWidget* setup_label_toggle(GtkWidget *label, GtkWidget *toggledlabel);
  GtkWidget* setup_label(const char*);

  GtkTooltips *tips;

  GtkWidget *frame;
  GtkWidget *table;

  GtkWidget *rl, *range;
  GtkWidget *tl, *type;
  GtkWidget *nl, *name;
  GtkWidget *pl, *product;
  GtkWidget *ll, *locus;
  GtkWidget *ol, *orientation;
  GtkWidget *dl, *dbmenu;
  GtkWidget *nol, *note;
  GtkWidget *ful, *function;
  GtkWidget *trl, *translation;

  GtkWidget *go;

  atIte anno;
  int sequence;     // native seq #

  bool dbmenuActive;
  vector<string> dbrefTable;
};

class gidPage;
class gid_browse;
class gid_search;


// ------------------------------
// pages

void gidPageClicked(GtkWidget *w, gpointer p);

// abstraction
class gidPage : public generalWidgets{
public:
  virtual GtkWidget* prepare() = 0;
  virtual void activate(GtkWidget*) = 0;
  void release(GtkWidget*){};
  virtual void cleanup() = 0;
  virtual void clear()   = 0;
  void flush_hilights();

  virtual ~gidPage() = 0;
protected:
  vector<gmhilight> gmhtable[MAXSEQ];
};

// browse
class gid_browse : public gidPage{
public:
  GtkWidget* prepare();
  void activate(GtkWidget*);
  void cleanup();
  void clear();

  void set_annotation(atIte, int);
  bool isSticky() const;
  bool showConnected() const;
  bool followMouse() const;
    
protected:
  atIte last_annotation;
  int last_sequence;

  GtkWidget *viewport;
  GtkWidget *table;
  int table_rows;

  vector<gid_infoWidgets> genetable[MAXSEQ];

  GtkWidget *follow_mouse_check;
  GtkWidget *sticky_disp_check;
  GtkWidget *show_connected_check;

  gid_infoWidgets *maingene;
};

// search
class gid_search : public gidPage {
public:
  GtkWidget* prepare();
  void activate(GtkWidget*);
  void cleanup();
  void clear();

  void hilight();
  void clear_hilights();

  void import_prefs(pref*);
  void export_prefs(pref*);

  gid_search(){
    opts.name = opts.product = opts.locus = opts.note = opts.func = true; 
    opts.casei = false;
  };

protected:
  GtkWidget *queryTerm;
  GtkWidget *queryButton;
  GtkWidget *ignoreCase;
  GtkWidget *seqCheck[MAXSEQ];
  GtkWidget *resultBox;
  GtkWidget *resultStatusLabel;
  GtkWidget *resultHead, *resultPrev, *resultNext, *resultTail;
  GtkWidget *name, *product, *locus, *note, *function, *orientation;

  //vector<gid_infoWidgets> queryResult;
  gid_infoWidgets queryResult[GID_SEARCH_RESULTS_TO_SHOW];

  typedef vector<pair<atIte, annotationIndex> > matchvec;
  matchvec matches;
  unsigned matchlist_from;

  bool resultsReady;

  void showResults();

  struct {
    bool name;
    bool product;
    bool locus;
    bool note;
    bool func;
    bool casei;
  } opts;
};


class geneInfoDialog : public generalDialog{
public:
  void init(GtkWidget *mainwindow);
  void release(GtkWidget*){}; 
  void activate(GtkWidget*);
  void response(int);

  void setByStatus();

  void set_annotation(atIte, int);
  void clear();
  void flush_hilights();
  void redraw_hilights();

  bool isActive() const {return isOpen;};

  bool isBrowserSticky() const;
  bool browserShowConnected() const;
  bool browserFollowMouse() const;
  void clearBrowser();

  geneInfoDialog(){ isOpen=false; };

  void import_prefs(pref* p){ search.import_prefs(p); };
  void export_prefs(pref* p){ search.export_prefs(p); };

protected:
  gid_browse browse;
  gid_search search;

  vector <gidPage*> pages;
  GtkWidget *notebook;
  bool isOpen;

  string annotationProfile;
  string sequenceProfile;
};

#endif
