#include "gmv.h"
#include <gtk/gtk.h>
#include <cairo.h>

#ifdef CAIRO_HAS_PDF_SURFACE
#include <cairo-pdf.h>
#endif 

#include "gmv_defs.h"
#include "export.h"
#include "mainwindow.h"
#include "mainwindow_handlers.h"
#include "messages.h"

// ----------------------------------------------------------------------
//  PDF

#ifdef CAIRO_HAS_PDF_SURFACE

void generatePDF_subthread(gpointer file){
  assert(cout << "generating PDF..." << endl);
  cairo_t *cr;
  cairo_surface_t *surface;

  surface = cairo_pdf_surface_create((char *)file, width(), height());
  
  cr = cairo_create(surface);
  draw(cr, &mr);
  cairo_show_page(cr);
  
  cairo_surface_destroy (surface);
  cairo_destroy (cr);
  assert(cout << "PDF wrote successfully" << endl);

  g_mutex_lock(status.itc_mutex);
  status.thread_running = false;
  g_mutex_unlock(status.itc_mutex);
}

void generatePDF(const char* file){
  handlers.block();

  g_mutex_lock(status.itc_mutex);
  status.thread_running = true;
  status.wait_for_disp  = true;
  g_mutex_unlock(status.itc_mutex);

  // launch sub-thread
  g_thread_create((GThreadFunc)generatePDF_subthread, (gpointer)file, false, NULL);

  // set idle process to display
  gint* tag = (gint*)malloc(sizeof(gint));
  *tag = background_add((GSourceFunc)wait_and_disp, tag);
}

void handle_FileSavePDF(GtkWidget  *widget,  gpointer useless_pointer){
  GtkWidget *dialog;
  bool filesave = false;
  string pdfname;
  
  dialog = gtk_file_chooser_dialog_new("Save PDF",
				       GTK_WINDOW(status.mainwindow),
				       GTK_FILE_CHOOSER_ACTION_SAVE,
				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				       GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
				       (gpointer)NULL);

  GtkFileFilter *filter = gtk_file_filter_new();
  gtk_file_filter_add_pattern(filter, "*.pdf");
  gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filter);
  
  gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
  gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), 
				(status.basename+"-"+plot_type_str()+".pdf").c_str());
  
  if (gtk_dialog_run (GTK_DIALOG (dialog))==GTK_RESPONSE_ACCEPT){
    char *filename;
    filesave = true;
    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    pdfname = filename;
    g_free (filename);
  }
  gtk_widget_destroy (dialog);

  if(filesave)
      generatePDF(pdfname.c_str());
}
#endif

// ----------------------------------------------------------------------
//  PNG

void generatePNG_subthread(gpointer file){
  assert(cout << "generating PNG..." << endl);
  cairo_t *cr;
  cairo_surface_t *surface;
  surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width(), height());
  
  cr = cairo_create (surface);

  // clear the surface
  cairo_set_source_rgba(cr, 1, 1, 1, 0);
  cairo_rectangle(cr, 0, 0, width(), height());
  cairo_set_line_width (cr, 1);
  cairo_stroke (cr);
  cairo_fill(cr);

  if(status.antialias) cairo_set_antialias(cr, CAIRO_ANTIALIAS_SUBPIXEL);
  else                  cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);

  draw (cr, &mr);
  cairo_surface_write_to_png(surface, (char *)file);
  
  cairo_surface_destroy (surface);
  cairo_destroy (cr);

  g_mutex_lock(status.itc_mutex);
  status.thread_running = false;
  g_mutex_unlock(status.itc_mutex);
}

void generatePNG(const char* file){
  handlers.block();
  g_mutex_lock(status.itc_mutex);
  status.thread_running = true;
  status.wait_for_disp  = true;
  g_mutex_unlock(status.itc_mutex);

  // launch sub-thread
  g_thread_create((GThreadFunc)generatePNG_subthread, 
		  (gpointer)file, false, (GError**)NULL);

  // set idle process to display
  gint* tag = (gint*)malloc(sizeof(gint));
  *tag = background_add((GSourceFunc)wait_and_disp, tag);
}

void handle_FileSavePNG(GtkWidget  *widget, gpointer useless_pointer){
  GtkWidget *dialog;
  bool filesave = false;
  string pngname;

  assert(cout << "handle_FileSavePNG()" << endl);
  
  dialog = gtk_file_chooser_dialog_new("Save PNG",
				       GTK_WINDOW(status.mainwindow),
				       GTK_FILE_CHOOSER_ACTION_SAVE,
				       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				       GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
				       (gpointer)NULL);

  GtkFileFilter *filter = gtk_file_filter_new();
  gtk_file_filter_add_pattern(filter, "*.png");
  gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filter);
  
  gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
  gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(dialog), 
				(status.basename+"-"
				 +plot_type_str()+".png").c_str());
  
  if (gtk_dialog_run (GTK_DIALOG (dialog))==GTK_RESPONSE_ACCEPT){
    char *filename;
    filesave = true;
    filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    pngname = filename;
    g_free (filename);
  }
  gtk_widget_destroy (dialog);

  if(filesave){
    // save pixbuf directly
    gdk_pixbuf_save(pixbuf,
		    pngname.c_str(),
		    "png", NULL, (gpointer)NULL);

    //    generatePNG(pngname.c_str());
  }
}

// ----------------------------------------------------------------------
//  GDK_PIXBUF

void free_tmpimg(guchar *pixels, gpointer data){
  assert(cout << "free_tmpimg(): freeing tempimg for gdk_pixbuf!" << endl);
  free(pixels);
}

GdkPixbuf *generatePixbuf(results *data){
  //  assert(cout << "generatePixbuf(): generating Pixbuf..." << endl);
  cairo_t *cr;
  cairo_surface_t *surface;
  GdkPixbuf *pixbuf;

  bool cacheable = (status.plot_type == PLOT_DOT || 
		    status.plot_type == PLOT_CHROM_LINKS ||
		    status.plot_type == PLOT_SUPERZOOM );

  // try the cache
  if (cacheable){
    pixbuf_cache_tag t; t.get_current_globs();
    pixbuf = pbcache.lookup(t);
    if (pixbuf != NULL){
      assert(cout << "cache hit!" << endl);

      pixbuf_cache_item *citem = pbcache.lookup_cache(t);
      status.basewidth    = citem->basewidth;
      status.gene_markers = citem->gene_markers;
      status.in_range_anchors = citem->in_range_anchors;
      for(set<int>::iterator i=status.in_range_anchors.begin();
	  i!=status.in_range_anchors.end(); i++){
	status.overwrap_ancs.erase(*i);
      }
      return pixbuf;
    }
  }
  
  // no hit...
  //  assert(cout << "generatePixbuf(): allocating tmpimg..." << endl);
  unsigned char* tmpimg = (unsigned char*)malloc(width()*height()*4);
  if(tmpimg==NULL){
    cerr << "generatePixbuf(): malloc() for tmpimg failed." << endl;
    cerr << "image size was " << width() << "x" << height() << endl;
    exit(-1);
  }
  //  assert(cout << "generatePixbuf(): tmpimg buf at " << (void*)tmpimg << endl);

  // background color
  //  assert(cout << "generatePixbuf(): filling by background color..." << endl);
  int bg = (int)(status.bgvalue * 255);
  if (bg>255) bg=255;
  bg = (256*256*256)*bg + (256*256)*bg + 256*bg + bg;
  memset((void *)tmpimg, bg, width()*height()*4); 

  // initialize & draw by cairo 
  //  assert(cout << "generatePixbuf(): initializing cairo..." << endl);
  surface = cairo_image_surface_create_for_data(tmpimg, CAIRO_FORMAT_ARGB32,
						width(), height(), width()*4);

  cr = cairo_create (surface);
  if(status.antialias) cairo_set_antialias(cr, CAIRO_ANTIALIAS_SUBPIXEL);
  else                  cairo_set_antialias(cr, CAIRO_ANTIALIAS_NONE);

  draw (cr, data);
   
  // compress ARGB to RGB. gdk_pixbuf doesn't support ARGB! 
  for(int i=0; i<width()*height(); i++){
#if defined(__i386__) || defined(__x86_64__)
    tmpimg[i*3+0] = tmpimg[i*4+2];
    tmpimg[i*3+1] = tmpimg[i*4+1];
    tmpimg[i*3+2] = tmpimg[i*4+0];
#else
    tmpimg[i*3+0] = tmpimg[i*4+1];
    tmpimg[i*3+1] = tmpimg[i*4+2];
    tmpimg[i*3+2] = tmpimg[i*4+3];
#endif
  }

  pixbuf = gdk_pixbuf_new_from_data(tmpimg, GDK_COLORSPACE_RGB, FALSE,
				    8, width(), height(), width()*3, 
				    free_tmpimg, tmpimg);
  
  cairo_surface_destroy (surface);
  cairo_destroy (cr);

  if (cacheable){
    pixbuf_cache_tag t; t.get_current_globs();
    pbcache.add(t, pixbuf, status.basewidth, status.gene_markers);
  }

  return pixbuf;
}
