#ifndef _GENERAL_LIST_H_
#define _GENERAL_LIST_H_

#include <stdlib.h>
#include <gtk/gtk.h>
#include <string.h>
#include <vector>
#include <string>
#include <set>

template <typename T, typename U, typename V, typename W> 
class generalListItem {
public:
  void* at(unsigned i) const {
  return ((i==0) ? (void*)&t :
	  (i==1) ? (void*)&u :
	  (i==2) ? (void*)&v :
	  (void*)&w );
  };

  T t; U u; V v; W w;
};

class generalList_base {
public:
  virtual void edited(GtkCellRendererText*, const gchar*, const gchar*) = 0;
  virtual void toggled(GtkCellRendererToggle*, const gchar*) = 0;
  virtual void dropped(gint, gint) = 0;
  virtual void move_up_selected() = 0;
  virtual void move_down_selected() = 0;
  virtual void remove_selected() = 0;
  virtual ~generalList_base(){};
};

void handle_list_edit(GtkCellRendererText*, const gchar*, const gchar*, gpointer);
void handle_list_toggle(GtkCellRendererToggle*, const gchar*, gpointer);
gboolean handle_list_drop(GtkWidget*, GdkDragContext*, gint, gint, guint, gpointer);

template <typename T> class generalList : public generalList_base {
public:
  virtual ~generalList(){};

  static const int renderer_text = 0;
  static const int renderer_progress = 1;
  static const int renderer_check = 2;

  class coltype {
  public:
    coltype(){
      data_type = G_TYPE_INT;
      is_editable = true;
      name = "default";
      view_type = renderer_text;
    };

    GType data_type;
    bool is_editable;
    std::string name;
    int view_type;
  };

  void set_types(std::vector<coltype>& t){
    coltypes = t;
  };

  void add_initial_item(T i){ items.push_back(i); };
  void add_initial_item(std::vector<T>& i){
    for(typename std::vector<T>::iterator ite=i.begin(); ite!=i.end(); ite++)
      items.push_back(*ite);
  };

  void add_item(T i){
    add_initial_item(i);
    
    GtkTreeIter iter;
    gtk_list_store_append(GTK_LIST_STORE(model), &iter);
    for(unsigned j=0; j<coltypes.size(); j++) 
      set_list_store(&iter, items.size()-1, j);
  };

  void replace(std::vector<T>& newitems){
    // size of newitems and items must be same
    if(items.size() != newitems.size()) return;

    GtkTreeIter iter;
    items = newitems;
    gtk_tree_model_get_iter_first(model, &iter);
    for(unsigned i=0; i<items.size(); i++){
      for(unsigned j=0; j<coltypes.size(); j++){
	set_list_store(&iter, i, j);
      }
      gtk_tree_model_iter_next(model, &iter);
    }
  };

  GtkTreeView* create_view(GtkSelectionMode s){			   
    treeview = GTK_TREE_VIEW(gtk_tree_view_new_with_model(model));
    gtk_tree_view_set_rules_hint(treeview, TRUE);
    gtk_tree_selection_set_mode(gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview)),
				s);
    
    GtkTargetEntry targets[] = {
      { (gchar*)"x-target/gmv", GTK_TARGET_SAME_WIDGET, 0 }
    };

    gtk_tree_view_enable_model_drag_source(treeview, GDK_BUTTON1_MASK, 
					   targets, 1, GDK_ACTION_MOVE);
    gtk_tree_view_enable_model_drag_dest(treeview, targets, 1, GDK_ACTION_MOVE);
    g_signal_connect(treeview, "drag-drop", G_CALLBACK(handle_list_drop), this);

    for(unsigned i=0; i<coltypes.size(); i++){
      GtkCellRenderer *renderer;
      switch(coltypes[i].view_type){
      case renderer_check:
	renderer = gtk_cell_renderer_toggle_new();
	gtk_tree_view_insert_column_with_attributes(treeview, -1, 
						    coltypes[i].name.c_str(),
						    renderer, 
						    "active", i, NULL);
	g_signal_connect(renderer, "toggled", G_CALLBACK(handle_list_toggle), this);
	break;

      case renderer_progress:
	renderer = gtk_cell_renderer_progress_new();
	gtk_tree_view_insert_column_with_attributes(treeview, -1, 
						    coltypes[i].name.c_str(),
						    renderer, 
						    "value", i, NULL);
	break;
      default: // renderer_text
	renderer = gtk_cell_renderer_text_new();
 	g_object_set(renderer, "editable",  
		     (coltypes[i].is_editable ? TRUE : FALSE), NULL);
 	g_signal_connect(renderer, "edited", G_CALLBACK(handle_list_edit), this);

 	gtk_tree_view_insert_column_with_attributes(treeview, -1, 
						    coltypes[i].name.c_str(),
 						    renderer,
						    "text", i, NULL);
	break;
      }
      GtkTreeViewColumn* tvc = gtk_tree_view_get_column(treeview, i);
      gtk_tree_view_column_set_resizable(tvc, true);
      g_object_set_data(G_OBJECT(renderer), "column", GINT_TO_POINTER(i));
    }

    g_object_unref(model);
    return treeview;
  };

  GtkTreeModel* create_model(){
    GtkTreeIter iter;
    
    std::vector<GType> types;
    for(unsigned i=0; i<coltypes.size(); i++)
      types.push_back(coltypes[i].data_type);

    model = GTK_TREE_MODEL(gtk_list_store_newv(types.size(), &types[0]));
    
    /* add items */
    for(unsigned i=0; i<items.size(); i++){
      gtk_list_store_append(GTK_LIST_STORE(model), &iter);

      for(unsigned j=0; j<types.size(); j++) set_list_store(&iter, i, j);
    }
    return model;
  };

  void toggled(GtkCellRendererToggle *cell,
	       const gchar         *path_string){
    GtkTreePath *path = gtk_tree_path_new_from_string(path_string);
    GtkTreeIter iter;

    gint column = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(cell), "column"));
    gtk_tree_model_get_iter(model, &iter, path);
    int row = gtk_tree_path_get_indices(path)[0];

    *(bool*)(items[row].at(column)) = ! *(bool*)(items[row].at(column));
    
    set_list_store(&iter, row, column);
    gtk_tree_path_free(path);
  };

  
  void edited(GtkCellRendererText *cell,
	      const gchar         *path_string,
	      const gchar         *new_text){
    GtkTreePath *path = gtk_tree_path_new_from_string(path_string);
    GtkTreeIter iter;

    gint column = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(cell), "column"));
    gtk_tree_model_get_iter(model, &iter, path);
    int row = gtk_tree_path_get_indices(path)[0];

    gchar *old_text;
    switch(coltypes[column].data_type){
    case G_TYPE_INT:
      *((int*)(items[row].at(column))) = atoi(new_text); 
      break;
    case G_TYPE_STRING:
      gtk_tree_model_get(model, &iter, column, &old_text, -1);
      g_free(old_text);

      *((std::string*)(items[row].at(column))) = std::string(new_text); 
      break;
    }

    set_list_store(&iter, row, column);
    gtk_tree_path_free(path);
  };

  void dropped(gint wx, gint wy){
    GtkTreePath *p;
    GtkTreeViewDropPosition pos;
    if(gtk_tree_view_get_dest_row_at_pos(treeview, wx, wy, &p, &pos)){
      // got destination reference
      GtkTreeRowReference* dest = gtk_tree_row_reference_new(model, p);

      std::vector<GtkTreeRowReference*> srl = get_selected_rows();

      bool reverse = (pos!=GTK_TREE_VIEW_DROP_BEFORE);
      std::vector<GtkTreeRowReference*>::iterator ite = srl.begin();
      std::vector<GtkTreeRowReference*>::reverse_iterator rite = srl.rbegin();
      do {
	GtkTreeIter i;
	GtkTreePath *pp;
	p = gtk_tree_row_reference_get_path(dest);
	gtk_tree_model_get_iter(model, &i, p);

	pp = gtk_tree_row_reference_get_path( !reverse ? *ite : *rite );

	// move items
	int dstrow = gtk_tree_path_get_indices(p)[0];
	int srcrow = gtk_tree_path_get_indices(pp)[0];

	if(srcrow<dstrow && !reverse) dstrow--;
	if(srcrow>dstrow &&  reverse) dstrow++;

	T tmp = items[srcrow];
	for(int j=0; j < abs(dstrow-srcrow); j++){
	  if(srcrow<dstrow) items[srcrow+j] = items[srcrow+j+1];
	  else items[srcrow-j] = items[srcrow-j-1];
	}
	items[dstrow] = tmp;

	// move list_store
	GtkTreeIter ii;
	gtk_tree_model_get_iter(model, &ii, pp);
	if(!reverse)
	  gtk_list_store_move_before(GTK_LIST_STORE(model), &ii, &i);
	else
	  gtk_list_store_move_after(GTK_LIST_STORE(model), &ii, &i);

	// advance the iterator
	ite++; rite++;
      } while( !reverse ? ite!=srl.end() : rite!=srl.rend());
      gtk_tree_row_reference_free(dest);
      free_selected_rows(srl);
    }
  };

  void move_up_selected(){
    std::vector<GtkTreeRowReference*> srl = get_selected_rows();
    std::set<int> moved;

    int bottomrow = 0;
    for(std::vector<GtkTreeRowReference*>::iterator i=srl.begin(); 
	i!=srl.end(); i++){
      GtkTreePath *p = gtk_tree_row_reference_get_path(*i);
      int row = gtk_tree_path_get_indices(p)[0];
      if(i==srl.begin()) bottomrow = row;
      else if(bottomrow<row) bottomrow = row;
    }

    if(bottomrow>=(int)srl.size()){
      for(std::vector<GtkTreeRowReference*>::iterator i=srl.begin(); 
	  i!=srl.end(); i++){
	GtkTreeIter iter, previter;
	GtkTreePath *p = gtk_tree_row_reference_get_path(*i);
	int row = gtk_tree_path_get_indices(p)[0];
	gtk_tree_model_get_iter(model, &iter, p);

	if(row > 0){
	  if(moved.find(row-1) != moved.end()) continue;
	  T tmp;
	  tmp = items[row];
	  items[row] = items[row-1];
	  items[row-1] = tmp;
	  moved.insert(row-1);
	
	  gtk_tree_path_prev(p);
	  row = gtk_tree_path_get_indices(p)[0];
	  gtk_tree_model_get_iter(model, &previter, p);
	  gtk_list_store_move_before(GTK_LIST_STORE(model), &iter, &previter);
	} else moved.insert(row);
      }
    }
    free_selected_rows(srl);
  };

  void move_down_selected(){
    std::vector<GtkTreeRowReference*> srl = get_selected_rows();
    std::set<int> moved;

    // prescan to find a row on the top
    int toprow = 0;
    for(std::vector<GtkTreeRowReference*>::reverse_iterator i=srl.rbegin(); 
	i!=srl.rend(); i++){
      GtkTreePath *p = gtk_tree_row_reference_get_path(*i);
      int row = gtk_tree_path_get_indices(p)[0];
      if(i==srl.rbegin()) toprow = row;
      else if(toprow>row) toprow = row;
    }
    
    if(toprow < (int)(items.size() - srl.size())){
      for(std::vector<GtkTreeRowReference*>::reverse_iterator i=srl.rbegin(); 
	  i!=srl.rend(); i++){
	GtkTreeIter iter;
	GtkTreePath *p = gtk_tree_row_reference_get_path(*i);
	int row = gtk_tree_path_get_indices(p)[0];
	gtk_tree_model_get_iter(model, &iter, p);
	GtkTreeIter nextiter = iter;

	if(gtk_tree_model_iter_next(model, &nextiter)){
	  if(moved.find(row+1) != moved.end()) continue;
	  moved.insert(row+1);
	  T tmp;
	  tmp = items[row];
	  items[row] = items[row+1];
	  items[row+1] = tmp;
	  gtk_list_store_move_after(GTK_LIST_STORE(model), &iter, &nextiter);
	} else moved.insert(row);
      }
    }
    free_selected_rows(srl);
  };

  void remove_selected(){
    std::vector<GtkTreeRowReference*> srl = get_selected_rows();

    for(std::vector<GtkTreeRowReference*>::iterator i=srl.begin(); 
	i!=srl.end(); i++){
      GtkTreeIter iter;
      GtkTreePath *p = gtk_tree_row_reference_get_path(*i);
      int row = gtk_tree_path_get_indices(p)[0];
      gtk_tree_model_get_iter(model, &iter, p);

      gtk_list_store_remove(GTK_LIST_STORE(model), &iter);
      items.erase(items.begin()+row);
    }
    free_selected_rows(srl);
  };

  void clear_only_data(){ items.clear();  };

  void clear(){
    gtk_list_store_clear(GTK_LIST_STORE(model));
    items.clear();
  }

  std::vector<T> items;

private:
  std::vector<coltype> coltypes;

  GtkTreeModel* model;
  GtkTreeView* treeview;

  std::vector<GtkTreeRowReference*> get_selected_rows() const {
    GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);

    std::vector<GtkTreeRowReference*> srl;
    GList *sl = gtk_tree_selection_get_selected_rows(selection, NULL);
    for(GList *s=sl; s; s=s->next){
      GtkTreePath *p = (GtkTreePath*)s->data;
      srl.push_back(gtk_tree_row_reference_new(model, p));
      gtk_tree_path_free(p);
    }
    g_list_free(sl);
    return srl;
  }

  void free_selected_rows(std::vector<GtkTreeRowReference*>& refs){
    for(std::vector<GtkTreeRowReference*>::iterator i=refs.begin(); 
	i!=refs.end(); i++)
      gtk_tree_row_reference_free(*i);
  }

  void set_list_store(GtkTreeIter* iter, int row, int col){
    switch(coltypes[col].data_type){
    case G_TYPE_STRING:
      gtk_list_store_set(GTK_LIST_STORE(model), iter, col, 
			 (void*)strdup(((std::string*)(items[row].at(col)))->c_str()), -1);
      break;
    case G_TYPE_BOOLEAN:
      gtk_list_store_set(GTK_LIST_STORE(model), iter, col, 
			 *(bool*)(items[row].at(col)), -1);
      break;
    case G_TYPE_INT:
      gtk_list_store_set(GTK_LIST_STORE(model), iter, col, 
			 *(int*)(items[row].at(col)), -1);
      break;
    }
  };
};

#endif
