//
//   GMV: a GTK+ based Murasaki Visualizer
//   
//   Copyright (C) Bioinformatics Lab., Keio University, 2006-2007.
//

/* 
   TODO (higher priority):

   High-priority wishlist:
     rift filter
     ellipsize label on splash screen ?
     "recent used" menu
     improved annotation track management interface
     improved histogram control sliders
     zoomable dotplot
     annotation management brushup
     repeat filter
     bitscore loader
     hits loader
     draft mode notification dialogs
     automatic U / R mask disabling
     Histogram abstraction
     Repeat histogram
     GC skew
     GC ratio

   Generals:
     NLS
     color selector for gene markers etc. (new confwin infrastructure!?)
     sequence length inference from anchor list

   Filters:
     "window" filter: show only limited anchors goin' through some range (s)
     modular structure for anchor scores & histograms (for bitscores etc.)
     separate expression filter for anchors and genes
     count based filter (anc# or tfidf:gene - not only for expr)
     histogram for expr (for anc:expr and gene:expr - not only tfidf:anchor)

   Reports:
     Integrate nucleotide report & filter report
     HTML report?

   Dialogs:
     "show details" option on GID? how to implement?
     additional statistics in GID (# of ancs, tfidf score, etc.)
       ... how about expression data? it's terrible!
     gene_name_onoff_method on confwin
     "snap to tics" option for sliders in inspector

   Anchor Display:
     "Abort Rendering" button...

   Chrom Display:
     GC skew
     better ticks method (consider the text label width)
     selectable "color base" scheme in chrom links view

   Internals:
     limit size of undo/redo stack
     further effort on filepath management
        - better path estimation with .opt file

   TODO (lower priority)
     speculative rendering in background
     separate advanced options from config menu?
     speculative rendering & caching (in background)?

   MAY BE FINISHED (someone please test)
   FIXED BUGS:
*/

#include <iostream>
#include <sstream>

#include <sys/time.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <cairo.h>
#include <assert.h>

#include "izfstream.h"
#include "mainwindow.h"
#include "pixbuf_cache.h"
#include "gmv.h"
#include "basestat.h"
#include "export.h"
#include "colormap.h"
#include "zoomer.h"
#include "handler_table.h"
#include "text.h"
#include "plotters.h"
#include "mainwindow_handlers.h"
#include "range_list.h"
#include "messages.h"
#include "opts.h"
#include "util.h"
#include "dblinks.h" // for launch_browser()
#include "../image/splash.h"

#ifdef __MINGW32__
# include <winsock2.h>
#endif

using namespace std;

int min(int a, int b){
  if(a<b) return a; else return b;
}

zoom_cursor zc;

// ----------------------------------------------------------------------
//  some global settings and accessors

GtkWidget *canvas, *canvas_ebox;
GdkPixbuf *pixbuf;
results mr;
handler_table handlers;
mv_status status;
pixbuf_cache pbcache;
zoom *zoomer;

int width (){ return zoomer->current().first;  }
int height(){ return zoomer->current().second; }


// ----------------------------------------------------------------------
// misc utility functions

string preffile(){
  return (string)g_get_home_dir() + G_DIR_SEPARATOR + ".gmv"; 
}

vector<string> split(string str, string delimiter){
  vector <string> tokens;
  
  tokens.clear();

  size_t pos = 0;
  size_t pos2;

  while( (pos2 = str.find("|", pos)) != string::npos){
    tokens.push_back(str.substr(pos, pos2-pos));
    pos=pos2+1;
  }

  return tokens;
}

// ----------------------------------------------------------------------
// reset_all(): for re-initialization 

void reset_all(){
  mr.clear();
  pbcache.clear();
  delete zoomer;
  status.set_defaults();
  status.undo_stack.clear();
  status.redo_stack.clear();
  status.rsb.clear();
  status.mb.bmark_clear();

  // close dialogs
  status.gid.close();
  status.ins.close();
  status.statd.close();
  status.repd.close();
  status.prefd.close();
#ifndef OLD_GTK_COMPAT
  status.dasd.close();
#endif
}

// ----------------------------------------------------------------------
void notify_update_response(GtkDialog *dialog, gint response, gpointer useless){
  string skipfile = (string)g_get_home_dir() + G_DIR_SEPARATOR + ".gmv_version";
  gtk_widget_destroy(GTK_WIDGET(dialog));

  switch(response){
  case GTK_RESPONSE_YES:
    launch_browser("http://murasaki.dna.bio.keio.ac.jp/gmv.html");
    break;
  case GTK_RESPONSE_CANCEL:
    std::ofstream sf;
    sf.open(skipfile.c_str());
    sf << status.latest_release << endl;
    sf.close();
    break;
    // default is do nothing:
  }
}

void notify_update(){
  string skipfile = (string)g_get_home_dir() + G_DIR_SEPARATOR + ".gmv_version";
  int latest = status.latest_release;
  if(latest > REVISION ){
    std::ifstream sf;
    sf.open(skipfile.c_str());
    if(sf){
      int skip_version;
      sf >> skip_version;
      sf.close();
      if(skip_version == latest) return;
    }

    GtkWidget* dialog = gtk_message_dialog_new(GTK_WINDOW(status.mainwindow),
					       GTK_DIALOG_DESTROY_WITH_PARENT,
					       GTK_MESSAGE_INFO,
					       GTK_BUTTONS_NONE,
					       "Newer GMV build is available.\n Check the website?\n");
   
    gtk_dialog_add_buttons(GTK_DIALOG(dialog),
			   GTK_STOCK_YES, GTK_RESPONSE_YES,
			   "_Skip this version", GTK_RESPONSE_CANCEL,
			   GTK_STOCK_NO,  GTK_RESPONSE_NO, NULL);

    gtk_window_set_title(GTK_WINDOW(dialog), "New version available!");
    g_signal_connect(dialog, "response", G_CALLBACK(notify_update_response),
		     NULL);
    gtk_widget_show_all(dialog);
  }
}

gboolean version_check_idle(gpointer useless){ 
  g_mutex_lock(status.vercheck_mutex);
  int latest = status.latest_release;
  g_mutex_unlock(status.vercheck_mutex);

  if(latest != 0 && (!status.has_any_modal_dialogs())){
    notify_update();
    return FALSE; 
  }

  return TRUE; // call me again!
}

gpointer version_check(gpointer mainwindow){
  string latest; 
#ifndef RELEASE_BUILD
  string revfile = "/gmv/snap_rev";
#else 
  string revfile = "/gmv/release_rev";
#endif
  string skipfile = (string)g_get_home_dir() + G_DIR_SEPARATOR + ".gmv_version";

  if(!get_by_http(&latest, "murasaki.dna.bio.keio.ac.jp", 
 		  80,  revfile.c_str())) latest="-1";

  g_mutex_lock(status.vercheck_mutex);
  status.latest_release = atoi(latest.c_str());
  g_mutex_unlock(status.vercheck_mutex);
  
  return NULL;
}

void* initialize(void *hoge){

  basestat** bs;
  vector <string> genename;

  // --------------------------------------------------
  // Here we go.
  zoomer = new zoom(status.default_width, status.default_height);
  assert(zoomer->sanityCheck());
      
  assert(cout << "initialize(): done with zoomer initialization" << endl);

  bs = mr.bs;

  // --------------------------------------------------
  // load sequence

  // load annotations and sequences
  int seqno = mr.load_seq_files(status.opt_load_seq, status.opt_load_annot);

  // --------------------------------------------------
  // load repeats

  if(status.repeatfile != "") mr.load_repeat_file(status.repeatfile, seqno);

  // --------------------------------------------------
  // read anchors
  IFSTREAM anchorfile, tfidffile;

  assert(cout << "initialize(): open anchor file: " << status.basename + status.anchorfile_extension << endl);
  string anchorfile_name = status.basename+status.anchorfile_extension;
  anchorfile.open(anchorfile_name.c_str());  
  if(!anchorfile){
    cerr << "initialize(): couldn't open "
	 << status.basename << ".anchors (this shouldn't happen!)" << endl;
    exit(-1); 
  }
    
  bool has_tfidf = false;
  if(status.tfidffile!=""){
    tfidffile.open(status.tfidffile.c_str());
    if(!tfidffile){
      cerr << "initialize(): couldn't open "
	   << status.tfidffile << " (this shouldn't happen!)" << endl;
      exit(-1); 
    }
    has_tfidf = true;
  }

  if(status.use_x11){
    lock_gdk();
    gtk_window_set_title(GTK_WINDOW(status.mainwindow),
			 ((string)"Murasaki Viewer: "+
			  status.basename + status.anchorfile_extension).c_str());
    unlock_gdk();
  }

  pbar_reset("Loading anchors...");
  anchorfile.seekg(0, std::ios::end);
  double anchorfilelen = anchorfile.tellg();
  anchorfile.seekg(0, std::ios::beg);

  // prepare anchor segments
  mr.prepare_anchor_segments(seqno);
 
  // check we have score or not
  bool has_hit = false;
  bool has_score = false;    // anc file has some score
  bool has_hashdetail = false;  // we can calculate tfidf score from anc file

  string test_str, test_str2;
  getline(anchorfile, test_str);
  anchorfile.seekg(0); // rewind before continue!

  istringstream test_stream(test_str);

  int fields=0;
  while(test_stream >> test_str2) fields++;

  if(fields >= seqno*3+1) has_hit = true;
  if(fields >= seqno*3+2) has_score = true;
  if(fields >  seqno*3+2) has_hashdetail = true; 

  // tfidf calc stats
  multimap<seqlen_t, int> bin_anc;
  vector< map<seqlen_t, int> > word_count;

  // let's read
  do {
    vector<seqlen_t> anchor(seqno*3);
    seqlen_t pos1, pos2;
    int dir;
    char d;

    for(int i=0; i<seqno; i++){
      anchorfile >> pos1;
      anchorfile >> pos2;
      anchorfile >> d;

      // do abs(posx)-1 since GMV's basecount is 0 origin
      pos1 = abs(pos1);
      pos2 = abs(pos2);
      pos1--;
      pos2--;

      dir = (d=='+') ? 1 : 0;
      if(pos2 < pos1) swap(pos1, pos2);
      
      anchor[i*3  ] = pos1;
      anchor[i*3+1] = pos2;
      anchor[i*3+2] = dir;
    }

    bool got_score = false;
    double score = 0;
      
    if(has_tfidf){
      // has .stats.tfidf
      tfidffile >> test_str2;
      score = (test_str2 == "-inf") ? 0 : atof(test_str2.c_str());
      got_score = true;
    } else {
      if(has_hit){
	// skip the first column (hits)
	anchorfile >> pos1;
      }
      if(has_score){
	anchorfile >> test_str2;
	score = (test_str2 == "-inf") ? 0 : atof(test_str2.c_str());
	got_score = true;
      }
      if(has_hashdetail){ 
	string hashdetail;
	getline(anchorfile, hashdetail);

	size_t p = hashdetail.find_first_not_of(" ");
	word_count.push_back(map<seqlen_t, int>());
	int anc_no = mr.arows();
	while( p != string::npos ){
	  size_t p2 = hashdetail.find_first_of(":", p);
	  seqlen_t bin_no = atoll(hashdetail.substr(p, p2-p).c_str());

	  p = p2+1;
	  p2 = hashdetail.find_first_of(",", p);
	  int bin_cnt = atoi(hashdetail.substr(p, p2-p).c_str());
	  // cout << "   bin: " << bin_no << " count: " << bin_cnt << endl;

	  bin_anc.insert( pair<seqlen_t, int>(bin_no, anc_no) );
	  word_count.at(anc_no).insert( pair<seqlen_t, int>(bin_no, bin_cnt));

	  p = hashdetail.find_first_not_of(",", p2);
	}
      }
    }

    if (anchorfile.eof()) break;

    if(got_score) mr.tfidf.push_back(score);
    mr.insert_anchor_row(anchor);
    pbar_set_fraction( anchorfile.tellg()/anchorfilelen );
  } while (true);
    
  if(!mr.tfidf.empty()){
    ofstream of;
    if(has_hashdetail && !has_tfidf) 
      of.open(string(status.basename + ".stats.tfidf").c_str());

    pbar_reset("Doing tf-idf scoring...");
    for(unsigned i=0; i<mr.arows(); i++){
      double score = 0;
	
      // calc tfidf if required & possible
      if(has_hashdetail && !has_tfidf){
	for(map<seqlen_t, int>::iterator j=word_count.at(i).begin();
	    j!=word_count.at(i).end(); j++){
	  seqlen_t bin_no = j->first;
	  int bin_cnt = j->second;
	  size_t anc_cnt = bin_anc.count(bin_no);
	  score += ( bin_cnt * log( mr.arows() / anc_cnt ));
	}
	if(of) of << score << endl;
	mr.tfidf.setval(i, score);
      }

      score = mr.tfidf[i];
      if(score<MIN_POSSIBLE_SCORE) score=MIN_POSSIBLE_SCORE;
      mr.tfidf.setval(i, score);

      pbar_set_fraction( (double)i/mr.arows());
    }
    of.close();
  }

  anchorfile.close();

  // build segment tree
  pbar_reset("Generating anchor index...");
  for(int i=0; i<seqno; i++){
    pbar_set_task((double)i/seqno, (double)(i+1)/seqno);
    mr.build_anchor_segment_tree(i, pbar_set_fraction);
  }
  pbar_set_task(0,1);

  // go histogram
  pbar_reset("Generating tf-idf histogram...");
  mr.tfidf.fixate(histogram::logarithmic);
  if(status.opt_min_tfidf != 0)
    mr.tfidf.histo.set_low(status.opt_min_tfidf);

  // 99.9% filter for "anchor color by tf-idf score"
  int visible=0;
  for(unsigned a=0; a<HISTOGRAM_RESOLUTION; a++){
    visible+=mr.tfidf.histo[a];
    mr.tfidf_color_scoremax = pow(10, a*mr.tfidf.histo.getstep() + log10(mr.tfidf.getmin()));
    if((double)visible >= 99.5 * (double)mr.arows()) break;
  }

  // tf-idf draft mode: -omin_tfidf= option overrides this.
  if(!mr.tfidf.empty() && status.opt_min_tfidf ==0){
    int disp_ancs = 0;
    for(unsigned x=HISTOGRAM_RESOLUTION; x>0; x--){
      disp_ancs += mr.tfidf.histo[x-1];

      if (disp_ancs > MAX_DRAFT_ANCS){
	if(x > HISTOGRAM_RESOLUTION - 3) break;

	mr.tfidf.histo.set_low(pow(10, log10(mr.tfidf.getmin()) 
				   + mr.tfidf.histo.getstep() * (x+1) ));

	status.infobar.push_back(string("There may be too many anchors, so minimum tfidf score is set to " + itos(mr.tfidf.histo.get_low()) + ".\nUse inspector to change the threshold (inspector appears by pressing ctrl+I)."));

	break;
      }
    }
  } else {
    if(status.opt_min_tfidf != 0)
      mr.tfidf.histo.set_low(status.opt_min_tfidf);
  }

  // finish with loading
  mr.rebuild_segment_offset_index();

  status.chk_toggle_seq();
  mr.order_seq = status.order_seq;
  mr.chk_order_seq();

  assert(cout << "initialize(): loaded everything!" << endl);

  // annotation
  opt_read_annotations(&status.annotation_files);

  // automatic sequence ordering
  if(status.auto_order) status.auto_reorder();

  // COG
  if(status.cog_file!="") mr.loadcog(status.cog_file);

  // anchor statistics
  if(status.opt_stat_html || status.opt_stat_text) opt_anchor_statistics();

  if(status.opt_stat_text){
    ofstream stats;
    stats.open((status.basename+"-stat.txt").c_str());
    if(!stats){
      stats << mr.summarize_in_text();
      stats.close();
    }
  }

  if(status.opt_stat_html){
    ofstream stats;
    stats.open((status.basename+"-stat.html").c_str());
    if(!stats){
      stats << mr.summarize_in_html();
      stats.close();
    }
  }

  // undo stack
  pixbuf_cache_tag t; 
  t.get_current_globs();
  status.undo_stack.push_back(t);

  if(status.use_x11) redraw_subthread(NULL);

  //  set_quickbar(); // DON'T CALL THIS HERE!
  return NULL;
}

// ----------------------------------------------------------------------
//  Main

int main(int argc, char **argv)
{
  status.argc = argc;
  status.argv = argv;

  if(glib_check_version(2,32,0)){
    // not necessary since 2.32
    
    // thread init
    //if(g_thread_get_initialized() /*g_thread_supported()*/){
    //  cerr << "Sorry, GLib thread support is required." << endl;
    //  exit(-1);
    //}

    g_thread_init (NULL);
  }
  
  status.itc_mutex = g_mutex_new();
  status.render_mutex = g_mutex_new();

  mr.clear();

#ifdef __MINGW32__
  // stupid winsock initialization
  WSADATA wsadata;
  WSAStartup(MAKEWORD(2,0), &wsadata);
#endif

  // double click timer init
  status.dc.init();

  // zoom cursor and misc init
  zc.init();

  procopts();
  status.use_x11 = !(status.generate_png || status.generate_pdf);

  if(status.use_x11){					
    // run with X
#ifndef __MINGW32__
    gdk_threads_init();
#endif
    gtk_init(&argc, &argv);
    
    if(status.cmdopt_forward) status.plot_bwd = false;
    else if(status.cmdopt_backward) status.plot_fwd = false;

    //  initialize widgets
    GtkWidget *window;
    GtkAccelGroup *accel_group;

    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    status.mainwindow = window;
    
    status.tips = gtk_tooltips_new();

    gtk_window_set_default_size(GTK_WINDOW (window),
				status.mainwindow_w, status.mainwindow_h);
    gtk_window_set_title(GTK_WINDOW (window), "Murasaki Viewer");

    g_signal_connect(window, "destroy-event", G_CALLBACK(handle_quit), NULL);
    g_signal_connect(window, "delete-event",  G_CALLBACK(handle_quit), NULL);
    g_signal_connect(window, "focus-out-event", G_CALLBACK(handle_focusout), NULL);

    accel_group = gtk_accel_group_new();
    gtk_window_add_accel_group(GTK_WINDOW(window), accel_group);

    GtkWidget *vbox;
    vbox = gtk_vbox_new(FALSE, 0);
    gtk_container_add(GTK_CONTAINER(window), vbox);
    

    // --------------------------------------------------
    // initialize stuff, order is important

    GtkWidget *sidebar = status.lsb.init();
    GtkWidget *thumbbar = status.rsb.init();
    GtkWidget *menubar = status.mb.init(window, accel_group);
    
    // --------------------------------------------------
    // menu bar
    gtk_box_pack_start(GTK_BOX(vbox), menubar, FALSE, FALSE, 0);

    // --------------------------------------------------
    // left tool bar + Anchor plot area
    GtkWidget *hbox, *vbox2;
    GtkWidget *viewport;

    // anchor plot area
    viewport = gtk_scrolled_window_new(NULL, NULL);
    gtk_scrolled_window_set_policy((GtkScrolledWindow*)viewport, 
				   GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

    // vbox2 = infobar + plot area + status bars
    vbox2 = gtk_vbox_new(false, 0);
    status.infobarbox = gtk_vbox_new(false, 0);
    gtk_box_pack_start(GTK_BOX(vbox2), status.infobarbox, false, false, 0);
    gtk_box_pack_start(GTK_BOX(vbox2), viewport, TRUE, TRUE, 0);

    // pack'em together
    hbox = gtk_hbox_new(FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox), sidebar, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(hbox), vbox2, TRUE, TRUE, 0);
    gtk_box_pack_start(GTK_BOX(hbox), thumbbar, FALSE, FALSE, 0);

    gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

    // --------------------------------------------------
    // drawing area
    GtkWidget *vpHbox, *vpVbox;

    vpHbox = gtk_hbox_new(FALSE, 0);
    vpVbox = gtk_vbox_new(FALSE, 0);

    canvas_ebox = gtk_event_box_new();
    canvas = gtk_image_new();
    gtk_container_add(GTK_CONTAINER(canvas_ebox), canvas);
    gtk_event_box_set_above_child(GTK_EVENT_BOX(canvas_ebox), true);

    gtk_widget_add_events(canvas_ebox,  GDK_POINTER_MOTION_MASK);
    gtk_widget_add_events(canvas_ebox,  GDK_BUTTON_PRESS_MASK);
    gtk_widget_add_events(canvas_ebox,  GDK_BUTTON_RELEASE_MASK);
    gtk_widget_add_events(canvas_ebox,  GDK_LEAVE_NOTIFY_MASK);
    gtk_widget_add_events(canvas_ebox,  GDK_ENTER_NOTIFY_MASK);

    handlers.set_sensitive(canvas_ebox, false);
    handlers.connect(canvas_ebox, "button-release-event", 
		     G_CALLBACK(handle_plotAreaReleased), NULL);
    handlers.connect(canvas_ebox, "button-press-event", 
		     G_CALLBACK(handle_plotAreaPressed), NULL);
    handlers.connect(canvas_ebox, "motion-notify-event", 
		     G_CALLBACK(handle_plotAreaMotion), NULL);
    handlers.connect(canvas_ebox, "leave-notify-event", 
		     G_CALLBACK(handle_plotAreaLeave), NULL);
    handlers.connect(canvas_ebox, "enter-notify-event", 
		     G_CALLBACK(handle_plotAreaEnter), NULL);

    // put the drawing_area at the center of scrollwindow
    gtk_box_pack_start((GtkBox*)vpHbox, (GtkWidget*)canvas_ebox, 
		       TRUE, FALSE, 0);
    gtk_box_pack_start((GtkBox*)vpVbox, vpHbox, TRUE, FALSE, 0);
    gtk_scrolled_window_add_with_viewport((GtkScrolledWindow*)viewport, vpVbox);

    // --------------------------------------------------
    // progress bar & justification indicator
    status.prog_hpaned = gtk_hpaned_new();
    status.progbar = gtk_progress_bar_new();
    gtk_progress_bar_set_text((GtkProgressBar*)status.progbar, "Starting up... You may have to open an \".anchors\" file.");

    gtk_box_pack_start(GTK_BOX(vbox2), status.progbar, FALSE, FALSE, 0);
    gtk_box_pack_start(GTK_BOX(vbox2), status.js.init(), FALSE, FALSE, 0);

    // ----------------------------------------------------------------------
    //  New splash

    status.splashwindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_decorated(GTK_WINDOW(status.splashwindow), false);
    g_signal_connect(status.splashwindow, "destroy",
		     G_CALLBACK(gtk_widget_destroyed), &status.splashwindow);
    gtk_window_set_position(GTK_WINDOW(status.splashwindow),
			    GTK_WIN_POS_CENTER_ALWAYS);

    GtkWidget* splashbox = gtk_vbox_new(false, 0);
    gtk_container_add(GTK_CONTAINER(status.splashwindow), splashbox);

    GtkWidget* splashbox2 = gtk_hbox_new(false, 0);
    gtk_box_pack_start(GTK_BOX(splashbox), splashbox2, false, false, 0);

    // ----------------------------------------------------------------------
    GtkWidget* splashmessage = gtk_label_new("");

    char buf[1024];
    snprintf(buf, 1024, 
	     "<span foreground=\"#612a74\">\
<b>GMV: a GTK+ powered Murasaki Visualizer</b>\n\
<small>Developed by Bioinformatics Lab., Keio Univ.\n\
version %s [%s]\n\
build %d: %s</small>\
</span>", VERSION, RELEASE_NAME, REVISION, RELEASE_DATE);
    gtk_label_set_markup(GTK_LABEL(splashmessage), buf);
    gtk_label_set_angle(GTK_LABEL(splashmessage), 90);
    gtk_box_pack_start(GTK_BOX(splashbox2), splashmessage, true, true, 0);

    // ----------------------------------------------------------------------

    gtk_box_pack_start(GTK_BOX(splashbox2), 
		       gtk_image_new_from_pixbuf(gdk_pixbuf_new_from_inline(-1, splashimg, false, NULL)),
		       false, false, 0);

    // ----------------------------------------------------------------------

    status.splash_progbar_backup = status.progbar;
    status.progbar = gtk_progress_bar_new();
    gtk_box_pack_start(GTK_BOX(splashbox), status.progbar, false, false, 0);

    gtk_widget_show_all(status.splashwindow);

    // ----------------------------------------------------------------------
    // Splash

//     g_signal_connect(canvas, "expose-event",
// 		     G_CALLBACK(handle_expose_pixbuf), pixbuf);

//    gtk_widget_set_size_request(canvas, SPLASH_WIDTH, SPLASH_HEIGHT);

    //    gtk_widget_show_all(window); // keep it, splash is on the screen
    status.js.clear(); // justification bar must be hidden

    // update checker
    status.vercheck_mutex = g_mutex_new();

    status.latest_release = 0;
    g_timeout_add(1000, (GSourceFunc)version_check_idle, NULL);
    if (status.use_x11) g_thread_create((GThreadFunc)version_check, 
					status.mainwindow, false, NULL);

    if(status.argc==0){
      if(!status.mb.openAnchorsDialog()) return 0;
    } else {
      if(!status.set_filenames()){
	// warning
	GtkWidget *dialog;
	dialog = gtk_message_dialog_new(GTK_WINDOW(status.mainwindow),
					GTK_DIALOG_DESTROY_WITH_PARENT,
					GTK_MESSAGE_INFO,
					GTK_BUTTONS_OK,
					"Please try another anchor file.");
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);

	// "open anchors" dialog
	if(!status.mb.openAnchorsDialog()) return 0;
      }else{
	g_idle_add((GSourceFunc)start_initialize, NULL);
      }
    }

    // keep it, splash is on the screen
    //    gtk_widget_show_now(window); // increase stability
    gdk_flush();

    gtk_main();

    if(status.mainwindow != NULL){ //handle_quit sets NULL to status.mainwindow 
      int window_w, window_h;
      gtk_window_get_size(GTK_WINDOW(window), &window_w, &window_h);

      if(status.mb.get_autosave()) status.save_prefs();

      pref p;
      p.load(preffile());

      p.setpref("gmv_auto", status.mb.get_autosave(), "autosave_prefs");

      if(status.mb.get_autosave()){
	p.setpref("gmv_auto", window_w, "window_width");
	p.setpref("gmv_auto", window_h, "window_height");
	
	string dasnames="";
	string dasurls="";
	for(vector<pair<string,string> >::iterator
	      ite=status.das_server_list.begin(); 
	    ite!=status.das_server_list.end(); ite++){
	  if(dasnames!="") dasnames+=",";
	  if(dasurls !="") dasurls +=",";
	  dasnames += ite->first;
	  dasurls  += ite->second;
	}
	p.setpref("gmv_auto", dasnames, "DAS_names");
	p.setpref("gmv_auto", dasurls, "DAS_URLs");

	p.setpref("gmv_auto", status.das_prev_server, "DAS_server");
	p.setpref("gmv_auto", status.das_prev_source, "DAS_source");
	p.setpref("gmv_auto", status.das_prev_segment, "DAS_segment");

	status.gid.export_prefs(&p);
	status.ins.export_prefs(&p);

	p.setpref("gmv_auto", status.lsb.sense_visible(), "sidebar_visible");
	p.setpref("gmv_auto", status.rsb.sense_visible(), "thumbbar_visible");
	p.setpref("gmv_auto", status.lsb.sense_schemes_visible(), "sidebar_schemes_visible");
	p.setpref("gmv_auto", status.lsb.sense_toggles_visible(), "sidebar_toggles_visible");
      }
      p.save(preffile());
    }

  } else {
    // run without X
    assert(cout << "gmv is running without GUI." << endl);

    if(!status.set_filenames()){
      cout << "gmv: couldn't load some file(s)" <<endl;
      exit(-1);
    }

    initialize(NULL);

    do{
      status.plot_fwd = true;  status.plot_bwd = true; fileplot();

      if(status.cmdopt_forward){
	status.plot_fwd = true;  status.plot_bwd = false; fileplot();  }

      if(status.cmdopt_backward){
	status.plot_fwd = false;  status.plot_bwd = true; fileplot();  }

      status.plot_type++;
    } while(status.plot_all && status.plot_type <2);

    for(vector<string>::iterator i=status.infobar.begin();
	i!=status.infobar.end(); i++){
      cout << "GMV Info: " << *i << endl;
    }
  }

#ifdef __MINGW32__
  WSACleanup();
#endif

  return 0;
}
