#include "handler_table.h"
#include <assert.h>
#include <gtk/gtk.h>
#include <iostream>

using namespace std;

handler_table::handler::handler(gpointer i, gulong h, int t){
  instance = i;
  handler_id = h;
  type = t;
}

// ----------------------------------------------------------------------
//  block/unblock table + signal handler management

void handler_table::connect(gpointer instance, int handler_type){
  table.push_back(handler(instance, 0, handler_type));
}

void handler_table::connect(gpointer instance, const char* signame, 
			    GCallback cb, gpointer data, int handler_type){
  gulong handlerid = g_signal_connect(instance, signame, cb, data);
  table.push_back(handler(instance, handlerid, handler_type));
}

void handler_table::connect(gpointer instance, const char* signame, 
			    GCallback cb, gpointer data){
  connect(instance, signame, cb, data, HANDLER_GENERAL);
}

void handler_table::block(){
  refcount++;
  assert(cout << "block : refcount=" << refcount << endl);
  if(refcount > 1) return;
  assert(cout << "blocking " << table.size() << " handlers" << endl);
  for(unsigned int i=0; i<table.size(); i++){
    gtk_widget_set_sensitive((GtkWidget*)table.at(i).instance, false);
    if(table.at(i).handler_id != 0)
      g_signal_handler_block(table.at(i).instance, table.at(i).handler_id);
  }
}

void handler_table::unblock(){
  refcount--;
  assert(cout << "unblock : refcount=" << refcount << endl);
  if(refcount > 0) return;
  if(refcount < 0){ refcount = 0; return; }

  assert(cout << "unblocking " << table.size() << " handlers" << endl);
  for(unsigned int i=0; i<table.size(); i++){
    GtkWidget *w = (GtkWidget*)table.at(i).instance;
    if(deactivated_widgets.find(w) == deactivated_widgets.end())
      gtk_widget_set_sensitive(w, true);
    if(table.at(i).handler_id != 0)
      g_signal_handler_unblock(table.at(i).instance, table.at(i).handler_id);
  }
}

void handler_table::remove_dialog(int t){
  vector<handler>::iterator i;

  for(i=table.begin(); i!=table.end();)
    if(i->type == t){ i=table.erase(i); }
    else i++;
}

void handler_table::remove_widget(GtkWidget *w){
  vector<handler>::iterator i;

  for(i=table.begin(); i!=table.end();)
    if(i->instance == (gpointer)w){ i=table.erase(i); }
    else i++;
}

void handler_table::remove_dialog(){
  remove_dialog(HANDLER_DIALOG);
}

void handler_table::remove(gpointer instance){
  vector<handler>::iterator i;

  for(i=table.begin(); i!=table.end();)
    if(i->instance == instance){ i=table.erase(i); break; }
    else i++;
}

void handler_table::set_sensitive(GtkWidget *w, bool active){
  if(active){
    set<GtkWidget*>::iterator ite = deactivated_widgets.find(w);
    if(ite != deactivated_widgets.end()) deactivated_widgets.erase(ite);
  } else {
    deactivated_widgets.insert(w);
  }
  gtk_widget_set_sensitive(w, active);
}
