#include <vector>
#include <gtk/gtk.h>

#include "dialog_base.h"
#include "gmv_defs.h"

class histogram : public generalWidgets {
public:
  static const int linear = 0;
  static const int logarithmic = 1;
  static const int resolution = HISTOGRAM_RESOLUTION;
  static const int height = HISTOGRAM_HEIGHT;

  histogram();
  ~histogram();

  void activate(GtkWidget*);
  void release (GtkWidget*);

  void init(int, double, double, const std::vector<double>&);  // mode, hi, lo, data
  bool reinit_widgets();

  void set_triggers(generalWidgets *s, generalWidgets *b){
    state_change_trig = s; button_release_trig = b; };
  void clear_triggers(){ state_change_trig=NULL; button_release_trig=NULL; };

  int visible();
  void fixhisto();
  void fixhisto(double, double);
  std::pair<double,double> getrange() const{
    return std::pair<double,double>(threshold_l, threshold_h); };
  bool inrange(double v) const { return(threshold_l <= v && v <= threshold_h);};

  void set_low(double l) { fixhisto(l, threshold_h); };
  void set_high(double h){ fixhisto(threshold_l, h); };
  double get_low() const { return threshold_l; };
  double get_high() const { return threshold_h; };

  // resource access0
  int operator[](unsigned int i) const { return histo_int[i]; };
  double getstep() const { return step; };

  GtkWidget *getimage() const { return histo_image; };
  GtkWidget *getslider_h() const { return slider_h; };
  GtkWidget *getslider_l() const { return slider_l; };
  GtkWidget *getspinbutton_h() const { return spinbutton_h; };
  GtkWidget *getspinbutton_l() const { return spinbutton_l; };

private:
  bool check_range(GtkWidget*);

  int histomode;
  bool ready;

  double log_geta; 
  double bottom, top, step;
  double threshold_h, threshold_l;

  std::vector<int> histo_int;
  std::vector<double> histo;
  
  unsigned char *histo_raw_image;

  GdkPixbuf *histo_pb;
  GtkWidget *histo_image;
  GtkWidget *slider_h, *slider_l;
  GtkWidget *spinbutton_h, *spinbutton_l;

  generalWidgets *state_change_trig, *button_release_trig;

  bool is_logmode() const {return histomode == logarithmic;};
  bool is_linearmode() const {return histomode == linear;};
};
