using namespace std;

#include <gtk/gtk.h>

#include "gmv.h"
#include "inspector.h"
#include "dasclient.h"

seqlen_t seqlen_from_spinbutton(GtkWidget* w){
  return (seqlen_t)gtk_spin_button_get_value(GTK_SPIN_BUTTON(w));
}

// ------------------------------
// DAS server managemnt dialog

#ifndef OLD_GTK_COMPAT // GtkAssistant is provided by GTK+-2.10 and later

void das_server_manage_dialog::init(GtkWidget* w){
  isOpen = true;
  dialog = gtk_dialog_new_with_buttons("DAS server management",
				       GTK_WINDOW(w),
				       GTK_DIALOG_DESTROY_WITH_PARENT,
				       GTK_STOCK_OK,
				       GTK_RESPONSE_OK,
				       GTK_STOCK_CANCEL,
				       GTK_RESPONSE_CANCEL,
				       NULL);
  gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(w));
  gtk_widget_set_size_request(dialog, 400, 300);

  GtkWidget *sw;
  sw = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), sw, true, true, 0);
  
  vector<generalList<srvItem>::coltype> cols;
  generalList<srvItem>::coltype col;
  col.data_type = G_TYPE_STRING; col.name = "Name";
  cols.push_back(col);
  col.name = "URL";
  cols.push_back(col);
  
  listedit.set_types(cols);
  listedit.create_model();
  treeview = GTK_WIDGET(listedit.create_view(GTK_SELECTION_MULTIPLE));

  gtk_container_add(GTK_CONTAINER(sw), treeview);


  // buttons
  GtkWidget *hbox;
  hbox = gtk_hbox_new(true, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, false, false, 0);

  add_button = gtk_button_new_with_mnemonic("_New server");
  g_signal_connect(add_button, "clicked", G_CALLBACK(handle_activate), this);
  gtk_box_pack_start(GTK_BOX(hbox), add_button, false, false, 0);
  
  remove_button = gtk_button_new_with_mnemonic("_Remove server");
  g_signal_connect(remove_button, "clicked", G_CALLBACK(handle_activate), this);
  gtk_box_pack_start(GTK_BOX(hbox), remove_button, false, false, 0);

  reset_button = gtk_button_new_with_mnemonic("R_eset to default");
  g_signal_connect(reset_button, "clicked", G_CALLBACK(handle_activate), this);
  gtk_box_pack_start(GTK_BOX(hbox), reset_button, false, false, 0);

  gtk_widget_show_all(GTK_DIALOG(dialog)->vbox);
  response(gtk_dialog_run(GTK_DIALOG(dialog)));
  gtk_widget_destroy(dialog);
  isOpen = false;
}

void das_server_manage_dialog::response(gint r){
  if(r == GTK_RESPONSE_OK){
    srvlist_orig->clear();
    for(vector<srvItem>::iterator i=listedit.items.begin(); 
	i!=listedit.items.end(); i++){
      srvlist_orig->push_back(i->get());
    }
  }
}

void das_server_manage_dialog::activate(GtkWidget *w){
  if(w==add_button){
    srvItem s;
    s.set(pair<string,string>("NAME", "http://URL.of/your/dsn"));
    listedit.add_item(s);
  }

  if(w==remove_button)
    listedit.remove_selected();

  if(w==reset_button)
    listedit.clear();
}

// ------------------------------
// DAS import dialog


void das_import_dialog::activate(GtkWidget* w){
  int on_types=0;
  bool type_toggled = false;

  if(w==servmng_btn){
    int current_server = gtk_combo_box_get_active(GTK_COMBO_BOX(servercombo));
    size_t combo_members = dsn_urls.size();
    status.dasd.set_server_list( &status.das_server_list );
    status.dasd.init(status.mainwindow);

    // remove 
    for(size_t i=0; i<combo_members; i++)
      gtk_combo_box_remove_text(GTK_COMBO_BOX(servercombo), 0);

    // then add 
    set_server_list();
    for(unsigned i=0; i<dsn_urls.size(); i++)
      gtk_combo_box_append_text(GTK_COMBO_BOX(servercombo),
				(dsn_names[i]+" ("+dsn_urls[i]+")").c_str());
    gtk_combo_box_set_active(GTK_COMBO_BOX(servercombo), current_server);
    return;
  }

  if(w==save_tracks){
    GtkWidget* chooser;
    chooser = gtk_file_chooser_dialog_new("Save DAS XML data",
					  GTK_WINDOW(status.mainwindow),
					  GTK_FILE_CHOOSER_ACTION_SAVE,
					  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					  GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
					  (gpointer)NULL);
    
    GtkFileFilter *filter = gtk_file_filter_new();
    gtk_file_filter_add_pattern(filter, "*.xml");
    gtk_file_filter_add_pattern(filter, "*.XML");

    gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(chooser), filter);
    gtk_file_chooser_set_do_overwrite_confirmation(GTK_FILE_CHOOSER(chooser), 
						   TRUE);

    if (gtk_dialog_run (GTK_DIALOG (chooser))==GTK_RESPONSE_ACCEPT){
      char *filename;
      filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));

      ofstream dasfile;
      dasfile.open(filename);
      dasfile << das_data;
      dasfile.close();

      g_free(filename);
    }
    gtk_widget_destroy (chooser);    
  }

  for(vector<GtkWidget*>::iterator i=type_checks.begin();
      i!=type_checks.end(); i++){
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(*i))) on_types++;
    if(w==*i) type_toggled=true;
  }

  if(type_toggled){
    gtk_assistant_set_page_complete(GTK_ASSISTANT(dialog), pages[3], 
				    (on_types != 0));
  }
}

void das_import_dialog::set_server_list(){
  dsn_names.clear();
  dsn_urls.clear();

  dsn_names.push_back("UCSC"); 
  dsn_urls.push_back("http://genome.cse.ucsc.edu/cgi-bin/das/dsn");
  dsn_names.push_back("EnsEMBL");
  dsn_urls.push_back("http://www.ensembl.org/das/dsn");
  // dsn_names.push_back("KEGG");
  // dsn_urls.push_back("http://das.hgc.jp/cgi-bin/das/kegg/dsn");
  // dsn_names.push_back("Wormbase");
  // dsn_urls.push_back("http://www.wormbase.org/db/das/dsn");
  dsn_names.push_back("BioMart");
  dsn_urls.push_back("http://www.biomart.org/biomart/das/dsn");
  dsn_names.push_back("Kazusa DNA Res. Inst.");
  dsn_urls.push_back("http://g.kazusa.or.jp/cgi-bin/das/dsn");

  for(vector<pair<string,string> >::iterator i=status.das_server_list.begin();
      i!=status.das_server_list.end(); i++){
    dsn_names.push_back(i->first);
    dsn_urls.push_back(i->second);
  }
}

das_import_result* das_import_dialog::run(GtkWidget* w){
  das_data = "";
  mutex = g_mutex_new();
  dialog = gtk_assistant_new();
  
  gtk_window_set_modal(GTK_WINDOW(dialog), true);
  gtk_window_set_transient_for(GTK_WINDOW(dialog), GTK_WINDOW(status.mainwindow));
  
  g_signal_connect(dialog, "apply", 
		   G_CALLBACK(assistant_apply), (gpointer)this);
  g_signal_connect(dialog, "close", 
		   G_CALLBACK(assistant_close), (gpointer)this);
  g_signal_connect(dialog, "cancel", 
		   G_CALLBACK(assistant_cancel), (gpointer)this);
  g_signal_connect(dialog, "prepare", 
		   G_CALLBACK(assistant_prepare), (gpointer)this);

  gtk_window_set_title(GTK_WINDOW(dialog), "DAS Annotation Import");

  // DAS servers
  set_server_list();

  // ------------------------------
  // setup pages
  
  GtkWidget* table;

  // ----------  
  // page 0: select DAS server
  table = gtk_table_new(5, 5, true);

  servercombo = gtk_combo_box_new_text();

  for(unsigned i=0; i<dsn_urls.size(); i++)
    gtk_combo_box_append_text(GTK_COMBO_BOX(servercombo),
			      (dsn_names[i]+" ("+dsn_urls[i]+")").c_str());
  gtk_combo_box_set_active(GTK_COMBO_BOX(servercombo), 
			   prev_server < 0 ? 0 : 
			   prev_server >= (int)dsn_urls.size() ? 0 : 
			   prev_server);

  gtk_table_attach_defaults(GTK_TABLE(table),
			    gtk_label_new("Select DAS server from below:"),
			    1, 4, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), servercombo,
			    1, 4, 2, 3);

  pages.push_back(table);
  page_widget_ready.push_back(true);

  gtk_assistant_append_page(GTK_ASSISTANT(dialog), table);
  gtk_assistant_set_page_title(GTK_ASSISTANT(dialog), table, 
			       "Select DAS server");
  gtk_assistant_set_page_type(GTK_ASSISTANT(dialog), table, 
			      GTK_ASSISTANT_PAGE_CONTENT);

  servmng_btn = gtk_button_new_with_mnemonic("_Manage DAS servers");
  gtk_table_attach_defaults(GTK_TABLE(table), servmng_btn,
			    3, 4, 3, 4);
  g_signal_connect(servmng_btn, "clicked", 
		   G_CALLBACK(handle_activate), this);

  // always GO for this page
  gtk_assistant_set_page_complete(GTK_ASSISTANT(dialog), table, true);

  // ----------
  // page 1: Select source

  table = gtk_table_new(5, 5, true);

  sourcecombo = NULL;
  mapmasters.clear();

  gtk_table_attach_defaults(GTK_TABLE(table),
			    gtk_label_new("Select source from below:"),
			    1, 4, 1, 2);
  
  pages.push_back(table);
  page_widget_ready.push_back(false);
  gtk_assistant_append_page(GTK_ASSISTANT(dialog), table);
  gtk_assistant_set_page_title(GTK_ASSISTANT(dialog), table, 
			       "Select source");
  gtk_assistant_set_page_type(GTK_ASSISTANT(dialog), table, 
			      GTK_ASSISTANT_PAGE_CONTENT);

  gtk_assistant_set_page_complete(GTK_ASSISTANT(dialog), table, false);

  // ----------
  // page 2: Select segment 

  table = gtk_table_new(5, 5, true);

  segmentcombo = NULL;
  gtk_table_attach_defaults(GTK_TABLE(table),
			    gtk_label_new("Select segment from below:"),
			    1, 4, 1, 2);
  
  pages.push_back(table);
  page_widget_ready.push_back(false);
  gtk_assistant_append_page(GTK_ASSISTANT(dialog), table);
  gtk_assistant_set_page_title(GTK_ASSISTANT(dialog), table, 
			       "Select segment");
  gtk_assistant_set_page_type(GTK_ASSISTANT(dialog), table, 
			      GTK_ASSISTANT_PAGE_CONTENT);

  gtk_assistant_set_page_complete(GTK_ASSISTANT(dialog), table, false);

  // ----------
  // page 3: Select type

  table = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(table),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  typetable = NULL;
  
  pages.push_back(table);
  page_widget_ready.push_back(false);
  gtk_assistant_append_page(GTK_ASSISTANT(dialog), table);
  gtk_assistant_set_page_title(GTK_ASSISTANT(dialog), table, 
			       "Select feature type");
  gtk_assistant_set_page_type(GTK_ASSISTANT(dialog), table, 
			      GTK_ASSISTANT_PAGE_CONTENT);

  gtk_assistant_set_page_complete(GTK_ASSISTANT(dialog), table, false);

  // ----------
  // page 4: Summary

  table = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(table),
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  summarytable = NULL;
  
  pages.push_back(table);
  page_widget_ready.push_back(false);
  gtk_assistant_append_page(GTK_ASSISTANT(dialog), table);
  gtk_assistant_set_page_title(GTK_ASSISTANT(dialog), table, 
			       "Summary & Confirm DAS import");
  gtk_assistant_set_page_type(GTK_ASSISTANT(dialog), table, 
			      GTK_ASSISTANT_PAGE_CONTENT);

  gtk_assistant_set_page_complete(GTK_ASSISTANT(dialog), table, false);

  // ----------
  // page 5: load features!

  table = gtk_table_new(5, 5, true);

  segmentcombo = NULL;
  
  pages.push_back(table);
  page_widget_ready.push_back(false);
  gtk_assistant_append_page(GTK_ASSISTANT(dialog), table);
  gtk_assistant_set_page_title(GTK_ASSISTANT(dialog), table, 
			       "Download");
  gtk_assistant_set_page_type(GTK_ASSISTANT(dialog), table, 
			      GTK_ASSISTANT_PAGE_CONTENT);

  gtk_assistant_set_page_complete(GTK_ASSISTANT(dialog), table, false);

  gtk_assistant_set_page_type(GTK_ASSISTANT(dialog), table, 
			      GTK_ASSISTANT_PAGE_CONFIRM);

  result_p = new das_import_result;

  // let's roll
  isOpen = true;
  gtk_widget_show_all(dialog);
  gtk_main();
  gtk_widget_destroy(dialog);
  isOpen = false;

  das_data = "";
  // and return what?
  
  return result_p;
}

void das_import_dialog::apply(){
//   cout << "apply\n";
//   cout << "result_p = " << result_p;
  result_p->separate_track = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(expand_sources));
  
  gtk_main_quit();
}

void das_import_dialog::cancel(){
  //  cout << "cancel\n";
  delete result_p;
  result_p = NULL;
  gtk_main_quit();
}

void das_import_dialog::close(){
  //  cout << "close\n";
  gtk_main_quit();
}

void assistant_subthread(gpointer did){
  ((das_import_dialog*)did)->background();
}


int assistant_idle(gpointer did){
  return ((das_import_dialog*)did)->idle();
}

int das_import_dialog::idle(){
  int return_value = true;
  int page = gtk_assistant_get_current_page(GTK_ASSISTANT(dialog));
  if(g_mutex_trylock(mutex)){
    if(!thread_running){
      if(exit_status==DAS_IMPORT_EXIT_STATUS_NORMAL){
	gtk_assistant_set_page_complete(GTK_ASSISTANT(dialog), 
					pages[page], true);
	GtkWidget *warndialog;
	switch(page){
	case 1:
	  gtk_combo_box_remove_text(GTK_COMBO_BOX(sourcecombo), 0);
	  gtk_widget_destroy(dsn_progbar);

	  // append sources
	  mapmasters.clear();
	  for(map<string,das_dsn>::iterator i=dsnmap.begin();
	      i!=dsnmap.end(); i++){
	    string title = i->first+" ("+i->second.desc+")";
	    if(title.size() > 60) title = title.substr(0,60) + "...";
	    gtk_combo_box_append_text(GTK_COMBO_BOX(sourcecombo), 
				      title.c_str());
	    sources.push_back(title);
	    mapmasters.push_back(i->second.mapmaster);
	  }
	  gtk_combo_box_set_active(GTK_COMBO_BOX(sourcecombo), 
				   prev_source < 0 ? 0 :
				   prev_source >= (int)dsnmap.size() ? 0 :
				   prev_source);
	  break;

	case 2:
	  gtk_combo_box_remove_text(GTK_COMBO_BOX(segmentcombo), 0);
	  gtk_widget_destroy(segtype_progbar);
	  warndialog=NULL;

	  if(epmap.size() > 250){
	    warndialog = gtk_message_dialog_new(GTK_WINDOW(status.mainwindow), 
						GTK_DIALOG_DESTROY_WITH_PARENT,
						GTK_MESSAGE_WARNING,
						GTK_BUTTONS_OK,
						"Too many segments (%d).\n This will take some time.", (int)epmap.size());

	    gtk_window_set_title(GTK_WINDOW(warndialog), "Warning!");
	  }
	  
	  // warning dialog locks the import dialog up... i don't know why
// 	  gtk_window_set_modal(GTK_WINDOW(dialog), false);
// 	  if(warndialog != NULL){
// 	    gtk_dialog_run(GTK_DIALOG(warndialog));
// 	    gtk_widget_destroy(warndialog);
// 	  }
// 	  gtk_window_set_modal(GTK_WINDOW(dialog), true);
	  
	  segments.clear();
	  for(map<string,das_entrypoint>::iterator i=epmap.begin(); 
	      i!=epmap.end(); i++){
	    gtk_combo_box_append_text(GTK_COMBO_BOX(segmentcombo), 
				      (i->first + " (" + 
				       itos(i->second.start) + "-" +
				       itos(i->second.stop) + ", " +
				       (i->second.reverse ? "-" : "+") + 
				       ")").c_str());
	    segments.push_back(i->first);
	  }

	  gtk_combo_box_set_active(GTK_COMBO_BOX(segmentcombo), 
				   prev_segment < 0 ? 0 :
				   prev_segment >= (int)epmap.size() ? 0 : 
				   prev_segment);
	  break;

	case 5:
	  gtk_widget_destroy(dl_progbar);
	  gtk_widget_destroy(dl_statLabel);
	  gtk_table_attach_defaults(GTK_TABLE(pages[5]),
				    dl_statLabel=gtk_label_new("Annotations loaded successfully."), 1, 4, 1, 2);

	  // expand sources to indivisual track
	  // save tracks 
	  expand_sources = gtk_check_button_new_with_label("Expand sources to indivisual tracks");
	  save_tracks = gtk_button_new_with_label("Save DAS XML file");
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(expand_sources), false);
	  gtk_table_attach_defaults(GTK_TABLE(pages[5]),
				    expand_sources, 1, 3, 3, 4);
	  gtk_table_attach_defaults(GTK_TABLE(pages[5]),
				    save_tracks, 3, 4, 3, 4);
	  g_signal_connect(save_tracks, "clicked", 
			   G_CALLBACK(handle_activate), this);

	  gtk_widget_show_all(pages[5]);
	  break;
	}
	page_widget_ready[page]=true;

	// unlock previous page
	handlers.set_sensitive(GTK_ASSISTANT(dialog)->back, true);
	handlers.set_sensitive(GTK_ASSISTANT(dialog)->cancel, true);
      } else {
	// got an error while downloading & parsing
	GtkWidget *errordialog;
	
	string emsg;
	switch(exit_status){
	case DAS_IMPORT_EXIT_STATUS_URL_ERROR: 
	  emsg = "Invalid URL."; break;
	case DAS_IMPORT_EXIT_STATUS_HTTP_ERROR:
	  emsg = "HTTP transfer error."; break;
	case DAS_IMPORT_EXIT_STATUS_PARSE_ERROR:
	  emsg = "XML parse error."; break;
	default: emsg = "An error occured while processing."; break;
	}
	
	errordialog = gtk_message_dialog_new(GTK_WINDOW(status.mainwindow), 
					     GTK_DIALOG_DESTROY_WITH_PARENT,
					     GTK_MESSAGE_ERROR,
					     GTK_BUTTONS_CLOSE,
					     "%s",
					     emsg.c_str());
	gtk_dialog_run(GTK_DIALOG(errordialog));
	gtk_widget_destroy(errordialog);
	gtk_assistant_set_current_page(GTK_ASSISTANT(dialog), page-1);
      }
      return_value = false; // finished
    } else {
      // thread is running
      switch(page){
      case 1:
	gtk_progress_bar_pulse(GTK_PROGRESS_BAR(dsn_progbar)); break;
      case 2:
	gtk_progress_bar_pulse(GTK_PROGRESS_BAR(segtype_progbar)); break;
      case 5:
	gtk_progress_bar_pulse(GTK_PROGRESS_BAR(dl_progbar)); break;
      }
    }
    g_mutex_unlock(mutex);
  } // if(g_mutex_trylock(mutex));
  
  return return_value; // if true, call me again!
}

void das_import_dialog::background(){
  int page = gtk_assistant_get_current_page(GTK_ASSISTANT(dialog));

  string host, path;
  int port;

  int currentstat;

  // string das_data; // moved to das_import_dialog's member variable
  string mapmaster;

  das_dsnParser dp;
  das_entrypointParser ep;
  das_typeParser tp;
  das_featureParser fp;

  assert(cout << "background thread launched!\n");

  switch(page){
  case 1:  // load DSN
    currentstat = gtk_combo_box_get_active(GTK_COMBO_BOX(servercombo));
    prev_server = currentstat;
    if(!parseURL(dsn_urls[currentstat].c_str(),
		 &host, &port, &path)) 
      exit_status = DAS_IMPORT_EXIT_STATUS_URL_ERROR;

    // get dsn by HTTP
    if(!get_by_http(&das_data, host.c_str(), port, path.c_str())) 
      exit_status = DAS_IMPORT_EXIT_STATUS_HTTP_ERROR;

    // parse dsn
    dsnmap.clear();
    if(!dp.parse(das_data, dsnmap))
      exit_status = DAS_IMPORT_EXIT_STATUS_PARSE_ERROR;
    break;

  case 2:  // load segment entrypoint & types
    epmap.clear();
    dtmap.clear();
    
    currentstat = gtk_combo_box_get_active(GTK_COMBO_BOX(sourcecombo));
    prev_source = currentstat;
    mapmaster = mapmasters[currentstat];
    
    // --------------------
    // get entrypoints

    // parse URL
    if( !parseURL(mapmaster+"/entry_points", &host, &port, &path))
      exit_status = DAS_IMPORT_EXIT_STATUS_URL_ERROR;

    // get by HTTP
    if(!get_by_http(&das_data, host.c_str(), port, path.c_str()))
      exit_status = DAS_IMPORT_EXIT_STATUS_HTTP_ERROR;

    // parse it
    if(!ep.parse(das_data, epmap))
      exit_status = DAS_IMPORT_EXIT_STATUS_PARSE_ERROR;

    // --------------------
    // parse URL
    if( !parseURL(mapmaster+"/types", &host, &port, &path))
      exit_status = DAS_IMPORT_EXIT_STATUS_URL_ERROR;

    // get by HTTP
    if(!get_by_http(&das_data, host.c_str(), port, path.c_str()))
      exit_status = DAS_IMPORT_EXIT_STATUS_HTTP_ERROR;

    // parse it
    if(!tp.parse(das_data, dtmap))
      exit_status = DAS_IMPORT_EXIT_STATUS_PARSE_ERROR;
    break;
    
  case 5: // load features
    result_p->fvec.clear();

    // parse URL
    if( !parseURL(feature_url, &host, &port, &path))
      exit_status = DAS_IMPORT_EXIT_STATUS_URL_ERROR;

    // get by HTTP
    if(!get_by_http(&das_data, host.c_str(), port, path.c_str()))
      exit_status = DAS_IMPORT_EXIT_STATUS_HTTP_ERROR;

    // parse it
    if(!fp.parse(das_data, result_p->fvec))
      exit_status = DAS_IMPORT_EXIT_STATUS_PARSE_ERROR;

//   for(vector<das_feature>::iterator i=fvec.begin(); i!=fvec.end(); i++){
//     cout 
//       << i->id << "\t"
//       << i->type << "\t"
//       << i->start << "\t"
//       << i->stop << "\t"
//       << ( i->reverse ? '-' : '+' ) << endl;
//   }

    break;
  }    

  g_mutex_lock(mutex);
  thread_running = false;
  g_mutex_unlock(mutex);
}

void das_import_dialog::prepare(GtkWidget* w){
  int page=0;

  int row=0;
  int col=0;

  // get active page
  for(unsigned i=0; i<pages.size(); i++)
    if(pages[i]==w) page=int(i);

  //  cout << "prepare for page " << page << endl;

  // ----------------------------------------------------------------------
  // invalidate if required
  bool invalidate = false;

  if(page==1){
    int current = gtk_combo_box_get_active(GTK_COMBO_BOX(servercombo));
    if(current!=prev_server) invalidate = true;
  }

  if(page==2){
    int current = gtk_combo_box_get_active(GTK_COMBO_BOX(sourcecombo));
    if(current!=prev_source) invalidate = true;
  }

  if(invalidate){
    for(unsigned i=(unsigned)page; i<pages.size(); i++){
      gtk_assistant_set_page_complete(GTK_ASSISTANT(dialog), w, false);
      page_widget_ready[i]=false;
    }
  }

  // ----------------------------------------------------------------------
  // setup widgets

  if(!page_widget_ready[page]){

    // ------------------------------
    // setup for page 1 & 2

    if(page==1){
      // flush source combo first
      if(sourcecombo != NULL) gtk_widget_destroy(sourcecombo);

      sourcecombo = gtk_combo_box_new_text();
      gtk_combo_box_append_text(GTK_COMBO_BOX(sourcecombo), 
				"Contacting server...");
      gtk_combo_box_set_active(GTK_COMBO_BOX(sourcecombo), 0);
    
      gtk_table_attach_defaults(GTK_TABLE(pages[page]), sourcecombo,
				1, 4, 2, 3);
      gtk_combo_box_set_active(GTK_COMBO_BOX(sourcecombo), 0);
      gtk_widget_show(sourcecombo);
    }
    if(page==2){
      if(segmentcombo != NULL) gtk_widget_destroy(segmentcombo);
    
      segmentcombo = gtk_combo_box_new_text();
      gtk_combo_box_append_text(GTK_COMBO_BOX(segmentcombo), 
				"Contacting server...");
      gtk_combo_box_set_active(GTK_COMBO_BOX(segmentcombo), 0);
      gtk_combo_box_set_active(GTK_COMBO_BOX(segmentcombo), 0);
      gtk_table_attach_defaults(GTK_TABLE(pages[page]), segmentcombo,
				1, 4, 2, 3);
      gtk_widget_show(segmentcombo);
    }

    // setup progressbar & idle functions
    if(page==1){ 
      dsn_progbar = gtk_progress_bar_new();
      gtk_table_attach_defaults(GTK_TABLE(pages[1]), dsn_progbar, 1, 4, 3, 4);
      gtk_widget_show(dsn_progbar);
    }
    if(page==2){
      segtype_progbar = gtk_progress_bar_new();
      gtk_table_attach_defaults(GTK_TABLE(pages[2]), segtype_progbar, 1, 4, 3, 4);
      gtk_widget_show(segtype_progbar);
    }
    if(page==5){
      dl_progbar = gtk_progress_bar_new();
      gtk_table_attach_defaults(GTK_TABLE(pages[5]), dl_progbar, 1, 4, 3, 4);
      gtk_widget_show(dl_progbar);
    }
    if(page==1 || page==2 || page==5){
      exit_status = DAS_IMPORT_EXIT_STATUS_NORMAL;
      g_mutex_lock(mutex);
      thread_running = true;
      g_mutex_unlock(mutex);
      g_thread_create((GThreadFunc)assistant_subthread, this, true, NULL);
      g_timeout_add(PROGBAR_UPDATE_INTERVAL, (GSourceFunc)assistant_idle, this);

      //lock previous page
      handlers.set_sensitive(GTK_ASSISTANT(dialog)->back, false);
      handlers.set_sensitive(GTK_ASSISTANT(dialog)->cancel, false);
    }

    // ------------------------------
    // setup for page 3
    if(page==3){
      if(typetable != NULL)
	gtk_widget_destroy(typetable);

      typetable = gtk_table_new(4, 2, false);
      type_checks.clear();
      gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(w), typetable);

      for(map<string,das_type>::iterator i=dtmap.begin(); i!=dtmap.end(); i++){
	GtkWidget* c = gtk_check_button_new_with_label((i->first + " (" +
							i->second.category + 
// 							", " +
// 							i->second.type +
							")").c_str());
	type_checks.push_back(c);
	types.push_back(i->first);
	gtk_table_attach_defaults(GTK_TABLE(typetable), c,
				  col, col+1, row, row+1);
	col++;
	if(col==2){ col=0; row++; }
      }

      if(dtmap.size()==0){
	GtkWidget *c = gtk_check_button_new_with_label("Default: this database has no selectable type.");
	type_checks.push_back(c);
	types.push_back("_DEFAULT_TYPE_");
	gtk_table_attach_defaults(GTK_TABLE(typetable), c,
				  col, col+1, row, row+1);
      }

      // connect signals
      for(vector<GtkWidget*>::iterator i=type_checks.begin();
	  i!=type_checks.end(); i++)
	g_signal_connect(*i, "toggled", 
			 G_CALLBACK(handle_activate), (gpointer)this);

      page_widget_ready[3]=true;
      gtk_widget_show_all(typetable);
    }

    // ------------------------------
    // setup for page 4 (confirm)
    if(page==4){

      if(summarytable != NULL)
	gtk_widget_destroy(summarytable);

      summarytable = gtk_table_new(4, 2, false);
      gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(w), summarytable);
      gtk_table_set_col_spacing(GTK_TABLE(summarytable), 0, 10);

      GtkWidget *l;
      l=gtk_label_new("Server: ");
      gtk_misc_set_alignment(GTK_MISC(l), 1, 0.5);
      gtk_table_attach_defaults(GTK_TABLE(summarytable), l, 0, 1, 0, 1);

      l=gtk_label_new("Source: ");
      gtk_misc_set_alignment(GTK_MISC(l), 1, 0.5);
      gtk_table_attach_defaults(GTK_TABLE(summarytable), l, 0, 1, 1, 2);

      l=gtk_label_new("Segment: ");
      gtk_misc_set_alignment(GTK_MISC(l), 1, 0.5);
      gtk_table_attach_defaults(GTK_TABLE(summarytable), l, 0, 1, 2, 3);
      
      l=gtk_label_new("Feature types: ");
      gtk_misc_set_alignment(GTK_MISC(l), 1, 0.5);
      gtk_table_attach_defaults(GTK_TABLE(summarytable), l, 0, 1, 3, 4);


      int srv = gtk_combo_box_get_active(GTK_COMBO_BOX(servercombo));
      string s = (dsn_names[srv]+ " (" + dsn_urls[srv] + ")");
      l=gtk_label_new(s.c_str());
      gtk_misc_set_alignment(GTK_MISC(l), 0, 0.5);
      gtk_table_attach_defaults(GTK_TABLE(summarytable), l, 1, 2, 0, 1);
      result_p->server = dsn_names[srv];

      int src = gtk_combo_box_get_active(GTK_COMBO_BOX(sourcecombo));
      l=gtk_label_new(sources[src].c_str());
      gtk_misc_set_alignment(GTK_MISC(l), 0, 0.5);
      gtk_table_attach_defaults(GTK_TABLE(summarytable), l, 1, 2, 1, 2);
      result_p->source = sources[src];

      int seg = gtk_combo_box_get_active(GTK_COMBO_BOX(segmentcombo));
      prev_segment = seg;
      s = ( segments[seg] + " (" +
	    itos_comma(epmap[segments[seg]].start) + "-" +
	    itos_comma(epmap[segments[seg]].stop) +
	    ", " + (epmap[segments[seg]].reverse ? "-" : "+") + ")");
      l=gtk_label_new(s.c_str());
      gtk_misc_set_alignment(GTK_MISC(l), 0, 0.5);
      gtk_table_attach_defaults(GTK_TABLE(summarytable), l, 1, 2, 2, 3);
      result_p->segment = s;

      s = "";
      for(unsigned i=0; i<type_checks.size(); i++){
	if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(type_checks[i]))){
	  if(s!="") s+= ", ";
	  s +=  (types[i]=="_DEFAULT_TYPE_" ? "All" : types[i] );
	}
      }

      l=gtk_label_new(s.c_str());
      gtk_misc_set_alignment(GTK_MISC(l), 0, 0.5);
      gtk_label_set_line_wrap(GTK_LABEL(l), true);
      gtk_table_attach_defaults(GTK_TABLE(summarytable), l, 1, 2, 3, 4);
      result_p->features = s;

      gtk_widget_show_all(summarytable);

      // generate URL
      feature_url = mapmasters[gtk_combo_box_get_active(GTK_COMBO_BOX(sourcecombo))];
      feature_url += "/features?segment=" + url_encode(segments[seg]);
      
      if(types[0]!="_DEFAULT_TYPE_"){
	for(unsigned i=0; i<type_checks.size(); i++){
	  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(type_checks[i])))
	    feature_url += ";type=" + url_encode(types[i]);
	}
      }

      //      cout << "features URL: " << feature_url << endl;
      gtk_assistant_set_page_complete(GTK_ASSISTANT(dialog), pages[4], true);
    }

    if(page==5){
      gtk_table_attach_defaults(GTK_TABLE(pages[5]),
				dl_statLabel=gtk_label_new("Loading... "),
				1, 4, 1, 2);
      gtk_widget_show_all(pages[5]);
    }
  }
}

#endif // OLD_GTK_COMPAT

// ------------------------------
// Inspector: the integrated control dialog

void inspectorPageChange(GtkWidget *w, GtkNotebookPage *useless, 
			 gint uselesss, gpointer insdialog){
  ((inspector*)insdialog)->pageChange();
}

inspectorPage::~inspectorPage(){};

gboolean handle_inspectorPageRelease(GtkWidget *w, GdkEventButton *ev, gpointer p){
  assert(cout << "handle_release(): for " << p << endl);
  ((inspectorPage*)p)->release(w);
  return false;
}

gboolean handle_inspectorPage(GtkWidget *w, gpointer p){
  assert(cout << "handle_spdialog(): for " << p << endl);
  ((inspectorPage*)p)->activate(w);
  return false;
}

// ------------------------------

void inspector::release(GtkWidget *w){
}

void inspector::activate(GtkWidget *w){
}

void inspector::setByStatus(){
  assert(cout << "inspector::setByStatus()" << endl);
  if(!isOpen) return;

  for(unsigned int i=0; i<pages.size(); i++) pages[i]->setByStatus();
}

void inspector::response(gint r){
  if(r == GTK_RESPONSE_OK || r == GTK_RESPONSE_APPLY){

    assert(cout << "inspector::response(): current page = :" <<
	   gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook)) << endl);

    // apply active page first
    unsigned current_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
    pages[current_page]->setToStatus();
    
    // and every other page
    for(unsigned int i=0; i<pages.size(); i++)
      if(i != current_page) pages[i]->setToStatus();
    
    redraw(); // inspector::setByStatus() will called later by this
  }

  if (r != GTK_RESPONSE_APPLY){
    handlers.remove_dialog(HANDLER_INSPECTOR);
    
    for(unsigned int i=0; i<pages.size(); i++) pages[i]->cleanup();

    gtk_widget_destroy(dialog);
    for(unsigned i=0; i<pages.realsize(); i++) pages.activate(i);
    
    isOpen = false;
  }
}

void inspector::pageChange(){
  assert(cout << "inspector::PageChange() at " << this  << endl);
  //  for(unsigned int i=0; i<pages.size(); i++) pages[i]->setByStatus();
}

void inspector::init(GtkWidget *mainwindow){
  if (isOpen==true)  return;
  isOpen = true;

  dialog = gtk_dialog_new_with_buttons ("GMV Inspector",
					GTK_WINDOW(mainwindow),
					GTK_DIALOG_DESTROY_WITH_PARENT,
					GTK_STOCK_OK,
					GTK_RESPONSE_OK,
					GTK_STOCK_APPLY,
					GTK_RESPONSE_APPLY,
					GTK_STOCK_CANCEL,
					GTK_RESPONSE_CANCEL,
					NULL);

  // move window to proper place
  int main_x, main_y, main_w, main_h;
  gtk_window_get_position(GTK_WINDOW(mainwindow), &main_x, &main_y);
  gtk_window_get_size(GTK_WINDOW(mainwindow), &main_w, &main_h);

  gtk_window_move(GTK_WINDOW(dialog), main_x, main_y + main_h);

  // prepare GtkNotebook
  notebook = gtk_notebook_new();
  gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), notebook);

  pagelabels.clear();
  pagelabels.push_back(gtk_label_new_with_mnemonic("_Zoom and Navigate"));
  pagelabels.push_back(gtk_label_new_with_mnemonic("_Sequences"));
  pagelabels.push_back(gtk_label_new_with_mnemonic("tf-idf Anchor _Filter"));
  pagelabels.push_back(gtk_label_new_with_mnemonic("_Annotation Tracks and Filters"));
  pagelabels.push_back(gtk_label_new_with_mnemonic("_Rifts filter"));
  pagelabels.push_back(gtk_label_new_with_mnemonic("_Color settings"));

  // deactivate tf-idf dialog if score isn't loaded
  for(unsigned i=0; i<pages.realsize(); i++){
    pages.activate(i);
    if(pages.at_direct(i) == &tid &&
       ! ( (!mr.tfidf.empty()) &&
	   mr.tfidf.getmin() != mr.tfidf.getmax())) {
      pages.deactivate(i);
      pagelabels.deactivate(i);
      assert(cout << "inspector: deactivate page " << i << endl);
    }
  }

  // embed dialogs
  for(unsigned int i=0; i<pages.size(); i++){
    GtkWidget *p = pages[i]->prepare();
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook), p, pagelabels[i]);
  }
  
  // connect signal(s)
  g_signal_connect(notebook, "switch-page", 
		   G_CALLBACK(inspectorPageChange), (gpointer)this);

  // connect signals in action area
  GList *buttons, *b;
  buttons = gtk_container_get_children(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area));
  for(b=buttons; b; b=b->next){
    handlers.connect(b->data, HANDLER_INSPECTOR);
  }
  g_list_free(buttons);

  // OK, let's go
  setByStatus();
  gtk_widget_show_all(notebook);
  gtk_notebook_set_current_page(GTK_NOTEBOOK(notebook), 0);

  g_signal_connect(dialog, "response", 
		   G_CALLBACK(handle_dialogResponse), 
		   (gpointer)this);

  gtk_widget_show_all(dialog);
}

void inspector::import_prefs(pref *p){
  for(rvector<inspectorPage*>::iterator i=pages.begin(); i!=pages.end(); i++)
    ((inspectorPage*)*i)->import_prefs(p);
}


void inspector::export_prefs(pref* p){
  for(rvector<inspectorPage*>::iterator i=pages.begin(); i!=pages.end(); i++)
    ((inspectorPage*)*i)->export_prefs(p);
}

inspector::inspector(){
  isOpen = false;

  pages.push_back(&mzd);   
  pages.push_back(&spd); 
  pages.push_back(&tid);
  pages.push_back(&amd); 
  pages.push_back(&rfd);
  pages.push_back(&col);
}

// ----------------------------------------------------------------------
// tf-idf filter

GtkWidget* tfIdfDialog::prepare(){
  GtkWidget *sbl_l, *sbl_h;

  mr.tfidf.histo.reinit_widgets();
  mr.tfidf.histo.set_triggers((generalWidgets*)this, (generalWidgets*)this);

  // prepare packing table
  table = gtk_table_new(8, 5, false);

  // label and spinbutton
  sbl_l = gtk_label_new("Lower threshold");
  gtk_misc_set_alignment(GTK_MISC(sbl_l), 1, 0.5);
  sb_l = mr.tfidf.histo.getspinbutton_l();

  sbl_h = gtk_label_new("Higher threshold");
  gtk_misc_set_alignment(GTK_MISC(sbl_h), 1, 0.5);
  sb_h = mr.tfidf.histo.getspinbutton_h();

  // sliders
  slider_l = mr.tfidf.histo.getslider_l();
  slider_h = mr.tfidf.histo.getslider_h();

  // autoapply
  autoApply = chkbox_setup(&opts.auto_apply, "_Apply Automatically");

  // status label
  statusLabel = gtk_label_new("");

  // pack 'em
  gtk_table_attach_defaults(GTK_TABLE(table), sbl_l, 0, 1, 7, 8);
  gtk_table_attach_defaults(GTK_TABLE(table), sb_l,  1, 2, 7, 8);
  gtk_table_attach_defaults(GTK_TABLE(table), sbl_h, 0, 1, 6, 7);
  gtk_table_attach_defaults(GTK_TABLE(table), sb_h,  1, 2, 6, 7);
  gtk_table_attach_defaults(GTK_TABLE(table), mr.tfidf.histo.getimage(), 0, 5, 1, 4);
  gtk_table_attach_defaults(GTK_TABLE(table), slider_h, 0, 5, 4, 5);
  gtk_table_attach_defaults(GTK_TABLE(table), slider_l, 0, 5, 5, 6);
  gtk_table_attach_defaults(GTK_TABLE(table), autoApply, 4, 5, 7, 8);
  gtk_table_attach_defaults(GTK_TABLE(table), statusLabel, 2, 4, 7, 8);

  handlers.connect(sb_l,     HANDLER_INSPECTOR);
  handlers.connect(slider_l, HANDLER_INSPECTOR);
  handlers.connect(sb_h,     HANDLER_INSPECTOR);
  handlers.connect(slider_h, HANDLER_INSPECTOR);

  setByStatus();
  activate(NULL); // show approx visibility count

  return table;
}

void tfIdfDialog::cleanup(){
  return;
}

void tfIdfDialog::activate(GtkWidget* w){
  // value-changed
  if(w==NULL){
    int visible_anchors = mr.tfidf.histo.visible();
    gtk_label_set_text(GTK_LABEL(statusLabel),
		       (itos(visible_anchors) + "/" + itos(mr.arows()) + 
			" visible anchors (approx.)").c_str());
  }
}

void tfIdfDialog::release(GtkWidget* w){
  // release
  if(w==NULL)
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(autoApply))) apply();
}

void tfIdfDialog::setToStatus(){
  // nothing to do: the value is saved by histogram itself
}

void tfIdfDialog::setByStatus(){
//   if(status.max_anchor_score==0) status.max_anchor_score = mr.tfidf.getmax();
  mr.tfidf.histo.fixhisto();
}

void tfIdfDialog::apply(){
  assert(cout << "tfIdfDialog::apply()" << endl);
  setToStatus();
  redraw();
}

void tfIdfDialog::import_prefs(pref *p){
  p->getprefa(opts.auto_apply, "INS_tfidf_autoapply");
}

void tfIdfDialog::export_prefs(pref *p){
  p->setpref("gmv_auto", opts.auto_apply, "INS_tfidf_autoapply");
}

// ----------------------------------------------------------------------
// sequence position dialog

void seqPositionDialog::release(GtkWidget *w){
  for(unsigned i=0; i<pos_scales.size(); i++){
    if(pos_scales[i] == w){
      no_auto_redraw = true;
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(pos_spbtns[i]),
				gtk_range_get_value(GTK_RANGE(pos_scales[i])));
    }
  }
  apply();
}

void seqPositionDialog::activate(GtkWidget *w){

  // justification
  if(w==ar || w==ac || w==al){
    int mode =
      (w==ar) ? JUSTIFY_RIGHT : (w==al) ? JUSTIFY_LEFT : JUSTIFY_CENTER;
      
    mr.justify(mode);
    setByStatus();
    apply();
    return;
  }

  // ordering
  if(w==order_auto){
    status.auto_reorder();

    setByStatus();
    apply();
    return;
  }

  if(w==order_reset){
    for(size_t i=0; i<order_cbox.size(); i++) status.order_seq[i]=(int)i;

    setByStatus();
    apply();
    return;
  }

  // spinbuttons and scales
  for(unsigned i=0; i<pos_scales.size(); i++){
    if(pos_spbtns[i]==w){
      double val = gtk_spin_button_get_value(GTK_SPIN_BUTTON(pos_spbtns[i]));
      no_auto_redraw = true;
      gtk_range_set_value(GTK_RANGE(pos_scales[i]), val);
      return;
    }
    if(pos_scales[i]==w){
      double val = gtk_range_get_value(GTK_RANGE(pos_scales[i]));
      no_auto_redraw = true;
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(pos_spbtns[i]), val);
      return;
    }    
  }

  // order comboboxes
  set<int> usedseq;
  for(unsigned i=0; i<order_cbox.size(); i++) usedseq.insert(i);
  for(unsigned i=0; i<order_cbox.size(); i++) 
    usedseq.erase(gtk_combo_box_get_active(GTK_COMBO_BOX(order_cbox[i])));

  for(unsigned i=0; i<order_cbox.size(); i++){
    if(order_cbox[i]==w){
      // if someone has my number, replace it with unused one
      for(unsigned j=0; j<order_cbox.size(); j++){
	if(i==j) continue;
	if(gtk_combo_box_get_active(GTK_COMBO_BOX(order_cbox[i])) ==
	   gtk_combo_box_get_active(GTK_COMBO_BOX(order_cbox[j])) ){
	  no_auto_redraw = true;
	  gtk_combo_box_set_active(GTK_COMBO_BOX(order_cbox[j]), *usedseq.begin());
	  usedseq.erase(usedseq.begin());
	}
      }
    }
  }
  
  // sequence toggles
  int visible_seqs = 0;
  GtkWidget *first_off = NULL;
  for(unsigned i=0; i<toggle_chk.size(); i++){
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle_chk[i]))) 
      visible_seqs++;
    else
      if(first_off==NULL) first_off=toggle_chk[i];
  }
  if(visible_seqs==1){
    no_auto_redraw = true;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(first_off), true);
  }

  // redraw by order comboboxes & sequence toggles
  for(unsigned i=0; i<toggle_chk.size(); i++){
    if(order_cbox[i]==w || toggle_chk[i]==w){ apply(); return; }
  }
}

void seqPositionDialog::apply(){
  if(apply_auto == NULL || setbystatus_running) return;
  if(no_auto_redraw){ no_auto_redraw=false; return; }
  if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(apply_auto))) return;

  assert(cout << "spd::apply() at " << this << endl);
  setToStatus();
  redraw();
  assert(cout << "spd::apply() return" << endl);
}

void seqPositionDialog::setToStatus(){
  assert(cout << "spd::setToStatus() at " << this << endl);
  if(setbystatus_running) return;
  mr.clear_offset();

  for(unsigned int i=0; i<mr.acols()/3; i++){
    assert(cout << "spd::setToStatus(): seq offset #" << i << " = " <<
	   gtk_spin_button_get_value(GTK_SPIN_BUTTON(pos_spbtns[i])) << endl);
    mr.set_offset(i, (seqlen_t)gtk_spin_button_get_value(GTK_SPIN_BUTTON(pos_spbtns[i])));
    mr.getbs_direct(i)->name = gtk_entry_get_text(GTK_ENTRY(seqname_boxes[i]));
    status.order_seq[gtk_combo_box_get_active(GTK_COMBO_BOX(order_cbox[i]))] = i;
    status.toggle_seq[i] = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(toggle_chk[i]));

  }

  // zoomout if necessary
  assert(cout << "spd::setToStatus(): status.plot_type = " << status.plot_type << endl);
  if(status.plot_type != PLOT_SUPERZOOM){
    status.superzoom_from = 0;
    status.superzoom_to = mr.maxbases(status.toggle_seq);
    status.ins.setByStatus(); // to fix manual zoom dialog
  }
  
  assert(cout << "spd::setToStatus(): return" << endl);
}

void seqPositionDialog::setByStatus(){
  setbystatus_running = true;
  for(unsigned int i=0; i<mr.acols()/3; i++){
    gtk_entry_set_text(GTK_ENTRY(seqname_boxes[i]), 
		       mr.getbs_direct(i)->name.c_str());

    no_auto_redraw = true;
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(pos_spbtns[i]), 
			      (gdouble)mr.seq_offset[i]);

    no_auto_redraw = true;
    gtk_combo_box_set_active(GTK_COMBO_BOX(order_cbox[status.order_seq[i]]), i);

    no_auto_redraw = true;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_chk[i]), 
				 status.toggle_seq[i]);
  }
  no_auto_redraw = false;
  setbystatus_running = false;
}

GtkWidget* seqPositionDialog::prepare(){
  assert(cout << "spd at: " << this << endl);
  no_auto_redraw = false;
  setbystatus_running = false;
  apply_auto = NULL;

  // prepare for packing
  table = gtk_table_new(7, 5, false);

  GtkWidget *seqtable = gtk_table_new(mr.acols()/3, 8, false);
  GtkWidget *viewport = gtk_scrolled_window_new(NULL,NULL);

  gtk_scrolled_window_add_with_viewport((GtkScrolledWindow*)viewport, seqtable);
  gtk_scrolled_window_set_policy((GtkScrolledWindow*)viewport, 
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  gtk_table_attach_defaults(GTK_TABLE(table), viewport, 0, 5, 0, 6);

  // arrange the dialog

  GtkWidget *l;

  unsigned nseqs = mr.acols()/3;
  seqlen_t longest = mr.maxbases(status.toggle_seq);

  // seqtable setup 
  l = gtk_label_new("Sequence");
  gtk_misc_set_alignment(GTK_MISC(l), 0.5, 0.5);
  gtk_table_attach(GTK_TABLE(seqtable), l, 0, 1, 0, 1, GTK_FILL, GTK_FILL, 0,0);
  l = gtk_label_new("Order");
  gtk_misc_set_alignment(GTK_MISC(l), 0.5, 0.5);
  gtk_table_attach(GTK_TABLE(seqtable), l, 1, 2, 0, 1, GTK_FILL, GTK_FILL, 0,0);
  l = gtk_label_new("Show");
  gtk_misc_set_alignment(GTK_MISC(l), 0.5, 0.5);
  gtk_table_attach(GTK_TABLE(seqtable), l, 2, 3, 0, 1, GTK_FILL, GTK_FILL, 0,0);
  l = gtk_label_new("Name");
  gtk_misc_set_alignment(GTK_MISC(l), 0.5, 0.5);
  gtk_table_attach(GTK_TABLE(seqtable), l, 3, 4, 0, 1, GTK_FILL, GTK_FILL, 0,0);
  l = gtk_label_new("Offset");
  gtk_misc_set_alignment(GTK_MISC(l), 0.5, 0.5);
  gtk_table_attach(GTK_TABLE(seqtable), l, 4, 8, 0, 1, GTK_FILL, GTK_FILL, 0,0);

  gtk_table_set_col_spacing(GTK_TABLE(seqtable), 0, 15);
  gtk_table_set_col_spacing(GTK_TABLE(seqtable), 1, 15);
  gtk_table_set_col_spacing(GTK_TABLE(seqtable), 2, 15);

  for(unsigned int i=0; i<nseqs; i++){
    // ------------------------------
    // label
    string s = "#" + itos(i+1);
    l = gtk_label_new(s.c_str());
    gtk_misc_set_alignment(GTK_MISC(l), 0.5 , 0.5);

    // position spinbutton
    pos_spbtns.push_back(gtk_spin_button_new_with_range(-longest, longest, 1));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(pos_spbtns[i]), 
			      (gdouble)mr.seq_offset[i]);

    // order combobox
    order_cbox.push_back(gtk_combo_box_new_text());
    for(unsigned int s=0; s<nseqs; s++){
      gtk_combo_box_append_text(GTK_COMBO_BOX(order_cbox[i]), itos(s+1).c_str());
    }
    gtk_widget_add_events(order_cbox[i],  GDK_BUTTON_RELEASE_MASK);
    g_signal_connect(order_cbox[i],  "changed", 
		     G_CALLBACK(handle_inspectorPage), (gpointer)this);
    handlers.connect(order_cbox[i], HANDLER_INSPECTOR);

    // toggle checkbox
    toggle_chk.push_back(gtk_check_button_new());
    gtk_widget_add_events(toggle_chk[i],  GDK_BUTTON_RELEASE_MASK);
    g_signal_connect(toggle_chk[i],  "toggled", 
		     G_CALLBACK(handle_inspectorPage), (gpointer)this);
    handlers.connect(toggle_chk[i], HANDLER_INSPECTOR);

    // position scale
    pos_scales.push_back(gtk_hscale_new_with_range(-longest, longest, 1));
    gtk_scale_set_draw_value(GTK_SCALE(pos_scales[i]), false);
    gtk_range_set_value(GTK_RANGE(pos_scales[i]), (gdouble)mr.seq_offset[i]);

    // sequence name entry
    seqname_boxes.push_back(gtk_entry_new());
    gtk_entry_set_text(GTK_ENTRY(seqname_boxes[i]), mr.getbs_direct(i)->name.c_str());

    // ------------------------------
    // attach 'em
    GtkWidget *order_cbox_ebox = gtk_event_box_new();
    gtk_container_add(GTK_CONTAINER(order_cbox_ebox), order_cbox[i]);

    gtk_table_attach(GTK_TABLE(seqtable), l,         0, 1, i+1, i+2,
		     GtkAttachOptions(GTK_FILL), GTK_FILL, 0, 0);
    gtk_table_attach(GTK_TABLE(seqtable), order_cbox_ebox, 1, 2, i+1, i+2,
		     GtkAttachOptions(GTK_FILL), GTK_FILL, 0, 0);
    gtk_table_attach(GTK_TABLE(seqtable), toggle_chk[i], 2, 3, i+1, i+2,
		     GtkAttachOptions(GTK_FILL), GTK_FILL, 0, 0);
    gtk_table_attach(GTK_TABLE(seqtable), seqname_boxes[i], 3, 4, i+1, i+2,
		     GtkAttachOptions(GTK_FILL), GTK_FILL, 0, 0);
    gtk_table_attach(GTK_TABLE(seqtable), pos_spbtns[i],  4, 5, i+1, i+2,
		     GtkAttachOptions(GTK_FILL), GTK_FILL, 0, 0);
    gtk_table_attach(GTK_TABLE(seqtable), pos_scales[i], 5, 8, i+1, i+2,
		     GtkAttachOptions(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

    // tips
    status.set_tip(order_cbox_ebox, "Select position of this sequence to reorder sequences");
    status.set_tip(toggle_chk[i], "Toggle show/hide this sequence");
    status.set_tip(seqname_boxes[i], "Enter alternative name of sequence here");
    status.set_tip(pos_spbtns[i], "Enter offset to shift this sequence");
    status.set_tip(pos_scales[i], "Move slider to shift this sequence");

    // signals
    gtk_widget_add_events(pos_spbtns[i], GDK_BUTTON_RELEASE_MASK);
    gtk_widget_add_events(pos_scales[i], GDK_BUTTON_RELEASE_MASK);
    g_signal_connect(pos_spbtns[i],  "button-release-event", 
		     G_CALLBACK(handle_inspectorPageRelease), (gpointer)this);
    g_signal_connect(pos_scales[i],  "button-release-event", 
		     G_CALLBACK(handle_inspectorPageRelease), (gpointer)this);

    g_signal_connect(pos_spbtns[i], "value-changed",
		     G_CALLBACK(handle_inspectorPage), (gpointer)this);
    g_signal_connect(pos_scales[i], "value-changed",
		     G_CALLBACK(handle_inspectorPage), (gpointer)this);

    handlers.connect(pos_spbtns[i], HANDLER_INSPECTOR);
    handlers.connect(pos_scales[i], HANDLER_INSPECTOR);
  }

  apply_auto = chkbox_setup(&opts.auto_apply, "_Apply Automatically");

  gtk_table_attach_defaults(GTK_TABLE(table), apply_auto, 0, 1, 6, 7);

  // ------------------------------
  // buttonbar
  GtkWidget *buttons = gtk_hbutton_box_new();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(buttons), GTK_BUTTONBOX_START);

  // order
  order_reset = gtk_button_new_with_label("Reset seq order");
  order_auto  = gtk_button_new_with_label("Auto seq reorder");

  gtk_box_pack_start_defaults(GTK_BOX(buttons), order_auto);
  gtk_box_pack_start_defaults(GTK_BOX(buttons), order_reset);

  status.set_tip(order_auto, "Reorder sequences automatically");
  status.set_tip(order_reset, "Reorder sequences in original order");

  g_signal_connect(order_reset, "clicked",
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);
  g_signal_connect(order_auto, "clicked", 
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);

  handlers.connect(order_reset, HANDLER_INSPECTOR);
  handlers.connect(order_auto,  HANDLER_INSPECTOR);

  // justification presets
  al = gtk_button_new_from_stock(GTK_STOCK_JUSTIFY_LEFT);
  ac = gtk_button_new_from_stock(GTK_STOCK_JUSTIFY_CENTER);
  ar = gtk_button_new_from_stock(GTK_STOCK_JUSTIFY_RIGHT);

  gtk_box_pack_start_defaults(GTK_BOX(buttons), al);
  gtk_box_pack_start_defaults(GTK_BOX(buttons), ac);
  gtk_box_pack_start_defaults(GTK_BOX(buttons), ar);

  gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(buttons), al, true);
  gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(buttons), ac, true);
  gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(buttons), ar, true);

  status.set_tip(al, "Justificate sequences to left");
  status.set_tip(ac, "Justificate sequences to center");
  status.set_tip(ar, "Justificate sequences to right");

  gtk_table_attach_defaults(GTK_TABLE(table), buttons, 1, 5, 6, 7);

  g_signal_connect(al, "clicked", G_CALLBACK(handle_inspectorPage), (gpointer)this);
  g_signal_connect(ac, "clicked", G_CALLBACK(handle_inspectorPage), (gpointer)this);
  g_signal_connect(ar, "clicked", G_CALLBACK(handle_inspectorPage), (gpointer)this);

  handlers.connect(al, HANDLER_INSPECTOR);
  handlers.connect(ac, HANDLER_INSPECTOR);
  handlers.connect(ar, HANDLER_INSPECTOR);

  assert(cout << "spd::prepare() return" << endl);
  return table;
}

void seqPositionDialog::cleanup(){
  seqname_boxes.clear();
  pos_spbtns.clear();
  pos_scales.clear();
  order_cbox.clear();
  toggle_chk.clear();
}

void seqPositionDialog::import_prefs(pref *p){
  p->getprefa(opts.auto_apply, "INS_seq_autoapply");
}

void seqPositionDialog::export_prefs(pref *p){
  p->setpref("gmv_auto", opts.auto_apply, "INS_seq_autoapply");
}

// ----------------------------------------------------------------------
// Manual zoom

bool mzDialog_idle_redraw(gpointer mzd){
  ((manualZoomDialog*)mzd)->setToStatus();
  redraw();
  return FALSE;
}

void manualZoomDialog::apply(){
  assert(cout << "mzd::apply() at" << this << endl);
  setToStatus();

  seqlen_t from = seqlen_from_spinbutton(fromBox);
  seqlen_t to   = seqlen_from_spinbutton(toBox);

  if (from < to && 0 <= from && to <= mr.maxbases(status.toggle_seq))
    redraw();
  assert(cout << "mzd::apply(): return" << endl);
}

void manualZoomDialog::setToStatus(){
  assert(cout << "mzd::setToStatus() at " << this << endl);
  assert(cout << "mzd::setToStatus() maxbases=" << mr.maxbases(status.toggle_seq) << endl);
  seqlen_t from = seqlen_from_spinbutton(fromBox);
  seqlen_t to   = seqlen_from_spinbutton(toBox);

  assert(cout << "mzd::setToStatus(): from=" << from << " to=" << to << endl);
  assert(cout << "mzd::setToStatus(): status.plot_type was " << status.plot_type << endl);
  if (from < to){
    if ( 0 < from || to < mr.maxbases(status.toggle_seq)){
      status.superzoom_from = from;
      status.superzoom_to = to;
      status.plot_type = PLOT_SUPERZOOM;

    } else {
      if ( 0 == from && to == mr.maxbases(status.toggle_seq)){
	status.superzoom_from = from;
	status.superzoom_to = to;
	status.plot_type = PLOT_CHROM_LINKS;
      }
    }
  }
  assert(cout << "mzd::setToStatus(): status.plot_type is " << status.plot_type << endl);
  assert(cout << "mzd::setToStatus(): return" << endl);
}

void manualZoomDialog::release(GtkWidget *w){
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(autoApply)))
    mzDialog_idle_redraw(this);
}

void manualZoomDialog::activate(GtkWidget *w){ // clicked / value-changed

  assert(cout << "mzd::activate() at " << this << endl);

  // ------------------------------
  // Range Scale
  if (MZD_RANGESCALE){
    if(!GTK_WIDGET_HAS_FOCUS(w)) return; // this avoids signal ping-pong.
                                       
    logzoomer lz;
    lz.init(mr.maxbases(status.toggle_seq));

    gdouble range = (gdouble)lz.size((seqlen_t)gtk_range_get_value(GTK_RANGE(w)));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(rangeBox), range);
  }

  // ------------------------------
  // From & To Scales
  if (MZD_FROMSCALE || MZD_TOSCALE){
    GtkWidget* sb;
    if(MZD_FROMSCALE) sb=fromBox; else sb=toBox;

    if(!GTK_WIDGET_HAS_FOCUS(w)) return; // to avoid signal ping-pong
    gdouble pos = gtk_range_get_value(GTK_RANGE(w));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb), pos);
  }

  // ------------------------------
  // from & to boxes
  if (MZD_FROMBOX || MZD_TOBOX){
    bool locked   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rangeLock));
    seqlen_t range_max = mr.maxbases(status.toggle_seq);
    seqlen_t from      = seqlen_from_spinbutton(fromBox);
    seqlen_t to        = seqlen_from_spinbutton(toBox);
    seqlen_t range     = seqlen_from_spinbutton(rangeBox);

    if(MZD_FROMBOX){
      if(from+range <= range_max){
	if (locked) to = from + range;
	else {
	  if ( to<=from ) to = from+1;
	  range = to-from;
	}
      } else {
	if (locked){ to = range_max; from = range_max-range; }
	else{ range = to-from;  } 
      }
    } else { // MZD_TOBOX
      if (to-range >= 0){
	if (locked) from = to - range;
	else {
	  if (from >= to) from = to-1; 
	  range = to-from;
	}
      } else {
	if (locked){ from = 0; to = range; }
	else {range = to-from; }
      }
    }
  

    assert(cout << "mzd::activate() setting values..." << endl);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(fromBox ), (gdouble)from);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(toBox   ), (gdouble)to);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(rangeBox), (gdouble)range);

    gtk_range_set_value(GTK_RANGE(fromScale), (gdouble)from);
    gtk_range_set_value(GTK_RANGE(toScale  ), (gdouble)to);
  }

  // ------------------------------
  // range box
  if (MZD_RANGEBOX){
    logzoomer lz;
    lz.init(mr.maxbases(status.toggle_seq));

    gdouble range = gtk_spin_button_get_value(GTK_SPIN_BUTTON(rangeBox));
    gtk_range_set_value(GTK_RANGE(rangeScale), (gdouble)lz.step((int)range));

    // adjust 'em
    bool locked   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rangeLock));
    seqlen_t range_max = mr.maxbases(status.toggle_seq);
    seqlen_t from      = seqlen_from_spinbutton(fromBox);
    seqlen_t to        = seqlen_from_spinbutton(toBox);

    if (from+range > range_max){
      if (locked) { to = range_max; from = range_max-(seqlen_t)range; }
      else { range = to-from; }
    } else {
      to = from+(seqlen_t)range;
    }
 
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(fromBox ), (gdouble)from);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(toBox   ), (gdouble)to);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(rangeBox), (gdouble)range);
  }

  // ------------------------------
  // zoom in & out
  if (MZD_ZOUT || MZD_ZIN){
    logzoomer lz;
    lz.init(mr.maxbases(status.toggle_seq));

    int step = (int)gtk_range_get_value(GTK_RANGE(rangeScale));
    if(MZD_ZOUT){
      step++;
      if(step > lz.steps) step = lz.steps;
    } else {
      step--;
      if(step < 1) step = 1;
    }

    seqlen_t from   = seqlen_from_spinbutton(fromBox);
    seqlen_t to     = seqlen_from_spinbutton(toBox);
    seqlen_t center = (from+to)/2;

    from = center-lz.size(step)/2;
    if(from<0) from=0;

    gtk_spin_button_set_value(GTK_SPIN_BUTTON(fromBox ), (gdouble)from);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(rangeBox), 
			      (gdouble)lz.size(step));
  }

  // ------------------------------
  // head & tail
  if (MZD_GOHEAD) gtk_spin_button_set_value(GTK_SPIN_BUTTON(fromBox), 0);

  if (MZD_GOTAIL) gtk_spin_button_set_value(GTK_SPIN_BUTTON(toBox), 
					  (gdouble)mr.maxbases(status.toggle_seq));

  // ------------------------------
  // prev & next
  if (MZD_GONEXT || MZD_GOPREV){
    int step = 1;
    for(int i=0; 
	i<gtk_combo_box_get_active(GTK_COMBO_BOX(step_check)); i++)
      step *= 2;

    seqlen_t from = seqlen_from_spinbutton(fromBox);
    seqlen_t to   = seqlen_from_spinbutton(toBox);
  
    seqlen_t range  = to-from;
    seqlen_t stride = range/step;
    bool locked   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rangeLock));

    if(MZD_GONEXT){ 
      seqlen_t mb = mr.maxbases(status.toggle_seq);
      if(to+stride < mb){ from += stride; to += stride; }
      else              { from = mb-range; to = mb; } }
    else { 
      if(from-stride < 0) { from = 0; to = range; }
      else { from -= stride; if (!locked) to -= stride; }
    }
 
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(fromBox), from);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(toBox), to);
  }

  bool need_to_redraw = (MZD_ZIN || MZD_ZOUT ||
			 MZD_GOHEAD || MZD_GOTAIL || MZD_GOPREV || MZD_GONEXT);

  // ------------------------------
  // redraw if you want
  if( need_to_redraw && 
      gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(autoApply)))
    g_idle_add((GSourceFunc)mzDialog_idle_redraw, (gpointer)this);
}

void manualZoomDialog::cleanup(){
}

void manualZoomDialog::setByStatus(){
  assert(cout << "mzd::setByStatus() at " << this << endl);
  logzoomer lz;
  seqlen_t maxbases=mr.maxbases(status.toggle_seq);

  assert(cout << "mzd::setByStatus() maxbases=" << maxbases << endl);
  
  gtk_range_set_range(GTK_RANGE(rangeScale), 1, lz.init(maxbases));
  gtk_range_set_range(GTK_RANGE(fromScale), 0, maxbases);
  gtk_range_set_range(GTK_RANGE(toScale), 0, maxbases);

  gtk_spin_button_set_range(GTK_SPIN_BUTTON(rangeBox), 1, maxbases);
  gtk_spin_button_set_range(GTK_SPIN_BUTTON(fromBox), 0, maxbases-1);
  gtk_spin_button_set_range(GTK_SPIN_BUTTON(toBox), 1, maxbases);
  
  seqlen_t current_from, current_to;
  current_from = status.superzoom_from;
  current_to   = status.superzoom_to;

  gtk_range_set_value(GTK_RANGE(rangeScale), (gdouble)lz.step(current_to-current_from));
  gtk_range_set_value(GTK_RANGE(fromScale),  (gdouble)current_from);
  gtk_range_set_value(GTK_RANGE(toScale),    (gdouble)current_to);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(rangeBox), (gdouble)(current_to-current_from));
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(fromBox),  (gdouble)current_from);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(toBox),    (gdouble)current_to);
}

GtkWidget* manualZoomDialog::prepare(){

  assert(cout << "mzd::prepare() at " << this << endl);

  table = gtk_table_new(8, 9, false);
  seqlen_t maxbases=mr.maxbases(status.toggle_seq);

  // ------------------------------

  logzoomer lz;
  seqlen_t current_from, current_to;
  current_from = status.superzoom_from;
  current_to   = status.superzoom_to;
 
  separator = gtk_hseparator_new();

  // control buttons
  range_zin  = gtk_button_new_from_stock(GTK_STOCK_ZOOM_IN);
  range_zout = gtk_button_new_from_stock(GTK_STOCK_ZOOM_OUT);

  status.set_tip(range_zin, "Zoom in by x2");
  status.set_tip(range_zout, "Zoom out by 1/2");

  goHead     = gtk_button_new_from_stock(GTK_STOCK_GOTO_FIRST);
  goPrev     = gtk_button_new_from_stock(GTK_STOCK_GO_BACK);
  goNext     = gtk_button_new_from_stock(GTK_STOCK_GO_FORWARD);
  goTail     = gtk_button_new_from_stock(GTK_STOCK_GOTO_LAST);

  status.set_tip(goHead, "Go to left end");
  status.set_tip(goPrev, "Move left by the stride selected above");
  status.set_tip(goNext, "Move right by the stride selected above");
  status.set_tip(goTail, "Go to right end");
  
  // scales
  rangeScale = gtk_hscale_new_with_range(1, lz.init(maxbases), 1);
  fromScale  = gtk_hscale_new_with_range(0, maxbases, 1);
  toScale    = gtk_hscale_new_with_range(0, maxbases, 1);

  gtk_scale_set_draw_value(GTK_SCALE(rangeScale), false);
  gtk_scale_set_draw_value(GTK_SCALE(fromScale), false);
  gtk_scale_set_draw_value(GTK_SCALE(toScale), false);
  
  status.set_tip(rangeScale, "Move slider to change range on display");
  status.set_tip(fromScale, "Move slider to change start position on display");
  status.set_tip(toScale, "Move slider to change end position on display");

  // spinbuttons
  rangeLabel = gtk_label_new("Range");
  fromLabel  = gtk_label_new("From");
  toLabel    = gtk_label_new("To");

//   gtk_label_set_justify(GTK_LABEL(rangeLabel), GTK_JUSTIFY_RIGHT);
//   gtk_label_set_justify(GTK_LABEL(fromLabel),  GTK_JUSTIFY_RIGHT);
  
  gtk_misc_set_alignment(GTK_MISC(rangeLabel), 1, 0.5);
  gtk_misc_set_alignment(GTK_MISC(fromLabel),  1, 0.5);
  gtk_misc_set_alignment(GTK_MISC(toLabel),    0, 0.5);

  rangeBox   = gtk_spin_button_new_with_range(1, maxbases, 1);
  fromBox    = gtk_spin_button_new_with_range(0, maxbases-1, 1);
  toBox      = gtk_spin_button_new_with_range(1, maxbases, 1);

  status.set_tip(rangeBox, "Enter #bp to change range on display");
  status.set_tip(fromBox, "Enter #bp to change start position on display");
  status.set_tip(toBox, "Enter #bp to change end position on display");

  setByStatus(); // set values for spinbuttons & scales

  rangeLock = gtk_check_button_new_with_mnemonic("Lock _Range");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rangeLock), status.plot_type==PLOT_SUPERZOOM);

  status.set_tip(rangeLock, "Check this to lock display range\nwhen from/to sliders are moved");

  // forward / backward step
  step_check = gtk_combo_box_new_text();
  gtk_combo_box_append_text(GTK_COMBO_BOX(step_check), "Move 1/1");
  gtk_combo_box_append_text(GTK_COMBO_BOX(step_check), "Move 1/2");
  gtk_combo_box_append_text(GTK_COMBO_BOX(step_check), "Move 1/4");
  gtk_combo_box_append_text(GTK_COMBO_BOX(step_check), "Move 1/8");
  gtk_combo_box_append_text(GTK_COMBO_BOX(step_check), "Move 1/16");
  gtk_combo_box_set_active( GTK_COMBO_BOX(step_check), 1);

  // misc
  autoApply = chkbox_setup(&opts.auto_apply, "Apply A_utomatically");

  status.set_tip(autoApply, "Apply automatically by sliders & buttons");

  // spacers 
  dummyLabel1 = gtk_label_new(" ");
  dummyLabel2 = gtk_label_new(" ");
  dummyLabel3 = gtk_label_new(" ");
  
  // layout
  gtk_widget_set_size_request(fromScale, 500, -1);
  gtk_widget_set_size_request(toScale, 500, -1);
  gtk_widget_set_size_request(rangeScale, 500, -1);

  gtk_table_attach_defaults(GTK_TABLE(table), dummyLabel1, 0, 1, 0, 1);

  gtk_table_attach_defaults(GTK_TABLE(table), rangeLabel,  0, 1, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), rangeBox,    1, 2, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), rangeScale,  2, 7, 1, 2);
  gtk_table_attach_defaults(GTK_TABLE(table), rangeLock,   7, 9, 1, 2);

  gtk_table_attach_defaults(GTK_TABLE(table), range_zin,   2, 3, 2, 3);
  gtk_table_attach_defaults(GTK_TABLE(table), range_zout,  6, 7, 2, 3);

  gtk_table_attach_defaults(GTK_TABLE(table), separator,   0, 9, 3, 4);

  gtk_table_attach_defaults(GTK_TABLE(table), fromLabel,   0, 1, 4, 5);
  gtk_table_attach_defaults(GTK_TABLE(table), fromBox,     1, 2, 4, 5);
  gtk_table_attach_defaults(GTK_TABLE(table), fromScale,   2, 7, 4, 5);

  gtk_table_attach_defaults(GTK_TABLE(table), toScale,     2, 7, 5, 6);
  gtk_table_attach_defaults(GTK_TABLE(table), toBox,       7, 8, 5, 6);
  gtk_table_attach_defaults(GTK_TABLE(table), toLabel,     8, 9, 5, 6);
  
  gtk_table_attach_defaults(GTK_TABLE(table), goHead,      2, 3, 6, 7); 
  gtk_table_attach_defaults(GTK_TABLE(table), goPrev,      3, 4, 6, 7);
  gtk_table_attach_defaults(GTK_TABLE(table), dummyLabel2, 4, 5, 6, 7);
  gtk_table_attach_defaults(GTK_TABLE(table), goNext,      5, 6, 6, 7);
  gtk_table_attach_defaults(GTK_TABLE(table), goTail,      6, 7, 6, 7);

  gtk_table_attach_defaults(GTK_TABLE(table), dummyLabel3, 0, 1, 6, 7);
  gtk_table_attach_defaults(GTK_TABLE(table), autoApply,   7, 9, 6, 7);

  gtk_table_attach_defaults(GTK_TABLE(table), step_check, 4, 5, 6, 7);

  // size request
  gtk_widget_set_size_request(separator,  -1, 20);

  gtk_widget_set_size_request(goHead, 100, -1);
  gtk_widget_set_size_request(goPrev, 100, -1);
  gtk_widget_set_size_request(goNext, 100, -1);
  gtk_widget_set_size_request(goTail, 100, -1);

  gtk_widget_set_size_request(dummyLabel1, -1, 20);
  gtk_widget_set_size_request(dummyLabel2, 100, 20);
  gtk_widget_set_size_request(dummyLabel3, -1, 20);

  gtk_widget_set_size_request(rangeLabel, 60, -1);
  gtk_widget_set_size_request(toLabel,    60, -1);

  gtk_widget_set_size_request(rangeBox, 100, -1);
  gtk_widget_set_size_request(toBox,    100, -1);

  // ------------------------------

  gtk_widget_add_events(fromBox,  GDK_BUTTON_RELEASE_MASK);
  gtk_widget_add_events(toBox,    GDK_BUTTON_RELEASE_MASK);
  gtk_widget_add_events(rangeBox, GDK_BUTTON_RELEASE_MASK);
  g_signal_connect(G_OBJECT(fromBox), "button-release-event",
		   G_CALLBACK(handle_inspectorPageRelease), (gpointer)this);
  g_signal_connect(G_OBJECT(toBox), "button-release-event",
		   G_CALLBACK(handle_inspectorPageRelease), (gpointer)this);
  g_signal_connect(G_OBJECT(rangeBox), "button-release-event",
		   G_CALLBACK(handle_inspectorPageRelease), (gpointer)this);

  g_signal_connect(G_OBJECT(rangeScale), "button-release-event",
		   G_CALLBACK(handle_inspectorPageRelease), (gpointer)this);
  g_signal_connect(G_OBJECT(fromScale),  "button-release-event",
		   G_CALLBACK(handle_inspectorPageRelease), (gpointer)this);
  g_signal_connect(G_OBJECT(toScale),    "button-release-event",
		   G_CALLBACK(handle_inspectorPageRelease), (gpointer)this);
  g_signal_connect(G_OBJECT(rangeScale), "key-release-event",
		   G_CALLBACK(handle_inspectorPageRelease), (gpointer)this);
  g_signal_connect(G_OBJECT(fromScale),  "key-release-event",
		   G_CALLBACK(handle_inspectorPageRelease), (gpointer)this);
  g_signal_connect(G_OBJECT(toScale),    "key-release-event",
		   G_CALLBACK(handle_inspectorPageRelease), (gpointer)this);

  g_signal_connect(G_OBJECT(fromScale), "value-changed", 
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);
  g_signal_connect(G_OBJECT(toScale), "value-changed", 
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);

  g_signal_connect(G_OBJECT(rangeScale), "value-changed", 
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);

  g_signal_connect(G_OBJECT(rangeBox), "value-changed", 
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);
  g_signal_connect(G_OBJECT(fromBox), "value-changed", 
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);
  g_signal_connect(G_OBJECT(toBox), "value-changed", 
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);

  g_signal_connect(G_OBJECT(range_zin),  "clicked",
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);
  g_signal_connect(G_OBJECT(range_zout), "clicked",
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);

  g_signal_connect(G_OBJECT(goHead), "clicked",
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);
  g_signal_connect(G_OBJECT(goTail), "clicked",
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);
  g_signal_connect(G_OBJECT(goPrev), "clicked",
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);
  g_signal_connect(G_OBJECT(goNext), "clicked",
		   G_CALLBACK(handle_inspectorPage), (gpointer)this);

  handlers.connect(rangeScale,  HANDLER_INSPECTOR);
  handlers.connect(fromScale,   HANDLER_INSPECTOR);
  handlers.connect(toScale,     HANDLER_INSPECTOR);

  handlers.connect(range_zin,  HANDLER_INSPECTOR);
  handlers.connect(range_zout, HANDLER_INSPECTOR);

  handlers.connect(goHead, HANDLER_INSPECTOR);
  handlers.connect(goPrev, HANDLER_INSPECTOR);
  handlers.connect(goNext, HANDLER_INSPECTOR);
  handlers.connect(goTail, HANDLER_INSPECTOR);

  return table;
}

void manualZoomDialog::import_prefs(pref *p){
  p->getprefa(opts.auto_apply, "INS_zoom_autoapply");
}

void manualZoomDialog::export_prefs(pref *p){
  p->setpref("gmv_auto", opts.auto_apply, "INS_zoom_autoapply");
}

// ----------------------------------------------------------------------
// annotation management dialog

GtkWidget* annotationManageDialog::prepare(){
  status.annotation_files.clear();
  status.das_tracks.clear();

  GtkWidget *vbox = gtk_vbox_new(false, 0);

  // allocate memory blocks
  frames=(GtkWidget**)malloc(sizeof(GtkWidget*)*MAXSEQ);
  tables=(GtkWidget**)malloc(sizeof(GtkWidget*)*MAXSEQ);
  addButtons=(GtkWidget**)malloc(sizeof(GtkWidget*)*MAXSEQ);
  rows=(int*)malloc(sizeof(int)*MAXSEQ);
  
  sw = gtk_scrolled_window_new(NULL, NULL);
  swbox = gtk_hbox_new(true, 0);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), swbox);
  gtk_scrolled_window_set_policy((GtkScrolledWindow*)sw, 
				 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  for(unsigned i=0; i<mr.acols()/3; i++){
    frames[i]=gtk_frame_new("");

    GtkWidget *lb = gtk_hbox_new(false, 0);
    GtkWidget *l = gtk_label_new(("#"+itos(i+1)+": "+mr.bs[i]->name).c_str());
    gtk_box_pack_start(GTK_BOX(lb), l, false, false, 5);

    addButtons[i]=gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX(addButtons[i]), "Add Track");
    gtk_combo_box_append_text(GTK_COMBO_BOX(addButtons[i]), "From file");
#ifndef OLD_GTK_COMPAT
    gtk_combo_box_append_text(GTK_COMBO_BOX(addButtons[i]), "DAS import");
#endif
    gtk_combo_box_set_active(GTK_COMBO_BOX(addButtons[i]), 0);
    gtk_box_pack_end(GTK_BOX(lb), addButtons[i], false, false, 5);
    gtk_frame_set_label_widget(GTK_FRAME(frames[i]), lb);
    gtk_frame_set_label_align(GTK_FRAME(frames[i]), 0.5, 0.5);

    status.set_tip(addButtons[i], "Add an GenBank or GFF file to this sequence");

    tables[i]=gtk_table_new(100, 3, false);

    gtk_box_pack_start(GTK_BOX(swbox), frames[i], true, true, 0);
    gtk_container_add(GTK_CONTAINER(frames[i]), tables[i]);

    g_signal_connect(addButtons[i], "changed", 
		     G_CALLBACK(handle_inspectorPage), (gpointer)this);
    handlers.connect(addButtons[i], HANDLER_INSPECTOR);
  }

  fill_table();

  // add filter control
  GtkWidget *hbox = gtk_hbox_new(true, 0);
  hide_unanchored = gtk_check_button_new_with_mnemonic("Hide _unanchored genes");
  hide_anchored = gtk_check_button_new_with_mnemonic("Hide _anchored genes");
  gtk_box_pack_start_defaults(GTK_BOX(hbox), hide_unanchored);
  gtk_box_pack_start_defaults(GTK_BOX(hbox), hide_anchored);

  status.set_tip(hide_unanchored, "Hide genes with no anchor");
  status.set_tip(hide_anchored, "Hide genes with any anchor");

  // pack'em and show
  gtk_box_pack_start_defaults(GTK_BOX(vbox), sw);
  gtk_box_pack_start(GTK_BOX(vbox), hbox, false, false, 0);

  setByStatus();

  gtk_widget_show_all(vbox);
  return vbox;
}

void annotationManageDialog::fill_table(){
  // fill up the table
  clear_tables();
  for(unsigned i=0; i<mr.acols()/3; i++){
    rows[i] = 1;
    GtkWidget *l = gtk_label_new("");
    gtk_widget_set_size_request(l, 5, 5);
    gtk_table_attach(GTK_TABLE(tables[i]), l, 0, 1, 0, 1, GTK_SHRINK, GTK_SHRINK, 0, 0);

    gtk_table_resize(GTK_TABLE(tables[i]), 100, 3);
   
    for(rvector<annotationTrack*>::iterator ite=mr.annotations[i].begin_all();
	ite!=mr.annotations[i].end_all(); ite++){
      annotationTrack* t = (annotationTrack*)*ite;

      amInfoWidgets* a = new amInfoWidgets(t->get_name(), 
			    !mr.annotations[i].get_active(rows[i]-1));
      a->attach(tables[i], rows[i]);
      a->sigconnect(this);

      amInfos[i].push_back(a);
      rows[i]+=2; // each aminfo require 2 rows
    }
    gtk_widget_show_all(tables[i]);
  }
}

void annotationManageDialog::cleanup(){
  // free memory blocks
  clear_tables();

  free(frames);
  free(tables);
  free(addButtons);
  free(rows);
}

void annotationManageDialog::clear_tables(){
  for(unsigned i=0; i<MAXSEQ; i++){
    for(unsigned j=0; j<amInfos[i].size(); j++){
      amInfos[i].at(j)->destroy();
      delete amInfos[i].at(j);
    }
    amInfos[i].clear();
  }
}

void annotationManageDialog::activate(GtkWidget* w){
  
  for(unsigned i=0; i<mr.acols()/3; i++){
    if(w==addButtons[i]){
      gint b = gtk_combo_box_get_active(GTK_COMBO_BOX(addButtons[i]));
      gtk_combo_box_set_active(GTK_COMBO_BOX(addButtons[i]), 0);

      if(b == 1){ // add from file
	GtkWidget *chooser;
	string newfile;
	string newfile_nameonly;
      
	chooser = gtk_file_chooser_dialog_new("Open Annotation File",
					      GTK_WINDOW(status.mainwindow), 
					      GTK_FILE_CHOOSER_ACTION_OPEN,
					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					      (gpointer)NULL);

	GtkFileFilter *filter = gtk_file_filter_new();
	gtk_file_filter_add_pattern(filter, "*.cds*");
	gtk_file_filter_add_pattern(filter, "*.CDS*");
	gtk_file_filter_add_pattern(filter, "*.gtf*");
	gtk_file_filter_add_pattern(filter, "*.gff*");
	gtk_file_filter_add_pattern(filter, "*.gbk*");
	gtk_file_filter_add_pattern(filter, "*.xml*");
	gtk_file_filter_add_pattern(filter, "*.GTF*");
	gtk_file_filter_add_pattern(filter, "*.GFF*");
	gtk_file_filter_add_pattern(filter, "*.GBK*");
	gtk_file_filter_add_pattern(filter, "*.XML*");

	gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(chooser), filter);
      
	if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_ACCEPT){
	  char *filename;
	
	  filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(chooser));
	  newfile = filename; 
	  g_free(filename);

	  if(newfile.find("/") != string::npos)
	    newfile_nameonly = newfile.substr(newfile.find_last_of("/")+1);
	  else{
	    if(newfile.find("\\") != string::npos)
	      newfile_nameonly = newfile.substr(newfile.find_last_of("\\")+1);
	    else newfile_nameonly = newfile;
	  }

	  // disallow to add annotations with existing name
	  bool alreadyLoaded = false;
	  for(vector<amInfoWidgets*>::iterator j=amInfos[i].begin();
	      j!=amInfos[i].end(); j++){
	    if(((amInfoWidgets*)*j)->nameString == newfile_nameonly) alreadyLoaded=true;
	  }

	  if(!alreadyLoaded){ // new annotation track entry on the table
	    amInfoWidgets* a = new amInfoWidgets(newfile_nameonly, false);
	    a->file = newfile;
	    a->loadFlag = true;
	    a->removeFlag = false;

	    a->attach(tables[i], rows[i]);
	    a->sigconnect(this);

	    a->das = NULL;
	
	    amInfos[i].push_back(a);
	    rows[i]+=2;

	    gtk_widget_show_all(tables[i]);
	  }
	}
      
	gtk_widget_destroy(chooser);
	break;
      } // add file

#ifndef OLD_GTK_COMPAT
      if( b == 2 ){
	// DAS import
	bool old_modal = has_modal_child;
	has_modal_child = true;

	das_import_dialog did;
	did.set_combobox_status(status.das_prev_server, status.das_prev_source,
				status.das_prev_segment);
	das_import_result* das_res;
	das_res = did.run(status.mainwindow);
	did.get_combobox_status(&status.das_prev_server,
				&status.das_prev_source,
				&status.das_prev_segment);

	has_modal_child = old_modal;

	if(das_res != NULL){

	  amInfoWidgets* a = new amInfoWidgets(("DAS: " + das_res->server), false);
	  a->loadFlag = true;
	  a->removeFlag = false;

	  a->attach(tables[i], rows[i]);
	  a->sigconnect(this);

	  a->das = das_res;
	
	  amInfos[i].push_back(a);
	  rows[i]+=2;

	  gtk_widget_show_all(tables[i]);

	  // FIXME: if there's any track with same name, 
	  //        change my name like as DAS: KEGG(2)
	}
      }
#endif // OLD_GTK_COMPAT
    } // add button
    
    unsigned track=0;
    for(vector<amInfoWidgets*>::iterator j=amInfos[i].begin(); 
	j!=amInfos[i].end(); j++){
      amInfoWidgets* a=(amInfoWidgets*)*j;

      if(w==a->remove){
 	if(!GTK_WIDGET_SENSITIVE(a->hide)){
 	  handlers.set_sensitive(a->name, true);
 	  handlers.set_sensitive(a->hide, true);
 	  gtk_button_set_label(GTK_BUTTON(a->remove), "Remove");
	  status.set_tip(a->remove, "Remove this annotation track");
	  a->removeFlag = false;
	  // cout << "restore seq #" << i << " track#" << track << endl;
 	} else {
	  handlers.set_sensitive(a->name, false);
	  handlers.set_sensitive(a->hide, false);
	  gtk_button_set_label(GTK_BUTTON(a->remove), "Restore");
	  status.set_tip(a->remove, "Cancel removing this annotation track");
	  a->removeFlag = true;
	  //  cout << "remove seq #" << i << " track#" << track << endl;
 	}
	break;
      }
      track++;
    }
  }
}

void annotationManageDialog::release(GtkWidget* w){
  // nothing to do... 
}

void annotationManageDialog::setToStatus(){

  // check whether flushAtRedraw required or not, then confirm if required
  bool flush = false;
  for(unsigned i=0; i<mr.acols()/3; i++){
    for(vector<amInfoWidgets*>::iterator j=amInfos[i].begin(); 
	j!=amInfos[i].end(); j++){
      amInfoWidgets* a=(amInfoWidgets*)*j;

      if( (a->loadFlag && !a->removeFlag) ||
	  (!a->loadFlag && a->removeFlag) ){
	flush = true; 
	break;
      }
    }
  }
  if(flush)
    if(!confirm_to_flush()) return;
  
  // OK, let's go.
  status.annotation_files.clear();
  status.das_tracks.clear();

  for(unsigned i=0; i<mr.acols()/3; i++){
    for(vector<amInfoWidgets*>::iterator j=amInfos[i].begin(); 
	j!=amInfos[i].end(); j++){
      amInfoWidgets* a=(amInfoWidgets*)*j;

      if(a->scoreAttached || a->optionAttached){
	a->setToStatus();
      }

      // add to annotation load request list
      if(a->loadFlag && !a->removeFlag){
	assert(cout << "new annotation track for seq#" 
	       << i << " : " << a->file << endl);
	if(a->das == NULL)
	  status.annotation_files.push_back(pair<int,string>(i, a->file));
	else
	  status.das_tracks.push_back(pair<int,das_import_result*>(i, a->das));
	status.flushAtRedraw = true;
	mr.annotationsChanged = true;
	continue;
      }

      unsigned kk=0;
      for(rvector<annotationTrack*>::iterator k=mr.annotations[i].begin_all();
	  k!= mr.annotations[i].end_all(); k++){
	if(((annotationTrack*)*k)->get_name() == a->nameString) break;
	kk++;
      }

      // Remove
      if(!a->loadFlag && a->removeFlag){
	assert(cout << "delete annotation track for seq#" 
	       << i << " : " << a->nameString << endl);
	delete mr.annotations[i].at_direct(kk);
	mr.annotations[i].erase_direct(kk);
	status.flushAtRedraw = true;
      }

      // Hide
      if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(a->hide)))
	mr.annotations[i].deactivate(kk);
      else 
	mr.annotations[i].activate(kk);
    }
  }

  // clear flags & remove deleted widgets
  for(unsigned i=0; i<mr.acols()/3; i++){
    vector<amInfoWidgets*>::iterator j=amInfos[i].begin();

    while(j!=amInfos[i].end()){
      amInfoWidgets* a=(amInfoWidgets*)*j;
      if(a->removeFlag){
	a->destroy();  // remove widgets
	amInfos[i].erase(j);
	j = amInfos[i].begin();
      } else {
	j++;
      }
      a->loadFlag = false;
    }
  }

  // filters
  status.hide_anchored_gene = 
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(hide_anchored));
  status.hide_unanchored_gene = 
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(hide_unanchored));

}

void annotationManageDialog::setByStatus(){
  // this function knows about only toggling because...
  //   - there is no other mechanism to manipulate annotation tracks 
  //   - filling up the table in setByStatus() seems to be buggy ): :( 

  // set show hide, but the track may not already loaded (default is show)
  for(unsigned i=0; i<mr.acols()/3; i++){
    for(unsigned j=0; j<amInfos[i].size(); j++){
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(amInfos[i].at(j)->hide),
				   false); // default to show
      for(rvector<annotationTrack*>::iterator ite=mr.annotations[i].begin_all();
	  ite!=mr.annotations[i].end_all(); ite++){
	if(((annotationTrack*)*ite)->get_name() == amInfos[i].at(j)->nameString){
	  bool a=mr.annotations[i].get_active(ite);
	  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(amInfos[i].at(j)->hide), !a);
	  amInfos[i].at(j)->setByStatus();
	}
      }
    }
  }

  // show filter control if score is available
  for(unsigned i=0; i<mr.acols()/3; i++){
    for(unsigned j=0; j<amInfos[i].size(); j++){
      for(rvector<annotationTrack*>::iterator ite=mr.annotations[i].begin_all();
	  ite!=mr.annotations[i].end_all(); ite++){
	if(((annotationTrack*)*ite)->get_name() == amInfos[i].at(j)->nameString){
	  amInfos[i].at(j)->setAnnoTrack(ite);
	  if(((annotationTrack*)*ite)->hasScore()){
	    amInfos[i].at(j)->attach_score(tables[i]);
	    amInfos[i].at(j)->af.setByStatus(); // annotation score filter
	  } else {
	    amInfos[i].at(j)->attach_options(tables[i]);
	  }
	}
      } 
    }
  }

  // toggles
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(hide_anchored),
			       status.hide_anchored_gene);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(hide_unanchored),
			       status.hide_unanchored_gene);
}

void annotationManageDialog::apply(){
}

// ------------------------------

colorControlDialog::colorControlDialog(){
  anno_cb_set.push_back(anno_cb_t("CDS",  "CDS"));
  anno_cb_set.push_back(anno_cb_t("RBS",  "RBS"));
  anno_cb_set.push_back(anno_cb_t("rRNA", "rRNA"));
  anno_cb_set.push_back(anno_cb_t("tRNA", "tRNA"));
  anno_cb_set.push_back(anno_cb_t("Misc RNA",      "misc_RNA"));
  anno_cb_set.push_back(anno_cb_t("Repeat region", "repeat_region"));
  anno_cb_set.push_back(anno_cb_t("Repeat unit",   "repeat_unit"));
  anno_cb_set.push_back(anno_cb_t("Stem loop",     "stem_loop"));
  anno_cb_set.push_back(anno_cb_t("Misc feature",  "misc_feature"));
  anno_cb_set.push_back(anno_cb_t("Others",        ""));
}

GtkWidget* colorControlDialog::prepare(){
  table = gtk_table_new(2, 3, false);
  
  // ----------
  // Rift anchor color button

  GtkWidget *riftframe = gtk_frame_new("Anchors with rifts on seq #0");
  GtkWidget *riftframebox = gtk_vbox_new(false, 0);
  gtk_container_add(GTK_CONTAINER(riftframe), riftframebox);
  gtk_table_attach(GTK_TABLE(table), riftframe, 0, 1, 0, 1, 
		   GTK_FILL, GTK_FILL, 0, 0);

  rift0col_b = gtk_color_button_new();
  gtk_color_button_set_title(GTK_COLOR_BUTTON(rift0col_b),
			     "Default color for anchors with rifts on seq #0");
  gtk_box_pack_start(GTK_BOX(riftframebox), rift0col_b, true, false, 0);

  rift0col_policy0 = gtk_radio_button_new_with_label(NULL, "Use the color above");
  rift0col_policy1 = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(rift0col_policy0), "Choose an alternative sequence for color assignment");

  gtk_box_pack_start(GTK_BOX(riftframebox), rift0col_policy0, false, false, 0);
  gtk_box_pack_start(GTK_BOX(riftframebox), rift0col_policy1, false, false, 0);

  rift0col_for_all = gtk_check_button_new_with_label("Also use this color for all ancs with rifts");
  gtk_box_pack_start(GTK_BOX(riftframebox), rift0col_for_all, false, false, 0);

  // ----------
  // Annotation Color Buttons

  GtkWidget *annoframe = gtk_frame_new("Annotations");
  GtkWidget *annoframetab = gtk_table_new(4, 5, true);
  
  gtk_container_add(GTK_CONTAINER(annoframe), annoframetab);
  gtk_table_attach(GTK_TABLE(table), annoframe, 1, 2, 0, 1, 
		   GTK_FILL, GTK_FILL, 0, 0);
  
  int i = 0;
  for( vector<anno_cb_t>::iterator ac = anno_cb_set.begin();
	 ac != anno_cb_set.end(); ac++ ){
    GtkWidget *b = ac->cb = gtk_color_button_new();
    GtkWidget *l = gtk_label_new(ac->label);

    int c = i%5;
    int r = (i/5)*2;

    gtk_table_attach_defaults(GTK_TABLE(annoframetab), b, c, c+1, r,   r+1);
    gtk_table_attach_defaults(GTK_TABLE(annoframetab), l, c, c+1, r+1, r+2);
    gtk_color_button_set_title(GTK_COLOR_BUTTON(b), ac->label);
    gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(b), false);

    i++; 
  }

  // ----------
  // Preset buttons

  GtkWidget *presetframe = gtk_frame_new("Color Presets");
  GtkWidget *presetbox = gtk_vbox_new(true, 0);

  gtk_container_add(GTK_CONTAINER(presetframe), presetbox);
  gtk_table_attach(GTK_TABLE(table), presetframe, 2, 3, 0, 1, 
		   GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
  
  black_button  = gtk_button_new_with_label("Default Color");
  rna_button    = gtk_button_new_with_label("RNAs in Red");
  repeat_button = gtk_button_new_with_label("Repeats in Red");
  random_button = gtk_button_new_with_label("Random Color");

  gtk_box_pack_start(GTK_BOX(presetbox), black_button,  false, false, 0);
  gtk_box_pack_start(GTK_BOX(presetbox), rna_button,    false, false, 0);
  gtk_box_pack_start(GTK_BOX(presetbox), repeat_button, false, false, 0);
  gtk_box_pack_start(GTK_BOX(presetbox), random_button, false, false, 0);

  handlers.connect(black_button, "clicked", G_CALLBACK(handle_activate), 
		   (gpointer)this, HANDLER_INSPECTOR);
  handlers.connect(repeat_button, "clicked", G_CALLBACK(handle_activate), 
		   (gpointer)this, HANDLER_INSPECTOR);
  handlers.connect(rna_button, "clicked", G_CALLBACK(handle_activate), 
		   (gpointer)this, HANDLER_INSPECTOR);
  handlers.connect(random_button, "clicked", G_CALLBACK(handle_activate), 
		   (gpointer)this, HANDLER_INSPECTOR);

  // transparency check
  use_transparency = gtk_check_button_new_with_label("Enable transparency control");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_transparency), false);
  gtk_table_attach(GTK_TABLE(table), use_transparency, 0, 1, 1, 2, 
		   GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
  handlers.connect(use_transparency, "toggled", G_CALLBACK(handle_activate),
		   (gpointer)this, HANDLER_INSPECTOR);
  return table;
}

void colorControlDialog::activate(GtkWidget *w){
  GdkColor red;
  red.red = 65535;  red.blue = 0;  red.green = 0;

  if(w==black_button){
    for(unsigned i=0; i<3; i++) status.rift0_col[i] = 0;
    for(unsigned i=0; i<10; i++) status.anno_col[i] = RGBA(0,0,0, 65535/2);
    setByStatus();
  }

  if(w==repeat_button){
    for(vector<anno_cb_t>::iterator cb = anno_cb_set.begin(); 
	cb != anno_cb_set.end(); cb++){
      if(string(cb->key).find("repeat") != string::npos)
	gtk_color_button_set_color(GTK_COLOR_BUTTON(cb->cb), &red);
    }
  }

  if(w==rna_button){
    for(vector<anno_cb_t>::iterator cb = anno_cb_set.begin(); 
	cb != anno_cb_set.end(); cb++){
      if(string(cb->key).find("RNA") != string::npos)
	gtk_color_button_set_color(GTK_COLOR_BUTTON(cb->cb), &red);
    }
  }

  if(w==random_button){
    GRand *r = g_rand_new();
    for(unsigned i=0; i<3; i++) 
      status.rift0_col[i] = g_rand_int_range(r, 0, 65535);
    for(unsigned i=0; i<10; i++)
      status.anno_col[i] = RGBA(g_rand_int_range(r, 0, 65535),
				g_rand_int_range(r, 0, 65535), 
				g_rand_int_range(r, 0, 65535), 0.5);
    setByStatus();
    g_rand_free(r);
  }

  if(w==use_transparency){
    for(vector<anno_cb_t>::iterator cb = anno_cb_set.begin(); 
	cb != anno_cb_set.end(); cb++){
      gtk_color_button_set_use_alpha(GTK_COLOR_BUTTON(cb->cb), 
				     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_transparency)));
    }
  }
}

void colorControlDialog::setByStatus(){
  rift0col.red   = status.rift0_col[0] * 65535;
  rift0col.green = status.rift0_col[1] * 65535;
  rift0col.blue  = status.rift0_col[2] * 65535;
  gtk_color_button_set_color(GTK_COLOR_BUTTON(rift0col_b), &rift0col);

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rift0col_policy1), 
			       status.rift0_use_alt_seq);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rift0col_for_all), 
			       status.rift0_col_for_all);

  for( vector<anno_cb_t>::iterator ac = anno_cb_set.begin();
	 ac != anno_cb_set.end(); ac++ ){
    GdkColor tempcol;
    int base = status.anno_col_map[string(ac->key)];

    tempcol.red   = status.anno_col[base].r() * 65535;
    tempcol.green = status.anno_col[base].g() * 65535;
    tempcol.blue  = status.anno_col[base].b() * 65535;
    gtk_color_button_set_color(GTK_COLOR_BUTTON(ac->cb), &tempcol);
    gtk_color_button_set_alpha(GTK_COLOR_BUTTON(ac->cb), 
			       status.anno_col[base].a() * 65535);
  }
}

void colorControlDialog::setToStatus(){
  gtk_color_button_get_color(GTK_COLOR_BUTTON(rift0col_b), &rift0col);
  status.rift0_col[0] = (double)rift0col.red / 65535;
  status.rift0_col[1] = (double)rift0col.green / 65535;
  status.rift0_col[2] = (double)rift0col.blue / 65535;

  status.rift0_use_alt_seq = 
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rift0col_policy1));
  status.rift0_col_for_all = 
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rift0col_for_all));

  for( vector<anno_cb_t>::iterator ac = anno_cb_set.begin();
	 ac != anno_cb_set.end(); ac++ ){
    GdkColor tempcol;
    int base = status.anno_col_map[string(ac->key)];
    gtk_color_button_get_color(GTK_COLOR_BUTTON(ac->cb), &tempcol);
    double alpha = gtk_color_button_get_alpha(GTK_COLOR_BUTTON(ac->cb));
    bool use_a = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(use_transparency));

    status.anno_col[base] = RGBA((double)tempcol.red/65535,
				 (double)tempcol.green/65535,
				 (double)tempcol.blue/65535,
				 use_a ? (double)alpha/65535 : 0.5);
  }  
}

// ------------------------------

GtkWidget* riftFilterDialog::prepare(){
  table = gtk_table_new(2, 1, false);
  frame = gtk_frame_new("Sequences");
  enable = gtk_check_button_new_with_label("Enable rifts filter");
  seqtable = gtk_table_new(1, 6, false);

  gtk_table_attach_defaults(GTK_TABLE(table), frame,  0, 1, 0, 1);
  gtk_table_attach(GTK_TABLE(table), enable, 0, 1, 1, 2, 
		   GTK_FILL, GTK_FILL, 0, 0);

  gtk_container_add(GTK_CONTAINER(frame), seqtable);

  for(unsigned i=0; i<mr.acols()/3; i++){
    GtkWidget *l = gtk_label_new(string("Seq #" + itos(i+1)).c_str());
    GtkWidget *c = gtk_combo_box_new_text();
    gtk_combo_box_append_text(GTK_COMBO_BOX(c), "Don't care");
    gtk_combo_box_append_text(GTK_COMBO_BOX(c), "Must have rifts");
    gtk_combo_box_append_text(GTK_COMBO_BOX(c), "Must not have rifts");
    gtk_combo_box_set_active(GTK_COMBO_BOX(c), 0);
    combo[i] = c;
    gtk_table_attach(GTK_TABLE(seqtable), l, i%3*2, i%3*2+1, i/3, i/3+1,
		     GTK_FILL, GTK_FILL, 5, 0);
    gtk_table_attach(GTK_TABLE(seqtable), c, i%3*2+1, i%3*2+2, i/3, i/3+1,
		     GtkAttachOptions(GTK_FILL|GTK_EXPAND), GTK_FILL, 0, 0);
  }
  g_signal_connect(enable, "toggled",
		   G_CALLBACK(handle_activate), (gpointer)this);

  return table; 
}

void riftFilterDialog::setByStatus(){
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(enable),
			       mr.rift_filter_enabled);
  for(unsigned i=0; i<mr.acols()/3; i++){
    gtk_combo_box_set_active(GTK_COMBO_BOX(combo[i]), mr.rift_filter_policy[i]);
  }  
  activate(enable);
}

void riftFilterDialog::setToStatus(){
  mr.rift_filter_enabled = 
    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable));
  for(unsigned i=0; i<mr.acols()/3; i++){
    mr.rift_filter_policy[i] = 
      gtk_combo_box_get_active(GTK_COMBO_BOX(combo[i]));
  }  
}

void riftFilterDialog::activate(GtkWidget *w){
  if(w == enable){
    bool active = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(enable));
    for(unsigned i=0; i<mr.acols()/3; i++){
      handlers.set_sensitive(combo[i], active);
    }
  }
}

// ------------------------------

void annotationFilterDialog::init(GtkWidget *mainwindow){
  if(isOpen) return;
  isOpen = true;

  // create dialog
  dialog = gtk_dialog_new_with_buttons(string("Score Filter: " +
					      annoTrack->get_name()).c_str(),
				       GTK_WINDOW(mainwindow),
				       GTK_DIALOG_DESTROY_WITH_PARENT,
				       GTK_STOCK_OK,
				       GTK_RESPONSE_OK,
				       GTK_STOCK_APPLY,
				       GTK_RESPONSE_APPLY,
				       GTK_STOCK_CANCEL,
				       GTK_RESPONSE_CANCEL,
				       NULL);

  // setup frames and radiobuttons
//   range_rbutton = gtk_radio_button_new_with_label(NULL, "<b>Value range filter</b>");
//   count_rbutton = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(range_rbutton), "Value count filter");
  range_rbutton = gtk_radio_button_new(NULL);
  count_rbutton = gtk_radio_button_new_from_widget(GTK_RADIO_BUTTON(range_rbutton));

  GtkWidget *label = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(label), "<b>Value range filter</b>");
  gtk_container_add(GTK_CONTAINER(range_rbutton), label);

  label = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(label), "<b>Value count filter</b>");
  gtk_container_add(GTK_CONTAINER(count_rbutton), label);

  range_frame = gtk_frame_new("");
  count_frame = gtk_frame_new("");
  gtk_frame_set_label_widget(GTK_FRAME(range_frame), range_rbutton);
  gtk_frame_set_label_widget(GTK_FRAME(count_frame), count_rbutton);
  gtk_frame_set_label_align(GTK_FRAME(range_frame), 0.5, 0.5);
  gtk_frame_set_label_align(GTK_FRAME(count_frame), 0.5, 0.5);

  filter_anchor = gtk_check_button_new_with_label("Enable anchor filtering");
  filter_gene   = gtk_check_button_new_with_label("Enable gene filtering");
  
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), range_frame, 
		     false, false, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), count_frame, 
		     false, false, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), filter_anchor, 
		     false, false, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), filter_gene, 
		     false, false, 0);

  GtkWidget *vbox_r = gtk_vbox_new(false, 0);
  gtk_container_add(GTK_CONTAINER(range_frame), vbox_r);

  // row #1

  GtkWidget* hb = gtk_hbox_new(false, 0);
  range_policy_rbutton[0] = gtk_radio_button_new_with_label(NULL, "Average / ");
  range_policy_rbutton[1] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(range_policy_rbutton[0]), "Max & Min must be in range:");
  gtk_box_pack_start(GTK_BOX(hb), range_policy_rbutton[0], false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), range_policy_rbutton[1], false, false, 0);
  //  gtk_box_pack_start(GTK_BOX(vbox_r), hb, false, false, 0);

  // setup table_r
  table_r = gtk_table_new(3, 10, false);

  double mi = annoTrack->minScore();
  double ma = annoTrack->maxScore();
  double sd = annoTrack->sdScore();

  GtkWidget *label_h = gtk_label_new("Higher");
  GtkWidget *label_l = gtk_label_new("Lower");

  sb_h = gtk_spin_button_new_with_range(mi, ma, sd/100);
  sb_l = gtk_spin_button_new_with_range(mi, ma, sd/100);
  
  av_h = gtk_button_new_with_label("Av");
  av_l = gtk_button_new_with_label("Av");
  min  = gtk_button_new_with_label("Min");
  max  = gtk_button_new_with_label("Max");

  add_h = gtk_button_new_with_label("+SD");
  add_l = gtk_button_new_with_label("+SD");
  sub_h = gtk_button_new_with_label("-SD");
  sub_l = gtk_button_new_with_label("-SD");

  slider_h = gtk_hscale_new_with_range(mi, ma, sd/10);
  slider_l = gtk_hscale_new_with_range(mi, ma, sd/10);
  gtk_scale_set_draw_value(GTK_SCALE(slider_h), false);
  gtk_scale_set_draw_value(GTK_SCALE(slider_l), false);

  // attach 'em
  gtk_table_attach(GTK_TABLE(table_r), hb, 0, 10, 0, 1,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  gtk_table_attach(GTK_TABLE(table_r), label_h, 0, 1, 1, 2,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table_r), label_l, 0, 1, 2, 3,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);
		   
  gtk_table_attach(GTK_TABLE(table_r), sb_h, 1, 2, 1, 2,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table_r), sb_l, 1, 2, 2, 3,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  gtk_table_attach(GTK_TABLE(table_r), gtk_vseparator_new(), 2, 3, 1, 3,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  gtk_table_attach(GTK_TABLE(table_r), sub_h, 3, 4, 1, 2,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table_r), sub_l, 3, 4, 2, 3,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  gtk_table_attach(GTK_TABLE(table_r), slider_h, 4, 5, 1, 2,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table_r), slider_l, 4, 5, 2, 3,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  gtk_table_attach(GTK_TABLE(table_r), add_h, 5, 6, 1, 2,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table_r), add_l, 5, 6, 2, 3,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  gtk_table_attach(GTK_TABLE(table_r), gtk_vseparator_new(), 6, 7, 1, 3,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  gtk_table_attach(GTK_TABLE(table_r), min, 7, 8, 2, 3,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  gtk_table_attach(GTK_TABLE(table_r), av_h, 8, 9, 1, 2,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table_r), av_l, 8, 9, 2, 3,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  gtk_table_attach(GTK_TABLE(table_r), max, 9, 10, 1, 2,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  gtk_widget_set_size_request(slider_h, 150, -1);
  gtk_widget_set_size_request(slider_l, 150, -1);

  // connect signals
  g_signal_connect(add_h, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(add_l, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(sub_h, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(sub_l, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  

  g_signal_connect(slider_h, "value-changed", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(slider_l, "value-changed", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(sb_h, "value-changed", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(sb_l, "value-changed", 
		   G_CALLBACK(handle_activate), (gpointer)this);


  g_signal_connect(min, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(max, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(av_h, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(av_l, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);

  gtk_box_pack_start(GTK_BOX(vbox_r), table_r, false, false, 0);


  // setup table_c

  // row #1

  table_c = gtk_vbox_new(false, 0);

  hb = gtk_hbox_new(false, 0);

  count_lower_rb[0] = gtk_radio_button_new_with_label(NULL, "Lower / ");
  count_lower_rb[1] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(count_lower_rb[0]), "higher than threshold: ");
  
  gtk_box_pack_start(GTK_BOX(hb), count_lower_rb[0], false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), count_lower_rb[1], false, false, 0);
  gtk_box_pack_start(GTK_BOX(table_c), hb, false, false, 0);

  // row #2

  hb = gtk_hbox_new(false, 0);

  count_th_sb = gtk_spin_button_new_with_range(mi, ma, sd/100);
  count_th_scale = gtk_hscale_new_with_range(mi, ma, sd/100);
  count_th_sub = gtk_button_new_with_label("-SD");
  count_th_add = gtk_button_new_with_label("+SD");
  count_th_avg = gtk_button_new_with_label("Av");
  count_th_min = gtk_button_new_with_label("Min");
  count_th_max = gtk_button_new_with_label("Max");

  gtk_scale_set_draw_value(GTK_SCALE(count_th_scale), false);
  gtk_widget_set_size_request(count_th_scale, 150, -1);

  gtk_box_pack_start(GTK_BOX(hb), gtk_label_new(" "), true, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), count_th_sb, false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), gtk_vseparator_new(), false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), count_th_sub, false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), count_th_scale, false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), count_th_add, false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), gtk_vseparator_new(), false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), count_th_min, false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), count_th_avg, false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), count_th_max, false, false, 0);

  gtk_box_pack_start(GTK_BOX(table_c), hb, false, false, 0);

  // row #3

  gtk_box_pack_start(GTK_BOX(table_c), gtk_hseparator_new(), false, false, 0);
  
  // row #4

  hb = gtk_hbox_new(false, 0);
  
  GtkWidget* vb1 = gtk_vbox_new(false, 0);

  count_less_rb[0] = gtk_radio_button_new_with_label(NULL, "Less");
  count_less_rb[1] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(count_less_rb[0]), "More");

  gtk_box_pack_start(GTK_BOX(vb1), count_less_rb[0], true, true, 0);
  gtk_box_pack_start(GTK_BOX(vb1), count_less_rb[1], true, true, 0);

  gtk_box_pack_start(GTK_BOX(hb), vb1, false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb), gtk_label_new(" matches than: "), false, false, 0);

  gtk_box_pack_start(GTK_BOX(table_c), hb, false, false, 0);

  // row #4a

  GtkWidget* hb2 = gtk_hbox_new(false, 0);
  GtkWidget* vb2 = gtk_vbox_new(false, 0);

  count_percent_rb[0] = gtk_radio_button_new_with_label(NULL, "");
  count_percent_rb[1] = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(count_percent_rb[0]), "");

  count_num_sb = gtk_spin_button_new_with_range(0, 10000, 1);

  gtk_box_pack_start(GTK_BOX(hb2), count_percent_rb[0], false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb2), count_num_sb, false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb2), gtk_label_new(" points"), false, false, 0);
  gtk_box_pack_start(GTK_BOX(vb2), hb2, false, false, 0);

  // row #4b

  hb2 = gtk_hbox_new(false, 0);

  count_ratio_sb = gtk_spin_button_new_with_range(0, 100, 0.1);

  gtk_box_pack_start(GTK_BOX(hb2), count_percent_rb[1], false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb2), count_ratio_sb, false, false, 0);
  gtk_box_pack_start(GTK_BOX(hb2), gtk_label_new(" %"), false, false, 0);
  gtk_box_pack_start(GTK_BOX(vb2), hb2, false, false, 0);

  gtk_box_pack_start(GTK_BOX(hb), vb2, false, false, 0);

  // OK!

  gtk_container_add(GTK_CONTAINER(count_frame), table_c);

  // signals on count filter control

  g_signal_connect(count_th_add, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(count_th_sub, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);

  g_signal_connect(count_th_scale, "value-changed", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(count_th_sb, "value-changed", 
		   G_CALLBACK(handle_activate), (gpointer)this);

  g_signal_connect(count_th_min, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(count_th_max, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(count_th_avg, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);

  // signals on radio buttons

  g_signal_connect(range_rbutton, "toggled", 
		   G_CALLBACK(handle_activate), (gpointer)this);

  g_signal_connect(count_rbutton, "toggled", 
		   G_CALLBACK(handle_activate), (gpointer)this);

  // signals on dialog
  g_signal_connect(dialog, "response", 
		   G_CALLBACK(handle_dialogResponse), (gpointer)this);

  setByStatus();

  gtk_widget_show_all(dialog);
  gtk_window_set_resizable(GTK_WINDOW(dialog), false);

}

void annotationFilterDialog::setSensitivity(){
  bool isRangeActive = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(range_rbutton));
  handlers.set_sensitive(table_r, isRangeActive);
  handlers.set_sensitive(table_c, !isRangeActive);
}

void annotationFilterDialog::setByStatus(){
  if(!isOpen) return;
  // range filter

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(filter_anchor), 
			       annoTrack->filterAnchor());
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(filter_gene), 
			       annoTrack->filterGene());
 
  if(annoTrack->filterPolicy() == FILTER_POLICY_RANGE)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(range_rbutton), true);
  else 
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(count_rbutton), true);
  setSensitivity();
  
  if(annoTrack->rangeFilterPolicy() == RANGE_FILTER_POLICY_AVG)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(range_policy_rbutton[0]),
				 true);
  else
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(range_policy_rbutton[1]),
				 true);

  gtk_range_set_value(GTK_RANGE(slider_h), annoTrack->filterMax());
  gtk_range_set_value(GTK_RANGE(slider_l), annoTrack->filterMin());
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_h), annoTrack->filterMax());
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_l), annoTrack->filterMin());

  // count filter

  if(annoTrack->countFilterLower())
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(count_lower_rb[0]), true);
  else 
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(count_lower_rb[1]), true);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(count_th_sb), 
			    annoTrack->countFilterThreshold());
  gtk_range_set_value(GTK_RANGE(count_th_scale), 
		      annoTrack->countFilterThreshold());

  if(annoTrack->countFilterLess())
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(count_less_rb[0]), true);
  else
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(count_less_rb[1]), true);

  if(annoTrack->countFilterPolicy() == COUNT_FILTER_POLICY_COUNT)
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(count_percent_rb[0]), true);
  else 
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(count_percent_rb[1]), true);

  gtk_spin_button_set_value(GTK_SPIN_BUTTON(count_num_sb), 
			    annoTrack->countFilterCount());
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(count_ratio_sb), 
			    annoTrack->countFilterPercentage());

}

void annotationFilterDialog::activate(GtkWidget *w){
  if(!isOpen) return;

  // +- buttons
  if (w==add_h || w==add_l || w==sub_h || w==sub_l){
    bool ah = (w==add_h);
    bool al = (w==add_l);
    bool sh = (w==sub_h);
    bool sl = (w==sub_l);

    double current = ((ah||sh) ? gtk_range_get_value(GTK_RANGE(slider_h)) :
		      gtk_range_get_value(GTK_RANGE(slider_l)));

    if(ah || al) current += annoTrack->sdScore();
    if(sh || sl) current -= annoTrack->sdScore();

    if(ah || sh){ 
      gtk_range_set_value(GTK_RANGE(slider_h), current);
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_h), current);
    }
      
    if(al || sl){
      gtk_range_set_value(GTK_RANGE(slider_l), current);
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_l), current);
    }
  }

  if (w==count_th_add || w==count_th_sub){
    bool a = (w==count_th_add);
    bool s = (w==count_th_sub);

    double current = gtk_range_get_value(GTK_RANGE(count_th_scale));

    if(a) current += annoTrack->sdScore();
    if(s) current -= annoTrack->sdScore();

    gtk_range_set_value(GTK_RANGE(count_th_scale), current);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(count_th_sb), current);
  }


  // spinbuttons (value-changed)
  if (w==sb_h)
    gtk_range_set_value(GTK_RANGE(slider_h), 
			gtk_spin_button_get_value(GTK_SPIN_BUTTON(sb_h)));

  if (w==sb_l)
    gtk_range_set_value(GTK_RANGE(slider_l), 
			gtk_spin_button_get_value(GTK_SPIN_BUTTON(sb_l)));

  if (w==count_th_sb)
    gtk_range_set_value(GTK_RANGE(count_th_scale), 
			gtk_spin_button_get_value(GTK_SPIN_BUTTON(count_th_sb)));


  // slider (value-changed)
  if (w==slider_h)
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_h),
			      gtk_range_get_value(GTK_RANGE(slider_h)));

  if (w==slider_l)
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_l),
			      gtk_range_get_value(GTK_RANGE(slider_l)));

  if (w==count_th_scale)
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(count_th_sb),
			      gtk_range_get_value(GTK_RANGE(count_th_scale)));

  // preset buttons (clicked)
  if (w==av_h){
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_h), annoTrack->avgScore());
    gtk_range_set_value(GTK_RANGE(slider_h), annoTrack->avgScore());
  }
  
  if (w==av_l){
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_l), annoTrack->avgScore());
    gtk_range_set_value(GTK_RANGE(slider_l), annoTrack->avgScore());
  }

  if (w==max){
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_h), annoTrack->maxScore());
    gtk_range_set_value(GTK_RANGE(slider_h), annoTrack->maxScore());
  }

  if (w==min){
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_l), annoTrack->minScore());
    gtk_range_set_value(GTK_RANGE(slider_l), annoTrack->minScore());
  }

  if (w==count_th_avg){
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(count_th_sb), 
			      annoTrack->avgScore());
    gtk_range_set_value(GTK_RANGE(count_th_scale), annoTrack->avgScore());
  }

  if (w==count_th_max){
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(count_th_sb),
			      annoTrack->maxScore());
    gtk_range_set_value(GTK_RANGE(count_th_scale), annoTrack->maxScore());
  }

  if (w==count_th_min){
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(count_th_sb),
			      annoTrack->minScore());
    gtk_range_set_value(GTK_RANGE(count_th_scale), annoTrack->minScore());
  }

  if (w==range_rbutton || w==count_rbutton){
    setSensitivity();
  }

  // range check
  double hi = gtk_spin_button_get_value(GTK_SPIN_BUTTON(sb_h));
  double lo = gtk_spin_button_get_value(GTK_SPIN_BUTTON(sb_l));
  double sd = annoTrack->sdScore();

  if(lo>=hi){
    if(w==sb_h || w==slider_h)
      lo = hi-sd/50;
    else 
      hi = lo+sd/50;

    gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_h), hi);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(sb_l), lo);

    gtk_range_set_value(GTK_RANGE(slider_h), hi);
    gtk_range_set_value(GTK_RANGE(slider_l), lo);
  }
}

void annotationFilterDialog::release(GtkWidget *w){
}

void annotationFilterDialog::setToStatus(){
  if(!isOpen) return;

  // filter activity
  annoTrack->filterAnchor(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(filter_anchor)));
  annoTrack->filterGene(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(filter_gene)));
  
  // range filter
  annoTrack->filterPolicy(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(range_rbutton)) ? FILTER_POLICY_RANGE : FILTER_POLICY_COUNT);
  annoTrack->rangeFilterPolicy(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(range_policy_rbutton[0])) ? RANGE_FILTER_POLICY_AVG : RANGE_FILTER_POLICY_MINMAX);

  annoTrack->filterMin(gtk_spin_button_get_value(GTK_SPIN_BUTTON(sb_l)));
  annoTrack->filterMax(gtk_spin_button_get_value(GTK_SPIN_BUTTON(sb_h)));

  // count filter

  annoTrack->countFilterLower(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(count_lower_rb[0])));
  annoTrack->countFilterLess(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(count_less_rb[0])));
  annoTrack->countFilterPolicy(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(count_percent_rb[0])) ? COUNT_FILTER_POLICY_COUNT : COUNT_FILTER_POLICY_PERCENTAGE);

  annoTrack->countFilterThreshold(gtk_spin_button_get_value(GTK_SPIN_BUTTON(count_th_sb)));
  annoTrack->countFilterCount((int)gtk_spin_button_get_value(GTK_SPIN_BUTTON(count_num_sb)));
  annoTrack->countFilterPercentage(gtk_spin_button_get_value(GTK_SPIN_BUTTON(count_ratio_sb)));

  parent->set_score_status("* Filter Modified *");
}

void annotationFilterDialog::response(gint r){
  if (r == GTK_RESPONSE_OK || r == GTK_RESPONSE_APPLY) 
    setToStatus();

  // destroy
  if (r != GTK_RESPONSE_APPLY){
    gtk_widget_destroy(dialog);
    isOpen = false;
  }
}


// ------------------------------

void amInfoWidgets::activate(GtkWidget *w){
  if(w==filter_config){
    af.init(annoTrack, this);
    af.init(status.mainwindow);
  }

  if(w==filter_toggle){
    // nothing to do with filter toggling
  }
}

amInfoWidgets::amInfoWidgets(string n, bool a){
  name = gtk_label_new(n.c_str());
  remove = gtk_button_new_with_label("Remove");
  hide = gtk_check_button_new_with_label("Hide");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(hide), a);
  score_range = gtk_label_new("");

  gtk_misc_set_alignment(GTK_MISC(name), 0, 0.5);
  gtk_misc_set_alignment(GTK_MISC(score_range), 1, 0.5);

  status.set_tip(remove, "Remove this annotation track");
  status.set_tip(hide, "Hide this annotation track");

  optionExpander = gtk_expander_new("Options");

  nameString = n;
  loadFlag = false;
  removeFlag = false;
  scoreAttached = optionAttached = false;
}

void amInfoWidgets::set_score_status(string s){
  gtk_label_set_text(GTK_LABEL(score_range), s.c_str());
}

void amInfoWidgets::attach_options(GtkWidget* table){
  if(optionAttached) return;
  optionAttached = true;

  gtk_table_attach(GTK_TABLE(table), optionExpander, 0, 3, r+1, r+2,
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);
  GtkWidget *optionTableBox = gtk_vbox_new(false, 0);
  GtkWidget *l = gtk_label_new("Visible Features:");
  gtk_box_pack_start(GTK_BOX(optionTableBox), l, true, false, 0);

  for(set<string>::iterator ite=annoTrack->features.begin();
      ite!=annoTrack->features.end(); ite++){
    GtkWidget *b = gtk_check_button_new_with_label(ite->c_str());
    features_toggle.push_back(pair<GtkWidget*,string>(b, *ite));
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b),
				 !annoTrack->is_hidden_feature(*ite));
    gtk_box_pack_start(GTK_BOX(optionTableBox), b, false, false, 0);
  }

  gtk_container_add(GTK_CONTAINER(optionExpander), optionTableBox);
  gtk_widget_show_all(optionExpander);
}

void amInfoWidgets::attach_score(GtkWidget* table){
  set_score_status("");
  if(scoreAttached) return;
  scoreAttached = true;

  // let's begin
  gtk_table_attach(GTK_TABLE(table), optionExpander, 0, 3, r+1, r+2, 
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  GtkWidget *optionTableBox = gtk_hbox_new(false, 0);
  GtkWidget *l = gtk_label_new("");
  optionTable = gtk_table_new(1, 3, false);
  gtk_widget_set_size_request(l, 10, 0);
  gtk_box_pack_start(GTK_BOX(optionTableBox), l, true, false, 0);
  gtk_box_pack_start(GTK_BOX(optionTableBox), optionTable, false, false, 0);

  gtk_container_add(GTK_CONTAINER(optionExpander), optionTableBox);

  // filter opts
  GtkWidget *filter_label = gtk_label_new("Filter: ");
  GtkWidget *filter_optbox = gtk_hbox_new(false, 0);
  gtk_misc_set_alignment(GTK_MISC(filter_label), 0, 0.5);
  filter_toggle = gtk_check_button_new_with_label("ON");
  filter_config = gtk_button_new_with_label("Setup");
  gtk_box_pack_start(GTK_BOX(filter_optbox), filter_toggle, false, false, 0);
  gtk_box_pack_start(GTK_BOX(filter_optbox), filter_config, false, false, 10);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(filter_toggle), 
			       annoTrack->filterActive());

  status.set_tip(filter_optbox, "Toggle anchor/gene filter by this track's score");
  status.set_tip(filter_config, "Configure anchor/gene filter by this track's score");

  gtk_table_attach(GTK_TABLE(optionTable), filter_label, 0, 1, 0, 1, 
		   GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(optionTable), filter_optbox, 1, 3, 0, 1, 
		   GTK_FILL, GTK_FILL, 0, 0);

  g_signal_connect(filter_toggle, "toggled", 
		   G_CALLBACK(handle_activate), (gpointer)this);
  g_signal_connect(filter_config, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);

  // style opts
  GtkWidget *style_label = gtk_label_new("Style: ");
  GtkWidget *style_optbox = gtk_hbox_new(false, 0);
  gtk_misc_set_alignment(GTK_MISC(style_label), 0, 0.5);

  em_toggle = gtk_check_button_new_with_label("Emphasize");
  style_combo = gtk_combo_box_new_text();
  gtk_combo_box_append_text(GTK_COMBO_BOX(style_combo), "Arrow style");
  gtk_combo_box_append_text(GTK_COMBO_BOX(style_combo), "Box style");

  GtkWidget *style_combo_box = gtk_event_box_new();
  gtk_container_add(GTK_CONTAINER(style_combo_box), style_combo);

  status.set_tip(em_toggle, "Emphasize markers on this annotation track");
  status.set_tip(style_combo_box, "Select marker style\n");

  gtk_box_pack_start(GTK_BOX(style_optbox), em_toggle, false, false, 0);
  gtk_box_pack_start(GTK_BOX(style_optbox), style_combo_box, false, false, 10);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(em_toggle), 
			       annoTrack->emphasizePlot());
  gtk_combo_box_set_active(GTK_COMBO_BOX(style_combo), 
			   annoTrack->plotStyle());

  gtk_table_attach(GTK_TABLE(optionTable), style_label, 0, 1, 1, 2, 
		   GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(optionTable), style_optbox, 1, 3, 1, 2, 
		   GTK_FILL, GTK_FILL, 0, 0);

  // display opts
  GtkWidget *display_label = gtk_label_new("Display: ");
  GtkWidget *display_optbox = gtk_hbox_new(false, 0);
  gtk_misc_set_alignment(GTK_MISC(display_label), 0, 0.5);

  gn_toggle = gtk_check_button_new_with_label("Gene name");
  gtk_box_pack_start(GTK_BOX(display_optbox), gn_toggle, false, false, 0);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(em_toggle), 
			       annoTrack->showName());

  status.set_tip(gn_toggle, "Toggle showing names on markers");

  gtk_table_attach(GTK_TABLE(optionTable), display_label, 0, 1, 2, 3, 
		   GTK_FILL, GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(optionTable), display_optbox, 1, 3, 2, 3, 
		   GTK_FILL, GTK_FILL, 0, 0);

  // and show all
  gtk_widget_show_all(optionExpander);
}

void amInfoWidgets::attach(GtkWidget* table, int row){
  r = row;

  gtk_table_attach(GTK_TABLE(table), name, 0, 1, row, row+1, 
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), remove, 1, 2, row, row+1, 
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);
  gtk_table_attach(GTK_TABLE(table), hide, 2, 3, row, row+1, 
		   (GtkAttachOptions)(GTK_EXPAND|GTK_FILL), GTK_FILL, 0, 0);

  // filter_toggle/config will be attached later by attach_score(), 
  // if score is available.
}

void amInfoWidgets::sigconnect(annotationManageDialog* d){
  g_signal_connect(remove, "clicked", 
		   G_CALLBACK(handle_inspectorPage), (gpointer)d);
  handlers.connect(remove, HANDLER_INSPECTOR);
}

void amInfoWidgets::setByStatus(){
  if(scoreAttached){
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(filter_toggle), annoTrack->filterActive());
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(em_toggle), annoTrack->emphasizePlot());
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gn_toggle), annoTrack->showName());
    gtk_combo_box_set_active(GTK_COMBO_BOX(style_combo), annoTrack->plotStyle());
  }

  if(optionAttached){
    for(vector<pair<GtkWidget*,string> >::iterator ite=features_toggle.begin();
	ite!=features_toggle.end(); ite++){
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ite->first),
				   !annoTrack->is_hidden_feature(ite->second));
    }
  }
}

void amInfoWidgets::setToStatus(){
  if(scoreAttached){
    annoTrack->filterActive(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(filter_toggle)));
    annoTrack->emphasizePlot(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(em_toggle)));
    annoTrack->showName(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gn_toggle)));
    annoTrack->plotStyle(gtk_combo_box_get_active(GTK_COMBO_BOX(style_combo)));
  }

  if(optionAttached){
    annoTrack->hidden_features.clear();
    for(vector<pair<GtkWidget*,string> >::iterator ite=features_toggle.begin();
	ite!=features_toggle.end(); ite++){
      if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ite->first))){
	annoTrack->hidden_features.insert(ite->second);
      }
    }
  }
}

void amInfoWidgets::destroy(){
  af.close();
  
  features_toggle.clear();

  handlers.remove(name);
  handlers.remove(remove);
  handlers.remove(hide);

  gtk_widget_destroy(name);
  gtk_widget_destroy(remove);
  gtk_widget_destroy(hide);
  gtk_widget_destroy(score_range);
  gtk_widget_destroy(optionExpander);
}

void amInfoWidgets::setAnnoTrack(rvector<annotationTrack*>::iterator a){
  annoTrack=(annotationTrack*)*a;
  af.init(annoTrack, this);
}
