using namespace std;

#ifndef _INSPECTOR_H_
#define _INSPECTOR_H_

#include <map>

#include "dialog_base.h"
#include "rvector.h"
#include "annotation.h"
#include "util.h"
#include "dasclient.h"
#include "general_list.h"

#define DAS_IMPORT_EXIT_STATUS_NORMAL 0
#define DAS_IMPORT_EXIT_STATUS_URL_ERROR 1
#define DAS_IMPORT_EXIT_STATUS_HTTP_ERROR 2
#define DAS_IMPORT_EXIT_STATUS_PARSE_ERROR 3

// ----------------------------------------------------------------------
// DAS import dialog

#ifndef OLD_GTK_COMPAT

void assistant_apply(GtkAssistant *a, gpointer p);
void assistant_close(GtkAssistant *a, gpointer p);
void assistant_cancel(GtkAssistant *a, gpointer p);
void assistant_prepare(GtkAssistant *a, GtkWidget *w, gpointer p);

void assistant_subthread(gpointer);
int assistant_idle(gpointer); 

class das_server_manage_dialog : public generalDialog {
public:

  void set_server_list(vector<pair<string,string> >* l){
    if(isOpen) return;
    listedit.clear_only_data();
    for(vector<pair<string,string> >::iterator i=l->begin(); i!=l->end(); i++){
      srvItem s; s.set(*i);
      listedit.add_initial_item(s);
    }
    
    srvlist_orig = l;
  };

  void init(GtkWidget*);
  void response(gint);
  void activate(GtkWidget*);
  void release(GtkWidget*){};

private:
  class srvItem : public generalListItem<string,string,int,int> {
  public: 
    void set(pair<string, string> s){ t=s.first; u=s.second; };
    pair<string, string> get(){ return pair<string,string>(t, u); };
  };

  generalList<srvItem> listedit;

  vector<pair<string,string> > *srvlist_orig;

  GtkWidget *treeview;
  GtkWidget *add_button, *remove_button, *reset_button;
};

class das_import_dialog : public generalAssistant {

public:
  das_import_result* run(GtkWidget*);
  void apply();
  void close();
  void cancel();
  void prepare(GtkWidget*);
  void activate(GtkWidget*);

  void background();
  int idle();


  das_import_dialog(){
    prev_server = prev_source = prev_segment = -1;
  };

  void set_combobox_status(int a, int b, int c){
    prev_server = a;
    prev_source = b;
    prev_segment = c;
  };

  void get_combobox_status(int *a, int *b, int *c){
    *a = prev_server;
    *b = prev_source;
    *c = prev_segment;
  };

private:
  void set_server_list();

  das_import_result* result_p;
  string das_data;

  GMutex *mutex;
  volatile bool thread_running;
  volatile int exit_status;
  
  vector<GtkWidget*> pages;
  vector<string> dsn_names, dsn_urls;

  int prev_server, prev_source, prev_segment;
  vector<bool> page_widget_ready;

  // page 0
  GtkWidget *servercombo, *servmng_btn;

  // page 1
  GtkWidget *dsn_progbar;
  map<string,das_dsn> dsnmap;

  GtkWidget *sourcecombo;
  vector<string> sources;
  vector<string> mapmasters;

  // page 2
  GtkWidget *segtype_progbar;
  map<string,das_entrypoint> epmap;
  map<string,das_type> dtmap;

  GtkWidget *segmentcombo;
  vector<string> segments;

  // page 3
  GtkWidget *typetable;
  vector<GtkWidget*> type_checks;
  vector<string> types;

  // page 4
  GtkWidget *summarytable;
  string feature_url;

  // page 5
  GtkWidget *dl_statLabel, *dl_progbar;
  GtkWidget *expand_sources, *save_tracks;
};

#endif // OLD_GTK_COMPAT

// ----------------------------------------------------------------------
// inspector

void inspectorPageChange(GtkWidget*, GtkNotebookPage*, gint, gpointer);

// inspector pages

gboolean handle_inspectorPageRelease(GtkWidget*, GdkEventButton*, gpointer);
gboolean handle_inspectorPage(GtkWidget*, gpointer);

class inspectorPage : public generalWidgets {
public:
  virtual GtkWidget* prepare() = 0;
  virtual void cleanup() = 0;
  
  virtual void release(GtkWidget*) = 0;
  virtual void activate(GtkWidget*) = 0;

  virtual void setToStatus() = 0;
  virtual void setByStatus() = 0;

  virtual void apply() = 0;

  virtual void import_prefs(pref*) = 0;
  virtual void export_prefs(pref*) = 0;

  virtual ~inspectorPage() = 0;
};

// ----------------------------------------------------------------------
// color

class colorControlDialog : public inspectorPage {
public:
  GtkWidget* prepare();
  void cleanup(){};
  
  void release(GtkWidget*){};
  void activate(GtkWidget*);

  void setToStatus();
  void setByStatus();

  void apply(){};

  void import_prefs(pref*){};
  void export_prefs(pref*){};

  colorControlDialog();
  ~colorControlDialog(){};
private:
  GtkWidget *table;

  // rift anchor color buttons
  GdkColor rift0col; 
  GtkWidget *rift0col_b;
  GtkWidget *rift0col_policy0, *rift0col_policy1;
  GtkWidget *rift0col_for_all;

  // annotation color buttons
  class anno_cb_t { 
  public:
    anno_cb_t(const char *l, const char *k){ label=(char*)l; key=(char*)k; };
    GtkWidget *cb;  
    char* label; 
    char* key;
  };

  vector<anno_cb_t> anno_cb_set;

  // preset buttons
  GtkWidget *black_button;
  GtkWidget *repeat_button;
  GtkWidget *rna_button;
  GtkWidget *random_button;
  
  GtkWidget *use_transparency;
};

// ----------------------------------------------------------------------
// rift filter

class riftFilterDialog : public inspectorPage {
public:
  GtkWidget* prepare();
  void cleanup(){};
  
  void release(GtkWidget*){};
  void activate(GtkWidget*);

  void setToStatus();
  void setByStatus();

  void apply(){};

  void import_prefs(pref*){};
  void export_prefs(pref*){};

  ~riftFilterDialog(){};
private:
  GtkWidget *table, *frame, *seqtable, *enable;
  GtkWidget *combo[MAXSEQ];
};

// ----------------------------------------------------------------------
// tf-idf filter

class tfIdfDialog : public inspectorPage{
public:
  GtkWidget* prepare();
  void cleanup();

  void activate(GtkWidget*);
  void release(GtkWidget*);
  void setToStatus();
  void setByStatus();

  void apply();

  void import_prefs(pref*);
  void export_prefs(pref*);
 
  tfIdfDialog(){ opts.auto_apply = true; };

private:
  GtkWidget *dialog;
  GtkWidget *table;

  GtkWidget *sb_l, *sb_h;
  GtkWidget *slider_l, *slider_h;
  GtkWidget *autoApply;

  struct {
    bool auto_apply;
  } opts;

  GtkWidget *statusLabel;
};

// ----------------------------------------------------------------------
// sequence position control

class seqPositionDialog : public inspectorPage{

public:
  GtkWidget* prepare();
  void cleanup();

  void activate(GtkWidget*);
  void release(GtkWidget*);
  void setToStatus();
  void setByStatus();

  void apply();

  void import_prefs(pref*);
  void export_prefs(pref*);
 
  seqPositionDialog(){ opts.auto_apply = true; };

protected:
  GtkWidget *dialog;
  GtkWidget *table;

  bool no_auto_redraw;
  bool setbystatus_running;

  vector<GtkWidget*> seqname_boxes, pos_spbtns, pos_scales;
  vector<GtkWidget*> order_cbox, toggle_chk;
  GtkWidget *apply_auto;

  GtkWidget *order_reset, *order_auto;
  GtkWidget *al, *ac, *ar;

  struct {
    bool auto_apply;
  } opts;
};

// ----------------------------------------------------------------------
// sequence position control

bool mzDialog_idle_redraw(gpointer useless);

class manualZoomDialog : public inspectorPage{
public:
  GtkWidget* prepare();
  void cleanup();

  void activate(GtkWidget*);
  void release(GtkWidget*);
  void setToStatus();
  void setByStatus();

  void apply();

  manualZoomDialog(){ opts.auto_apply = true; };

  void import_prefs(pref*);
  void export_prefs(pref*);

private:
  GtkWidget *table;
  GtkWidget *rangeScale, *rangeLock, *rangeLabel, *rangeBox;
  GtkWidget *range_zin, *range_zout;
  
  GtkWidget *separator;
  GtkWidget *fromScale, *toScale, *fromLabel, *toLabel, *fromBox, *toBox;
  GtkWidget *goHead, *goTail, *goPrev, *goNext;

  GtkWidget *autoApply;
  GtkWidget *dummyLabel1, *dummyLabel2, *dummyLabel3;

  GtkWidget *step_check;

  struct {
    bool auto_apply;
  } opts;
};


// ----------------------------------------------------------------------
// annotation management

class amInfoWidgets;
class annotationManageDialog;
class annotationFilterDialog : public generalDialog{
public:
  void release(GtkWidget*);
  void activate(GtkWidget*);
  void response(gint);
  void init(GtkWidget *mainwindow);
  void init(annotationTrack* a, amInfoWidgets *p){
    annoTrack=a; parent=p;};
  
  void setByStatus();
  void setToStatus();

  void setSensitivity();

  annotationFilterDialog(){ isOpen=false; }

private:
  GtkWidget *filter_anchor, *filter_gene;

  GtkWidget *table_r;
  GtkWidget *range_frame, *range_rbutton;
  GtkWidget *sb_h, *sb_l;
  GtkWidget *av_h, *av_l, *min, *max;
  GtkWidget *add_h, *add_l, *sub_h, *sub_l;
  GtkWidget *slider_h, *slider_l;
  GtkWidget *range_policy_rbutton[2];

  GtkWidget *table_c;
  GtkWidget *count_frame, *count_rbutton;
  GtkWidget *count_lower_rb[2];
  GtkWidget *count_less_rb[2];
  GtkWidget *count_percent_rb[2];
  GtkWidget *count_th_sb, *count_num_sb, *count_ratio_sb;
  GtkWidget *count_th_scale;
  GtkWidget *count_th_min, *count_th_max, *count_th_avg;
  GtkWidget *count_th_add, *count_th_sub;

  annotationTrack* annoTrack;  
  amInfoWidgets *parent;
};

class amInfoWidgets : public generalWidgets {
public:
  int r;

  amInfoWidgets(){ das=NULL; };

  GtkWidget *name;   // label
  GtkWidget *remove; // button
  GtkWidget *hide;   // checkbox
  GtkWidget *filter_toggle; // checkbox
  GtkWidget *filter_config; // button
  GtkWidget *em_toggle; // emphasis
  GtkWidget *style_combo; // arrow or box
  GtkWidget *gn_toggle; // show gene name or not
  GtkWidget *score_range;
  GtkWidget *optionExpander, *optionTable;
  annotationFilterDialog af;
    
  string nameString;
  string file;

  bool removeFlag;
  bool loadFlag;
  bool scoreAttached, optionAttached;

  das_import_result* das;

  amInfoWidgets(string, bool);
  void attach(GtkWidget*, int);
  void attach_score(GtkWidget*);
  void attach_options(GtkWidget*);
  void sigconnect(annotationManageDialog*);

  void activate(GtkWidget*);
  void release(GtkWidget*){};
  void setToStatus();
  void setByStatus();

  void destroy();
    
  void set_score_status(string);
  void setAnnoTrack(rvector<annotationTrack*>::iterator);

private:
  annotationTrack* annoTrack;
  vector<pair<GtkWidget*,string> > features_toggle;
};

class annotationManageDialog : public inspectorPage{
public:
  annotationManageDialog(){ has_modal_child = false;};
  GtkWidget* prepare();
  void cleanup();

  void activate(GtkWidget*);
  void release(GtkWidget*);
  void setToStatus();
  void setByStatus();

  void apply();

  void fill_table();

  void import_prefs(pref*){};
  void export_prefs(pref*){ return; };
  bool has_any_modal_dialogs() const { return has_modal_child; };
  
private:
  bool has_modal_child;
  
  GtkWidget **frames, **tables, **addButtons;
  GtkWidget *hide_unanchored, *hide_anchored;
  int *rows;

  vector<amInfoWidgets*> amInfos[MAXSEQ];

  void clear_tables();

  GtkWidget *sw, *swbox;
};

// ----------------------------------------------------------------------
// inspector window

class inspector : public generalDialog{
public:
  void release(GtkWidget*);
  void activate(GtkWidget*);
  void response(gint);
  void pageChange();
  void init(GtkWidget *mainwindow);
  
  void setByStatus();
  bool has_any_modal_dialogs() const { return amd.has_any_modal_dialogs(); };

  inspector();

  void import_prefs(pref*);
  void export_prefs(pref*);

private:
  GtkWidget *notebook;

  manualZoomDialog mzd;
  seqPositionDialog spd;
  tfIdfDialog tid;
  annotationManageDialog amd;
  riftFilterDialog rfd;
  colorControlDialog col;

  rvector <inspectorPage*> pages;
  rvector <GtkWidget*> pagelabels;

  GtkWidget *spDialog;
};

#endif
