using namespace std;

#include <iostream>
#include <gtk/gtk.h>
#include <cairo.h>

#ifdef __MINGW32__
#include <io.h>
#include <fcntl.h>
#endif

#include "gmv.h"
#include "mainwindow.h"
#include "plotters.h"
#include "mainwindow_handlers.h"
#include "export.h"
#include "dialogs.h"
#include "arrow.xpm"

// ----------------------------------------------------------------------
// thumbnail bar

void thumb_bar::tb_buttonbox::init(bool bookmark_button_active){
  box = gtk_hbox_new(false, 0);
  tab = gtk_table_new(3, 1, true);
  thumb_button = gtk_button_new();

  if (bookmark_button_active){
    dismiss  = gtk_button_new_with_label("X");
    protect  = gtk_check_button_new();
    gtk_table_attach_defaults(GTK_TABLE(tab), dismiss, 0, 1, 0, 1);
    gtk_table_attach_defaults(GTK_TABLE(tab), protect, 0, 1, 2, 3);
  } else {
    dismiss = NULL;
    protect = NULL;
  }

  gtk_box_pack_start(GTK_BOX(box), thumb_button, false, false, 0);
  gtk_box_pack_start(GTK_BOX(box), tab,          false, false, 0);
}

void thumb_bar::tb_buttonbox::set_thumb(GdkPixbuf* i, pixbuf_cache_tag t){
  thumb = i;
  tag = t;
  gtk_container_add(GTK_CONTAINER(thumb_button), gtk_image_new_from_pixbuf(i));
}

void thumb_bar::tb_buttonbox::set_tip(GtkTooltips* tip, const string& s){
  name = s;
  gtk_tooltips_set_tip(tip, thumb_button, s.c_str(), "");
  if(dismiss != NULL)
    gtk_tooltips_set_tip(tip, dismiss, "Delete this thumbnail", "");
  if(protect != NULL)
    gtk_tooltips_set_tip(tip, protect, "Keep this thumbnail", "");
}

void thumb_bar::tb_buttonbox::activate(GtkWidget* w){

}

void thumb_bar::remove_button(GtkWidget* box){
  buttonboxes.erase(buttonboxes.find(box));
  handlers.remove_widget(box);
  gtk_widget_destroy(box);  
}

void thumb_bar::activate(GtkWidget *w){

  // ------------------------------
  // history flush button

  if(w==histo_flush){
    pixbuf_cache_tag current_tag;
    current_tag.get_current_globs();

    GList *bl = gtk_container_get_children(GTK_CONTAINER(histobox));
    GList *b  = bl;
    
    while(b){
      GtkWidget *box = (GtkWidget*)b->data;
      if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
				       (buttonboxes[box].protect)) &&
	 !(buttonboxes[box].tag == current_tag)){
	remove_button(box);
      }
      b = b->next;
    };
    g_list_free(bl);

    return;
  }

  // ------------------------------
  // add / flush bookmark button

  if(w==bmark_add){
    status.mb.bmark_add();
    return;
  }

  if(w==bmark_flush){
    status.mb.bmark_clear();
    return;
  }

  // ------------------------------
  // one of thumbnail buttons

  GtkWidget *box = w->parent->parent; // dismiss button
  if (buttonboxes.find(box) == buttonboxes.end()) box = w->parent; // thumb

  if (buttonboxes.find(box) != buttonboxes.end()){

    if(w == buttonboxes[box].thumb_button){
      if(box->parent == histobox)
	buttonboxes[box].tag.set_to_status(); 
      else 
	buttonboxes[box].tag.bookmark_to_status(); 
      status.lsb.setByStatus();
      redraw();
      return; 
    }

    if(w == buttonboxes[box].dismiss){
      pixbuf_cache_tag now;
      now.get_current_globs();
      if (now == buttonboxes[box].tag) return; // can't delete current

      // check history size
      GList *bl = gtk_container_get_children(GTK_CONTAINER(histobox));
      GList *b  = bl;
      int count = 0;
      while(b){
	if(++count > 2) break;
	b = b->next;
      };
      g_list_free(bl);
      if(count < 2) return;

      // then remove it
      remove_button(box);
      return;
    }
  }

  // unknown event
  return;
}

void thumb_bar::release(GtkWidget *w){
  if( w==show_button ){
    visible = true;
    gtk_widget_show(notebook);
    gtk_widget_hide(show_button);
    gtk_widget_show(hide_button);
  }
  if( w==hide_button ){
    visible = false;
    gtk_widget_hide(notebook);
    gtk_widget_hide(hide_button);
    gtk_widget_show(show_button);
  }
  status.mb.setByStatus();
}

void thumb_bar::add_img(pixbuf_cache_tag tag, GdkPixbuf *p){
  int w = gdk_pixbuf_get_width(p);
  int h = gdk_pixbuf_get_height(p);

  GdkPixbuf *pb = gdk_pixbuf_scale_simple(p, 80, 80*h/w, GDK_INTERP_BILINEAR);
  thumbnails[tag]=pb;
}

GtkWidget* thumb_bar::new_button(const pixbuf_cache_tag tag, 
				 GtkWidget *buttonbox,
				 const string& name, bool with_bmark_button){
  tb_buttonbox bb;

  bb.init(with_bmark_button);
  bb.set_thumb(thumbnails[tag], tag);
  bb.set_tip(tips, name);

  gtk_widget_show_all(bb.box);
  gtk_box_pack_start(GTK_BOX(buttonbox), bb.box, false, false,0);

  g_signal_connect(bb.thumb_button, "clicked",
		   G_CALLBACK(handle_activate), this);
  if(with_bmark_button){
    g_signal_connect(bb.dismiss, "clicked",
		     G_CALLBACK(handle_activate), this);
  }
  handlers.connect(bb.box, HANDLER_THUMBBAR);

  buttonboxes[bb.box] = bb;
  return bb.box;
}

void thumb_bar::add_bmark(pixbuf_cache_tag tag, string& name){
  new_button(tag, bmarkbox, name, false);
  gtk_widget_hide(bmarkbox);
  gtk_widget_show_all(bmarkbox);
}

void thumb_bar::add_histo(pixbuf_cache_tag tag){
  GList *bl = gtk_container_get_children(GTK_CONTAINER(histobox));
  GList *b  = bl;

  // test "hold" checkbutton ( !b -> histobox isn't empty )
  if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(histo_lock)) &&
     !b) return;
  
  // no duplicated entry & count children
  int boxes = 0;
  while(b){
    if(buttonboxes[(GtkWidget*)b->data].tag == tag){
      g_list_free(bl);
      return;
    }
    b = b->next;
    boxes++;
  }

  // limit is 50, but "protected" thumbs aren't deleted 
  if(boxes >= 50){
    GList *b = g_list_last(bl);

    while(b != bl){
      if(!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON
				       (buttonboxes[(GtkWidget*)b->data].
					protect))){
	remove_button(buttonboxes[(GtkWidget*)b->data].box);
	break;
      } else {
	b=b->prev;
      }
    }
  }
  g_list_free(bl);

  time_t now;
  char *date_str;
  time(&now);
  date_str = strdup(ctime(&now));

  char *c = date_str;
  while(*c != 0){
    if( *c == '\n' ) *c=0; c++;
  }

  GtkWidget* button = new_button(tag, histobox, string(date_str), true);
  gtk_box_reorder_child(GTK_BOX(histobox), button, 0); // show in reverse order
  
  free(date_str);

  /*if(history.size()==1) */ gtk_widget_hide(histobox); // to resize the box
  gtk_widget_show_all(histobox);
}

void thumb_bar::restore_bmark(){
  string filename = status.basename + ".bmk";

  // restore only when bookmark box is empty
  GList *l = gtk_container_get_children(GTK_CONTAINER(bmarkbox));
  gsize bookmarks = g_list_length(l);
  g_list_free(l);
  if(bookmarks != 0) return;
  
  // try to open
  int fd;
  fd = ::open(filename.c_str(), O_RDONLY);
  if(fd == -1) return;

  // check timestamp
  string ancfile = status.basename + status.anchorfile_extension;
  if(!check_timestamp(ancfile, filename)){
    status.infobar.push_back("Bookmark file is older than anchor file. Bookmarks may be obsolete.");
  }

  // read header
  char lenbuf[9];
  int keyval_len;
  lenbuf[8]=0;
  ::read(fd, lenbuf, 8);
  keyval_len = atoi(lenbuf);
  
  // do nothing if length of keyval part is 0
  if(keyval_len == 0) return;

  // get key-value part
  char* keyval_buf = (char*)malloc(keyval_len+1);
  keyval_buf[keyval_len]=0;
  ::read(fd, keyval_buf, keyval_len);

  GKeyFile* keyval = g_key_file_new();
  g_key_file_load_from_data(keyval, keyval_buf, keyval_len, 
			    G_KEY_FILE_NONE, NULL);
  
  // parse key-value part & restore them
  char** names;
  names = g_key_file_get_groups(keyval, &bookmarks);

  vector<char*> imgs;
  char* pngbuf = (char*)malloc(100);
  for(gsize i=0; i<bookmarks; i++){
    int imgsize = g_key_file_get_integer(keyval, names[i], "imgsize", NULL);
    pngbuf = (char*)realloc((void*)pngbuf, imgsize);
    ::read(fd, pngbuf, imgsize);
    GdkPixbufLoader* loader = gdk_pixbuf_loader_new_with_type("png", NULL);
    gdk_pixbuf_loader_write(loader, (const guchar*)pngbuf, imgsize, NULL);

    pixbuf_cache_tag t;
    t.bookmark_from_keyval(keyval, names[i]);
    thumbnails[t] = gdk_pixbuf_loader_get_pixbuf(loader);
    status.mb.bmark_add(t, string(names[i]).substr(8).c_str());
    gdk_pixbuf_loader_close(loader, NULL);
  }
  
  // cleanup and goodbye
  ::close(fd);
  free(keyval_buf);
  g_key_file_free(keyval);
  g_strfreev(names);
}

void thumb_bar::dump_bmark(){
  /* format:
     [0]-[7]        : length of key-value part (decimal ASCII string)
     [8]-[7+length] : key-value list ab
     [8+length]-    : PNG images 
   */

  string filename = status.basename + ".bmk";

  // get the list
  GList *l = gtk_container_get_children(GTK_CONTAINER(bmarkbox));
  GList *w;
  if( g_list_length(l) ==0 ){ g_list_free(l); return; }
  string buf = "";
  std::vector<pair<char*, int> > imgs;


  // try to open file
  int fd;
  fd = ::open(filename.c_str(), O_WRONLY|O_CREAT|O_TRUNC, 0644);
  if(fd == -1) return;

  // dump the bookmarks
  int cnt=0;
  for(w=l; w; w=w->next){
    char num_buf[9];
    snprintf(num_buf, 9, "%08d", ++cnt);
    tb_buttonbox* bb = &buttonboxes[(GtkWidget*)w->data];
    buf += "[" + string(num_buf) + bb->name + "]\n";
    buf += bb->tag.bookmark_to_string();
    
    gsize imgsize; char* imgbuf;
    gdk_pixbuf_save_to_buffer(bb->thumb, &imgbuf, &imgsize, "png",
			      NULL, "compression", "9", NULL);
    buf += "imgsize=" + itos((int)imgsize) + "\n";
    imgs.push_back(pair<char*, int>(imgbuf, imgsize));
  }
  g_list_free(l);

  // write info out
  char lenbuf[9];

  snprintf(lenbuf, 9, "%08d", (int)buf.length());
  ::write(fd, lenbuf, 8);
  ::write(fd, buf.c_str(), buf.length());

  // write images out & free images
  for(std::vector<pair<char*, int> >::iterator i=imgs.begin(); 
      i!=imgs.end(); i++){
    ::write(fd, i->first, i->second);
    g_free(i->first);
  }

  ::close(fd);
}

void thumb_bar::clear_bmark(){
  GList *l = gtk_container_get_children(GTK_CONTAINER(bmarkbox));
  GList *w;
  for(w=l; w; w=w->next){
    remove_button(buttonboxes[(GtkWidget*)w->data].box);
  }
  g_list_free(l);
}

void thumb_bar::clear_history(){
  GList *l = gtk_container_get_children(GTK_CONTAINER(histobox));
  GList *w;
  for(w=l; w; w=w->next){
    remove_button(buttonboxes[(GtkWidget*)w->data].box);
  }
  g_list_free(l);
}

void thumb_bar::clear(){
  // clear all widgets in histobox & bmarkbox
  GList *w, *l;
  l = gtk_container_get_children(GTK_CONTAINER(histobox));
  for(w=l; w; w=w->next) gtk_widget_destroy((GtkWidget*)w->data);
  g_list_free(l);

  l = gtk_container_get_children(GTK_CONTAINER(bmarkbox));
  for(w=l; w; w=w->next) gtk_widget_destroy((GtkWidget*)w->data);
  g_list_free(l);

  // deref pixbufs
  for(std::map<pixbuf_cache_tag, GdkPixbuf*>::iterator i=thumbnails.begin();
      i!=thumbnails.end(); i++) g_object_unref(i->second);

  // clear dataset
  handlers.remove_dialog(HANDLER_THUMBBAR);
  thumbnails.clear();
  buttonboxes.clear();
}

GtkWidget* thumb_bar::init(){
  box = gtk_hbox_new(false, 0);
  
  // tool tip
  tips = gtk_tooltips_new();
  gtk_tooltips_enable(tips);

  // on-off button
  GdkPixbuf* rightarrow_buf = gdk_pixbuf_new_from_xpm_data(rightarrow_pixmap);
  GdkPixbuf* leftarrow_buf = gdk_pixbuf_new_from_xpm_data(leftarrow_pixmap);

  rightarrow = gtk_image_new_from_pixbuf(rightarrow_buf);
  leftarrow  = gtk_image_new_from_pixbuf(leftarrow_buf);

  show_button = gtk_event_box_new();
  hide_button = gtk_event_box_new();

  gtk_container_add(GTK_CONTAINER(show_button), leftarrow);
  gtk_container_add(GTK_CONTAINER(hide_button), rightarrow);

  gtk_box_pack_start(GTK_BOX(box), hide_button, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(box), show_button, FALSE, FALSE, 0);

  gtk_widget_add_events(show_button, GDK_BUTTON_PRESS_MASK);
  gtk_widget_add_events(hide_button, GDK_BUTTON_PRESS_MASK);

  handlers.connect(show_button, "button-press-event",
		   G_CALLBACK(handle_release), this);
  handlers.connect(hide_button, "button-press-event", 
		   G_CALLBACK(handle_release), this);

  // contents boxes
  histobox = gtk_vbox_new(false, 0);
  bmarkbox = gtk_vbox_new(false, 0);

  // scrolled window
  GtkWidget *histowin = gtk_scrolled_window_new(NULL, NULL);
  GtkWidget *bmarkwin = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(histowin),
				 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(bmarkwin),
				 GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(histowin),histobox);
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(bmarkwin),bmarkbox);

  // additional stuff
  GtkWidget *histonotebox = gtk_vbox_new(false, 0);
  histo_lock = gtk_check_button_new_with_label("Hold");
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(histo_lock), false);
  histo_flush = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
  gtk_box_pack_start(GTK_BOX(histonotebox), histowin, true, true, 0);
  gtk_box_pack_start(GTK_BOX(histonotebox), histo_lock,  false, false, 0);
  gtk_box_pack_start(GTK_BOX(histonotebox), histo_flush, false, false, 0);

  GtkWidget *bmarknotebox = gtk_vbox_new(false, 0);
  bmark_add = gtk_button_new_from_stock(GTK_STOCK_ADD);
  bmark_flush = gtk_button_new_from_stock(GTK_STOCK_CLEAR);
  gtk_box_pack_start(GTK_BOX(bmarknotebox), bmarkwin, true, true, 0);
  gtk_box_pack_start(GTK_BOX(bmarknotebox), bmark_add, false, false, 0);
  gtk_box_pack_start(GTK_BOX(bmarknotebox), bmark_flush, false, false, 0);

  handlers.connect(histo_flush, "clicked", G_CALLBACK(handle_activate), this);
  handlers.connect(bmark_flush, "clicked", G_CALLBACK(handle_activate), this);
  handlers.connect(bmark_add,   "clicked", G_CALLBACK(handle_activate), this);

  // notebook
  notebook = gtk_notebook_new();
  gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_RIGHT);
  gtk_box_pack_start(GTK_BOX(box), notebook, FALSE, FALSE, 0);
  
  GtkWidget *label;
  label = gtk_label_new("History");
  gtk_label_set_angle(GTK_LABEL(label), 90);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), histonotebox, label);

  label = gtk_label_new("Bookmarks");
  gtk_label_set_angle(GTK_LABEL(label), 90);
  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), bmarknotebox, label);

  return box;
}


// ----------------------------------------------------------------------
// core canvas controls

void draw(cairo_t *cr, void *data){
  switch(status.plot_type){
  case PLOT_CHROM_LINKS: 
    draw_chrom_links(cr, data); 
    break;
  case PLOT_DOT:
    draw_dot_plot(cr, data); 
    break;
  case PLOT_SUPERZOOM:
    draw_chrom_links(cr, data);  
    break;
  }
}

bool start_initialize(gpointer useless_pointer){ 
  // this function must be called as an idle function of GTK,
  // to set up the GUI correctly.

  assert(cout << "start_initialize(): called" << endl);
  // load bookmarks
  status.rsb.restore_bmark();

  // prepare to launch sub-thread
  g_mutex_lock(status.itc_mutex);
  status.thread_running = true;
  status.wait_for_disp = true;
  g_mutex_unlock(status.itc_mutex);

  // launch sub-thread
  assert(cout << "start_initialize(): launching sub thread" << endl);
  g_thread_create((GThreadFunc)initialize, NULL, false, NULL);

  // set idle process to display
  gint* tag = (gint*)malloc(sizeof(gint));
  *tag = background_add((GSourceFunc)wait_and_disp, tag);

  return FALSE; // called for only once.
}

gboolean wait_and_disp(gpointer me){
  
  g_mutex_lock(status.itc_mutex);
  bool wait_for_disp = status.wait_for_disp;
  bool thread_running = status.thread_running;
  g_mutex_unlock(status.itc_mutex);

  if(wait_for_disp){
    if(!thread_running){
      if(status.splashwindow != NULL){
	status.progbar = status.splash_progbar_backup;
	gtk_widget_destroy(status.splashwindow);
	gtk_widget_show_all(status.mainwindow);
      }

      // sequence on/off on quickbar
      //      if(status.splash)
      status.lsb.packSeqChecks();

      // display
      assert(cout << "wait_and_disp(): OK, displaying the new figure" <<endl);

      gtk_image_set_from_pixbuf(GTK_IMAGE(canvas), pixbuf);
      handlers.set_sensitive(canvas_ebox, true);
      handlers.unblock();

      if(status.using_temp_pixbuf) g_object_unref(status.temp_pixbuf);
      status.using_temp_pixbuf = false;

      for(vector<string>::iterator i=status.infobar.begin();
 	  i!=status.infobar.end(); i++){
	infobar* ib = new infobar;
	gtk_box_pack_start(GTK_BOX(status.infobarbox), 
			   ib->init(*i), false, false, 0);
      }
      status.infobar.clear();
      gtk_widget_show_all(status.infobarbox);

      gtk_progress_bar_set_fraction((GtkProgressBar*)status.progbar, 0);
      gtk_progress_bar_set_text((GtkProgressBar*)status.progbar, "Idle");
      gtk_widget_hide(status.progbar);

      status.lsb.setByStatus();
      status.lsb.setTypeButton();
      status.rsb.set_visible();

      status.lsb.set_sensitive_for_linkplot(status.plot_type!=PLOT_DOT);

      // set zoomcursor width 
      int chrwidth = status.chrom_right - status.chrom_left;
      int longestbases = mr.maxbases(status.toggle_seq);
      double orig_basewidth = chrwidth/longestbases;
      double bwratio =  orig_basewidth/status.basewidth;

      int zcwidth = (int)(status.basewidth * 
			  status.zoomcursor_width_bp * bwratio);
      if (zcwidth < status.zoomcursor_width_px) zcwidth = status.zoomcursor_width_px;
      status.zcwidth = zcwidth;

      zc.set_yh(10, height()-20);
      zc.set_default_w(zcwidth);
      zc.set_bound( status.chrom_left, status.chrom_right, 
		    10, height()-20 );

      status.ins.setByStatus();
      status.gid.setByStatus();

      // thumbnail bar
      pixbuf_cache_tag t;
      t.get_current_globs();
      status.rsb.add_img(t, pixbuf);
      status.rsb.add_histo(t);

      // gid's highlights again
      if(status.plot_type != PLOT_DOT) status.gid.redraw_hilights();

      if(status.justify_by_anchor){
	if(status.dc_counter == 0){
	  status.js.set_status(status.overwrap_ancs.size(), status.overwrap_all);
	} else {
	  status.js.set_status(status.overwrap_ancs.size(), status.overwrap_all,
				status.dc_counter);
	}
      } else {
	status.js.clear();
      }
      status.justify_by_anchor = false;

      return FALSE; // remove me!
    } else {
      // manage the progress bar
      set_progbar();
    }
  }
  return TRUE;
}

// ----------------------------------------------------------------------
// Justification status indicator

GtkWidget* justificationStatus::init(){
  label = gtk_label_new("");
  clear();
  return label;
}

void justificationStatus::clear(){
  gtk_widget_hide(label);
  gtk_label_set_text(GTK_LABEL(label), "");
}

void justificationStatus::set_status(size_t remain, size_t all){
  gtk_widget_show(label);
  gtk_label_set_text(GTK_LABEL(label),
		     string("Justification mode: showing all anchors (" + itos(int(all)) + ")").c_str());
}

void justificationStatus::set_status(size_t remain, size_t all, int count){
  gtk_widget_show(label);
  gtk_label_set_text(GTK_LABEL(label),
		     string("Justification mode: showing partially (turn #" +
			    itos(count) + 
			    " / Remaining: " +
			    itos((int)remain) + " out of " +
			    itos((int)all) + ") Double-click to continue.").c_str());
  
}

// ----------------------------------------------------------------------
// Infobar

#include "../image/mvwarnicon.h"

GtkWidget* infobar::init(const string& message){
  GdkPixbuf* pixbuf;
  pixbuf = gdk_pixbuf_new_from_inline (-1, mvwarnicon, FALSE, NULL);

  GtkWidget *image = gtk_image_new_from_pixbuf(pixbuf);
  GtkWidget *label = gtk_label_new(message.c_str());
  GtkWidget *clear = gtk_button_new_with_label("OK");

  // gtk_label_set_line_wrap(GTK_LABEL(label), true); 

  box = gtk_hbox_new(false, 0);
  gtk_box_pack_start(GTK_BOX(box), image, false, false, 0);
  gtk_box_pack_start(GTK_BOX(box), label, true, true, 0);
  gtk_box_pack_start(GTK_BOX(box), clear, false, false, 0);

  g_signal_connect(clear, "clicked", 
		   G_CALLBACK(handle_activate), (gpointer)this);

  return box;
}

void infobar::activate(GtkWidget* w){
  delete this;
}

infobar::~infobar(){
  gtk_widget_destroy(box);
}


string plot_type_str(){
  string t;
  switch(status.plot_type){
  case PLOT_CHROM_LINKS: t = "links"; break;
  default:               t = "dot";   break;
  }

  if(status.plot_fwd && !status.plot_bwd) t += "-f";
  if(!status.plot_fwd && status.plot_bwd) t += "-b";

  return t;
}

void fileplot(){
  string PDFname=status.basename+"-"+plot_type_str()+".pdf";
  string PNGname=status.basename+"-"+plot_type_str()+".png";

#ifdef CAIRO_HAS_PDF_SURFACE
  if(status.generate_pdf){
    assert(cout << "writing on " << PDFname << endl);
    generatePDF_subthread((char*)PDFname.c_str());
  }
#endif
  if(status.generate_png){
    assert(cout << "writing on " << PNGname << endl);
    generatePNG_subthread((char*)PNGname.c_str());
  }
}

// ----------------------------------------------------------------------
// progress bar

guint background_add(GSourceFunc f, gpointer d){
  //  return g_idle_add(f, d);
  return g_timeout_add(PROGBAR_UPDATE_INTERVAL, f, d);
}

void set_progbar()
{
  static int prev_fraction;
  double fraction;
  
  fraction = status.prog->fraction();

  if(fraction < 0.01 && prev_fraction!=0) prev_fraction=0;
  if(fraction*1000 < prev_fraction) prev_fraction= 0;

  if (fraction*1000 > prev_fraction+5 || fraction == -1){ 
    gtk_widget_show(status.progbar);
    string t = status.prog->jobtitle();
    if(t != "") 
      gtk_progress_bar_set_text((GtkProgressBar*)status.progbar, t.c_str());
    
    if(fraction != -1)
      gtk_progress_bar_set_fraction((GtkProgressBar*)status.progbar, fraction);
    else 
      gtk_progress_bar_pulse(GTK_PROGRESS_BAR(status.progbar));
    prev_fraction = (int)(fraction*1000);
  }
}

// ----------------------------------------------------------------------
// redrawing

void* redraw_subthread(void* hoge){
  // zc.inactivate(); // this one here causes crash!
  if(status.loadSeqAtRedraw){
    if(!mr.has_sequence_statistics) mr.load_seq_files(true, false);
    status.loadSeqAtRedraw=false;
  }

  if(mr.annotationsChanged){
    opt_read_annotations(&status.annotation_files);
    opt_read_annotations(&status.das_tracks);
    mr.annotationsChanged = false;
  }

  GdkPixbuf* pb = generatePixbuf(&mr);
  assert(cout << "redraw_subthread(): pixbuf generated!" << endl);
  pixbuf = pb;
  
  g_mutex_lock(status.itc_mutex);
  status.thread_running = false;
  g_mutex_unlock(status.itc_mutex);
  return NULL;
}

void redraw_operation(){
  status.gmh.flush(pixbuf); // flush hilight
  status.gid.flush_hilights(); // flush all other hilights

  handlers.block();
  g_mutex_lock(status.itc_mutex);
  status.thread_running = true;
  status.wait_for_disp  = true;
  g_mutex_unlock(status.itc_mutex);

  zc.inactivate(); // moved from redraw_subthread. this is MPsafe!

  // launch sub-thread
  g_thread_create((GThreadFunc)redraw_subthread, NULL, false, NULL);

  // set idle process to display
  gint* tag = (gint*)malloc(sizeof(gint));
  *tag = background_add((GSourceFunc)wait_and_disp, tag);

  zc.force_inside(); 
}

void flush_and_redraw(){
  status.flushAtRedraw = true;
  redraw();
}

void redraw(){
  if(status.flushAtRedraw){
    status.undo_stack.clear();
    status.redo_stack.clear();

    status.rsb.clear_history();
    
    pbcache.clear();
    zoomer->init(status.default_width, status.default_height);

    status.flushAtRedraw=false;
  }

  redraw_operation();

  pixbuf_cache_tag t;
  t.get_current_globs();

  status.redo_stack.clear();
  status.undo_stack.push_back(t);
}

// ----------------------------------------------------------------------
// genetip

void genetip::setup_tip(){
  if(!active){
    window=gtk_window_new(GTK_WINDOW_POPUP);
    label=gtk_label_new("hogehoge");
    gtk_container_add(GTK_CONTAINER(window), label);
    active=true;
    gtk_widget_show_all(window);

  }
  int x, y;
  gdk_display_get_pointer(gdk_display_get_default(), NULL, &x, &y, NULL);
  x += 20;
  gtk_window_move(GTK_WINDOW(window), x, y);
}

void genetip::set_annotation(atIte a){
  setup_tip();
  gtk_label_set_text(GTK_LABEL(label),
		     mr.rat_native(a)->getname(status.name_by).c_str());
}

void genetip::set_score(atIte a){
  setup_tip();
  char score[32];
  char strand[4];
  annotation* aa = mr.rat_native(a);
  if(!aa->reverse) strcpy(strand, "+");
  else strcpy(strand, "-");

  snprintf(score, 255, "%s %f %s", aa->name.c_str(), aa->score, strand);
  
  gtk_label_set_text(GTK_LABEL(label), score);
}

void genetip::clear(){
  if(active){
    gtk_widget_destroy(window);
    active=false;
  }
}

		     
// ----------------------------------------------------------------------
// gene marker highlighter

void gmhilight::focus(GdkPixbuf* p, rectangle r){
  if(active){
    if( (int)(r.p1.x) == x1 && (int)(r.p1.y) == y1 ) return; // same place: already marked
    else clear(); // different place: clear prior to highlighting
  }

  original = p;

  x1 = (int)(r.p1.x);
  y1 = (int)(r.p1.y);
  x2 = (int)(r.p2.x);
  y2 = (int)(r.p2.y);

//   assert(cout << "gmhilight::focus(): focus (" << x1 << "," << y1 << ")-("
// 	 << x2 << "," << y2 << ")" << endl);

  w = abs(x2-x1)+1;
  h = abs(y2-y1)+1;

  // backup
  backup = gdk_pixbuf_new(GDK_COLORSPACE_RGB, false, 8, w, h);
  gdk_pixbuf_copy_area(original, x1, y1, w, h, backup, 0, 0);

  // mark on pixbuf (required for redrawing by zoom cursor etc)
  g_assert(gdk_pixbuf_get_colorspace(original) == GDK_COLORSPACE_RGB);
  g_assert(gdk_pixbuf_get_bits_per_sample(original) == 8);
  g_assert(!gdk_pixbuf_get_has_alpha(original));

  int rowstride = gdk_pixbuf_get_rowstride(original);
  guchar* pixels = gdk_pixbuf_get_pixels(original);

  for(int i=0; i<w-1; i++){
    pixels[rowstride*(y2-2) + (x1+i) *3 + 0] = 255;
    pixels[rowstride*(y2-2) + (x1+i) *3 + 1] = 0;
    pixels[rowstride*(y2-2) + (x1+i) *3 + 2] = 0;
    pixels[rowstride*(y2-1) + (x1+i) *3 + 0] = 255;
    pixels[rowstride*(y2-1) + (x1+i) *3 + 1] = 0;
    pixels[rowstride*(y2-1) + (x1+i) *3 + 2] = 0;
  }

  // and mark (appears on screen directly)
  GdkGC* gc = canvas->style->fg_gc[GTK_STATE_NORMAL];
  GdkColor gc_fore = {0,65535,0,0}; // red 
  gdk_colormap_alloc_color(gdk_colormap_get_system(),
                           &gc_fore, true, true);
  gdk_gc_set_foreground(gc, &gc_fore);
  gdk_draw_rectangle(canvas->window, gc, false,
                     x1, y2-2, w-2, 1);

  GdkRectangle region = { x1, y2-2, w-2, 1 };
  gdk_window_invalidate_rect(canvas->window, &region, false);

  // again, foreground color is black
  gc_fore.pixel = gc_fore.red = gc_fore.green = gc_fore.blue = 0;
  gdk_colormap_alloc_color(gdk_colormap_get_system(), 
			   &gc_fore, true, true);
  gdk_gc_set_foreground(gc, &gc_fore);

  active = true;
}

void gmhilight::clear(){
  if(!active) return;

  //  gdk_pixbuf_copy_area(backup, 0, 0, w, h, original, x1, y1);

  gdk_draw_pixbuf(canvas->window,
		  canvas->style->fg_gc[GTK_STATE_NORMAL],
		  backup, 0, 0, x1, y1, w, h, 
		  GDK_RGB_DITHER_NORMAL, 0, 0 );
  flush();
}

void gmhilight::flush(GdkPixbuf *p){
  if(!active) return;
  
  original = p;
  flush();
}

void gmhilight::flush(){
  if(!active) return;

  gdk_pixbuf_copy_area(backup, 0, 0, w, h, original, x1, y1);
  g_object_unref(backup);
  active = false;
}
