using namespace std;

#ifndef _MAINWINDOW_H_
#define _MAINWINDOW_H_

#define MAINMENU_ZIN  0
#define MAINMENU_ZOUT 1
#define MAINMENU_NORMALSIZE 2

#define MAINMENU_LEFT 0
#define MAINMENU_CENTER 1
#define MAINMENU_RIGHT 2

#include <vector>
#include <cairo.h>
#include <gtk/gtk.h>

#include "gmv_defs.h"
#include "opts.h"
#include "pixbuf_cache.h"
#include "dialog_base.h" // generalWidgets
#include "general_list.h"

// ----------------------------------------------------------------------
// menu bar
void handle_menubar(GtkWidget *w, gpointer useless);

class mainmenu : public generalWidgets {
 public:
  mainmenu(){ isReady=false; autosave=true; };

  void file_open();
  GtkWidget* init(GtkWidget *window, GtkAccelGroup *accel_group);
  void activate(GtkWidget *);
  void release(GtkWidget *w){};

  bool openAnchorsDialog();
  void loadAnnotationDialog();
  void undo();
  void redo();
  void zoom(int);

  void bmark_add();
  void bmark_add(pixbuf_cache_tag, const char*);
  void bmark_clear();

  void setByStatus();

  void set_autosave(bool);
  bool get_autosave();

  bool has_any_modal_dialogs() const;

 private:
  bool hasDialog, isReady, autosave;

  GtkAccelGroup *ag;

  GtkWidget *mainwindow;
  GtkWidget *m_file_open,  *m_file_loadanno, *m_file_loadseq;
  GtkWidget *m_file_png,   *m_file_pdf;
  GtkWidget *m_file_reset, *m_file_savepref, *m_file_autosave, *m_file_quit;
  GtkWidget *m_edit_undo,  *m_edit_redo, *m_edit_sort, *m_edit_msort;
  GtkWidget *m_view_larger, *m_view_smaller, *m_view_normalsize;
  GtkWidget *m_view_type, *m_view_sidebar,   *m_view_thumbbar;
  GtkWidget *m_view_al,   *m_view_ac, *m_view_ar;
  GtkWidget *m_view_zi,   *m_view_zo, *m_view_zn;
  GtkWidget *m_view_gm,   *m_view_hs, *m_view_nm, *m_view_rm, *m_view_or;
  GtkWidget *m_view_gofwd,*m_view_gobwd;
  GtkWidget *m_view_fwd,  *m_view_bwd, *m_view_clip;
  GtkWidget *m_view_stencil, *m_view_pfill;
  GtkWidget *m_view_alpha_const, *m_view_alpha_tfidf;
  GtkWidget *m_view_col_pos, *m_view_col_len, *m_view_col_tfidf;
  GtkWidget *m_view_namm_auto, *m_view_namm_gene, *m_view_namm_locus, *m_view_namm_prod;
  GtkWidget *m_view_namp_auto, *m_view_namp_show, *m_view_namp_hide;
  GtkWidget *m_view_hs_den, *m_view_hs_lden, *m_view_hs_red, *m_view_hs_pat, *m_view_hs_pnt, *m_view_hs_com;
  GtkWidget *m_view_seqticks, *m_view_source, *m_view_range, *m_view_seqno;

  GtkWidget *m_bmark_add, *m_bmark_clear, *m_bmark_save, *m_bmark_separator;
  GtkWidget *m_tool_stat, *m_tool_conf, *m_tool_ins, *m_tool_gid, *m_tool_repd;
  GtkWidget *m_tool_das;
  GtkWidget *m_help_about;

  GtkWidget* add_radio_menu(GtkWidget *menu, GSList **group, const char* label){
    GtkWidget *w = gtk_radio_menu_item_new_with_label(*group, label);
    *group = gtk_radio_menu_item_get_group(GTK_RADIO_MENU_ITEM(w));
    gtk_menu_shell_append(GTK_MENU_SHELL(menu), w);
    return w;
  };

  void set_radio_active(GtkWidget *w, bool active){
    gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(w), active);
  };

  vector< pair<GtkWidget*, pixbuf_cache_tag> > bookmarks;
};

// ----------------------------------------------------------------------
// justfication status indicator

class justificationStatus{ // is not a dialog, but embedded in mainwindow
public:
  void init(GtkWidget *);

  void clear();
  void set_status(size_t remain, size_t all);
  void set_status(size_t remain, size_t all, int count);
  GtkWidget* init();

protected:
  GtkWidget *label;
};

// ----------------------------------------------------------------------
// infobar in mainwindow

class infobar : generalWidgets{
public:
  GtkWidget* init(const string&);
  void activate(GtkWidget*);
  void release(GtkWidget*){};
  ~infobar();
protected:
  GtkWidget *box;
};

// ----------------------------------------------------------------------
// thumbnail bar

class thumb_bar : public generalWidgets {
public:
  thumb_bar(){ visible = true; };
  GtkWidget* init();
  void activate(GtkWidget*);
  void release(GtkWidget*);
  void clear();
  void clear_bmark();
  void clear_history();
  void restore_bmark();
  void dump_bmark();

  void set_visible_silent(bool vis){ visible = vis; };
  void set_visible(bool vis){ set_visible_silent(vis); set_visible(); };
  void set_visible(){ release( visible ? show_button : hide_button ); };
  bool sense_visible() const { return visible; };

  void add_histo(pixbuf_cache_tag);
  void add_bmark(pixbuf_cache_tag, string&);

  void add_img(pixbuf_cache_tag, GdkPixbuf*);

private:
  class tb_buttonbox : public generalWidgets {
  public:
    void init(bool);
    void activate(GtkWidget*);
    void release(GtkWidget*){};
    void set_thumb(GdkPixbuf*, pixbuf_cache_tag);
    void set_tip(GtkTooltips*, const string&);

    string name;
    GdkPixbuf *thumb;
    GtkWidget *box, *tab;
    GtkWidget *thumb_button;
    GtkWidget *dismiss, *bookmark, *protect;
    pixbuf_cache_tag tag;
  };

  bool visible;

  GtkWidget *new_button(const pixbuf_cache_tag, GtkWidget*, 
			const string&, bool);
  void remove_button(GtkWidget*);

  GtkTooltips *tips;
  GtkWidget *box;
  GtkWidget *show_button, *hide_button, *rightarrow, *leftarrow;
  GtkWidget *notebook, *histobox, *bmarkbox;
  GtkWidget *histo_lock, *histo_flush, *bmark_add, *bmark_flush;

  map <GtkWidget*, tb_buttonbox> buttonboxes; // [box] = tb_buttonbox
  map <pixbuf_cache_tag, GdkPixbuf*> thumbnails;
};

// ----------------------------------------------------------------------
// left_sidebar

string plot_type_str();
void fileplot();
 
class left_sidebar : public generalWidgets {
 public:
  left_sidebar(){ ready=false; visible=true; }

  GtkWidget* init();
  void packSeqChecks();
  void activate(GtkWidget *w);
  void release(GtkWidget *w){ activate(w); };
  void setToStatus();
  void setByStatus();
  void setTypeButton();
  void set_sensitive_for_linkplot(gboolean b);
  void togglePlotType();

  bool toggle_visible();
  void set_visible(bool v);
  void set_visible_silent(bool v);
  bool sense_visible(){ return visible; };
  bool sense_schemes_visible() const { return gtk_expander_get_expanded(GTK_EXPANDER(scheme_expander)); };
  bool sense_toggles_visible() const { return gtk_expander_get_expanded(GTK_EXPANDER(toggle_expander)); };

 private:
  class seqItem : public generalListItem<string, bool, int, int> {
  public:
    int number() const { return v; };
    int number(int i){ t = string("Seq #"+itos(i+1)); v = i;  return v; };
    int visible() const { return u; };
    int visible(bool b){ u = b; return u; };
  };

  bool ready;
  bool visible;

  GtkWidget *show_button, *hide_button, *rightarrow, *leftarrow;

  GtkWidget *qbbox;

  GtkWidget *loadseq_button;

  GtkWidget *arrange_frame, *arrange_frame_box;
  GtkWidget *go_fwd, *go_bwd;
  GtkWidget *zoom_out, *zoom_in, *zoom_normal;

  GtkWidget *opt_frame, *opt_frame_box;
  GtkWidget *gm_check;
  GtkWidget *fwd_check, *bwd_check;
  GtkWidget *rep_check, *unknown_check, *orf_check, *hspot_check;
  GtkWidget *stencil_check, *pfill_check;

  GtkWidget *clip_check;
  GtkWidget *redraw_button;
  GtkWidget *seq_expander, *seq_box, *seq_box2, *seq_check[MAXSEQ];
  generalList<seqItem> seqlist;
  std::vector<seqItem> seqlist_original;

  GtkWidget *scheme_expander, *scheme_box;
  GtkWidget *toggle_expander, *toggle_box;

  GtkWidget *type_button;

  vector <string> alpha_method;   GtkWidget *alpha_combo;
  vector <string> color_method;   GtkWidget *color_combo;
  vector <string> name_method;    GtkWidget *name_combo;
  vector <string> name_onoff;     GtkWidget *name_onoff_combo;
  vector <string> hotspot_method; GtkWidget *hotspot_combo;
};

// gene tip
class genetip{
 public:
  void clear();
  void set_annotation(atIte a);
  void set_score(atIte a);

  genetip(){ active = false; };
  //  ~genetip(){ if(active) gtk_widget_destroy(window); };

 private:
  void setup_tip();
  bool active;
  GtkWidget *window;
  GtkWidget *label;
};


// gene marker highlight
class gmhilight{
 public:
  void focus(GdkPixbuf*, rectangle);
  void clear(); // cleanup with screen recovery
  void flush(GdkPixbuf*); 
  void flush(); // cleanup without screen recovery

  gmhilight(){ active = false; }

 private:
  bool active;
  GdkPixbuf* original, *backup;
  int x1, y1, x2, y2, w, h;
};

// ----------------------------------------------------------------------
// core canvas control
void draw(cairo_t *cr, void *data);
bool start_initialize(gpointer useless_pointer); 
gboolean wait_and_disp(gpointer me);

// progress bar
guint background_add(GSourceFunc f, gpointer d);
void set_progbar();

// redrawing
void flush_and_redraw();
void* redraw_subthread(void* hoge);
void redraw_operation();
void redraw();

#endif
