#include "gmv_defs.h"
#include "mainwindow_handlers.h"
#include "gmv_base.h"
#include "gmv.h"
#include "mainwindow.h"
#include "export.h"
#include "messages.h"
#include "dblinks.h"

#include <iostream>
#include <sstream>
#include <set>

#include <glib.h>
#include <glib/gstdio.h>

using namespace std;

gboolean handle_quit(GtkWidget *w, GdkEvent *ev, gpointer useless){
  // to quit by delete/destroy signals on mainwindow, not normal termination.

  status.mainwindow = NULL;
  gtk_main_quit();
  return false;
}


gboolean handle_focusout(GtkWidget *w, GdkEventFocus *ev, gpointer useless){
  // mainwindow lost focus
  status.gt.clear();

  return false;
}


// ----------------------------------------------------------------------
// Plot area cursor control

bool is_superzoomable(int mpx, int mpy){
  for(vector< pair<int,int> >::iterator i=status.clickable_y.begin();
      i!=status.clickable_y.end(); i++){
    if( i->first<=mpy && mpy<=i->second){
      return false;
    }
  }
  return true;
}

int clicked_sequence(int mpx, int mpy){ // geometrycal seq#
  int s= 0;
  for(vector< pair<int,int> >::iterator i=status.clickable_y.begin();
      i!=status.clickable_y.end(); i++){
    s++;
    if( i->first<=mpy && mpy<=i->second)
      break;
  }
  //  cerr << s/2 << endl;
  return s/2;
}

gboolean handle_plotAreaEnter(GtkWidget *w, GdkEventCrossing *ev, 
			      gpointer useless_pointer){

  //  assert(cout << "handle_plotAreaEnter()" << endl);
  zc.enter();
  return FALSE;
}

gboolean handle_plotAreaLeave(GtkWidget *w, GdkEventCrossing *ev, 
			      gpointer useless_pointer){

  //  assert(cout << "handle_plotAreaLeave()" << endl);
  zc.leave();
  return FALSE;
}

gboolean handle_plotAreaMotion(GtkWidget *w, GdkEventMotion *ev, 
			       gpointer useless_pointer){

  if(status.plot_type == PLOT_DOT) return FALSE;

  int mpx = (int)ev->x;
  int mpy = (int)ev->y;

  // sense key code
  bool shift = (( ev->state & GDK_SHIFT_MASK) != 0 );
  
  // check & do various things
  if(!is_superzoomable(mpx, mpy)){
    // non-zoomable zone: gene may exist
    seqlen_t useless;
    gene_markers_type::iterator gm;

    if(check_gene_marker_existence(mpx, mpy, useless, useless, gm)){
      // there's a gene
      int seq = clicked_sequence(mpx, mpy);
      
      int aseq = gm->ai.seq;
      int atrk = gm->ai.track;
      int aidx = gm->ai.no;

      atIte gene = mr.annotations[aseq].at(atrk)->at(aidx);

      // focus & info if the track is traceable
      if(mr.annotations[aseq].at(atrk)->isTraceable()){
	if(shift || status.gid.browserFollowMouse()){
	  status.gid.set_annotation(gene, seq);
	  status.gmh.focus(pixbuf, gm->rect);
	}
      }

      // genetip
      if(!mr.annotations[aseq].at(atrk)->hasScore())
	status.gt.set_annotation(gene);
      else
	status.gt.set_score(gene);
    } else {
      // there's no gene

      status.gt.clear();

      if(status.gid.isActive()){
	if(!status.gid.isBrowserSticky()){
	  if (shift) {
	    status.gmh.clear();
	    status.gid.clearBrowser();
	  }
	}
      } else
	status.gmh.clear();
    }
    
    zc.leave();
  } else {
    // in zoomable zone

    zc.force_inside();
  }
  
  zc.move(mpx, mpy);

  return FALSE;
}

bool check_gene_marker_existence(int mpx, int mpy, 
				 seqlen_t &base_f, seqlen_t &base_t,
				 gene_markers_type::iterator &gm){

  // check gene markers
  for(gene_markers_type::iterator i=status.gene_markers.begin();
      i!=status.gene_markers.end(); i++){
    
    if(i->rect.includes(mpx,mpy)){
      //      cout << "found" << endl;
      base_f = (int)((i->rect.p1.x-status.chrom_left)/
		     status.basewidth + status.superzoom_from);
      base_t = (int)((i->rect.p2.x-status.chrom_left)/
		     status.basewidth + status.superzoom_from);
      gm = i;
      return true;
    }
  }
  
  return false;
}

int check_anchor_existence(int mpx, int mpy){
  int base_f = (int)((mpx-status.chrom_left)/status.basewidth + 
		     status.superzoom_from);
  int base_t = (int)((mpx+1-status.chrom_left)/status.basewidth + 
		     status.superzoom_from);
  int seq  = clicked_sequence(mpx,mpy);
  int offset = mr.get_offset_t(seq);

  base_f -= offset;
  base_t -= offset;
    
  assert(cout << "from=" << base_f << " to=" << base_t << endl);

  int arows = (int)mr.arows();
  int acols = (int)mr.acols();

  int offsets[MAXSEQ];
  for(int s=0; s<acols/3; s++) offsets[s]=0;
    
  // find anchor there
  int overwraps=0;
  for(int i=0; i<arows; i++){
    int af, at;
    af = abs(mr.raa_physical(i, seq*3));
    at = abs(mr.raa_physical(i, seq*3+1));
    if(af>at){ int tmp=af; af=at; at=tmp;  }

    if((af<=base_f && base_f<=at) || (af<=base_t && base_t<=at) ||
       (base_f <= af && at <= base_t)){
      overwraps++;
    }
  }
  return overwraps;
}

void justify_for_anchor(int mpx, int mpy, bool check_genes, bool center, 

			int anc_from, int anc_to){
  // ( anc_from == -1 && anc_to == -1 ) for all, other to align by specific anc

  set<int> *overwraps = &status.overwrap_ancs; // list of overwrapping anchors

  if(mpx==status.last_dc_x && mpy==status.last_dc_y && !overwraps->empty()){
    // second or later time to align here
    status.dc_counter++;
  } else {
    // first time to align here
    overwraps->clear();
    status.last_dc_x = mpx;
    status.last_dc_y = mpy;
    status.dc_counter = 0;
    status.js.clear();
  }

  assert(cout 
	 << "dc_counter=" << status.dc_counter 
	 << " size=" << overwraps->size() << endl);

  seqlen_t base_f = (seqlen_t)((mpx-status.chrom_left)/status.basewidth) + 
		     status.superzoom_from;
  seqlen_t base_t = (seqlen_t)((mpx+1-status.chrom_left)/status.basewidth) + 
		     status.superzoom_from;
  int seq  = clicked_sequence(mpx,mpy);
  seqlen_t offset = mr.get_offset_t(seq);

  assert(cout 
	 << "clicked: x=" << mpx << " y=" << mpy 
	 << " logical=" << base_f << " physical=" << base_f-offset 
	 << " seq #" << clicked_sequence(mpx,mpy) << endl);

  gene_markers_type::iterator useless;

  // modify base_f & base_t, if gene exists
  if(check_genes)
    check_gene_marker_existence(mpx, mpy, base_f, base_t, useless);

  base_f -= offset;
  base_t -= offset;
    
  assert(cout << "from=" << base_f << " to=" << base_t << endl);

  int arows = (int)mr.arows();
  int acols = (int)mr.acols();

  // find anchor there
  int count=0;   // to select specific anchor to justify

  if(overwraps->empty()){
    for(int i=0; i<arows; i++){
      if (!mr.anchorFilter(i)) continue;

      seqlen_t af, at;
      af = abs(mr.raa_physical(i, seq*3));
      at = abs(mr.raa_physical(i, seq*3+1));
      if(af>at){ seqlen_t tmp=af; af=at; at=tmp;  }
    
      if((af<=base_f && base_f<=at) || (af<=base_t && base_t<=at) ||
	 (base_f <= af && at <= base_t)){
	if( (anc_from == -1 && anc_to == -1)  ||
	    (anc_from == -1 && count<=anc_to) ||
	    (anc_from != -1 && anc_from <= count && anc_to == -1) ||
	    (anc_from <= count && count <= anc_to) ){
	  overwraps->insert(i);

	  assert(cout << "base_f=" << base_f << " base_t=" << base_t << endl);
	  assert(cout << "af=" << af << " at=" << at << endl);
	}
	count++;
      }
    }
  }

  if(status.dc_counter == 0)
    status.overwrap_all = status.overwrap_ancs.size();

  seqlen_t offsets[MAXSEQ];
  int offset_count[MAXSEQ];
  for(int s=0; s<acols/3; s++){ offsets[s]=0; offset_count[s]=0; }

  for(set<int>::iterator i=overwraps->begin(); i!=overwraps->end(); i++){
    for(int s=0; s<acols/3; s++){
      seqlen_t af = abs(mr.raa_physical(*i, seq*3));
      seqlen_t at = abs(mr.raa_physical(*i, seq*3+1));

      seqlen_t of = abs(mr.raa_physical(*i, s*3));
      seqlen_t ot = abs(mr.raa_physical(*i, s*3+1));
      
      if(of != ot){ // if not a rift
	offsets[s] += ((af+at)-(of+ot))/2;
	offset_count[s]++;
      }
    }
    status.justify_by_anchor = true;
    if(status.dc_counter != 0) break;   // 1st anchor only if 2nd time or later
  }

  // OK, there are some anchors 
  if(overwraps->size() != 0){
    assert(cout << "overwraps = " << overwraps->size() << endl);

    // adjust offset
    mr.clear_offset();

    for(int s=0; s<acols/3; s++){
      int num_overwraps = (status.dc_counter==0) ? offset_count[s] : 1;
      if(num_overwraps == 0) num_overwraps = 1;  // this happens when 1st anchor has rift

      offsets[s]/=num_overwraps;
      mr.set_offset(status.seq_lookup(s), offsets[s]);
      assert(cout << "offset = " << offsets[s] << endl);
    }

    // keep clicked location
    if(status.plot_type == PLOT_SUPERZOOM){
      seqlen_t new_offset = mr.get_offset_t(seq);
      status.superzoom_from = status.superzoom_from + (new_offset - offset);
      status.superzoom_to   = status.superzoom_to   + (new_offset - offset);
    }

    // center the anchor (if requested)
    if(center){
      if(status.plot_type == PLOT_SUPERZOOM){
	seqlen_t newcenter = (base_f + base_t)/2 + mr.get_offset_t(seq);
	seqlen_t plot_width = status.superzoom_to - status.superzoom_from;
	
	status.superzoom_from = newcenter - plot_width/2;
	status.superzoom_to = newcenter + plot_width/2;
      }
    }
    redraw();
  }
}

void justify_for_anchor(int mpx, int mpy, bool check_genes, bool center){
  justify_for_anchor(mpx, mpy, check_genes, center, -1, -1);
}

gboolean handle_plotAreaPressed(GtkWidget *w, GdkEventButton *ev, 
				gpointer useless_pointer){
  // button press event on canvas: may trigger zoom cursor drugging,
  //                                           or justification

  if(status.plot_type == PLOT_DOT) return FALSE;

  int mpx = (int)ev->x;
  int mpy = (int)ev->y;

#ifdef GDK_WINDOWING_QUARTZ  // right click emulation on quartz: MOD1 = cmd key
  bool command = (( ev->state & GDK_MOD1_MASK) != 0 );
#else
  bool command = false;
#endif

  // single click
  if(ev->button == 1 && !command){ // left click
    if(!is_superzoomable(mpx,mpy)){
      if(status.dc.check(mpx, mpy)){
	// it's doubleclick
	justify_for_anchor(mpx, mpy, true, false);
      } else {
	seqlen_t useless;
	gene_markers_type::iterator gm;
	if(check_gene_marker_existence(mpx, mpy, useless, useless, gm)){
	  status.gmh.clear();
	  int seq = clicked_sequence(mpx, mpy);
	  
	  int aseq = gm->ai.seq;
	  int atrk = gm->ai.track;
	  int aidx  = gm->ai.no;
	  
	  atIte gene = mr.annotations[aseq].at(atrk)->at(aidx);
	  
	  if(mr.annotations[aseq].at(atrk)->isTraceable()){
	    status.gid.init(status.mainwindow);
	    status.gid.set_annotation(gene, seq);
	    status.gmh.focus(pixbuf, gm->rect);
	  }
	}
      }
    } else { // just about cursor
      zc.pressed(mpx, mpy);
    }
  } else { // right or middle click
    if(!is_superzoomable(mpx,mpy)) status.acm.open(w, mpx, mpy);
  }
  
  assert(cout 
	 << "Press @ " << mpx << "," << mpy << endl);

  return FALSE; // call me again!
}

gboolean handle_plotAreaReleased(GtkWidget *w, GdkEventButton *ev, 
				 gpointer useless_pointer){
  // button release event on canvas: may trigger superzoom

  if(status.plot_type == PLOT_DOT) return FALSE;

  int mpx = (int)ev->x;
  int mpy = (int)ev->y;

  if(ev->button == 1){
    if(!is_superzoomable(mpx,mpy)){ zc.force_release(); return FALSE; }

    zc.released(mpx, mpy);

    if(!zc.cursor_inside) return FALSE;

    seqlen_t startbase = (seqlen_t)((double)(zc.x1-status.chrom_left)/status.basewidth);
    seqlen_t endbase   = (seqlen_t)((double)(zc.x2-status.chrom_left)/status.basewidth);

    assert(cout 
	   << "Release @ " << mpx << "," << mpy 
	   << " "<< startbase << "-" << endbase << endl);

    status.plot_type = PLOT_SUPERZOOM;
  
    seqlen_t from = status.superzoom_from;
    startbase += from;
    endbase += from;

    if(startbase != endbase){
      status.superzoom_from = startbase;
      status.superzoom_to = endbase;

      redraw();
    }
  }

  return FALSE;
}

// ----------------------------------------------------------------------
// ACM: Anchor Context Menu

void acm_callback(GtkMenuItem *mi, gpointer acm){
  ((gmv_anchor_context_menu*)acm)->activate(mi);
}

void gmv_anchor_context_menu::activate(GtkMenuItem *mi){
  unsigned int index;

  for(index=0; index<menuItems.size(); index++)
    if(GTK_MENU_ITEM(menuItems[index].widget)==mi) break;

  //  cout << "menu index is: " << index << endl;

  bool check_gene = false;
  bool center = false;

  int type = menuItems[index].type;

  switch(type){
  case ACM_ALIGN_ANCHOR: case ACM_CENTER_ANCHOR:
  case ACM_ALIGN_GENE:   case ACM_CENTER_GENE:
    if(type == ACM_CENTER_ANCHOR || type == ACM_CENTER_GENE) center = true;
    if(type == ACM_ALIGN_GENE || type == ACM_CENTER_GENE) check_gene = true;
    
    justify_for_anchor(mpx, mpy, check_gene, center);
    return;

  case ACM_OPENDB:
    launch_browser(menuItems[index].opt.c_str());
    return;

  case ACM_OPENDB_ALL:
    for(unsigned int i=0; i<menuItems.size(); i++){
      if(menuItems[i].type==ACM_OPENDB)
	launch_browser(menuItems[i].opt.c_str());
    }
    return;
  }

  // overwrapping anchor submenu should be selected
  for(index=0; index<menuItems_anchors.size(); index++){
    if(GTK_MENU_ITEM(menuItems_anchors[index].widget)==mi){
      justify_for_anchor(mpx, mpy, check_gene, center, index, index);
    } 
  }
}

void gmv_anchor_context_menu::open(GtkWidget *w, int x, int y){
  menu = gtk_menu_new();
  overwrap_anchor_menu = gtk_menu_new();
  mpx = x;
  mpy = y;

  menuItems.clear();
  menuItems_anchors.clear();

  int numAnchors=check_anchor_existence(x, y);

  if(numAnchors!=0){
    menuItem newitem;

    if(numAnchors>1){
      newitem.widget = NULL;
      newitem.label  = "Align for...";
      newitem.type   = ACM_OVERWRAP_ANCHORS;
      newitem.opt    = "";
      menuItems.push_back(newitem);

      for(int i=0; i<numAnchors; i++){
	newitem.widget = NULL;
	newitem.label  = "Anchor #" + itos(i);
	newitem.type   = i;
	newitem.opt    = "";
	menuItems_anchors.push_back(newitem);
      }
    }

    newitem.widget = NULL;
    newitem.label  = (numAnchors==1) ? "Align for this anchor" :
      "Align for anchors here";
    newitem.type   = ACM_ALIGN_ANCHOR;
    newitem.opt    = "";
    menuItems.push_back(newitem);

    newitem.label  = (numAnchors==1) ? "Align & center this ancnhor": 
      "Align & center anchors here";
    newitem.type   = ACM_CENTER_ANCHOR;
    menuItems.push_back(newitem);
  }

  assert(cout << "# of overwrapping anchors: " << numAnchors << endl);

  seqlen_t useless;
  gene_markers_type::iterator gm;
  if(check_gene_marker_existence(x, y, useless, useless, gm)){
    if(menuItems.size() != 0){
      menuItem separator = {NULL, "", ACM_SEPARATOR, ""};
      menuItems.push_back(separator);
    }

    menuItem newitem = {NULL, "Align for this gene", ACM_ALIGN_GENE, ""};
    menuItems.push_back(newitem);

    newitem.label = "Align & center this gene";
    newitem.type  = ACM_CENTER_GENE;
    menuItems.push_back(newitem);

    // find the gene
    int aseq = gm->ai.seq;
    int atrk = gm->ai.track;
    int aidx = gm->ai.no;

    atIte gene = mr.annotations[aseq].at(atrk)->at(aidx);

    string locus_tag = mr.rat_native(gene)->locus;
    if(locus_tag != ""){
      menuItem separator = {NULL, "", ACM_SEPARATOR, ""};
      menuItems.push_back(separator);

      char url[1025];
      snprintf(url, 1024, "http://www.genome.ad.jp/dbget-bin/www_bfind_sub?mode=bfind&max_hit=1000&dbkey=kegg&keywords=%s", locus_tag.c_str());
      menuItem newitem = {NULL, "KEGG search by locus tag",
			  ACM_OPENDB, (string)url};
      menuItems.push_back(newitem);

      if(locus_tag.substr(0, 5)=="BSNT_"){
	seqlen_t left, right;
	left = mr.rat_native(gene)->begin;
	right = mr.rat_native(gene)->end;
	if(right<left) swap(left,right);

	snprintf(url, 1024, "http://browser.natto-genome.org:8000/cgi-bin/gb2/gbrowse/chromosome/?name=natto0:%lld..%lld", left, right);
	menuItem newitem = {NULL, "Open Natto Browser",
			    ACM_OPENDB, (string)url};
	menuItems.push_back(newitem);
      }
    }

    set<string> *dbref = &mr.rat_native(gene)->dbref;
    int numOfDBLinks=0;
    for(set<string>::iterator i=dbref->begin(); i!=dbref->end(); i++){
      //      cout << "dbref=" << *i << endl;
      dblink* d=dblinks;
      string dbname, accession;
      char url[1025];
      while(d->dbname != NULL){
	if(i->compare(0, strlen(d->dbtag), (string)d->dbtag) == 0){
	  int pos = i->find(":");
	  dbname = (string)d->dbname;
	  accession = i->substr(pos+1, i->length()-pos-1);
	  snprintf(url, 1024, d->url, accession.c_str());
	}
	d++;
      }
      if( dbname!="" && accession!="" ){
	if(numOfDBLinks == 0){
	  menuItem separator = {NULL, "", ACM_SEPARATOR, ""};
	  menuItems.push_back(separator);
	}

	menuItem newitem = {NULL, 
			    "Open " + dbname + "(" + accession + ")",
			    ACM_OPENDB, (string)url};
	menuItems.push_back(newitem);
	numOfDBLinks++;
      }
    }

    if(numOfDBLinks >= 2){
      menuItem newitem = {NULL, "", ACM_SEPARATOR, ""};
      menuItems.push_back(newitem);

      newitem.label = "Open all DB entries above";
      newitem.type  = ACM_OPENDB_ALL;
      menuItems.push_back(newitem);
    }
  }

  if(menuItems.size()==0) return;

  for(vector<menuItem>::iterator i=menuItems.begin(); 
      i!=menuItems.end(); i++){
    GtkWidget *mi;
    if(i->type != ACM_SEPARATOR){
      mi=gtk_menu_item_new_with_label(i->label.c_str());
      i->widget=mi;
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);

      if(i->type == ACM_OVERWRAP_ANCHORS){
	gtk_menu_item_set_submenu(GTK_MENU_ITEM(mi), overwrap_anchor_menu);

	for(vector<menuItem>::iterator j=menuItems_anchors.begin(); 
	    j!=menuItems_anchors.end(); j++){
	  mi=gtk_menu_item_new_with_label(j->label.c_str());
	  j->widget=mi;
	  gtk_menu_shell_append(GTK_MENU_SHELL(overwrap_anchor_menu), mi);
	  g_signal_connect(mi, "activate", G_CALLBACK(acm_callback), this);
	}
	
      } else {
	g_signal_connect(mi, "activate", G_CALLBACK(acm_callback), this);
      }
    } else {
      mi=gtk_separator_menu_item_new();
      gtk_menu_shell_append(GTK_MENU_SHELL(menu), mi);
    }
  }
   
  gtk_widget_show_all(menu);
    
  gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL, 3, 0);

  //     gtk_container_add(GTK_CONTAINER(window), menu);
  //     gtk_widget_show_all(menu);

  //     // display that
  //     int x_org, y_org;

  //     gtk_window_set_screen(GTK_WINDOW(window), screen);
  //     gdk_window_get_origin(w->window, &x_org, &y_org);
    
  //     gtk_window_move(GTK_WINDOW(window), x_org+x, y_org+y);
  //     gtk_widget_show_all(window);
};


