#ifndef _MAINWINDOW_HANDLERS_H_
#define _MAINWINDOW_HANDLERS_H_

#define MZ_DIALOG 0
#define MZ_FROMBOX 1
#define MZ_TOBOX 2
#define MZ_RANGEBOX 3
#define MZ_FROMSCALE 4
#define MZ_RANGESCALE 5
#define MZ_RANGELOCK 6
#define MZ_TOSCALE 7
#define MZ_AUTOAPPLY 8
#define MZ_STEP_1 9
#define MZ_STEP_2 10 
#define MZ_STEP_4 11 
#define MZ_STEP_8 12
#define MZ_STEP_16 13

#define ACM_ALIGN_ANCHOR  1
#define ACM_CENTER_ANCHOR 2
#define ACM_ALIGN_GENE    3
#define ACM_CENTER_GENE   4
#define ACM_OPENDB        5
#define ACM_OPENDB_ALL    6
#define ACM_SEPARATOR     7
#define ACM_OVERWRAP_ANCHORS 8

using namespace std;

#include <gtk/gtk.h>
#include <string>
#include <vector>
#include "gmv_defs.h"

// exit
gboolean handle_quit(GtkWidget*, GdkEvent*, gpointer);
gboolean handle_focusout(GtkWidget *w, GdkEventFocus *ev, gpointer useless);

// plotarea
gboolean handle_plotAreaEnter(GtkWidget*, GdkEventCrossing*, gpointer);
gboolean handle_plotAreaLeave(GtkWidget*, GdkEventCrossing*, gpointer);
gboolean handle_plotAreaMotion(GtkWidget*, GdkEventMotion*, gpointer);
gboolean handle_plotAreaPressed(GtkWidget*, GdkEventButton*, gpointer);
gboolean handle_plotAreaReleased(GtkWidget*, GdkEventButton*, gpointer);

bool is_clickable(int mpx, int mpy);
bool check_gene_marker_existence(int mpx, int mpy, 
				 seqlen_t &base_f, seqlen_t &base_t,
				 gene_markers_type::iterator &);

// doubleclick checker
class doubleclicker{
private:
  GTimer* timer;
  int clicked_x, clicked_y;

public:
  void init(){
    timer = g_timer_new();
    g_timer_start(timer);
    
    clicked_x = 0;
    clicked_y = 0;
  };

  bool check(int x, int y){
    bool ret = ( clicked_x == x && clicked_y == y && 
		 g_timer_elapsed(timer, NULL) < 0.3 );

    clicked_x = x;
    clicked_y = y;
    g_timer_reset(timer);

    return ret;
  };
};


// ACM
class gmv_anchor_context_menu{
  struct menuItem{
    GtkWidget *widget;
    string label;
    int type;
    string opt;
  };

 private:
  GtkWidget *menu;
  GtkWidget *overwrap_anchor_menu;
  vector< menuItem > menuItems, menuItems_anchors;
  int mpx, mpy;

 public:
  void open(GtkWidget *w, int x, int y);
  void activate(GtkMenuItem *mi);
};


// zoom util
class logzoomer{
 public:
  seqlen_t maxval;
  int steps;
  
  int init(seqlen_t m){
    maxval = m;
    seqlen_t a = 1;
    seqlen_t i, j;

    for(i=0; a*10<m; i++) a*=10;

    if(a*2 >= m) j=1;
    else if(a*5 >= m) j=2;
    else j=3;

    steps=i*3+j+1;
    return steps;
  }

  seqlen_t size(int s){  // step -> size
    seqlen_t j=1;
    for(int i=0; i<s/3; i++) j*=10;
    if(s%3 == 1) j*=2;
    if(s%3 == 2) j*=5;

    if(j>maxval) j=maxval;

    return j;
  }

  int step(seqlen_t s){  // size -> (nearest) step 
    for(int i=0; i<steps; i++)
      if(size(i) >= s) return i;

    return 0;
    // FIXME: this can't find really nearest one!
  }
}; 

#endif
