#include <glib/gstdio.h>  // for gunlink()

#include "mainwindow.h"
#include "gmv.h"
#include "export.h"

#define menuconnect(w)  handlers.connect(w, "activate", G_CALLBACK(handle_activate),  this);


// ------------------------------------------------------------
// Main Menubar

bool mainmenu::has_any_modal_dialogs() const {
  if(!isReady) return false; else return hasDialog; 
}

void mainmenu::setByStatus(){
  handlers.block();

  set_radio_active(m_view_sidebar,  status.lsb.sense_visible());
  set_radio_active(m_view_thumbbar, status.rsb.sense_visible());

  set_radio_active(m_view_gm, status.show_gene_marker);
  set_radio_active(m_view_hs, status.show_hotspots);
  set_radio_active(m_view_nm, status.plot_U);
  set_radio_active(m_view_rm, status.plot_R);
  set_radio_active(m_view_or, status.plot_O);

  set_radio_active(m_view_fwd, status.plot_fwd);
  set_radio_active(m_view_bwd, status.plot_bwd);
  set_radio_active(m_view_clip, status.clip_hard);
  set_radio_active(m_view_stencil, status.rift_stencil);
  set_radio_active(m_view_pfill, status.rift_pattern_fill);

  set_radio_active(m_view_range,    status.disp_range);
  set_radio_active(m_view_seqno,    status.disp_seqno);
  set_radio_active(m_view_seqticks, status.disp_seqticks);
  set_radio_active(m_view_source,   status.disp_source);

  set_radio_active((status.alpha_by == ALPHA_BY_CONST) ? m_view_alpha_const : 
		   m_view_alpha_tfidf, true);

  set_radio_active((status.color_by == COLOR_BY_TFIDF)     ? m_view_col_tfidf :
		   (status.color_by == COLOR_BY_ANCHORLEN) ? m_view_col_len :
		   m_view_col_pos, true );

  set_radio_active((status.name_by == NAME_BY_PRODUCT) ? m_view_namm_prod :
		   (status.name_by == NAME_BY_GNAME)   ? m_view_namm_gene :
		   (status.name_by == NAME_BY_LOCUS)   ? m_view_namm_locus :
		   m_view_namm_auto, true );
  
  set_radio_active((status.gene_name_onoff_method==DISP_NAME_OFF) ? m_view_namp_hide:
		   (status.gene_name_onoff_method==DISP_NAME_ON)  ? m_view_namp_show:
		   m_view_namp_auto, true);

  set_radio_active((status.hotspot_by == HOTSPOT_BY_COMBINATION) ? m_view_hs_com :
		   (status.hotspot_by == HOTSPOT_BY_DENSITY_LOG) ? m_view_hs_lden :
		   (status.hotspot_by == HOTSPOT_BY_RED) ? m_view_hs_red :
		   (status.hotspot_by == HOTSPOT_BY_PATTERN) ? m_view_hs_pat :
		   (status.hotspot_by == HOTSPOT_BY_NUMBER) ? m_view_hs_pnt :
		   m_view_hs_den, true);

  handlers.set_sensitive(m_view_stencil, status.plot_type != PLOT_DOT);
  handlers.set_sensitive(m_view_pfill,   status.plot_type != PLOT_DOT);

  handlers.set_sensitive(m_file_loadseq, !mr.has_sequence_statistics);
  handlers.set_sensitive(m_file_loadseq, !mr.has_sequence_statistics);

  handlers.set_sensitive(m_view_gofwd, status.plot_type == PLOT_SUPERZOOM);
  handlers.set_sensitive(m_view_gobwd, status.plot_type == PLOT_SUPERZOOM);

  handlers.set_sensitive(m_view_hs, mr.has_hotspots());
  handlers.set_sensitive(m_view_nm, mr.has_sequence_statistics);
  handlers.set_sensitive(m_view_rm, mr.has_sequence_statistics);
  handlers.set_sensitive(m_view_or, mr.has_sequence_statistics);

  handlers.unblock();
}

void mainmenu::set_autosave(bool a){
  if(!isReady)
    autosave = a;
  else
    set_radio_active(m_file_autosave, a);
}

bool mainmenu::get_autosave(){
  return gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_file_autosave));
}

void mainmenu::file_open(){
  g_signal_emit_by_name((gpointer)m_file_open, "activate");
}


void mainmenu::activate(GtkWidget *w){
  // file
  if (w == m_file_open) openAnchorsDialog();

  if (w == m_file_loadanno) loadAnnotationDialog();

  if (w == m_file_loadseq){
    if(confirm_to_flush()){
      status.loadSeqAtRedraw = true;
      flush_and_redraw();
    }
  }

#ifdef CAIRO_HAS_PDF_SURFACE
  if (w == m_file_pdf ) handle_FileSavePDF(w, NULL);
#endif
  if (w == m_file_png ) handle_FileSavePNG(w, &mr);
  if (w == m_file_savepref){
    status.save_prefs();
    GtkWidget *infodialog = gtk_message_dialog_new(GTK_WINDOW(status.mainwindow),
						   GTK_DIALOG_DESTROY_WITH_PARENT,
						   GTK_MESSAGE_INFO,
						   GTK_BUTTONS_OK,
						   "Preferences have been saved.");
    hasDialog = true;
    gtk_dialog_run(GTK_DIALOG(infodialog));
    gtk_widget_destroy(infodialog);
    hasDialog = false;
  }

  if (w == m_file_reset){
    GtkWidget *confirm = gtk_message_dialog_new(GTK_WINDOW(status.mainwindow),
						GTK_DIALOG_DESTROY_WITH_PARENT,
						GTK_MESSAGE_QUESTION,
						GTK_BUTTONS_OK_CANCEL,
						"Reset saved preferences and exit.\n You have to launch GMV again.");
    hasDialog = true;
    if(gtk_dialog_run(GTK_DIALOG(confirm)) == GTK_RESPONSE_OK){
      g_unlink(preffile().c_str());
      status.mainwindow = NULL;
      gtk_main_quit();
    }
    gtk_widget_destroy(confirm);
    hasDialog = false;
  }

  if (w == m_file_quit){
    status.rsb.dump_bmark();
    gtk_main_quit();
  }

  // edit
  if (w == m_edit_undo)  undo();
  if (w == m_edit_redo)  redo();
  if (w == m_edit_sort){
    source_sort_dialog ssd;
    hasDialog = true;
    ssd.init(status.mainwindow);
    hasDialog = false;
  }

  if (w == m_edit_msort){
    manual_source_sort_dialog mssd;
    hasDialog = true;
    mssd.init(status.mainwindow);
    hasDialog = false;
  }

  // bookmark
  if (w == m_bmark_add)   bmark_add();
  if (w == m_bmark_clear) bmark_clear();
  if (w == m_bmark_save)  status.rsb.dump_bmark();

  // view
  if (w == m_view_sidebar){  
    status.lsb.set_visible(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(w))); setByStatus(); }

  if (w == m_view_thumbbar)
    status.rsb.set_visible(gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(w)));

  if (w == m_view_larger )  zoom(MAINMENU_ZIN);
  if (w == m_view_smaller)  zoom(MAINMENU_ZOUT);
  if (w == m_view_normalsize) zoom(MAINMENU_NORMALSIZE);

  bool *b = NULL;
  if(w == m_view_gm) b = &status.show_gene_marker;
  if(w == m_view_hs) b = &status.show_hotspots;
  if(w == m_view_nm) b = &status.plot_U;
  if(w == m_view_rm) b = &status.plot_R;
  if(w == m_view_or) b = &status.plot_O;
  if(w == m_view_range)    b = &status.disp_range;
  if(w == m_view_seqno)    b = &status.disp_seqno;
  if(w == m_view_seqticks) b = &status.disp_seqticks;
  if(w == m_view_source)   b = &status.disp_source;
  if(w == m_view_fwd)  b = &status.plot_fwd;
  if(w == m_view_bwd)  b = &status.plot_bwd;
  if(w == m_view_clip) b = &status.clip_hard;
  if(w == m_view_stencil) b = &status.rift_stencil;
  if(w == m_view_pfill)   b = &status.rift_pattern_fill;

  if( b != NULL){
    *b = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(w));
    redraw();
  }

  if (w == m_view_al  ){ mr.justify(JUSTIFY_LEFT);   redraw(); }
  if (w == m_view_ac  ){ mr.justify(JUSTIFY_CENTER); redraw(); }
  if (w == m_view_ar  ){ mr.justify(JUSTIFY_RIGHT);  redraw(); }
  if (w == m_view_type)  status.lsb.togglePlotType();

  if (w == m_view_gofwd){ status.go_fwd(); redraw(); }
  if (w == m_view_gobwd){ status.go_bwd(); redraw(); }
  if (w == m_view_zi) { status.zoom_in(); redraw(); }
  if (w == m_view_zo) { status.zoom_out(); redraw(); }
  if (w == m_view_zn) { status.zoom_normal(); redraw(); }


  if (w == m_view_alpha_const || w == m_view_alpha_tfidf){
    status.alpha_by = 
      (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_alpha_tfidf)) 
       && !mr.tfidf.empty()) ?
      ALPHA_BY_TFIDF : ALPHA_BY_CONST;
    redraw();
  } 

  if (w == m_view_col_pos || w == m_view_col_len || w == m_view_col_tfidf ){
    status.color_by = 
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_col_pos)) ?
      COLOR_BY_POSITION :
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_col_len)) ?
      COLOR_BY_ANCHORLEN : COLOR_BY_TFIDF;
    if(status.color_by == COLOR_BY_TFIDF && mr.tfidf.empty())
      status.color_by = COLOR_BY_POSITION;
    redraw();
  }

  if (w == m_view_namm_auto || w == m_view_namm_gene ||
      w == m_view_namm_locus || w == m_view_namm_prod ){
    status.name_by = 
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_namm_auto)) ?
      NAME_AUTO :
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_namm_gene)) ?
      NAME_BY_GNAME :
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_namm_locus)) ?
      NAME_BY_LOCUS : NAME_BY_PRODUCT;
    redraw();
  }

  if (w == m_view_namp_auto || w == m_view_namp_show || w == m_view_namp_hide ){
    status.gene_name_onoff_method = 
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_namp_auto)) ?
      DISP_NAME_AUTO :
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_namp_show)) ?
      DISP_NAME_ON : DISP_NAME_OFF;
    redraw();
  }

  if (w == m_view_hs_den || w == m_view_hs_lden || w == m_view_hs_red ||
      w == m_view_hs_pat || w == m_view_hs_pnt  || w == m_view_hs_com){
    status.hotspot_by = 
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_hs_den)) ?
      HOTSPOT_BY_DENSITY :
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_hs_lden)) ?
      HOTSPOT_BY_DENSITY_LOG :
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_hs_red)) ?
      HOTSPOT_BY_RED :
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_hs_pat)) ?
      HOTSPOT_BY_PATTERN :
      gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(m_view_hs_pnt)) ?
      HOTSPOT_BY_NUMBER : HOTSPOT_BY_COMBINATION;
    redraw();
  }

  // tool
  if (w == m_tool_stat)  status.statd.init(mainwindow);
  if (w == m_tool_conf)  status.prefd.init(mainwindow);
  if (w == m_tool_ins)   status.ins.init(mainwindow);
  if (w == m_tool_gid)   status.gid.init(mainwindow);
  if (w == m_tool_repd)  status.repd.init(mainwindow);
#ifndef OLD_GTK_COMPAT
  if (w == m_tool_das){ 
    status.dasd.set_server_list( &status.das_server_list );
    status.dasd.init(mainwindow); }
#endif

  // help
  if (w == m_help_about){
    hasDialog = true;
    aboutDialog(mainwindow);
    hasDialog = false;
  }

  // other: may be bookmark
  for(unsigned i=0; i<bookmarks.size(); i++){
    if(w == bookmarks[i].first){
      bookmarks[i].second.bookmark_to_status();
      status.lsb.setByStatus();
      redraw();
      break;
    }
  }


}

void mainmenu::bmark_add(){
  pixbuf_cache_tag t;
  t.get_current_globs();
  bmark_add(t, ""); 
}

void mainmenu::bmark_add(pixbuf_cache_tag t, const char* n){
  GtkWidget *d;
  string name = string(n);

  if( name == "" ){
    d = gtk_dialog_new_with_buttons ("Add Bookmark",
				     GTK_WINDOW(status.mainwindow),
				     GTK_DIALOG_DESTROY_WITH_PARENT,
				     GTK_STOCK_OK,
				     GTK_RESPONSE_OK,
				     GTK_STOCK_CANCEL,
				     GTK_RESPONSE_CANCEL,
				     NULL);

    gtk_window_set_modal(GTK_WINDOW(d), true);
    gtk_window_set_transient_for(GTK_WINDOW(d), GTK_WINDOW(status.mainwindow));
   
    GtkWidget *l = gtk_label_new("Name:");
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->vbox), l, false, false, 0);
    gtk_widget_show(l);

    GtkWidget *e = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(e), 
		       string("Bookmark " + itos((int)bookmarks.size())).c_str());

    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(d)->vbox), e, false, false, 0);
    gtk_widget_show(e);

    hasDialog = true;
    if(gtk_dialog_run(GTK_DIALOG(d)) == GTK_RESPONSE_OK)
      name = string(gtk_entry_get_text(GTK_ENTRY(e)));
    gtk_widget_destroy(d);
    hasDialog = false;
  }

  if ( name != "" ){
    // add separator if bookmark list is empty
    if(bookmarks.size()==0){
      m_bmark_separator = gtk_separator_menu_item_new();
      gtk_menu_shell_append(GTK_MENU_SHELL(status.bmark_menu), 
			    m_bmark_separator);
      gtk_widget_show(m_bmark_separator);
    }

    // add bookmark
    GtkWidget *m = gtk_menu_item_new_with_label(name.c_str());
    gtk_menu_shell_append(GTK_MENU_SHELL(status.bmark_menu), m);
    gtk_widget_show(m);

    bookmarks.push_back(pair<GtkWidget*, pixbuf_cache_tag>(m, t));
    status.rsb.add_bmark(t, name);
   
    g_signal_connect(m, "activate", G_CALLBACK(handle_activate), this);
    handlers.connect(m, HANDLER_BMARK);

    // and keyboard accelerator
    if(bookmarks.size()<10)
      gtk_widget_add_accelerator(m, "activate", ag,
				 0x30 + bookmarks.size()-1,
				 GdkModifierType(GDK_SHIFT_MASK|GDK_CONTROL_MASK), GTK_ACCEL_VISIBLE);
  }
}

void mainmenu::bmark_clear(){
  handlers.remove_dialog(HANDLER_BMARK);
  if(bookmarks.size() != 0) gtk_widget_destroy(m_bmark_separator);
  for(unsigned i=0; i<bookmarks.size(); i++)
    gtk_widget_destroy(bookmarks[i].first);

  status.rsb.clear_bmark();
  bookmarks.clear();
}

GtkWidget* mainmenu::init(GtkWidget *window, GtkAccelGroup *accel_group){
  hasDialog = false;
  isReady = true;
  mainwindow = window;

  // --------------------
  // file 
  GtkWidget *file_menu, *file_item;
  ag = accel_group;

  file_menu = gtk_menu_new();
    
  // ----------
  m_file_open = gtk_menu_item_new_with_mnemonic("_Open Anchor File");
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_file_open);


  m_file_loadanno = gtk_menu_item_new_with_mnemonic("Load _Annotation File");
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_file_loadanno);

  m_file_loadseq = gtk_menu_item_new_with_label("Load sequences");
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_file_loadseq);

  // ----------
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu),
			gtk_separator_menu_item_new());

  m_file_png  = gtk_menu_item_new_with_mnemonic("Save P_NG");
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_file_png);

#ifdef CAIRO_HAS_PDF_SURFACE
  m_file_pdf  = gtk_menu_item_new_with_mnemonic("Save P_DF");
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_file_pdf);
#endif

  // ----------
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu),
			gtk_separator_menu_item_new());

  m_file_savepref = gtk_menu_item_new_with_mnemonic("Save P_references");
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_file_savepref);

  m_file_autosave = gtk_check_menu_item_new_with_label("Auto-save preferences");
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_file_autosave);
  set_autosave(autosave);

  m_file_reset = gtk_menu_item_new_with_mnemonic("_Reset GMV");
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_file_reset);

  // ----------
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu),
			gtk_separator_menu_item_new());

  m_file_quit = gtk_menu_item_new_with_mnemonic("_Quit");
  gtk_menu_shell_append(GTK_MENU_SHELL(file_menu), m_file_quit);

  // ----------

  file_item = gtk_menu_item_new_with_mnemonic("_File");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_item), file_menu);

  gtk_widget_add_accelerator(m_file_open, "activate", accel_group,
			     'o', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_file_loadanno, "activate", accel_group,
			     'a', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_file_quit, "activate", accel_group,
			     'q', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  // --------------------
  // edit
  GtkWidget *edit_menu, *edit_item;

  edit_menu = gtk_menu_new();

  m_edit_undo  = gtk_menu_item_new_with_mnemonic("_Undo");
  m_edit_redo  = gtk_menu_item_new_with_mnemonic("_Redo");
  m_edit_sort  = gtk_menu_item_new_with_mnemonic("_Sort segments/chromosomes by anchor");
  m_edit_msort = gtk_menu_item_new_with_mnemonic("Sort segments/chromosomes by _manual");

  gtk_menu_shell_append(GTK_MENU_SHELL(edit_menu), m_edit_undo);
  gtk_menu_shell_append(GTK_MENU_SHELL(edit_menu), m_edit_redo);
  gtk_menu_shell_append(GTK_MENU_SHELL(edit_menu), gtk_separator_menu_item_new());
  gtk_menu_shell_append(GTK_MENU_SHELL(edit_menu), m_edit_sort);
  gtk_menu_shell_append(GTK_MENU_SHELL(edit_menu), m_edit_msort);

  edit_item = gtk_menu_item_new_with_mnemonic("_Edit");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(edit_item), edit_menu);

  gtk_widget_add_accelerator(m_edit_undo, "activate", accel_group,
			     'z', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_edit_redo, "activate", accel_group,
			     'y', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
    
  // view
  GtkWidget *view_menu, *view_item;

  view_menu = gtk_menu_new();
    
  m_view_sidebar = gtk_check_menu_item_new_with_label("Sidebar");
  m_view_thumbbar = gtk_check_menu_item_new_with_label("Thumbnail bar");

  m_view_type = gtk_menu_item_new_with_mnemonic("_Toggle plot type");

  m_view_al   = gtk_menu_item_new_with_mnemonic("Justify _Left");
  m_view_ar   = gtk_menu_item_new_with_mnemonic("Justify _Right");
  m_view_ac   = gtk_menu_item_new_with_mnemonic("Justify _Center");

  m_view_gofwd  = gtk_menu_item_new_with_mnemonic("Move _Forward");
  m_view_gobwd  = gtk_menu_item_new_with_mnemonic("Move _Backward");

  m_view_zi   = gtk_menu_item_new_with_mnemonic("_Zoom in");
  m_view_zo   = gtk_menu_item_new_with_mnemonic("Zoom ou_t");
  m_view_zn   = gtk_menu_item_new_with_mnemonic("Zoom out _full");

  m_view_gm   = gtk_check_menu_item_new_with_label("Show gene markers");
  m_view_hs   = gtk_check_menu_item_new_with_label("Show repeat (from murasaki) markers");
  m_view_nm   = gtk_check_menu_item_new_with_label("Show N masks on sequence");
  m_view_rm   = gtk_check_menu_item_new_with_label("Repeat (from sequence) masks on sequence");
  m_view_or   = gtk_check_menu_item_new_with_label("Show ORFs");

  // color menus
  GtkWidget *alpha_menu = gtk_menu_new();
  GtkWidget *alpha_item = gtk_menu_item_new_with_label("Anchor transparency");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(alpha_item), alpha_menu);

  GSList *rg = NULL;
  m_view_alpha_const = add_radio_menu(alpha_menu, &rg, "Constant");
  m_view_alpha_tfidf = add_radio_menu(alpha_menu, &rg, "tf-idf score"); 

  GtkWidget *color_menu = gtk_menu_new();
  GtkWidget *color_item = gtk_menu_item_new_with_label("Anchor color");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(color_item), color_menu);

  rg = NULL;
  m_view_col_pos   = add_radio_menu(color_menu, &rg, "Position on seq #1");
  m_view_col_len   = add_radio_menu(color_menu, &rg, "Anchor length");
  m_view_col_tfidf = add_radio_menu(color_menu, &rg, "tf-idf score");

  GtkWidget *naming_menu = gtk_menu_new();
  GtkWidget *naming_item = gtk_menu_item_new_with_label("Annotation name policy");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(naming_item), naming_menu);

  rg = NULL;
  m_view_namm_auto  = add_radio_menu(naming_menu, &rg, "Automatic");
  m_view_namm_gene  = add_radio_menu(naming_menu, &rg, "Gene name");
  m_view_namm_locus = add_radio_menu(naming_menu, &rg, "Locus tag");
  m_view_namm_prod  = add_radio_menu(naming_menu, &rg, "Product");

  GtkWidget *namedisp_menu = gtk_menu_new();
  GtkWidget *namedisp_item = gtk_menu_item_new_with_label("Annotation name display policy");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(namedisp_item), namedisp_menu);
  rg = NULL;
  m_view_namp_auto = add_radio_menu(namedisp_menu, &rg, "Automatic");
  m_view_namp_show = add_radio_menu(namedisp_menu, &rg, "Show always");
  m_view_namp_hide = add_radio_menu(namedisp_menu, &rg, "Hide always");

  GtkWidget *hotspot_menu = gtk_menu_new();
  GtkWidget *hotspot_item = gtk_menu_item_new_with_label("Repeat (mergefilter) display");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(hotspot_item), hotspot_menu);
  rg = NULL;
  m_view_hs_den  = add_radio_menu(hotspot_menu, &rg, "Density");
  m_view_hs_lden = add_radio_menu(hotspot_menu, &rg, "log(Density)");
  m_view_hs_red  = add_radio_menu(hotspot_menu, &rg, "Red markers");
  m_view_hs_pat  = add_radio_menu(hotspot_menu, &rg, "Color for each pattern");
  m_view_hs_pnt  = add_radio_menu(hotspot_menu, &rg, "Color by copy # of the repeat");
  m_view_hs_com  = add_radio_menu(hotspot_menu, &rg, "Color by combination of the repeat");

  // rift menu
  GtkWidget *rift_menu = gtk_menu_new();
  GtkWidget *rift_item = gtk_menu_item_new_with_label("Rift display options");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(rift_item), rift_menu);

  m_view_stencil = gtk_check_menu_item_new_with_label("Stencil at rifts");
  m_view_pfill   = gtk_check_menu_item_new_with_label("Do pattern fill on anchors with rifts");
  gtk_menu_shell_append(GTK_MENU_SHELL(rift_menu), m_view_stencil);
  gtk_menu_shell_append(GTK_MENU_SHELL(rift_menu), m_view_pfill);

  // plot size & adv options
  GtkWidget *size_menu = gtk_menu_new();
  GtkWidget *size_item = gtk_menu_item_new_with_label("Plot size");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(size_item), size_menu);
  m_view_larger  = gtk_menu_item_new_with_mnemonic("_Enlarge (+10%)");
  m_view_smaller = gtk_menu_item_new_with_mnemonic("Shr_ink (-10%)");
  m_view_normalsize = gtk_menu_item_new_with_mnemonic("N_ormal (100%)");

  gtk_menu_shell_append(GTK_MENU_SHELL(size_menu), m_view_larger);
  gtk_menu_shell_append(GTK_MENU_SHELL(size_menu), m_view_smaller);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), gtk_separator_menu_item_new());
  gtk_menu_shell_append(GTK_MENU_SHELL(size_menu), m_view_normalsize);

  GtkWidget *adv_menu = gtk_menu_new();
  GtkWidget *adv_item = gtk_menu_item_new_with_label("Advanced options");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(adv_item), adv_menu);
  m_view_clip = gtk_check_menu_item_new_with_label("Strict clipping");
  m_view_fwd  = gtk_check_menu_item_new_with_label("Show forward anchors");
  m_view_bwd  = gtk_check_menu_item_new_with_label("Show backward anchors");
  m_view_range = gtk_check_menu_item_new_with_label("Show bp range in plot");
  m_view_seqno = gtk_check_menu_item_new_with_label("Show sequence #");

  m_view_seqticks = gtk_check_menu_item_new_with_label("Show small bp ticks on each sequence");
  m_view_source = gtk_check_menu_item_new_with_label("Show sequence source (below the sequence)");
  gtk_menu_shell_append(GTK_MENU_SHELL(adv_menu), m_view_clip);
  gtk_menu_shell_append(GTK_MENU_SHELL(adv_menu), m_view_fwd);
  gtk_menu_shell_append(GTK_MENU_SHELL(adv_menu), m_view_bwd);
  gtk_menu_shell_append(GTK_MENU_SHELL(adv_menu), m_view_range);
  gtk_menu_shell_append(GTK_MENU_SHELL(adv_menu), m_view_seqno);
  gtk_menu_shell_append(GTK_MENU_SHELL(adv_menu), m_view_seqticks);
  gtk_menu_shell_append(GTK_MENU_SHELL(adv_menu), m_view_source);

  // attach view menu
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_sidebar);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_thumbbar);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), gtk_separator_menu_item_new());

  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_gofwd);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_gobwd);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), gtk_separator_menu_item_new());

  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_zi);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_zo);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_zn);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), gtk_separator_menu_item_new());

  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), alpha_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), color_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), gtk_separator_menu_item_new());

  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_gm);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), naming_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), namedisp_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), gtk_separator_menu_item_new());

  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_hs);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), hotspot_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), gtk_separator_menu_item_new());

  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_nm);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_rm);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_or);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), rift_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), gtk_separator_menu_item_new());

  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_al);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_ac);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_ar);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), gtk_separator_menu_item_new());

  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), m_view_type);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), gtk_separator_menu_item_new());

  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), size_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(view_menu), adv_item);

  view_item = gtk_menu_item_new_with_mnemonic("_View");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(view_item), view_menu);

  gtk_widget_add_accelerator(m_view_gofwd, "activate", accel_group,
			     'f', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_view_gobwd, "activate", accel_group,
			     'b', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_view_zn, "activate", accel_group,
			     '0', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_view_zi, "activate", accel_group,
			     '+', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_view_zo, "activate", accel_group,
			     '-', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_view_type, "activate", accel_group,
			     't', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_view_al, "activate", accel_group,
			     '<', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_view_ac, "activate", accel_group,
			     '|', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_view_ar, "activate", accel_group,
			     '>', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  // bookmark
  GtkWidget *bmark_menu, *bmark_item;

  bmark_menu = gtk_menu_new();
  status.bmark_menu = bmark_menu;

  m_bmark_add   = gtk_menu_item_new_with_mnemonic("_Add bookmark");
  m_bmark_clear = gtk_menu_item_new_with_mnemonic("_Clear bookmark");
  m_bmark_save  = gtk_menu_item_new_with_mnemonic("_Save bookmark now");

  gtk_menu_shell_append(GTK_MENU_SHELL(bmark_menu), m_bmark_add);
  gtk_menu_shell_append(GTK_MENU_SHELL(bmark_menu), m_bmark_clear);
  gtk_menu_shell_append(GTK_MENU_SHELL(bmark_menu), m_bmark_save);

  bmark_item = gtk_menu_item_new_with_mnemonic("_Bookmark");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(bmark_item), bmark_menu);

  gtk_widget_add_accelerator(m_bmark_add, "activate", accel_group,
			     'd', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  // tool
  GtkWidget *tool_menu, *tool_item;

  tool_menu = gtk_menu_new();

  m_tool_conf = gtk_menu_item_new_with_mnemonic("_Configure");
  m_tool_stat = gtk_menu_item_new_with_mnemonic("_Sequence statistics");
  m_tool_ins  = gtk_menu_item_new_with_mnemonic("_Inspector");
  m_tool_gid  = gtk_menu_item_new_with_mnemonic("_Gene Information");
  m_tool_repd = gtk_menu_item_new_with_mnemonic("_Reports");
  m_tool_das  = gtk_menu_item_new_with_mnemonic("_DAS server manager");

  gtk_menu_shell_append(GTK_MENU_SHELL(tool_menu), m_tool_conf);
  gtk_menu_shell_append(GTK_MENU_SHELL(tool_menu), m_tool_stat);
  gtk_menu_shell_append(GTK_MENU_SHELL(tool_menu), m_tool_ins);
  gtk_menu_shell_append(GTK_MENU_SHELL(tool_menu), m_tool_gid);
  gtk_menu_shell_append(GTK_MENU_SHELL(tool_menu), m_tool_repd);
#ifndef OLD_GTK_COMPAT
  gtk_menu_shell_append(GTK_MENU_SHELL(tool_menu), m_tool_das);
#endif

  tool_item = gtk_menu_item_new_with_mnemonic("_Tool");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(tool_item), tool_menu);

  gtk_widget_add_accelerator(m_tool_conf, "activate", accel_group,
			     'c', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_tool_stat, "activate", accel_group,
			     's', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_tool_ins,  "activate", accel_group,
			     'i', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_tool_gid,  "activate", accel_group,
			     'g', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(m_tool_repd,  "activate", accel_group,
			     'r', GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);

  // help
  GtkWidget *help_menu, *help_item;

  help_menu = gtk_menu_new();
  m_help_about= gtk_menu_item_new_with_mnemonic("_About");
  gtk_menu_shell_append(GTK_MENU_SHELL(help_menu), m_help_about);

  help_item = gtk_menu_item_new_with_mnemonic("_Help");
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(help_item), help_menu);

  gtk_menu_item_set_right_justified(GTK_MENU_ITEM(help_item), true);

  // attaching on menubar 
  GtkWidget *mb;
    
  mb = gtk_menu_bar_new();

  gtk_menu_shell_append(GTK_MENU_SHELL(mb), file_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(mb), edit_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(mb), bmark_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(mb), view_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(mb), tool_item);
  gtk_menu_shell_append(GTK_MENU_SHELL(mb), help_item);


  menuconnect(m_file_open);
  menuconnect(m_file_loadanno);
  menuconnect(m_file_loadseq);
#ifdef CAIRO_HAS_PDF_SURFACE
  menuconnect(m_file_pdf);
#endif
  menuconnect(m_file_png);
  menuconnect(m_file_savepref);
  menuconnect(m_file_reset);
  menuconnect(m_file_autosave);
  menuconnect(m_file_quit);

  menuconnect(m_edit_undo);
  menuconnect(m_edit_redo);
  menuconnect(m_edit_sort);
  menuconnect(m_edit_msort);

  menuconnect(m_bmark_add);
  menuconnect(m_bmark_clear);
  menuconnect(m_bmark_save);

  menuconnect(m_view_sidebar);
  menuconnect(m_view_thumbbar);

  menuconnect(m_view_zi);
  menuconnect(m_view_zo);
  menuconnect(m_view_zn);
  menuconnect(m_view_gofwd);
  menuconnect(m_view_gobwd);

  menuconnect(m_view_gm);
  menuconnect(m_view_hs);
  menuconnect(m_view_nm);
  menuconnect(m_view_rm);
  menuconnect(m_view_or);

  menuconnect(m_view_normalsize);
  menuconnect(m_view_larger);
  menuconnect(m_view_smaller);

  menuconnect(m_view_al);
  menuconnect(m_view_ac);
  menuconnect(m_view_ar);

  menuconnect(m_view_type);

  menuconnect(m_view_fwd);
  menuconnect(m_view_bwd);
  menuconnect(m_view_clip);
  menuconnect(m_view_stencil);
  menuconnect(m_view_pfill);

  menuconnect(m_view_range);
  menuconnect(m_view_seqno);
  menuconnect(m_view_seqticks);
  menuconnect(m_view_source);

  menuconnect(m_view_alpha_const);
  menuconnect(m_view_alpha_tfidf);

  menuconnect(m_view_col_pos);
  menuconnect(m_view_col_len);
  menuconnect(m_view_col_tfidf);

  menuconnect(m_view_namm_auto);
  menuconnect(m_view_namm_gene);
  menuconnect(m_view_namm_locus);
  menuconnect(m_view_namm_prod);

  menuconnect(m_view_namp_auto);
  menuconnect(m_view_namp_show);
  menuconnect(m_view_namp_hide);

  menuconnect(m_view_hs_den);
  menuconnect(m_view_hs_lden);
  menuconnect(m_view_hs_red);
  menuconnect(m_view_hs_pat);
  menuconnect(m_view_hs_pnt);
  menuconnect(m_view_hs_com);

  menuconnect(m_tool_stat);

  menuconnect(m_tool_conf);
  menuconnect(m_tool_ins);
  menuconnect(m_tool_gid);
  menuconnect(m_tool_repd);
#ifndef OLD_GTK_COMPAT
  menuconnect(m_tool_das);
#endif

  menuconnect(m_help_about);

  return mb;
}

// ------------------------------
// [ File ]

bool mainmenu::openAnchorsDialog(){
  status.rsb.dump_bmark();

  GtkWidget *dialog;
  bool file_selected = false;
  
  dialog = gtk_file_chooser_dialog_new ("Open Anchors File",
					GTK_WINDOW(status.mainwindow),
					GTK_FILE_CHOOSER_ACTION_OPEN,
					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					(gpointer)NULL);
  
  GtkFileFilter *filter = gtk_file_filter_new();
  gtk_file_filter_add_pattern(filter, "*.anchors");
  gtk_file_filter_add_pattern(filter, "*.block");
  gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filter);

  hasDialog = true;
  if (gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
    file_selected = true;

  if(file_selected){
    char *filename;
    
    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));

    // windows compatibility hack
    char *fn = filename;
    while(*fn != 0){
      if(*fn=='\\') *fn='/';
      fn++;
    }
    // end windows compatibility hack

    string new_anchor_file = filename;

    g_free (filename);

    gtk_widget_destroy (dialog);
    hasDialog = false;

    // check existence of the files
    status.anchorfile_fromGUI = true;
    status.anchorfile = new_anchor_file;
    
    // check existence of the files
    status.anchorfile_fromGUI = true;
    status.anchorfile = new_anchor_file;

    // mr.seq{file,name}s are cleared by mr.clear(), 
    // but it's no good for this case... so make a backup set
    vector <string> seqfiles_copy;
    vector <string> seqnames_copy;

    seqfiles_copy = mr.seqfiles;
    seqnames_copy = mr.seqnames;

    if(!status.set_filenames()){
      GtkWidget *errordialog;
      errordialog = gtk_message_dialog_new (GTK_WINDOW(status.mainwindow),
					    GTK_DIALOG_DESTROY_WITH_PARENT,
					    GTK_MESSAGE_ERROR,
					    GTK_BUTTONS_CLOSE,
					    "Error loading file '%s'",
					    new_anchor_file.c_str());
      hasDialog = true;
      gtk_dialog_run (GTK_DIALOG (errordialog));
      gtk_widget_destroy (errordialog);
      hasDialog = false;

      mr.seqfiles = seqfiles_copy;
      mr.seqnames = seqnames_copy;
      return false; 
    }

    seqfiles_copy = mr.seqfiles;
    seqnames_copy = mr.seqnames;

    // quit superzoom 
    if (status.plot_type == PLOT_SUPERZOOM){
      status.plot_type=PLOT_CHROM_LINKS;
      status.superzoom_from = 0;
      status.superzoom_to   = mr.maxbases(status.toggle_seq);
    }

    // clear everything but some settings
    bool sidebar_visible = status.lsb.sense_visible();
    reset_all();
    status.lsb.set_visible_silent(sidebar_visible);
    
    // then load new anchors/sequences
    status.anchorfile_fromGUI = true;
    status.anchorfile = new_anchor_file;
    status.set_filenames(); 

    mr.seqfiles = seqfiles_copy;
    mr.seqnames = seqnames_copy;

    assert(cout << "handle_openAnchorsbutton(): OK, going to reload." << endl);
    handlers.block();

    g_idle_add((GSourceFunc)start_initialize, NULL);
    return true;
  } else {
    gtk_widget_destroy(dialog);
    hasDialog = false;
  }
  return false;
}

void mainmenu::loadAnnotationDialog(){
  GtkWidget *dialog, *seqcombo;
  bool file_selected = false;
  
  dialog = gtk_file_chooser_dialog_new ("Load Annotation File",
					GTK_WINDOW(status.mainwindow),
					GTK_FILE_CHOOSER_ACTION_OPEN,
					GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
					GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
					(gpointer)NULL);

  // prepare filter
  GtkFileFilter *filter = gtk_file_filter_new();
  gtk_file_filter_add_pattern(filter, "*.cds*");
  gtk_file_filter_add_pattern(filter, "*.CDS*");
  gtk_file_filter_add_pattern(filter, "*.gtf*");
  gtk_file_filter_add_pattern(filter, "*.gff*");
  gtk_file_filter_add_pattern(filter, "*.gbk*");
  gtk_file_filter_add_pattern(filter, "*.xml*");
  gtk_file_filter_add_pattern(filter, "*.GTF*");
  gtk_file_filter_add_pattern(filter, "*.GFF*");
  gtk_file_filter_add_pattern(filter, "*.GBK*");
  gtk_file_filter_add_pattern(filter, "*.XML*");
  gtk_file_chooser_set_filter(GTK_FILE_CHOOSER(dialog), filter);

  // prepare seqname combo
  GtkWidget *seqcombobox;
  seqcombobox = gtk_hbox_new(false, 0);
  gtk_box_pack_start(GTK_BOX(seqcombobox), 
		     gtk_label_new("For sequence "), false, false, 0);
  seqcombo = gtk_combo_box_new_text();
  for(unsigned s=0; s<mr.acols()/3; s++){
    gtk_combo_box_append_text(GTK_COMBO_BOX(seqcombo), 
			      string("#" + itos(s+1)).c_str());
  }
  gtk_combo_box_set_active(GTK_COMBO_BOX(seqcombo), 0);
  gtk_box_pack_start(GTK_BOX(seqcombobox), seqcombo, false, false, 0);

  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), 
		     seqcombobox, false, false, 0);
  gtk_widget_show_all(seqcombobox);

  // launch the dialog
  hasDialog = true;
  if (gtk_dialog_run(GTK_DIALOG (dialog)) == GTK_RESPONSE_ACCEPT)
    file_selected = true;

  if(file_selected){
    char *filename;
    filename = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(dialog));

    // windows compatibility hack
    char *fn = filename;
    while(*fn != 0){
      if(*fn=='\\') *fn='/';
      fn++;
    }
    // end windows compatibility hack

    // get filename
    string new_annotation_file = filename;
    g_free (filename);

    // get seq#
    int seq = gtk_combo_box_get_active(GTK_COMBO_BOX(seqcombo));

    gtk_widget_destroy (dialog);
    hasDialog = false;

    // confirm
    if(!confirm_to_flush()) return;

    // prepare to load
    status.annotation_files.push_back(pair<int,string>(seq, new_annotation_file));
    status.flushAtRedraw = true;
    mr.annotationsChanged = true;

    redraw();
  } else {
    // canceled, just close the dialog
    gtk_widget_destroy(dialog);
    hasDialog = false;
  }
}

// ------------------------------
// [ Edit ]

void mainmenu::undo(){
  if(status.undo_stack.size() < 2) return;

  pixbuf_cache_tag t;
  t.get_current_globs();
  status.redo_stack.push_back(t);

  t = status.undo_stack[status.undo_stack.size()-2];
  status.undo_stack.pop_back();

  t.set_to_status();
  status.lsb.setByStatus();

  redraw_operation();
}

void mainmenu::redo(){
  if(status.redo_stack.size() == 0) return;

  pixbuf_cache_tag t;
  t.get_current_globs();
  t = status.redo_stack[status.redo_stack.size()-1];
  status.redo_stack.pop_back();
  status.undo_stack.push_back(t);

  t.set_to_status();
  status.lsb.setByStatus();

  redraw_operation();
}

void mainmenu::zoom(int inout){
  switch(inout){
  case MAINMENU_ZIN:   zoomer->in(); break;
  case MAINMENU_ZOUT:  zoomer->out(); break;
  default:             zoomer->reset();
  }
  redraw();
}
