#include "mainwindow.h"
#include "gmv.h"
#include "arrow.xpm.h"

// ----------------------------------------------------------------------
// Left sidebar (QuickBar in older version)

void left_sidebar::set_sensitive_for_linkplot(gboolean b){
  handlers.set_sensitive(color_combo, b);
  handlers.set_sensitive(name_combo,  b);
  handlers.set_sensitive(name_onoff_combo, b);
  handlers.set_sensitive(hotspot_combo, b);
}

void left_sidebar::packSeqChecks(){
  int seq=0;
  while(mr.bs[seq] != NULL) seq++;

  if( (unsigned)seq != seqlist.items.size() ){
    seqlist.clear();
    seqlist_original.clear();
    
    for(int i=0; i<seq; i++){
      seqItem si;
      si.number(i);
      seqlist.add_item(si);
      seqlist_original.push_back(si);
    }
  }
  gtk_expander_set_expanded(GTK_EXPANDER(seq_expander), seq<=4);
  gtk_widget_show_all(seq_box);
}

GtkWidget* left_sidebar::init(){
  ready = true;

  GtkWidget *qbhbox;
  
  qbhbox = gtk_hbox_new(FALSE, 0);
  qbbox = gtk_vbox_new(FALSE, 0);
  
  gtk_box_pack_start(GTK_BOX(qbhbox), qbbox, FALSE, FALSE, 0);

  // sidebar toggle button

  GdkPixbuf* rightarrow_buf = gdk_pixbuf_new_from_xpm_data(rightarrow_pixmap);
  GdkPixbuf* leftarrow_buf = gdk_pixbuf_new_from_xpm_data(leftarrow_pixmap);

  rightarrow = gtk_image_new_from_pixbuf(rightarrow_buf);
  leftarrow  = gtk_image_new_from_pixbuf(leftarrow_buf);

  show_button = gtk_event_box_new();
  hide_button = gtk_event_box_new();

  gtk_container_add(GTK_CONTAINER(show_button), rightarrow);
  gtk_container_add(GTK_CONTAINER(hide_button), leftarrow);

  gtk_box_pack_start(GTK_BOX(qbhbox), show_button, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(qbhbox), hide_button, FALSE, FALSE, 0);

  gtk_widget_add_events(show_button, GDK_BUTTON_PRESS_MASK);
  gtk_widget_add_events(hide_button, GDK_BUTTON_PRESS_MASK);

  handlers.connect(show_button, "button-press-event",
		   G_CALLBACK(handle_release), this);
  handlers.connect(hide_button, "button-press-event", 
		   G_CALLBACK(handle_release), this);

  // sidebar main part

  go_fwd = gtk_button_new_from_stock(GTK_STOCK_GO_FORWARD);
  go_bwd = gtk_button_new_from_stock(GTK_STOCK_GO_BACK);

  status.set_tip(go_fwd, "Move forward");
  status.set_tip(go_bwd, "Move backward");

  zoom_out = gtk_button_new_from_stock(GTK_STOCK_ZOOM_OUT);
  zoom_in  = gtk_button_new_from_stock(GTK_STOCK_ZOOM_IN);
  zoom_normal = gtk_button_new_from_stock(GTK_STOCK_ZOOM_100);

  status.set_tip(zoom_in, "Zoom in center");
  status.set_tip(zoom_out, "Zoom out");
  status.set_tip(zoom_normal, "Show whole sequence");

  //

  std::vector<generalList<seqItem>::coltype> cols;
  generalList<seqItem>::coltype col;

  col.data_type = G_TYPE_STRING; col.is_editable = false;
  col.view_type = generalList<seqItem>::renderer_text;  cols.push_back(col);

  col.data_type = G_TYPE_BOOLEAN; col.is_editable = true;
  col.view_type = generalList<seqItem>::renderer_check; cols.push_back(col);

  seqlist.set_types(cols);
  seqlist.create_model();
  seq_box = GTK_WIDGET(seqlist.create_view(GTK_SELECTION_MULTIPLE));
  gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(seq_box), false);
  
  gm_check  = gtk_check_button_new_with_mnemonic("Gene _marker");
  hspot_check = gtk_check_button_new_with_mnemonic("Show _Repeats");
  fwd_check = gtk_check_button_new_with_mnemonic("F_orward");
  bwd_check = gtk_check_button_new_with_mnemonic("_Backward");
  unknown_check = gtk_check_button_new_with_mnemonic("_N mask");
  rep_check = gtk_check_button_new_with_mnemonic("R_epeat mask");
  orf_check = gtk_check_button_new_with_mnemonic("Show _ORF");
  clip_check = gtk_check_button_new_with_mnemonic("Strict _clipping");

  stencil_check = gtk_check_button_new_with_label("Stencils");
  pfill_check = gtk_check_button_new_with_label("Pattern fills");

  status.set_tip(gm_check, "Show annotations");
  status.set_tip(hspot_check, "Show heavy repeats (detected by murasaki)");
  status.set_tip(fwd_check, "Show forward anchors");
  status.set_tip(bwd_check, "Show backward anchors");
  status.set_tip(unknown_check, "Show \"N\" base masks on chromosome");
  status.set_tip(rep_check, "Show repeat masks on chromosome");
  status.set_tip(orf_check, "Show ORFs");
  status.set_tip(clip_check, "Hide anchors which has both ends outside of screen");
  status.set_tip(stencil_check, "Enable stencil for anchors with rifts");
  status.set_tip(pfill_check, "Enable pattern fill for anchors with rifts");

  // alpha by
  alpha_combo = gtk_combo_box_new_text();
  alpha_method.push_back("Alpha by...");
  alpha_method.push_back("tf-idf score");
  alpha_method.push_back("Constant");

  for(vector<string>::iterator ite=alpha_method.begin(); 
      ite != alpha_method.end(); ite++)
    gtk_combo_box_append_text(GTK_COMBO_BOX(alpha_combo), ite->c_str());

  GtkWidget* alpha_combo_ebox = gtk_event_box_new();
  gtk_container_add(GTK_CONTAINER(alpha_combo_ebox), alpha_combo);

  gtk_combo_box_set_active(GTK_COMBO_BOX(alpha_combo), 0);
  status.set_tip(alpha_combo_ebox, "Set anchor transparency scheme");

  // color by
  color_combo = gtk_combo_box_new_text();
  color_method.push_back("Color by...");
  color_method.push_back("Position");
  color_method.push_back("Length");
  color_method.push_back("tf-idf");
  for(vector<string>::iterator ite=color_method.begin(); 
      ite != color_method.end(); ite++)
    gtk_combo_box_append_text(GTK_COMBO_BOX(color_combo), ite->c_str());

  GtkWidget* color_combo_ebox = gtk_event_box_new();
  gtk_container_add(GTK_CONTAINER(color_combo_ebox), color_combo);

  gtk_combo_box_set_active(GTK_COMBO_BOX(color_combo), 0);
  status.set_tip(color_combo_ebox, "Set anchor coloring scheme");

  // name by 
  name_combo = gtk_combo_box_new_text();
  name_method.push_back("Name by...");
  name_method.push_back("Auto");
  name_method.push_back("Gene name");
  name_method.push_back("Locus tag");
  name_method.push_back("Product");
  for(vector<string>::iterator ite=name_method.begin(); 
      ite != name_method.end(); ite++)
    gtk_combo_box_append_text(GTK_COMBO_BOX(name_combo), ite->c_str());

  GtkWidget* name_combo_ebox = gtk_event_box_new();
  gtk_container_add(GTK_CONTAINER(name_combo_ebox), name_combo);

  gtk_combo_box_set_active(GTK_COMBO_BOX(name_combo), 0);
  status.set_tip(name_combo_ebox, "Set gene naming scheme");

  // name on/off
  name_onoff_combo = gtk_combo_box_new_text();
  name_onoff.push_back("Show Name...");
  name_onoff.push_back("Auto");
  name_onoff.push_back("Show always");
  name_onoff.push_back("Hide always");
  for(vector<string>::iterator ite=name_onoff.begin(); 
      ite != name_onoff.end(); ite++)
    gtk_combo_box_append_text(GTK_COMBO_BOX(name_onoff_combo), ite->c_str());

  GtkWidget* name_onoff_combo_ebox = gtk_event_box_new();
  gtk_container_add(GTK_CONTAINER(name_onoff_combo_ebox), name_onoff_combo);

  gtk_combo_box_set_active(GTK_COMBO_BOX(name_onoff_combo), 0);
  status.set_tip(name_onoff_combo_ebox, "Set gene name show/hide scheme");

  // hotspot coloring scheme
  hotspot_combo = gtk_combo_box_new_text();
  hotspot_method.push_back("Hotspot by...");
  hotspot_method.push_back("Density (Bar)");
  hotspot_method.push_back("log(Density) (Bar)");
  hotspot_method.push_back("Always Red");
  hotspot_method.push_back("Pattern");
  hotspot_method.push_back("# Points");
  hotspot_method.push_back("# Combination");

  for(vector<string>::iterator ite=hotspot_method.begin(); 
      ite != hotspot_method.end(); ite++)
    gtk_combo_box_append_text(GTK_COMBO_BOX(hotspot_combo), ite->c_str());

  GtkWidget* hotspot_combo_ebox = gtk_event_box_new();
  gtk_container_add(GTK_CONTAINER(hotspot_combo_ebox), hotspot_combo);

  gtk_combo_box_set_active(GTK_COMBO_BOX(hotspot_combo), 0);
  status.set_tip(hotspot_combo_ebox, "Set hotspot coloring scheme");

  redraw_button = gtk_button_new_from_stock(GTK_STOCK_APPLY);
  status.set_tip(redraw_button, "Apply options above");

  // Plot type button
  type_button = gtk_button_new_with_label("Plot Type");
  status.set_tip(type_button, "Change plot type (Dot plot / Link plot / Zoomed Link Plot)");

  // Load SEQs
  loadseq_button = gtk_button_new();
  GtkWidget *label = gtk_label_new(NULL);
  gtk_label_set_markup(GTK_LABEL(label), "<b><span foreground=\"red\">Load Sequences\nto show ORFs</span></b>");
  gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
  gtk_container_add(GTK_CONTAINER(loadseq_button), label);

  // pack 'em: commented out because it's for students only :)
  //  gtk_box_pack_start(GTK_BOX(qbbox), loadseq_button, FALSE, FALSE, 0);

  // arrange_frame
  arrange_frame = gtk_frame_new("Arrange");
  arrange_frame_box = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(arrange_frame), arrange_frame_box);
  gtk_box_pack_start(GTK_BOX(qbbox), arrange_frame, TRUE, TRUE, 0);

  gtk_box_pack_start(GTK_BOX(arrange_frame_box), go_bwd, false, false, 0);
  gtk_box_pack_start(GTK_BOX(arrange_frame_box), go_fwd, false, false, 0);

  gtk_box_pack_start(GTK_BOX(arrange_frame_box), gtk_hseparator_new(),
		     false, false, 0);
  
  gtk_box_pack_start(GTK_BOX(arrange_frame_box), zoom_in,     false, false, 0);
  gtk_box_pack_start(GTK_BOX(arrange_frame_box), zoom_normal, false, false, 0);
  gtk_box_pack_start(GTK_BOX(arrange_frame_box), zoom_out,    false, false, 0);

  gtk_button_set_alignment(GTK_BUTTON(go_bwd),      0, 0.5);
  gtk_button_set_alignment(GTK_BUTTON(go_fwd),      0, 0.5);

  gtk_button_set_alignment(GTK_BUTTON(zoom_in),     0, 0.5);
  gtk_button_set_alignment(GTK_BUTTON(zoom_normal), 0, 0.5);
  gtk_button_set_alignment(GTK_BUTTON(zoom_out),    0, 0.5);

  // opt_frame
  opt_frame = gtk_frame_new("Options");
  opt_frame_box = gtk_vbox_new(FALSE, 0);
  gtk_container_add(GTK_CONTAINER(opt_frame), opt_frame_box);
  gtk_box_pack_start(GTK_BOX(qbbox), opt_frame, TRUE, TRUE, 0);

  // seq
  seq_expander = gtk_expander_new("Sequences");
  gtk_expander_set_expanded(GTK_EXPANDER(seq_expander), false);
  seq_box2 = gtk_vbox_new(false, 0);
  gtk_box_pack_start(GTK_BOX(seq_box2), seq_box, false, false, 0);
  gtk_box_pack_end(GTK_BOX(seq_box2), gtk_hseparator_new(), false, false, 0);
  gtk_container_add(GTK_CONTAINER(seq_expander), seq_box2);

  gtk_box_pack_start(GTK_BOX(opt_frame_box), seq_expander, FALSE, FALSE, 0);

  // scheme
  scheme_expander = gtk_expander_new("Colors/Names");
  scheme_box = gtk_vbox_new(FALSE, 0);
  gtk_expander_set_expanded(GTK_EXPANDER(scheme_expander), 
			    status.show_sidebar_schemes);
  gtk_container_add(GTK_CONTAINER(scheme_expander), scheme_box);
  
  gtk_box_pack_start(GTK_BOX(opt_frame_box), scheme_expander, FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(scheme_box), alpha_combo_ebox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(scheme_box), color_combo_ebox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(scheme_box), name_combo_ebox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(scheme_box), name_onoff_combo_ebox, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(scheme_box), hotspot_combo_ebox, FALSE, FALSE, 0);
  gtk_box_pack_end(GTK_BOX(scheme_box), gtk_hseparator_new(), FALSE, FALSE, 0);

  // toggles
  toggle_expander = gtk_expander_new("Toggle options");
  toggle_box = gtk_vbox_new(FALSE, 0);
  gtk_expander_set_expanded(GTK_EXPANDER(toggle_expander), 
			    status.show_sidebar_toggles);
  gtk_container_add(GTK_CONTAINER(toggle_expander), toggle_box);
  
  gtk_box_pack_start(GTK_BOX(opt_frame_box), toggle_expander, FALSE, FALSE, 0);

  gtk_box_pack_start(GTK_BOX(toggle_box), gm_check,  FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(toggle_box), hspot_check,  FALSE, FALSE, 0);
  //  gtk_box_pack_start(GTK_BOX(toggle_box), fwd_check, FALSE, FALSE, 0);
  //  gtk_box_pack_start(GTK_BOX(toggle_box), bwd_check, FALSE, FALSE, 0);
  //  gtk_box_pack_start(GTK_BOX(toggle_box), unknown_check, FALSE, FALSE, 0);
  //  gtk_box_pack_start(GTK_BOX(toggle_box), rep_check, FALSE, FALSE, 0);
  gtk_box_pack_start(GTK_BOX(toggle_box), orf_check, FALSE, FALSE, 0);
  //  gtk_box_pack_start(GTK_BOX(toggle_box), clip_check, FALSE, FALSE, 0);
  //  gtk_box_pack_start(GTK_BOX(toggle_box), stencil_check, FALSE, FALSE, 0);
  //  gtk_box_pack_start(GTK_BOX(toggle_box), pfill_check, FALSE, FALSE, 0);

  //
  gtk_box_pack_start(GTK_BOX(opt_frame_box), redraw_button, FALSE, FALSE, 0);

  // 
  gtk_box_pack_start(GTK_BOX(qbbox), type_button , FALSE, FALSE, 0);

  //  handlers.connect(loadseq_button,"clicked", G_CALLBACK(handle_activate), this);

  handlers.connect(go_bwd,        "clicked", G_CALLBACK(handle_activate), this);
  handlers.connect(go_fwd,        "clicked", G_CALLBACK(handle_activate), this);
  handlers.connect(zoom_normal,   "clicked", G_CALLBACK(handle_activate), this);
  handlers.connect(zoom_in,       "clicked", G_CALLBACK(handle_activate), this);
  handlers.connect(zoom_out,      "clicked", G_CALLBACK(handle_activate), this);

  handlers.connect(type_button,   "clicked", G_CALLBACK(handle_activate), this);
  handlers.connect(redraw_button, "clicked", G_CALLBACK(handle_activate), this);

  setTypeButton();

  return qbhbox;
}

void left_sidebar::activate(GtkWidget *w){

  if(w==hide_button || w==show_button){
    toggle_visible();
    status.mb.setByStatus();
  }

  if(w==redraw_button){
    setToStatus();
    setByStatus();
    redraw();
  }

  if(w==go_fwd || w==go_bwd){
    if (w==go_fwd) status.go_fwd();
    if (w==go_bwd) status.go_bwd();
    status.ins.setByStatus();
    redraw();
  }

  if(w==zoom_normal || w==zoom_in || w==zoom_out){
    if (w==zoom_normal) status.zoom_normal();
    if (w==zoom_in)     status.zoom_in();
    if (w==zoom_out)    status.zoom_out();
    status.ins.setByStatus();
    redraw();
  }

//   if(w==loadseq_button){
//     if(confirm_to_flush()){
//       status.loadSeqAtRedraw = true;
//       status.plot_O = true;
//       flush_and_redraw();
//     }
//   }

  if(w==type_button) togglePlotType();
}

void left_sidebar::togglePlotType(){
  switch(status.plot_type){
  case PLOT_CHROM_LINKS: // will be dot plot
    status.plot_type = PLOT_DOT;
    assert(cout << "handle_PlotTypebutton(): changing to Dot Plot" << endl);
    break;
  case PLOT_DOT: // will be chrom links
  case PLOT_SUPERZOOM:
    status.zoom_normal();
    status.plot_type = PLOT_CHROM_LINKS;
    assert(cout << "handle_PlotTypebutton(): changing to Chrom Links" << endl);
    break;
  }
  redraw();
}

void left_sidebar::setToStatus(){
  status.show_gene_marker = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(gm_check));
  status.show_hotspots = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(hspot_check));
  status.gene_name_onoff_method = gtk_combo_box_get_active(GTK_COMBO_BOX(name_onoff_combo));
  status.hotspot_by = gtk_combo_box_get_active(GTK_COMBO_BOX(hotspot_combo));
  status.plot_fwd = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(fwd_check));
  status.plot_bwd = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(bwd_check));
  status.plot_R   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(rep_check));
  status.plot_O   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(orf_check));
  status.plot_U   = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(unknown_check));
  status.clip_hard = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(clip_check));

  status.rift_stencil = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(stencil_check));
  status.rift_pattern_fill = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(pfill_check));

  status.alpha_by = gtk_combo_box_get_active(GTK_COMBO_BOX(alpha_combo));

  status.color_by = gtk_combo_box_get_active(GTK_COMBO_BOX(color_combo));
  if(status.color_by == COLOR_BY_TFIDF && mr.tfidf.empty())
    status.color_by = COLOR_BY_POSITION;

  status.name_by  = gtk_combo_box_get_active(GTK_COMBO_BOX(name_combo));

  for(unsigned s=0; s<mr.acols()/3; s++){
    status.toggle_seq[seqlist.items[s].number()] = seqlist.items[s].visible();
    status.order_seq[s] = seqlist.items[s].number();
  }
  
  status.chk_toggle_seq(); // prevent all seqs to be false
  mr.chk_order_seq();
}

bool left_sidebar::toggle_visible(){
  set_visible(!visible);
  return visible;
}

void left_sidebar::set_visible(bool v){
  visible = v;
  if(ready){
    if(v) gtk_widget_show_all(qbbox); else gtk_widget_hide_all(qbbox);
    setByStatus(); // show & hide loadseq_button 
  }
}

void left_sidebar::set_visible_silent(bool v){
  visible = v;
  if(ready){
    if(v) gtk_widget_show_all(qbbox); else gtk_widget_hide_all(qbbox);
  }
}

void left_sidebar::setByStatus(){
  handlers.block();
  status.mb.setByStatus();

  if(visible){
    gtk_widget_show_all(qbbox);
    gtk_widget_show(leftarrow);
    gtk_widget_hide(rightarrow);
  } else {
    gtk_widget_hide_all(qbbox);
    gtk_widget_hide(leftarrow);
    gtk_widget_show(rightarrow);
  }

  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(gm_check), status.show_gene_marker);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(hspot_check), status.show_hotspots);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(fwd_check), status.plot_fwd);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(bwd_check), status.plot_bwd);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(rep_check), status.plot_R);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(orf_check), status.plot_O);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(unknown_check), status.plot_U);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(clip_check), status.clip_hard);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(stencil_check), status.rift_stencil);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(pfill_check), status.rift_pattern_fill);

  gtk_combo_box_set_active(GTK_COMBO_BOX(alpha_combo), status.alpha_by);
  gtk_combo_box_set_active(GTK_COMBO_BOX(color_combo), status.color_by);
  gtk_combo_box_set_active(GTK_COMBO_BOX(name_combo), status.name_by);
  gtk_combo_box_set_active(GTK_COMBO_BOX(name_onoff_combo), status.gene_name_onoff_method);
  gtk_combo_box_set_active(GTK_COMBO_BOX(hotspot_combo), status.hotspot_by);

  std::vector<seqItem> sl(seqlist_original.size());
  for(unsigned s=0; s<mr.acols()/3; s++){
    sl[s] = seqlist_original[status.order_seq[s]];
    sl[s].visible(status.toggle_seq[status.order_seq[s]]);
  }
  seqlist.replace(sl);


  handlers.set_sensitive(hspot_check, mr.has_hotspots());
  if(mr.has_sequence_statistics){
    //    gtk_widget_hide(loadseq_button);
    handlers.set_sensitive(rep_check, true);
    handlers.set_sensitive(orf_check, true);
    handlers.set_sensitive(unknown_check, true);
  }
  else{
    //    gtk_widget_show(loadseq_button);
    handlers.set_sensitive(rep_check, false);
    handlers.set_sensitive(orf_check, false);
    handlers.set_sensitive(unknown_check, false);
  }
 
  handlers.set_sensitive(go_fwd,        status.plot_type == PLOT_SUPERZOOM);
  handlers.set_sensitive(go_bwd,        status.plot_type == PLOT_SUPERZOOM);
  handlers.set_sensitive(zoom_normal,   status.plot_type == PLOT_SUPERZOOM);
  handlers.set_sensitive(zoom_in,       status.plot_type != PLOT_DOT);
  handlers.set_sensitive(zoom_out,      status.plot_type != PLOT_DOT);

  handlers.set_sensitive(stencil_check, status.plot_type != PLOT_DOT);
  handlers.set_sensitive(pfill_check,   status.plot_type != PLOT_DOT);

  handlers.unblock();
}

void left_sidebar::setTypeButton(){
  //  return;
  switch(status.plot_type){
  case PLOT_DOT: // will be dot plot
    assert(cout << "handle_PlotTypebutton(): changing to Dot Plot" << endl);
    gtk_button_set_label(GTK_BUTTON(type_button), "Chrom\nLinks"); // button label is to go back
    break;
  case PLOT_SUPERZOOM:
    gtk_button_set_label(GTK_BUTTON(type_button), "Zoom\nOut"); // button label is to go back
    break;
  case PLOT_CHROM_LINKS: // will be chrom links
    assert(cout << "handle_PlotTypebutton(): changing to Chrom Links" << endl);
    gtk_button_set_label(GTK_BUTTON(type_button), "Dot\nPlot");
    break;
  }
}
