using namespace std;

#include <getopt.h>
#include <iostream>
#include <iomanip>
#include "gmv.h"
#include "messages.h"
#include "annotation.h"
#include "gbmisc.h"

void procopts(){
  assert(cout << "procopts(): called" << endl);

  status.set_defaults();
  mr.order_seq = status.order_seq;

  // --------------------------------------------------
  // Command Line Opts.

  int optch;
  extern char *optarg;
  extern int  optind;
  int longindex=0;
    
  static struct option long_options[] = {
    {"annotation",1,0,'a'},
    {"mask-seq",1,0,'m'},
    {"reorder-seq",1,0,'r'},
    {"plot-type",1,0,'t'},
    {"sequence-names",1,0,'n'},
    {"shift-sequences",1,0,'s'},
    {"file-output",1,0,'f'},
    {"options",1,0,'o'},
    {"enable",1,0,'e'},
    {"disable",1,0,'d'},
    {"help",0,0,'h'},
    {"version",0,0,'v'},
    {0,0,0,0}
  };
    
  string available_options = "a:d:e:f:hm:n:o:r:s:t:v";
    
  // every variables controlled by cmdline opts must be in
  // backup() and restore() !!
  while ((optch = getopt_long(status.argc, status.argv, 
			      available_options.c_str(), 
			      long_options, &longindex)) != -1){

    string n;
    size_t f=0;
    size_t t;
    string fn;
    int sn=0;
    int seqcount=0;

    switch (optch){
      // ------------------------------------------------------------
    case 't':
      n=optarg;
      if(n=="all"  || n=="a") status.plot_all=true;
      if(n=="dot"  || n=="d") status.plot_type=PLOT_DOT;
      if(n=="link" || n=="l") status.plot_type=PLOT_CHROM_LINKS;
      break;

      // ------------------------------------------------------------
    case 'r':  // reorder-seq
      n=optarg;
      t=n.find(",", f); 
      do{
	if(t==string::npos) t=n.length();
	  fn = n.substr(f,(t-f));
	  mr.order_seq[seqcount] = atoi(fn.c_str());
	  f=t+1; t = n.find(",", f);
	  seqcount++;
      } while(f < n.length());
      break;

      // ------------------------------------------------------------
    case 'm':  // mask-seq
      for(unsigned int i=0; i<strlen(optarg); i++)
	if (optarg[i]=='0') status.toggle_seq[i]=false;
	// status.chk_toggle_seq(); // never do this because seqs aren't loaded
      break;

      // ------------------------------------------------------------
    case 's': // shift-seqs
      n=optarg;
      t=n.find(",", f); 
      do{
	if(t==string::npos) t=n.length();
	  fn = n.substr(f,(t-f));
	  //	  cout << "offset for " << seqcount << " is " << fn <<endl;
	  mr.set_offset(seqcount, atoi(fn.c_str()));
	  f=t+1; t = n.find(",", f);
	  seqcount++;
      } while(f < n.length());
      break;

      // ------------------------------------------------------------
    case 'a': // fill vector<string> annotation_files up.
      n=optarg;
      t=n.find(",", f); 
      do{
	if(t==string::npos) t=n.length();
	sn=0;   // seqence number
	int pos;

	fn = n.substr(f,(t-f));

	if(fn.find(":") != string::npos){
	  pos = fn.find(":");
	  sn=atoi(fn.substr(0, pos).c_str())-1;
	  if(sn<0) sn=0;
	  fn=fn.substr(pos+1, fn.size()-pos);
	}

	status.annotation_files.push_back(pair<int,string>(sn,fn));

	f=t+1; t = n.find(",", f);
      } while(f < n.length());
      break;

      // ------------------------------------------------------------
    case 'f': // file-output
      n=optarg;
      t=n.find(",", f); 
      do{
	if(t==string::npos) t=n.length();
	  fn = n.substr(f,(t-f));
	  
	  if(fn=="png" || fn=="all") status.generate_png = true;
	  if(fn=="pdf" || fn=="all") status.generate_pdf = true;
	  
	  f=t+1; t = n.find(",", f);
      } while(f < n.length());
      break;

      // ------------------------------------------------------------
    case 'n': // fill vector<string> sequence_names up.
      n=optarg;
      t=n.find(",", f); 
      do{
	if(t==string::npos) t=n.length();
	  fn = n.substr(f,(t-f));
	  status.sequence_names.push_back(fn);
	  f=t+1; t = n.find(",", f);
      } while(f < n.length());
      break;

      // ------------------------------------------------------------
    case 'o': // --options
      n=optarg;
      assert(cout << "--options=" << n << endl);

      t=n.find(",", f); 
      do{
	if(t==string::npos) t=n.length();
	fn = n.substr(f,(t-f));

	size_t pos = fn.find("=");
	if(pos!=string::npos){
	  string fnA, fnB;
	  fnA=fn.substr(0,pos);
	  fnB=fn.substr(pos+1, fn.length());

	  double opt_f = atof(fnB.c_str());
	  int    opt_i = atoi(fnB.c_str());

	  // analysis
	  if(fnA=="min_orf")        status.min_orf             = opt_i;

	  // plotting 
	  if(fnA=="seq_thickness")  status.sequence_thickness  = opt_f;
	  if(fnA=="orf_thickness")  status.orf_thickness       = opt_f;
	  if(fnA=="font_size")      status.font_size_general   = opt_i;
	  if(fnA=="font_size_gene") status.font_size_gene      = opt_i;

	  if(fnA=="cursor_bp")      status.zoomcursor_width_bp = opt_i;
	  if(fnA=="cursor_px")      status.zoomcursor_width_px = opt_i;

	  if(fnA=="width")          status.default_width       = opt_i;
	  if(fnA=="height")         status.default_height      = opt_i;
	  if(fnA=="min_size")       status.min_size            = opt_i;
	  if(fnA=="max_size")       status.max_size            = opt_i;

	  if(fnA=="zoom_factor")    status.zoom_factor         = opt_f;

	  if(fnA=="min_tfidf")      status.opt_min_tfidf       = opt_f;

	  if(fnA=="rift_filter"){
	    mr.rift_filter_enabled = true;
	    for(unsigned pos=0; pos<fnB.size(); pos++){
	      mr.rift_filter_policy[pos] = ( (fnB[pos] == '1') ? 1:
					     (fnB[pos] == '2') ? 2: 0 );
	    }
	  }

	  // color
	  if(fnA=="alpha")          status.alpha               = opt_f;
	  if(fnA=="saturation")     status.saturation          = opt_f;
	  if(fnA=="value_fwd")      status.fwdvalue            = opt_f;
	  if(fnA=="value_bwd")      status.bwdvalue            = opt_f;
	  if(fnA=="bgvalue")        status.bgvalue             = opt_f;
	  if(fnA=="fl_contrast")    status.fill_stroke_ratio   = opt_f;
	  if(fnA=="min_thickness")  status.min_thickness       = opt_f;
	  if(fnA=="max_thickness")  status.max_thickness       = opt_f;
	  if(fnA=="huediff")        status.huediff_fb          = opt_f;
	  if(fnA=="endcap")         status.chrlink_endcap      = opt_f;
	  if(fnA=="endcap_na")      status.chrlink_endcap_na   = opt_f;
	  if(fnA=="gene_name_auto"){status.gene_name_onoff_method = 0;
	    status.show_gene_name_threshold                    = opt_i; }
	  if(fnA=="color_by"){
	    status.color_by = COLOR_BY_POSITION;
	    if(fnB=="length") status.color_by = COLOR_BY_ANCHORLEN;
	    if(fnB=="tfidf")  status.color_by = COLOR_BY_TFIDF;
	  }
	  if(fnA=="cog"){
	    status.cog_file = fnB;
	  }
	}

	if(fn.find("min_orf") == string::npos &&
	   fn.find("seq_thickness") == string::npos &&
	   fn.find("orf_thickness") == string::npos &&
	   fn.find("font_size") == string::npos && 
	   fn.find("font_size_gene") == string::npos && 
	   fn.find("cursor_bp") == string::npos && 
	   fn.find("cursor_px") == string::npos && 
	   fn.find("width") == string::npos && 
	   fn.find("height") == string::npos && 
	   fn.find("min_size") == string::npos && 
	   fn.find("max_size") == string::npos && 
	   fn.find("zoom_factor") == string::npos && 
	   fn.find("min_tfidf") == string::npos && 
	   fn.find("rift_filter") == string::npos && 
	   fn.find("alpha") == string::npos && 
	   fn.find("saturation") == string::npos && 
	   fn.find("value_fwd") == string::npos && 
	   fn.find("value_bwd") == string::npos && 
	   fn.find("bgvalue") == string::npos && 
	   fn.find("fl_contrast") == string::npos && 
	   fn.find("min_thickness") == string::npos && 
	   fn.find("max_thickness") == string::npos && 
	   fn.find("huediff") == string::npos && 
	   fn.find("endcap") == string::npos && 
	   fn.find("endcap_na") == string::npos && 
	   fn.find("gene_name_auto") == string::npos && 
	   fn.find("color_by") == string::npos &&
	   fn.find("cog") == string::npos){
	  cerr << "gmv: invalid option: -o" << fn << endl;
	  exit(-1);
	}

	f=t+1; t = n.find(",", f);
      } while(f < n.length());
      break;

      // ------------------------------------------------------------
    case 'e': // --enable
      n=optarg;
      assert(cout << "--enable=" << n << endl);

      t=n.find(",", f); 
      do{
	if(t==string::npos) t=n.length();
	fn = n.substr(f,(t-f));
	if(fn=="load_seq")         status.opt_load_seq  = true;
	if(fn=="statistics")       status.opt_stat_text = true; 
	if(fn=="statistics_html")  status.opt_stat_html = true; 
	if(fn=="auto_order")       status.auto_order    = true; 
	if(fn=="force_gene_name")
	  status.gene_name_onoff_method   = DISP_NAME_ON;

	if(fn=="u_mask")    status.plot_U          = true;
	if(fn=="r_mask")    status.plot_R          = true;
	if(fn=="repeat")    status.show_hotspots   = true;

	if(fn=="rift_stencil") status.rift_stencil = true;

	if(fn!="load_seq" &&
	   fn!="statistics" && 
	   fn!="statistics_html" && 
	   fn!="auto_order" && 
	   fn!="force_gene_name" &&
	   fn!="u_mask" && 
	   fn!="o_mask" &&
	   fn!="rift_stencil" &&
	   fn!="repeat" ){
	  cerr << "gmv: invalid option: -e" << fn << endl;
	  exit(-1);
	}

	f=t+1; t = n.find(",", f);
      } while(f < n.length());
      break;

      // ------------------------------------------------------------
    case 'd': // --disable
      n=optarg;
      t=n.find(",", f); 
      do{
	if(t==string::npos) t=n.length();
	fn = n.substr(f,(t-f));

	if(fn=="load_annotation") status.opt_load_annot = false;
	if(fn=="force_gene_name") status.gene_name_onoff_method = DISP_NAME_OFF;
	if(fn=="gene_markers")        status.show_gene_marker    = false;
	if(fn=="range")     status.disp_range      = false;
	if(fn=="seqno")     status.disp_seqno      = false;
	if(fn=="bwd")       status.cmdopt_forward  = true;
	if(fn=="fwd")       status.cmdopt_backward = true;
	if(fn=="o_mask")    status.plot_R          = false;
	if(fn=="repeat")    status.show_hotspots   = false;
	if(fn=="rift_pattern_fill") status.rift_pattern_fill = false;

	if(fn!="load_annotation" &&
	   fn!="force_gene_name" && 
	   fn!="gene_markers" &&
	   fn!="range" &&
	   fn!="seqno" &&
	   fn!="bwd" &&
	   fn!="fwd" &&
	   fn!="o_mask" &&
	   fn!="repeat" &&
	   fn!="rift_pattern_fill"){
	  cerr << "gmv: invalid option: -d" << fn << endl;
	  exit(-1);
	}

	f=t+1; t = n.find(",", f);
      } while(f < n.length());
      break;
    case 'v':
      version();
      exit(-1);
      break;
    case 'h': default:
      usage();
      exit(-1);
      break;
    }
  }
  status.argc -= optind;
  status.argv += optind;


  // option constraints
  if (status.plot_all && 
      !(status.generate_png || status.generate_pdf)) 
    status.plot_type=PLOT_CHROM_LINKS;

  if (status.saturation > 1) status.saturation=1;
  if (status.bgvalue > 1) status.bgvalue=1;
  if (status.fwdvalue > 1) status.fwdvalue=1;
    
  if (status.saturation < 0) status.saturation=0;
  if (status.bgvalue < 0) status.bgvalue=0;
  if (status.fwdvalue < 0) status.fwdvalue=0;
  if (status.min_thickness < 0) status.min_thickness=0;

  if (status.opt_stat_html || status.opt_stat_text) status.opt_load_seq = true;

  // is anchorfile specified?
  if (status.argc > 1){
    usage();
    exit(-1);
  }

  // save backup (will be restored after loading pref files)
  status.backup();

  assert(cout << "procopts(): return" << endl);
}

void opt_read_annotations(vector<pair<int, string> >* af){

  pbar_reset("Reading annotation files...");

  int annocount=0;
  for(vector< pair<int,string> >::iterator i=af->begin(); 
      i!= af->end(); i++){
    
    // read it
    IFSTREAM anno;
    int seqno = i->first;
    anno.open(i->second.c_str());
    assert(cout << "opt_read_annotations(): reading " << i->second 
	   << " for sequence #" << i->first <<endl);

    if(anno == NULL){ 
      cerr << "Can't load annotation file: " << i->second <<endl;
      continue; 
    }

    pbar_set_task((double)annocount/af->size(), 
		  (double)(annocount+1)/af->size());

    annotationTrack* at = new annotationTrack;
    at->readstream(anno, i->second, seqno, mr, pbar_set_fraction);
    mr.annotations[seqno].push_back(at);

    anno.close();
 
    annocount++;
  }
}

void opt_read_annotations(vector<pair<int, das_import_result*> >* af){
  pbar_reset("Converting DAS tracks...");
  
  for(unsigned i=0; i<af->size(); i++){
    volatile int seqno = af->at(i).first;
    assert(cout << "das_tracks entry #" << i << " for seqno #" << seqno << endl);

    pbar_set_task((double)i/af->size(), (double)(i+1)/af->size());

    annotationTrack* at = new annotationTrack("DAS: " + af->at(i).second->server);

    for(vector<das_feature>::iterator j=af->at(i).second->fvec.begin();
	j != af->at(i).second->fvec.end(); j++){
      at->push_back(annotation(*j));
    }
    // FIXME: oh, how about track'has_score?
    at->isTraceable(true);
    at->hasScore(false);
    at->fixate();
    mr.annotations[seqno].push_back(at);
    assert(cout << at->size() << " genes imported\n");

    delete af->at(i).second;
  }
  af->clear();
}

void opt_anchor_statistics(){
  results *r = &mr;

  int seqno = r->acols()/3;

  if(!mr.has_detailed_statistics){

    pbar_reset("Generating statistics...");

    vector< range_list > *anchor_ranges, *anchor_ranges_f, *anchor_ranges_b;
    anchor_ranges = &r->anchor_ranges;
    anchor_ranges_f = &r->anchor_ranges_f;
    anchor_ranges_b = &r->anchor_ranges_b;
    
    for(int i=0; i<seqno; i++){
      range_list r;
      anchor_ranges->push_back(r);
      anchor_ranges_f->push_back(r);
      anchor_ranges_b->push_back(r);
      mr.anchors_num_f[i] = 0;
      mr.anchors_num_b[i] = 0;
    }

    for(unsigned int i=0; i<mr.arows(); i++){
      for(int s=0; s<seqno; s++){
	seqlen_t from=abs(mr.raa_physical(i,3*s));
	seqlen_t to=abs(mr.raa_physical(i,3*s+1));
	if(from>to) swap(from, to);
	anchor_ranges->at(s).add(fromto(from,to+1));

	if(mr.raa_physical(i, 3*s+2) == 1){
	  anchor_ranges_f->at(s).add(fromto(from,to+1));
	  mr.anchors_num_f[s]++;
	} else {
	  anchor_ranges_b->at(s).add(fromto(from,to+1));
	  mr.anchors_num_b[s]++;
	}
      }
      pbar_set_fraction( (double)i/mr.arows() );
    }

    mr.has_detailed_statistics = true;
  }
}
