#include "gmv.h"
#include "pixbuf_cache.h"

#include <sstream>

using namespace std;

// **********************************************************************
//                     CAUTION! CAUTION!! CAUTION!!!
//     Placing many "&&" s in a ( ) block may kill your Emacs :-$
// **********************************************************************

// ----------------------------------------------------------------------
// Pixbuf Cache tag

bool operator==(const pixbuf_cache_tag& a, const pixbuf_cache_tag& b){
  bool tmpbool = ( a.img_width  == b.img_width &&
		   a.img_height == b.img_height &&
		   a.plot_type  == b.plot_type &&
		   a.stencil    == b.stencil &&
		   a.pfill      == b.pfill &&
		   a.fwd        == b.fwd &&
		   a.bwd        == b.bwd &&
		   a.unknown    == b.unknown &&
		   a.rep        == b.rep &&
		   a.orf        == b.orf &&
		   a.hotspot    == b.hotspot &&
		   a.alpha_by   == b.alpha_by );
  if(!tmpbool) return false;

  tmpbool = ( tmpbool &&
	      a.color_by   == b.color_by &&
	      a.name_by    == b.name_by &&
	      a.hotspot_by == b.hotspot_by &&
	      a.rift0_col  == b.rift0_col &&
	      a.rift0_col_for_all == b.rift0_col_for_all &&
	      a.rift0_use_alt_seq == b.rift0_use_alt_seq &&
	      a.anno_col         == b.anno_col &&
	      a.show_gene_marker == b.show_gene_marker &&
	      a.gene_name_onoff_method   == b.gene_name_onoff_method &&
	      a.disp_seqno       == b.disp_seqno &&
	      a.disp_range       == b.disp_range &&
	      a.disp_seqticks    == b.disp_seqticks &&
	      a.disp_source      == b.disp_source &&
	      a.min_anchor_score == b.min_anchor_score &&
	      a.max_anchor_score == b.max_anchor_score &&
	      a.clip_hard  == b.clip_hard &&
	      a.annotation_profile == b.annotation_profile &&
	      a.disabled_features == b.disabled_features );
  if(!tmpbool) return false;

  tmpbool = ( tmpbool &&
	      a.hide_anchored_gene == b.hide_anchored_gene &&
	      a.hide_unanchored_gene == b.hide_unanchored_gene &&
	      a.rift_filter_enabled == b.rift_filter_enabled &&
	      a.seqnames == b.seqnames );
  if(!tmpbool) return false;

  for(int i=0; i<MAXSEQ; i++){
    tmpbool = ( tmpbool && 
		(a.segment_offsets[i].size() == b.segment_offsets[i].size()) &&
		(a.toggle_seq[i] == b.toggle_seq[i]) &&
		(a.order_seq[i]  == b.order_seq[i]) &&
		(a.rift_filter_policy[i] == b.rift_filter_policy[i]) &&
		(a.seq_offset[i] == b.seq_offset[i] ) );
    if(!tmpbool) return false;



    for(unsigned j=0; j<a.segment_offsets[i].size(); j++)
      tmpbool = ( tmpbool && 
		  (a.segment_offsets[i].at(j) == b.segment_offsets[i].at(j)));
    if(!tmpbool) return false;
  }

  if( a.plot_type == PLOT_SUPERZOOM )
    tmpbool = 
      tmpbool &&
      (a.superzoom_from == b.superzoom_from ) &&
      (a.superzoom_to   == b.superzoom_to   );

  return tmpbool;
}

bool operator<(const pixbuf_cache_tag& a, const pixbuf_cache_tag& b){
  bool tmpbool = ( a.img_width  < b.img_width &&
		   a.img_height < b.img_height &&
		   a.plot_type  < b.plot_type &&
		   a.stencil    < b.stencil &&
		   a.pfill      < b.pfill &&
		   a.fwd        < b.fwd &&
		   a.bwd        < b.bwd &&
		   a.unknown    < b.unknown &&
		   a.rep        < b.rep &&
		   a.orf        < b.orf &&
		   a.hotspot    < b.hotspot &&
		   a.alpha_by   < b.alpha_by );

  tmpbool = ( tmpbool &&
	      a.color_by   < b.color_by &&
	      a.name_by    < b.name_by &&
	      a.hotspot_by < b.hotspot_by &&
	      a.rift0_col  < b.rift0_col &&
	      a.rift0_col_for_all < b.rift0_col_for_all &&
	      a.rift0_use_alt_seq < b.rift0_use_alt_seq &&
	      a.anno_col   < b.anno_col &&
	      a.show_gene_marker < b.show_gene_marker &&
	      a.gene_name_onoff_method   < b.gene_name_onoff_method &&
	      a.disp_seqno       < b.disp_seqno &&
	      a.disp_range       < b.disp_range &&
	      a.disp_seqticks    < b.disp_seqticks &&
	      a.disp_source      < b.disp_source &&
	      a.min_anchor_score < b.min_anchor_score &&
	      a.max_anchor_score < b.max_anchor_score &&
	      a.clip_hard  < b.clip_hard &&
	      a.annotation_profile < b.annotation_profile &&
	      a.disabled_features < b.disabled_features );

  tmpbool = ( tmpbool &&
	      a.hide_anchored_gene < b.hide_anchored_gene &&
	      a.hide_unanchored_gene < b.hide_unanchored_gene &&
	      a.rift_filter_enabled  < b.rift_filter_enabled &&
	      a.seqnames < b.seqnames );

  for(int i=0; i<MAXSEQ; i++){
    tmpbool = ( tmpbool && 
		(a.segment_offsets[i].size() < b.segment_offsets[i].size()) &&
		(a.toggle_seq[i] < b.toggle_seq[i]) &&
		(a.order_seq[i]  < b.order_seq[i]) &&
		(a.rift_filter_policy[i] < b.rift_filter_policy[i]) &&
		(a.seq_offset[i] < b.seq_offset[i] ) );

    for(unsigned j=0; j<a.segment_offsets[i].size(); j++)
      tmpbool = ( tmpbool && 
		  (a.segment_offsets[i].at(j) < b.segment_offsets[i].at(j)));
  }


  if( a.plot_type < PLOT_SUPERZOOM )
    tmpbool = 
      tmpbool &&
      (a.superzoom_from < b.superzoom_from ) &&
      (a.superzoom_to   < b.superzoom_to   );

  return tmpbool;
}

pixbuf_cache_tag::pixbuf_cache_tag(){
  img_width = img_height = plot_type = 0;
  stencil = pfill = fwd = bwd = unknown = orf = rep = true;

  for(int i=0; i<MAXSEQ; i++){
    toggle_seq[i] = true;
    order_seq[i] =  i;
    seq_offset[i] = 0;
    segment_offsets[i].clear();
    rift_filter_policy[i] = 0;
  }

  rift_filter_enabled = false;
  superzoom_from = superzoom_to = 0;

  alpha_by = color_by = name_by = hotspot_by = gene_name_onoff_method = 0;
  rift0_col.resize(3);
  rift0_col[0] = rift0_col[1] = rift0_col[2] = 0;
  rift0_col_for_all = rift0_use_alt_seq = false;
  
  anno_col.resize(30);
  for(unsigned i=0; i<10; i++) anno_col[i] = RGBA();

  show_gene_marker = hotspot = clip_hard 
    = disp_seqno = disp_range = disp_seqticks = disp_source = true;

  annotation_profile = "";
  disabled_features = "";
  min_anchor_score = max_anchor_score = 0;
  hide_anchored_gene = hide_unanchored_gene = true;
  seqnames="";
}

void pixbuf_cache_tag::get_current_globs(){
  img_width = width();
  img_height = height();
  plot_type = status.plot_type;
  stencil = status.rift_stencil;
  pfill = status.rift_pattern_fill;
  fwd = status.plot_fwd;
  bwd = status.plot_bwd;
  unknown = status.plot_U;
  orf = status.plot_O;
  rep = status.plot_R;
  rift_filter_enabled = mr.rift_filter_enabled;
  for(int i=0; i<MAXSEQ; i++){
    toggle_seq[i] = status.toggle_seq[i];
    order_seq[i] = status.order_seq[i];
    seq_offset[i] = mr.seq_offset[i];
    rift_filter_policy[i] = mr.rift_filter_policy[i];
  }

  superzoom_from = status.superzoom_from;
  superzoom_to   = status.superzoom_to;

  alpha_by         = status.alpha_by;
  color_by         = status.color_by;
  name_by          = status.name_by;
  
  rift0_col        = status.rift0_col;
  rift0_col_for_all = status.rift0_col_for_all;
  rift0_use_alt_seq = status.rift0_use_alt_seq;
  anno_col         = status.anno_col;

  show_gene_marker = status.show_gene_marker;
  gene_name_onoff_method   = status.gene_name_onoff_method;
  hotspot          = status.show_hotspots;
  hotspot_by       = status.hotspot_by;
  disp_range       = status.disp_range;
  disp_seqno       = status.disp_seqno;
  disp_seqticks    = status.disp_seqticks;
  disp_source      = status.disp_source;
  clip_hard        = status.clip_hard;
  annotation_profile = mr.annotationProfile();
  disabled_features = mr.disabledFeatures();
  assert(cout << "cache: " << annotation_profile << endl);

  min_anchor_score = mr.tfidf.histo.get_low();
  max_anchor_score = mr.tfidf.histo.get_high();

  hide_anchored_gene = status.hide_anchored_gene;
  hide_unanchored_gene = status.hide_unanchored_gene;

  seqnames=""; // no "|" at head required!
  unsigned i=0;
  while(mr.getbs_direct(i) != NULL){
    seqnames += mr.getbs_direct(i)->name + "|";
    segment_offsets[i] = mr.segment_offsets[i];
    i++;
  }
}

void pixbuf_cache_tag::bookmark_from_keyval(const GKeyFile* keyval,
					    const char* name){
  char* buf;
  char** lbuf; gboolean* bbuf;
  gsize n;

  buf = g_key_file_get_string((GKeyFile*)keyval, name, "plot_type", NULL);
  plot_type = atoll(buf); g_free(buf);

  buf = g_key_file_get_string((GKeyFile*)keyval, name, "zoom_from", NULL);
  superzoom_from = atoll(buf); g_free(buf);

  buf = g_key_file_get_string((GKeyFile*)keyval, name, "zoom_to", NULL);
  superzoom_to = atoll(buf); g_free(buf);

  lbuf = g_key_file_get_string_list((GKeyFile*)keyval, name, "offset", &n, NULL);
  for(gsize i=0; i<n; i++) seq_offset[i] = atoll(lbuf[i]);
  g_strfreev(lbuf);

  lbuf = g_key_file_get_string_list((GKeyFile*)keyval, name, "order", &n, NULL);
  for(gsize i=0; i<n; i++) order_seq[i] = atoi(lbuf[i]);
  g_strfreev(lbuf);
  
  bbuf = g_key_file_get_boolean_list((GKeyFile*)keyval, name, "toggle", &n, NULL);
  for(gsize i=0; i<n; i++) toggle_seq[i] = bbuf[i];

  // segment offsets: doesn't exist in bookmark files generated by old versions
  lbuf = g_key_file_get_string_list((GKeyFile*)keyval, name, 
				    "segment_offset", &n, NULL);
  if(lbuf!=NULL){
    unsigned s=0;
    for(gsize i=0; i<n; i+=3){
      while(atoll(lbuf[i])==-1){ // avoid "segment_offset=-1;-1;-1"
	i++;
	s++;
	if( i>=n ) break;
      }
      if( i>=n ) break;
      segment_offsets[s].push_back(atoll(lbuf[i]));
      segment_offsets[s].push_back(atoll(lbuf[i+1]));
      segment_offsets[s].push_back(atoll(lbuf[i+2]));
    }
    g_strfreev(lbuf);
  }

  g_free(bbuf);
}

string pixbuf_cache_tag::bookmark_to_string(){
  stringstream ss;
  int seqs = mr.acols()/3;
  
  ss << "plot_type=" << plot_type << "\n"
     << "zoom_from=" << superzoom_from << "\n"
     << "zoom_to=" << superzoom_to << "\n";

  ss << "toggle=";  
  for(int i=0; i<seqs; i++)
    ss << (toggle_seq[i] ? "true" : "false")
       << (i != seqs-1 ? ";" : "\n");
  
  ss << "order=";
  for(int i=0; i<seqs; i++)
    ss << order_seq[i] << (i != seqs-1 ? ";" : "\n");

  ss << "offset=";
  for(int i=0; i<seqs; i++)
    ss << seq_offset[i] << (i != seqs-1 ? ";" : "\n");

  // segment offset: pos;pos;offset;-1;pos;pos;offset;...
  ss << "segment_offset=";
  for(int i=0; i<seqs; i++){
    for(unsigned j=0; j<segment_offsets[i].size(); j++)
      ss << segment_offsets[i].at(j) << ";";
    ss << "-1" << (i != seqs-1 ? ";" : "\n");
  }
  
  return ss.str();
}

void pixbuf_cache_tag::bookmark_to_status(){
  // subset of set_to_status(): does only essential stuff
  status.plot_type = plot_type;
  status.superzoom_from = superzoom_from;
  status.superzoom_to   = superzoom_to;

  mr.clear_offset();
  for(int i=0; i<MAXSEQ; i++){
    status.toggle_seq[i] = toggle_seq[i];
    status.order_seq[i] = order_seq[i];
    mr.set_offset(i, seq_offset[i]);
    mr.segment_offsets[i] = segment_offsets[i];
  }
  mr.rebuild_segment_offset_index();
}

void pixbuf_cache_tag::set_to_status(){
  bookmark_to_status();

  zoomer->jump_by_width(img_width); // set width & height

  status.rift_stencil = stencil;
  status.rift_pattern_fill = pfill;
  status.plot_fwd = fwd;
  status.plot_bwd = bwd;
  status.plot_U = unknown;
  status.plot_R = rep;
  status.plot_O = orf;
  status.show_hotspots = hotspot;

  status.alpha_by         = alpha_by;
  status.color_by         = color_by;
  status.name_by          = name_by;

  status.rift0_col        = rift0_col;
  status.rift0_col_for_all = rift0_col_for_all;
  status.rift0_use_alt_seq = rift0_use_alt_seq;
  status.anno_col         = anno_col;

  status.show_gene_marker = show_gene_marker;
  status.gene_name_onoff_method = gene_name_onoff_method;
  status.hotspot_by       = hotspot_by;
  status.clip_hard        = clip_hard;
  status.disp_seqno       = disp_seqno;
  status.disp_range       = disp_range;
  status.disp_seqticks    = disp_seqticks;
  status.disp_source      = disp_source;

  mr.rift_filter_enabled = rift_filter_enabled;
  for(int i=0; i<MAXSEQ; i++) mr.rift_filter_policy[i] = rift_filter_policy[i];


  assert(cout << "import: " << annotation_profile << endl);
  mr.annotationProfileImport(annotation_profile);
  mr.disabledFeaturesImport(disabled_features);

  mr.tfidf.histo.fixhisto(min_anchor_score, max_anchor_score);

  unsigned i=0;
  vector<string> snvec = split(seqnames, "|");
  while(mr.getbs_direct(i) != NULL){
    mr.getbs_direct(i)->name = snvec[i];
    i++;
  }
}

unsigned int pixbuf_cache_tag::bytesize(){
  return (unsigned int)(img_width * img_height);
}

// ----------------------------------------------------------------------
// Pixbuf Cache item

unsigned int pixbuf_cache_item::bytesize(){
  return tag.bytesize();
}

// ----------------------------------------------------------------------
// Pixbuf Caching

unsigned int pixbuf_cache::total_bytes(){
  unsigned int s=0;
  for(unsigned int i=0; i<c.size(); i++){
    s += c[i].bytesize();
  }
  return s;
}

bool pixbuf_cache::request_size(unsigned int s){
  unsigned int required = total_bytes() + s;
  int entries = (int)c.size();

  assert(cout 
	 << "pixbuf_cache::request_size(): current=" << total_bytes()
	 << " requested=" << s 
	 << " total=" << required << endl);

  // remove the oldest one 
  while(required > PBCACHE_CAPACITY){
    if(entries==0) return false;   // couldn't allocate...
    for(vector<pixbuf_cache_item>::iterator i=c.begin(); i!=c.end(); i++){
      if(i->age==entries){
	assert(cout 
	       << "pixbuf_cache::request_size(): remove " << i->age
	       << " (size " << i->bytesize() 
	       << ") / size will be" << required-i->bytesize() << endl);
	required -= i->bytesize();
	c.erase(i);
      }
    }
    entries--;
  }

  return true;

}

pixbuf_cache_item* pixbuf_cache::lookup_cache(pixbuf_cache_tag t){
  for(unsigned int i=0; i<c.size(); i++){
    if(c[i].tag == t){
      return &c[i];
    }
  }

  return &c[0]; // this shouldn't happen
}

gene_markers_type pixbuf_cache::lookup_gm(pixbuf_cache_tag t){
  for(unsigned int i=0; i<c.size(); i++){
    if(c[i].tag == t){
      return c[i].gene_markers;
    }
  }
  
  return gene_markers_type(); // this shouldn't happen
}

GdkPixbuf* pixbuf_cache::lookup(pixbuf_cache_tag t){
  assert(cout << "pixbuf_cache::lookup(): lookup " << t.img_width << "x" << t.img_height << endl);
  assert(sanityCheck());
  assert(cout << "pixbuf_cache::lookup(): current size = " << total_bytes() <<endl);

  for(unsigned int i=0; i<c.size(); i++){
    if(c[i].tag == t){
      if(c[i].age != 0){
	assert(cout << "re-aging " << i << endl);
	int old_ci_age = c[i].age;
	for(unsigned int j=0; j<c.size(); j++)  if(c[j].age < old_ci_age) c[j].age++;
      }
      c[i].age = 0;

      assert(cout << "pixbuf_cache::lookup(): found!" << endl);
      assert(sanityCheck());
      return c[i].pixbuf;
    }
  }
  assert(cout << "pixbuf_cache::lookup(): not found..." << endl);
  return NULL;
}

double pixbuf_cache::lookup_bw(pixbuf_cache_tag t){

  for(unsigned int i=0; i<c.size(); i++){
    if(c[i].tag == t){
      return c[i].basewidth;
    }
  }
  assert(cout << "pixbuf_cache::lookup(): not found..." << endl);
  return 0;
}


void pixbuf_cache::add(pixbuf_cache_tag t, GdkPixbuf* pb, double bw, 
		       gene_markers_type gm){
  // NOTE: call this function after displaying the pixbuf

  assert(cout << "pixbuf_cache::add(): adding " << t.img_width << "x" << t.img_height << endl);
  assert(sanityCheck());  

  vector<pixbuf_cache_item>::iterator i;
  vector<pixbuf_cache_item>::iterator oldest;

  // if already registered, do nothing
  if( lookup(t)!=NULL ){
    assert(cout << "pixbuf_cache::add(): already registered." << endl);
    return;
  }

  // if full capacity, delete the oldest
  /*
  if(c.size()==PBCACHE_MAX){
    int  a=0, j=0, oldestj=0;

    for(i=c.begin(); i!=c.end(); i++){
      if(a< i->age){ a=i->age; 
	oldest= i; 
	oldestj=j; }
      j++;
    }
    assert(cout << "pixbuf_cache::add(): deleting entry #" << oldestj << endl);
    gdk_pixbuf_unref(oldest->pixbuf);
    c.erase(oldest);
  }
  */

  // try to allocate
  if(!request_size(t.bytesize())){
    cerr << "pixbuf_cache::add() : failed to add... (image too big?)" << endl;
    exit(-1);
  }
  
  // then register
  for(i=c.begin(); i!=c.end(); i++)
    i->age++; 

  c.push_back(pixbuf_cache_item(t, 0, pb, bw, status.gene_markers, status.in_range_anchors));

  assert(sanityCheck());
}

void pixbuf_cache::clear(){
  assert(cout << "pixbuf_cache::clear()" << endl);
  for(int i=(int)c.size()-1; i>=0; i--)
    gdk_pixbuf_unref(c[i].pixbuf);
  c.clear();

  assert(sanityCheck());
}

int pixbuf_cache::sanityCheck(){
  vector<pixbuf_cache_item>::iterator i;

  cout << "pixbuf_cache::sanityCheck(): age list: ";
  for(unsigned int i=0; i<c.size(); i++)
    cout << c[i].age << "(" << c[i].tag.img_width << "x" << c[i].tag.img_height <<  ")" << " ";
  cout << endl;

  for(int a=0; a<(int)c.size(); a++ ){
    for( i=c.begin(); i!=c.end(); i++){
      if( i->age == a ) break;
    }
    if( i==c.end() ) {
      cerr << "pixbuf_cache::sanityCheck(): age inconsistency!" << endl;
      return false;
    }
  }
  return true;
}

