#ifndef _PIXBUF_CACHE_H_
#define _PIXBUF_CACHE_H_

using namespace std;

#include <iostream>
#include <vector>
#include <gtk/gtk.h>

#include "gmv_defs.h"
#include "plotters.h"
#include "opts.h"
#include "annotation.h"

#define PBCACHE_MAX 5
#define PBCACHE_CAPACITY ((unsigned int)status.pbcache_cap*1024*1024)

// ----------------------------------------------------------------------
// tag

class pixbuf_cache_tag {
 public:
  int img_width;
  int img_height;
  int plot_type;
  bool stencil, pfill;
  bool fwd;
  bool bwd;
  bool unknown;
  bool rep;
  bool orf;
  bool hotspot;
  bool toggle_seq[MAXSEQ];
  seqlen_t  seq_offset[MAXSEQ];
  seqlen_t  order_seq[MAXSEQ];
  vector<seqlen_t> segment_offsets[MAXSEQ];
  bool rift_filter_enabled;
  int  rift_filter_policy[MAXSEQ];

  int  alpha_by, color_by, name_by, gene_name_onoff_method, hotspot_by;
  vector<double> rift0_col;
  bool rift0_col_for_all;
  bool rift0_use_alt_seq;
  vector<RGBA> anno_col;

  bool show_gene_marker;
  bool disp_seqno;
  bool disp_range;
  bool disp_seqticks;
  bool disp_source;
  bool clip_hard;
  double min_anchor_score, max_anchor_score;
  string annotation_profile, disabled_features;
  bool hide_unanchored_gene, hide_anchored_gene;
  string seqnames;

  seqlen_t superzoom_from, superzoom_to; // PLOT_SUPERZOOM

  friend bool operator==(const pixbuf_cache_tag& a, const pixbuf_cache_tag& b);
  friend bool operator< (const pixbuf_cache_tag& a, const pixbuf_cache_tag& b);

  //  pixbuf_cache_tag(int w, int h, int p, bool f, bool b, bool n, bool r);
  pixbuf_cache_tag();
  void get_current_globs(); // get current global status
  void set_to_status();
  void bookmark_to_status();
  string bookmark_to_string();
  void bookmark_from_keyval(const GKeyFile*, const char*);
  unsigned int bytesize();
};


class pixbuf_cache_item{
public:
  pixbuf_cache_tag tag;
  int age;
  GdkPixbuf* pixbuf;
  double basewidth;

  gene_markers_type gene_markers;
  set<int> in_range_anchors;

  pixbuf_cache_item(pixbuf_cache_tag t, int a, GdkPixbuf* pb, double bw, 
		    gene_markers_type gm, set<int> ia){ 
    tag=t;
    age=a;
    pixbuf=pb;
    basewidth=bw;
    gene_markers=gm;
    in_range_anchors=ia;
  };

  unsigned int bytesize();
};

class pixbuf_cache{
public:
  pixbuf_cache(){};
 
  GdkPixbuf* lookup(pixbuf_cache_tag t);
  pixbuf_cache_item* lookup_cache(pixbuf_cache_tag t);
  gene_markers_type lookup_gm(pixbuf_cache_tag t);
  double lookup_bw(pixbuf_cache_tag t);
  void add(pixbuf_cache_tag, GdkPixbuf*, double, gene_markers_type);
  int sanityCheck();
  void clear();

private:
  vector <pixbuf_cache_item> c;
  unsigned int total_bytes();
  bool request_size(unsigned int);
};

#endif
