#ifndef _RANGE_LIST_H_
#define _RANGE_LIST_H_

#include <iostream>
#include <sstream>
#include <set>

using namespace std;

typedef long long int fromto_t;

class fromto{
public:
  fromto_t from;
  fromto_t to;

  fromto(fromto_t f, fromto_t t){ from=f; to=t; };
  bool overlap_f (fromto a) const { return(a.from<from  && from<=a.to+1); };
  bool overlap_t (fromto a) const { return(a.from<=to+1 && to< a.to); };
  bool covered_by(fromto a) const { return(a.from<=from && to<=a.to); };
  bool covers_on (fromto a) const { return(from<=a.from && a.to<=to); };
  void replace_by(fromto a){ from=a.from; to=a.to; };

  bool coalesceable(fromto a) const{
//     if(covered_by(a)) cout << "C"; 
//     if(overlap_f(a)) cout << "F"; 
//     if(overlap_t(a)) cout << "T"; 

    if(covered_by(a) || overlap_f(a) || overlap_t(a)) return true;
    return false;
  }     

  void coalesce(fromto a){
    // cout << "coalesce:" << *this << "+" << a << endl;
    if (covered_by(a)){ replace_by(a); return;}
    if (overlap_f (a)){ from = a.from; return;}
    if (overlap_t (a)){ to   = a.to;   return;}
  }

  friend ostream& operator<<(ostream &os, const fromto a){
    return os << "[" << a.from << "," << a.to << "]";  }

  friend bool operator<(const fromto a, const fromto b){
    return(a.from < b.from);
  }
};


class range_list{
public:
  typedef multiset<fromto>::iterator ListIte;
  multiset<fromto> l;

  ListIte coalesce(ListIte i, ListIte j){
    fromto a=*i;
    fromto b=*j;
    l.erase(i);
    l.erase(j);
    
    a.coalesce(b);
    ListIte me=l.insert(l.begin(), a);
    
    ListIte next=me; next++;
    if(next!=l.end())
      if(next->coalesceable(*me)) coalesce(next, me);
    return me;
  }

  void add(const fromto a){
    //    cout << "add: " << *this << "+" << a ;
    //  cout << "add: "<< a <<endl;
    ListIte me=l.insert(l.begin(), a);
    //ListIte me=l.find(a);
    for(ListIte i=l.begin(); i!=l.end(); i++){
      if(i==me) continue;
      if(i->covers_on(a)){ l.erase(me); continue; }
      if(i->coalesceable(a)){ 
	ListIte prev, next;
	
	me=coalesce(i, me);
	break;
      }
    }
    //    cout << "=" << *this << endl;
  }	

  fromto_t total_length(){
    fromto_t len=0;
    for(ListIte i=l.begin(); i!=l.end(); i++)
      len+=i->to-i->from;
    return len;
  }

  int size(){
    return l.size();
  }

  friend ostream& operator<<(ostream &os, const range_list r){
    ostringstream ost;
    for(ListIte i=r.l.begin(); i!=r.l.end(); i++)
      ost << *i << " ";
    return os << ost.str();
  }
};


///add: [0,17] [54,71] +[36,53]=[0,17] [36,53] [54,71]
//1 add: [0,17] [36,71] +[36,53]=[0,17] [36,53]

// int main(){
//   range_list r;
  
//   r.add(fromto(0,17));
//   cout  << r << endl;

//   r.add(fromto(36,71));
//   cout << r <<  endl;

//   r.add(fromto(36,53));
//   cout << r <<  endl;

//   return 0;
// }



#endif
