using namespace std;

#ifndef _REORDERABLE_VECTOR_H_
#define _REORDERABLE_VECTOR_H_

#include <vector>

template <typename T> class rvIterator;

template <typename T> class rvector{
public:
  static const unsigned npos = 0-1;
  typedef rvIterator<T> iterator;

  rvector(){ clear(); };

  void clear(); 

  void push_back(T a);
  void pop_back();
  iterator erase(unsigned);
  iterator erase(iterator);

  unsigned realsize() const;
  unsigned size() const; 

  T& at(unsigned i);
  T& at(unsigned i, bool d);
  T& operator[](unsigned i);

  iterator begin(){ return rvIterator<T>(this, 0); };
  iterator end(){ return rvIterator<T>(this, size()); };

  // direct access
  iterator begin_all(){ return rvIterator<T>(this, 0, true); };
  iterator end_all(){ return rvIterator<T>(this, realsize(), true); };

  T& at_direct(unsigned i);
  iterator erase_direct(unsigned);

  // status control
  void activate(unsigned i);
  void deactivate(unsigned i);
  bool get_active(unsigned i){ return(active[i]); }; 
  bool get_active(iterator i){ return(active[i.pos]); }; // "direct" iterator only

  void promote(unsigned i);
  void demote(unsigned i);

  string profileString();
  
  // importing orders and activities
  void load_actives(bool* t);
  void load_order(int* o);

  // for debugger
  bool sanityCheck();

private:
  vector <T> v;
  vector <unsigned> order;
  vector <bool> active;

  void fixate_order();
  unsigned lookup(unsigned); // order lookup
};

template <typename T> class rvIterator{
public:
  rvIterator(){};
  rvIterator( const rvIterator<T>& a ){ rv=a.rv; pos=a.pos; direct=a.direct; };
  rvIterator( rvector<T>* a, unsigned p ){ rv=a; pos=p; direct=false; };
  rvIterator( rvector<T>* a, unsigned p, bool d ){ rv=a; pos=p; direct=d; };

  T& operator=(const rvIterator<T>& a){ 
    rv=a.rv; pos=a.pos; direct=a.direct; 
    return rv->at(pos,direct); }; // was *rv->at(pos,direct). is it true???

  T& operator*(){ return rv->at(pos, direct); };
  T* operator->(){ return &rv->at(pos, direct); };

  T& operator++(){ pos++; return rv->at(pos, direct); };
  T& operator--(){ pos--; return rv->at(pos, direct); };
  T& operator++(int){ pos++; return rv->at(pos-1, direct);  };
  T& operator--(int){ pos--; return rv->at(pos+1, direct);  };

  bool operator==(const rvIterator<T>& a){
    return pos==a.pos; };
  bool operator!=(const rvIterator<T>& a){
    return pos!=a.pos; };
  
  unsigned pos;
  rvector<T>* rv;
  bool direct;
};

// ----------------------------------------------------------------------
// public functions

template <typename T> void rvector<T>::clear(){
  v.clear(); order.clear(); active.clear(); 
  fixate_order();
}

template <typename T> typename rvector<T>::iterator rvector<T>::erase(iterator a){
  return( a.direct ? erase=_direct(a.pos) : erase(a.pos));
}

template <typename T> typename rvector<T>::iterator rvector<T>::erase(unsigned a){
  erase_direct(lookup(a));
  return(rvIterator<T>(this, a));
}

template <typename T> typename rvector<T>::iterator rvector<T>::erase_direct(unsigned a){
  unsigned c=0;
  for(typename vector<T>::iterator i=v.begin(); i!=v.end(); i++){
    if(a==c){ v.erase(i); break; } c++; }
  
  c=0;
  for(typename vector<bool>::iterator i=active.begin(); i!=active.end(); i++){
    if(a==c){ active.erase(i); break; } c++; }

  c=0;
  for(typename vector<unsigned>::iterator i=order.begin(); i!=order.end(); i++){
    if(a==c){ order.erase(i); break; } c++; }

  fixate_order();
  return(rvIterator<T>(this, a, true));
}


template <typename T> void rvector<T>::push_back(T a){
  v.push_back(a); 
  order.push_back(order.size()); 
  active.push_back(true); 
}

template <typename T> void rvector<T>::pop_back(){
  v.pop_back(); 
  order.pop_back();
  active.pop_back();
}

template <typename T> unsigned rvector<T>::realsize() const {
  return v.size(); };

template <typename T> unsigned rvector<T>::size() const { 
  unsigned s=0; 
  for(unsigned i=0; i<v.size(); i++) if(active[i]) s++; 
  return s; 
}

template <typename T> T& rvector<T>::at(unsigned i){ return v.at(lookup(i)); };
template <typename T> T& rvector<T>::at(unsigned i, bool d){ 
  return v.at(d ? i : lookup(i)); };

template <typename T> T& rvector<T>::at_direct(unsigned i){ return v.at(i); };


template <typename T> T& rvector<T>::operator[](unsigned i){ return v[lookup(i)]; };

template <typename T> void rvector<T>::activate(unsigned i){ active[i]=true; };
template <typename T> void rvector<T>::deactivate(unsigned i){ active[i]=false; };

template <typename T> void rvector<T>::promote(unsigned i){ 
  if(i>0){ int a=order[i-1]; order[i-1]=order[i]; order[i]=a; } 
}

template <typename T> void rvector<T>::demote(unsigned i){
  if(i<order.size()-1) { int a=order[i+1]; order[i+1]=order[i]; order[i]=a; } 
}

template <typename T> string rvector<T>::profileString(){
  string prof="";
  char tmpstr[10];

  for(vector<unsigned>::iterator i=order.begin(); i!=order.end(); i++){
    snprintf(tmpstr, 9, "%d", *i);
    prof=prof+tmpstr+"|";
  }

  for(vector<bool>::iterator i=active.begin(); i!=active.end(); i++)
    prof=prof+( (*i) ? "1" : "0" )+"|";

  return prof;
}
  
template <typename T> void rvector<T>::load_actives(bool* t){
  for(unsigned i=0; i<active.size(); i++) active[i]=t[i]; 
}
  
template <typename T> void rvector<T>::load_order(int* o){
  for(unsigned i=0; i<order.size(); i++) order[i]=o[i]; 
  fixate_order();
}

template <typename T> bool rvector<T>::sanityCheck(){
  for(unsigned i=0; i<size(); i++)
    cout << "[" << i << "] " << active[lookup(i)] << " " << v[lookup(i)] << endl;

  return(v.size()==order.size() && order.size()==active.size());
}



// ----------------------------------------------------------------------
// private functions

template <typename T> void rvector<T>::fixate_order(){
  vector<bool> chk(order.size());

  for(unsigned i=0; i<order.size(); i++) chk[i]=false;
    
  for(unsigned i=0; i<order.size(); i++){
    unsigned smallest=(unsigned)0-1;
    unsigned smallesti=smallest;
    for(unsigned j=0; j<order.size(); j++){
      if(order[j]<smallest && chk[j]==false){
	smallesti=j; smallest=order[j]; };
    }
    chk[smallesti]=true;
    order[smallesti]=i;
  }
}

template <typename T> unsigned rvector<T>::lookup(unsigned a){
  unsigned cnt = 0;
  for(unsigned i=0; i<active.size(); i++){
    if(active[order[i]]){
      if(a==cnt){
	return order[i];
      }
      cnt++;
    }
  }
  return npos;
}
#endif
