using namespace std;

#include <stdlib.h>
#include <gtk/gtk.h>
#include <string>
#include <string.h>
#include <iostream>
#include <fstream>
#include <sstream>
#include <vector>
#include <sys/types.h>
#include <glib/gstdio.h>
#include <gmv_base.h>
#include <ctype.h>

#if defined (__MINGW32__)
#include <windef.h>
#include <windows.h>
#endif

#include "util.h"

#if defined (__FreeBSD__)  || (__APPLE__)
# include <sys/types.h>
# include <sys/sysctl.h>
#endif

#if defined (__linux__)
# include <unistd.h>
#endif

// ----------------------------------------------------------------------
// compare two files' timestamps

bool check_timestamp(const string& older, const string& newer){
  struct stat stat_old, stat_new;

  bool time_is_ok = false;

  if( g_stat(older.c_str(), &stat_old) == 0 &&
      g_stat(newer.c_str(), &stat_new) == 0 ){
#if defined (__FreeBSD__) || defined (__APPLE__)
    time_is_ok = ( stat_old.st_mtimespec.tv_sec <=
		   stat_new.st_mtimespec.tv_sec );
#else
    time_is_ok = ( stat_old.st_mtime <= stat_new.st_mtime );
#endif
  }

  return time_is_ok;
}

// ----------------------------------------------------------------------
// gdk_threads

void lock_gdk(){
#ifndef __MINGW32__
      gdk_threads_enter();
#endif
}

void unlock_gdk(){
#ifndef __MINGW32__
      gdk_threads_leave();
#endif
}

// ----------------------------------------------------------------------
// swap and some math alternatives

#ifndef MICROBIAL
void swap(seqlen_t& a, seqlen_t& b){ seqlen_t c; c=a; a=b; b=c; };
#endif

void swap(int& a, int& b){ int c; c=a; a=b; b=c; };

#ifdef __FreeBSD__
long long int abs(long long int a){
  return (a<0) ? -a : a;;
}
#endif

// ----------------------------------------------------------------------
// checkbox with flag variable synchronization

void chkbox_toggled(GtkWidget* w, gpointer s){
  *(bool*)s = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(w));
}

GtkWidget* chkbox_setup(bool *b, const char* s){
  GtkWidget* w = gtk_check_button_new_with_mnemonic(s);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(w), *b);
  g_signal_connect(w, "toggled", G_CALLBACK(chkbox_toggled), (void*)b);
  
  return w;
}

// ----------------------------------------------------------------------
// decode URL-like encoded string, like as %20%31

string decodecharcode(const string& s){
  size_t pos=0;
  size_t len=s.size();
  string ss="";

  do{
    if(s[pos]!='%'){
      ss += s[pos];
    } else {
      if(pos+3 >= len) ss+=s.substr(pos); // just copy when run over the end
      else {
	char c=0;
	bool invalid=false;
	for(size_t p=1; p<=2; p++){
	  char cc=s[pos+p];
	  if(islower(cc)) cc-= 0x20;
	  
	  if(isdigit(cc)) c += cc-0x30;
	  else if('A' <= cc && cc <= 'F') c += cc-55;
	  else invalid = true;

	  if(p==1) c*=16;
	}

	if(invalid)
	  ss+=s.substr(pos, 3);
	else
	  ss+=c;
      }
      pos+=2; // move forward by 2 ( +1 at the end of this loop)
    }
  } while(++pos < len);
  return ss;
}

//----------------------------------------------------------------------
// integer -> string

string itos(double i, int keta){
  ostringstream os;
  os.setf(ios::fixed);
  os.precision(keta);
  os << i;
  return os.str();
}

string itos(double i){
  ostringstream os;
  os << i;
  return os.str();
}

string itos(unsigned i){
  ostringstream os;
  os << i;
  return os.str();
}

string itos(long long int i){
  ostringstream os;
  os << i;
  return os.str();
}

string itos(int i){
  ostringstream os;
  os << i;
  return os.str();
}

string itos_comma(string ss){
  char *s1=strdup(ss.c_str());
  char *s2=(char*)malloc(strlen(s1) + strlen(s1)/3 + 3);
  char *s1a=s1;

  s2[0]=0;

  /*
     0 1 2 3 
     A,B C D
   */

  for(int i=strlen(s1); i>0; i--){
    if(i%3==0 && i!=(int)strlen(s1)) strcat(s2, ",");
    strncat(s2, s1a, 1);
    s1a++;
  }

  string s = string(s2);

  free(s1);
  free(s2);

  return s;  
}

string itos_comma(long long int i){
  ostringstream os;
  os << i;
  return itos_comma(os.str());
}

// ----------------------------------------------------------------------
// progressbar update inteval calculation

unsigned mkinterval::setval(unsigned a){
  unsigned i=1;
  unsigned int b=0;

  while(i<a){ i=i*2+1; b++; if(b>sizeof(unsigned)*8) break;}

  i = i >> 7;

  itv = i;
  return i;
}


//----------------------------------------------------------------------
// system tweaks


#if defined (__MINGW32__)
LPSYSTEM_INFO win_sysinfo(LPSYSTEM_INFO lpsysinfo)
{
  HMODULE	kdll;

  void (CALLBACK* getnsi)(LPSYSTEM_INFO);
  memset(lpsysinfo, sizeof(SYSTEM_INFO), 0);
	
  kdll = LoadLibrary("Kernel32.dll");
  if(kdll == NULL) return NULL;

  (*(FARPROC*)&getnsi) = 
    GetProcAddress(kdll,"GetNativeSystemInfo");

  if(getnsi == NULL){
    FreeLibrary(kdll);
    return  NULL;
  }

  getnsi(lpsysinfo);
  FreeLibrary(kdll);

  return lpsysinfo;
}
#endif

int num_cpus(){
  int n = 1;
  return n; // debug

#if defined (__MINGW32__)
  SYSTEM_INFO sinfo;
  if(win_sysinfo(&sinfo) != NULL) n = (int)sinfo.dwNumberOfProcessors;
#endif

#if defined (__FreeBSD__)  || (__APPLE__)
  size_t len = sizeof(n);
  if (sysctlbyname("hw.ncpu", &n, &len, NULL, 0) < 0) n = 1;
#endif

#if defined (__linux__)
  long nn = sysconf(_SC_NPROCESSORS_ONLN);
  if(nn < 1) nn=1;
  n = (int)nn;
#endif

#if defined (__sun__)
  long nn = sysconf(_SC_NPROCESSORS_CONF);
  if(nn < 1) nn=1; // failsafe
  n = (int)nn;
#endif

  n=2;
  return n;
}

//----------------------------------------------------------------------
// GLib key-vale file tweaks

bool pref::load(string f){
  // will fail if empty, but please don't care...
  g_key_file_set_list_separator(pf, ';');
  return g_key_file_load_from_file(pf, f.c_str(), G_KEY_FILE_NONE, NULL);
}

bool pref::save(string f){
  ofstream pf_s;
  pf_s.open(f.c_str());
  g_key_file_set_list_separator(pf, ';');

  if(pf_s){
    gchar* p = g_key_file_to_data(pf, NULL, NULL);
    pf_s << p;
    g_free(p);
    pf_s.close();
    return true;
  }

  return false;
}

// setters

void pref::setpref(const char* group, int& val, const char* name){
  g_key_file_set_integer(pf, group, name, val);
}

void pref::setpref(const char* group, double& val, const char* name){
  g_key_file_set_double(pf, group, name, val);
}

void pref::setpref(const char* group, bool val, const char* name){
  g_key_file_set_boolean(pf, group, name, val);
}

void pref::setpref(const char* group, string& val, const char* name){
  g_key_file_set_string(pf, group, name, val.c_str());
}

void pref::setpref(const char* group, vector<double>& val, const char* name){
  g_key_file_set_double_list(pf, group, name, &val[0], val.size());
}

void pref::setpref(const char* group, vector<RGBA>& val, const char* name){
  vector<double> templist;
  for(size_t i=0; i<val.size(); i++){
    templist.push_back(val[i].r());
    templist.push_back(val[i].g());
    templist.push_back(val[i].b());
    templist.push_back(val[i].a());
  }
  
  g_key_file_set_double_list(pf, group, name, &templist[0], templist.size());
}

// getters

void pref::getpref(const char* group, int& val, const char* name){
  GError *err = NULL;
  int v = g_key_file_get_integer(pf, group, name, &err);
  if(err == NULL) val = v;
}

void pref::getpref(const char* group, double& val, const char* name){
  GError *err = NULL;
  double v = g_key_file_get_double(pf, group, name, &err);
  if(err == NULL) val = v;
}

void pref::getpref(const char* group, bool& val, const char* name){
  GError *err = NULL;
  int v = g_key_file_get_boolean(pf, group, name, &err);
  if(err == NULL) val = v;
}

void pref::getpref(const char* group, vector<string>& val, const char* name){
  char **l, **ll;
  gsize useless;
  GError *e = NULL;

  l = g_key_file_get_string_list(pf, group, name, &useless, &e);
  if(l!=NULL){
    ll = l;
    while(*ll != NULL){ val.push_back(*ll); ll++; }
  }
  g_strfreev(l);
}

void pref::getpref(const char* group, vector<double>& val, const char* name){
  gdouble *l;
  gsize listsize;
  GError *e = NULL;

  l = g_key_file_get_double_list(pf, group, name, &listsize, &e);
  if(l!=NULL){
    for(gsize i=0; i<listsize; i++) val.push_back(l[i]);
  } else {
    cerr << ( (e->code == G_KEY_FILE_ERROR_KEY_NOT_FOUND) ? "KEY_NOT_FOUND" 
	      : "INVALID_VALUE") << ": " << e->message << endl;
  }
}

void pref::getpref(const char* group, vector<RGBA>& val, const char* name){
  gdouble *l;
  gsize listsize;
  GError *e = NULL;

  vector<double> templist;

  l = g_key_file_get_double_list(pf, group, name, &listsize, &e);
  if(l!=NULL){
    for(gsize i=0; i<listsize; i++) templist.push_back(l[i]);
  } else {
    cerr << ( (e->code == G_KEY_FILE_ERROR_KEY_NOT_FOUND) ? "KEY_NOT_FOUND" 
	      : "INVALID_VALUE") << ": " << e->message << endl;
  }
 
  if( templist.size() % 4 != 0 ){
    cerr << "Color setting is broken. "
	 << "Stop to get color list from preference.\n";
    return;
  }

  val.clear();
  for(size_t i=0; i<templist.size(); i+=4)
    val.push_back(RGBA(templist, i));
}
