using namespace std;

#include <gtk/gtk.h>
#include <vector>
#include <iostream>
#include "gmv_defs.h"

#ifndef _UTIL_H_
#define _UTIL_H_


// ----------------------------------------------------------------------
// compare file timestamp

bool check_timestamp(const string&, const string&);

// ----------------------------------------------------------------------
// gdk_threads

void lock_gdk();
void unlock_gdk();

// ----------------------------------------------------------------------
// checkbox handling 

GtkWidget* chkbox_setup(bool*, const char*);

// ----------------------------------------------------------------------
// swap and math alternatives

#ifndef MICROBIAL
void swap(seqlen_t&, seqlen_t&);
#endif
void swap(int&, int&);

#ifdef __FreeBSD__
long long int abs(long long int a);
#endif

// ----------------------------------------------------------------------
// decode URL-like encoded string, like as %20%31

string decodecharcode(const string& s);

// ----------------------------------------------------------------------
// integer -> string

string itos(double, int);
string itos(long long int);
string itos(int);
string itos(double);
string itos(unsigned);
string itos_comma(long long int);

// ----------------------------------------------------------------------
// progressbar update inteval calculation

class mkinterval{
 public:
  unsigned setval(unsigned);
  bool check(unsigned a) const { return((a&itv) == itv); };
 private:
  unsigned itv;
};

// ----------------------------------------------------------------------
// system tweaks

int num_cpus();

// ----------------------------------------------------------------------
// GLib key-value file tweaks

class RGBA;
class pref{
public:
  pref(){
    pf = g_key_file_new();
    g_key_file_set_list_separator(pf, ';');
  };
  
  ~pref(){
    g_key_file_free(pf);
  };

  bool load(string);
  bool save(string);

  // setters
  void setpref(const char*, int&, const char*);
  void setpref(const char*, double&, const char*);
  void setpref(const char*, bool, const char*);
  void setpref(const char*, string&, const char*);
  void setpref(const char*, vector<double>&, const char*);
  void setpref(const char*, vector<RGBA>&, const char*);

  // getters
  void getpref(const char*, int&, const char*);
  void getpref(const char*, double&, const char*);
  void getpref(const char*,  bool&, const char*);
  void getpref(const char*, vector<string>&, const char*);
  void getpref(const char*, vector<double>&, const char*);
  void getpref(const char*, vector<RGBA>&, const char*);

  // from / to "gmv" section 
  template<typename T> void setpref(T& val, const char* name ){
    setpref("gmv", val, name); };

  template<typename T> void getpref(T& val, const char* name ){
    getpref("gmv", val, name); };

  // from "gmv_auto" section
  template<typename T> void getprefa(T& val, const char* name ){
    getpref("gmv_auto", val, name); };

private:
  GKeyFile* pf;
};

#endif
