using namespace std;

#include "zoom_cursor.h"
#include "gmv.h"

zoom_cursor::zoom_cursor(){
  active=false; 
  button_on=false; 
  dragging=false;
  cursor_inside=true;
}

void zoom_cursor::init(){
  mutex = g_mutex_new();
}

void zoom_cursor::set_yh(int new_y, int new_h){ 
  g_mutex_lock(mutex);
  cursor_y=new_y;
  cursor_h=new_h; 
  g_mutex_unlock(mutex);
}

void zoom_cursor::set_default_w(int new_w){
  g_mutex_lock(mutex);
  default_w=new_w; 
  g_mutex_unlock(mutex);
}

void zoom_cursor::set_bound(int new_x1, int new_x2, int new_y1, int new_y2){
  g_mutex_lock(mutex);
  bd_x1=new_x1; bd_x2=new_x2; bd_y1=new_y1; bd_y2=new_y2; 
  g_mutex_unlock(mutex);
  //     assert(cout 
  // 	   << "zc::set_bound() "
  // 	   << bd_x1 << ", "
  // 	   << bd_x2 << ", "
  // 	   << bd_y1 << ", "
  // 	   << bd_y2 << endl);
}

bool zoom_cursor::check_bound(int x, int y){
  bool inside = false;
  
  if ( bd_x1 <= x && x <= bd_x2 &&  
       bd_y1 <= y && y <= bd_y2) inside = true;
  //    assert(cout << "zc::check_bound() " << x << "," << y << " = " << inside << endl);
  return inside;
}

void zoom_cursor::force_inside(){
  g_mutex_lock(mutex);
  cursor_inside=true;
  g_mutex_unlock(mutex);
}

void zoom_cursor::show(){
//     assert(cout 
// 	   << "zc::show()  x1=" << x1 << " x2=" << x2 
// 	   << " button_on=" << button_on << endl);

  g_mutex_lock(mutex);
  paste();
  
  // copy
  if (!active){
    cpb1 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, false, 8, 1, cursor_h+1);
    cpb2 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, false, 8, 1, cursor_h+1);
    cpb3 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, false, 8, width()+1, 1);
    //  cbp4 = gdk_pixbuf_new(GDK_COLORSPACE_RGB, false, 8, width(), 1);
  }
				      
  gdk_pixbuf_copy_area(pixbuf, x1, cursor_y, 1, cursor_h+1, cpb1, 0, 0);
  gdk_pixbuf_copy_area(pixbuf, x2, cursor_y, 1, cursor_h+1, cpb2, 0, 0);
  gdk_pixbuf_copy_area(pixbuf,  0, cursor_y, width(), 1, cpb3, 0, 0);
  // gdk_pixbuf_copy_area(pixbuf(), 0, cursor_y+height, 1,  width(), cpb4, 0, 0);
  prev_x1 = x1;
  prev_x2 = x2;
    
  // draw
  GdkGC* gc = canvas->style->fg_gc[GTK_STATE_NORMAL];
  GdkColor gc_fore = {0,0,0,0}; // black
  gdk_colormap_alloc_color(gdk_colormap_get_system(), 
			   &gc_fore, true, true);
  gdk_gc_set_foreground(gc, &gc_fore);


  int x = min(x1, x2);
  int y = cursor_y;
  int w = abs(x2-x1);
  int h = cursor_h;
  
  gdk_draw_rectangle(canvas->window, gc, false, x, y, w, h);
  GdkRectangle region = { x+1, y+1, w-2, h-2 };
  gdk_window_invalidate_rect(canvas->window, &region, false);
  active = true;

  g_mutex_unlock(mutex);
}

void zoom_cursor::paste(){
  if (active){
    //       assert(cout 
    // 	     << "zc::paste() x1=" << prev_x1 << " x2=" << prev_x2 << endl);

    GdkGC* gc = canvas->style->fg_gc[GTK_STATE_NORMAL];
    GdkColor gc_fore = {0,65535,65535,65535};  // white
    gdk_colormap_alloc_color(gdk_colormap_get_system(), 
			     &gc_fore, true, true);
    gdk_gc_set_foreground(gc, &gc_fore);

    gdk_draw_rectangle(canvas->window, gc, false,
		       min(prev_x1, prev_x2), cursor_y, 
		       abs(prev_x2-prev_x1), cursor_h);

    gdk_draw_pixbuf(canvas->window,
		    canvas->style->fg_gc[GTK_STATE_NORMAL],
		    cpb1, 0, 0, prev_x1, cursor_y, 1, cursor_h+1, 
		    GDK_RGB_DITHER_NORMAL, 0, 0 );
    gdk_draw_pixbuf(canvas->window,
		    canvas->style->fg_gc[GTK_STATE_NORMAL],
		    cpb2, 0, 0, prev_x2, cursor_y, 1, cursor_h+1, 
		    GDK_RGB_DITHER_NORMAL, 0, 0 );
    gdk_draw_pixbuf(canvas->window,
		    canvas->style->fg_gc[GTK_STATE_NORMAL],
		    cpb3, 0, 0, 0, cursor_y, width(), 1, 
		    GDK_RGB_DITHER_NORMAL, 0, 0 );

    gc_fore.pixel = gc_fore.red = gc_fore.green = gc_fore.blue = 0;
    gdk_colormap_alloc_color(gdk_colormap_get_system(), 
			     &gc_fore, true, true);
    gdk_gc_set_foreground(gc, &gc_fore);

  }
}

void zoom_cursor::pressed(int x, int y){
  g_mutex_lock(mutex);
  p_x = x;
  if(check_bound(x, y)){ 
    x1 = x;
    x2 = x+default_w;
  } else {
    if(x>=bd_x2){ x1=bd_x2; x2=bd_x2-default_w; }
    if(x<=bd_x1){ x1=bd_x1; x2=bd_x1+default_w; }
  }
  button_on = true; 

  //     assert(cout 
  // 	   << "zc::pressed()  x1=" << x1 << " x2=" << x2 
  // 	   << " button_on=" << button_on << endl);
  g_mutex_unlock(mutex);
}

void zoom_cursor::force_release(){
  g_mutex_lock(mutex);
  button_on = false;
  g_mutex_unlock(mutex);
}

bool zoom_cursor::released(int x, int y){
  force_release();

  g_mutex_lock(mutex);
  //    if(!active) return false;

  r_x = x;
  //     assert(cout 
  // 	   << "zc::released()  x1=" << x1 << " x2=" << x2 
  // 	   << " button_on=" << button_on << endl);

  if(!cursor_inside){ g_mutex_unlock(mutex); return FALSE; }
  //    if(!active) return false; // someone may drag from outside

  int new_x = x;
  if(new_x>=bd_x2){ new_x=bd_x2; }
  if(new_x<=bd_x1){ new_x=bd_x1; }

  if(p_x == r_x){
    // stick
    //  assert(cout << "sticking" << endl);
    if (new_x < bd_x1){ x1=bd_x1; x2=bd_x1+default_w; }
    if (new_x >= bd_x2-default_w){ x1=bd_x2-default_w; x2=bd_x2;}
  }
  else
    if(new_x != x1) x2 = new_x; // drag

  g_mutex_unlock(mutex);
  show();
  g_mutex_lock(mutex);

  if(x1>x2) swap(x1, x2);
    
  dragging=false;
  g_mutex_unlock(mutex);
  return true;
}

void zoom_cursor::move(int x, int y){ 
  g_mutex_lock(mutex);
  if(!cursor_inside){ g_mutex_unlock(mutex); return; }
  //     assert(cout 
  // 	   << "zc::move()  x1=" << x1 << " x2=" << x2 
  // 	   << " button_on=" << button_on << endl);

  if(!button_on){
    // not in draggin'
    x1 = x;
    x2 = x1+default_w;
    if(!check_bound(x2,y) && x2>=bd_x2){ x2=bd_x2; x1=x2-default_w; }
    if(!check_bound(x1,y) && x1<=bd_x1){ x1=bd_x1; x2=x1+default_w; }
  } else {
    // draggin' 
    if(check_bound(x,y)) x2 = x;
    dragging = true;
  }

  g_mutex_unlock(mutex);
  show();
}

void zoom_cursor::enter(){
  force_inside();
}

void zoom_cursor::leave(){
  g_mutex_lock(mutex);
  //  assert(cout << "zc::leave()" << endl);
  paste();

  int x = min(prev_x1, prev_x2);
  int y = cursor_y;
  int w = abs(prev_x2-prev_x1);
  int h = cursor_h;
  
  GdkRectangle region = { x+1, y+1, w-2, h-2 };
  gdk_window_invalidate_rect(canvas->window, &region, false);

  //  button_on = false;
  active = false;
  dragging = false;
  cursor_inside = false;
  
//   g_object_unref(cpb1);
//   g_object_unref(cpb2);
//   g_object_unref(cpb3);

  g_mutex_unlock(mutex);
}

void zoom_cursor::inactivate(){ 
  leave(); 
}

