#include "zoomer.h"
#include "gmv.h"
#include <iostream>

#define ZOOM_FACTOR (status.zoom_factor)

using namespace std;

zoom::zoom(){
  // use init(w,h) later.
}

zoom::zoom(int w, int h){
  init(w, h);
}

void zoom::init(int default_w, int default_h){
  int w, h;
  vector < size_wh > temp;
  
  steps.clear();
  
  // smaller ones
  w = default_w;
  h = default_h;

  while(1){
    if ( w < status.min_size || h < status.min_size ) break;
    temp.push_back(pair<int,int> (w,h));

    w = (int)(w* (double)1/ZOOM_FACTOR);
    h = (int)(h* (double)1/ZOOM_FACTOR);
  }

  for(unsigned int i=temp.size(); i>0; i--) steps.push_back(temp[i-1]);
  nowat=(int)steps.size()-1;
  if(nowat<0) nowat=0;

  default_size = nowat;

  // larger ones
  w = (int)(default_w * ZOOM_FACTOR);
  h = (int)(default_h * ZOOM_FACTOR);
  while(1){
    if ( w > status.max_size || h > status.max_size ) break;
    steps.push_back(pair<int,int> (w,h));
    w = (int)(w*ZOOM_FACTOR);
    h = (int)(h*ZOOM_FACTOR);
  }
}

size_wh zoom::current(){
  return steps.at(nowat);
}

size_wh zoom::reset(){
  nowat = default_size;
  return steps.at(nowat);
}

size_wh zoom::in(){
  nowat++;
  if(nowat == (int)steps.size()) nowat--;
  return steps.at(nowat);
}

size_wh zoom::out(){
  if( nowat>0 ) nowat--;
  return steps.at(nowat);
}

size_wh zoom::jump(int a){
  int i=a;

  if(i >= (int)steps.size()) i=(int)steps.size()-1;
  if(i < 0) i=0;

  nowat=i;
  return steps.at(nowat);
}

size_wh zoom::jump_by_width(int w){
  nowat=0;
  for(unsigned int i=0; i<steps.size(); i++){
    if(steps[i].first >= w){ nowat=i; break; }
  }
  return steps.at(nowat);
}

size_wh zoom::jump_by_height(int h){
  nowat=0;
  for(unsigned int i=0; i<steps.size(); i++){
    if(steps[i].second >= h){ nowat=i; break; }
  }
  return steps.at(nowat);
}

int zoom::sanityCheck(){

  cout << "sanityCheck(): step sizes " << steps.size() << endl;

  cout << "sanityCheck(): zoomsteps: ";
  for( vector<size_wh>::iterator i=steps.begin(); i!=steps.end(); i++ ){
    cout << "(" << i->first << "," << i->second << ") ";
  }
  cout << endl;

  cout << "now at: " <<  "(" << current().first << "," << current().second << ") " 
       << endl;

  return 1;
}

